#include "stdafx.h"
#include "SensorSim.h"
#include "EACircuit.h"
#include "SensorSimCore.h"
#include "crypto/sha1.h"
#include "crypto/md5.h"
#include "CommonFnc.h"


extern CSensorSimCore*	pSensorSim;

// THIS STRUCTURE MUST BE GLOBAL (UNFORTUNATELLY), AS GALIB NEEDS STATIC CALLBACKS 
// AND WE NEED THEM TO BE ABLE TO MANIPULATE CLASS MEMBERS
GA_CIRCUIT*		pGACirc = NULL;	

//#define STATIC_CIRCUIT

#ifdef STATIC_CIRCUIT
static void circuit(BYTE inputs[MAX_INPUTS], BYTE outputs[MAX_OUTPUTS]) {
    BYTE VAR_IN_0 = inputs[0];
    BYTE VAR_IN_1 = inputs[1];
    BYTE VAR_IN_2 = inputs[2];
    BYTE VAR_IN_3 = inputs[3];
    BYTE VAR_IN_4 = inputs[4];
    BYTE VAR_IN_5 = inputs[5];
    BYTE VAR_IN_6 = inputs[6];
    BYTE VAR_IN_7 = inputs[7];
    BYTE VAR_IN_8 = inputs[8];
    BYTE VAR_IN_9 = inputs[9];
    BYTE VAR_IN_10 = inputs[10];
    BYTE VAR_IN_11 = inputs[11];
    BYTE VAR_IN_12 = inputs[12];
    BYTE VAR_IN_13 = inputs[13];
    BYTE VAR_IN_14 = inputs[14];
    BYTE VAR_IN_15 = inputs[15];

    BYTE VAR_1_1_NOP = VAR_IN_1 ;
    BYTE VAR_1_2_NOP = VAR_IN_2 ;
    BYTE VAR_1_3_NOP = VAR_IN_3 ;
    BYTE VAR_2_1_ROR_7 = VAR_1_1_NOP >> 7 ;
    BYTE VAR_2_2_NOP = VAR_1_2_NOP ;
    BYTE VAR_2_3_SUB = VAR_1_3_NOP -  VAR_1_2_NOP - VAR_1_3_NOP - 0;
    BYTE VAR_3_0_XOR = VAR_2_3_SUB ^ 0;
    BYTE VAR_3_1_NOP = VAR_2_1_ROR_7 ;
    BYTE VAR_3_2_NOP = VAR_2_2_NOP ;
    BYTE VAR_4_1_OR_ = VAR_3_0_XOR | 0;
    BYTE VAR_4_2_SUB = VAR_3_2_NOP -  VAR_3_1_NOP - VAR_3_2_NOP - 0;
    BYTE VAR_5_1_ROL_1 = VAR_4_1_OR_ << 1 ;
    BYTE VAR_5_3_AND = VAR_4_2_SUB & 0xff;
    BYTE VAR_6_0_CONST_202 = 202 ;
    BYTE VAR_6_1_SUM = VAR_5_1_ROL_1 + VAR_5_3_AND + 0;
    BYTE VAR_7_0_NOP = VAR_6_0_CONST_202 ;
    BYTE VAR_7_1_NOP = VAR_6_1_SUM ;
    BYTE VAR_7_2_OR_ = VAR_6_1_SUM | 0;
    BYTE VAR_8_0_SUM = VAR_7_0_NOP + VAR_7_2_OR_ + 0;
    BYTE VAR_8_1_NOP = VAR_7_1_NOP ;
    BYTE VAR_9_1_SUB = VAR_8_1_NOP -  VAR_8_0_SUM - VAR_8_1_NOP - 0;
    BYTE VAR_10_1_NOP = VAR_9_1_SUB ;

    outputs[1] = VAR_10_1_NOP;

} 
#endif

CEACircuit::CEACircuit() {
    // ALLOW ALL FNCS:
    memset(m_gaCirc.allowedFNC, 1, sizeof(m_gaCirc.allowedFNC));    

	// ASSIGN POINTER TO INTERNAL MEMBER FOR ACCESS OF INTERNAL STATIC FUNCTION 	
	pGACirc = &m_gaCirc;
    pGACirc->allocate();        
}

CEACircuit::~CEACircuit() {
    m_gaCirc.release();
}

void CEACircuit::garng_initializer(GAGenome &g) {
    GA1DArrayGenome<DWORD> &genome = (GA1DArrayGenome<DWORD>&) g;
    int     offset = 0;
    
    // CLEAR GENOM
    for (int i = 0; i < genome.size(); i++) genome.gene(i, 0);
    
	// INITIALIZE ALL LAYERS
    for (int layer = 0; layer < 2 * pGACirc->numLayers; layer++) {
        offset = layer * MAX_INTERNAL_LAYER_SIZE; 
        // LAST LAYER CAN HAVE DIFFERENT NUMBER OF FUNCTIONS
        int numLayerInputs = 0;
        if (layer == 0) {
            // WHEN SECTORING IS USED THEN FIRST LAYER OBTAIN internalLayerSize INPUTS
            if (pGACirc->bSectorInputData) numLayerInputs = pGACirc->internalLayerSize; 
            else numLayerInputs = pGACirc->numInputs;
        }
        else numLayerInputs = pGACirc->internalLayerSize;
        
        int numFncInLayer = ((layer == 2 * pGACirc->numLayers - 1) || (layer == 2 * pGACirc->numLayers - 2)) ? pGACirc->outputLayerSize : pGACirc->internalLayerSize;
        
        for (int slot = 0; slot < numFncInLayer; slot++) {
            DWORD   value;
            if (layer % 2 == 0) {
                // CONNECTION SUB-LAYER
                if (layer / 2 < pGACirc->numSelectorLayers) {
                    // SELECTOR LAYER - TAKE INPUT ONLY FROM PREVIOUS NODE IN SAME COORDINATES
                    value = pGACirc->precompPow[slot];
                }
                else {
                    // FUNCTIONAL LAYERS
                    pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(pGACirc->precompPow[numLayerInputs], &value);
                    //value = 0xffffffff;
                }
                genome.gene(offset + slot, value);
            }
            else {
                // FUNCTION SUB-LAYER, SET ONLY ALLOWED FUNCTIONS  
                if (layer / 2 < pGACirc->numSelectorLayers) {
                    // SELECTOR LAYER - PASS INPUT WITHOUT CHANGES (ONLY SIMPLE COMPOSITION OF INPUTS IS ALLOWED)
                    genome.gene(offset + slot, FNC_XOR);
                }
                else {
                    // FUNCTIONAL LAYER
                    BOOL bFNCNotSet = TRUE;
                    while (bFNCNotSet) {
                        pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(FNC_MAX, &value);
                        if (pGACirc->allowedFNC[value] != 0) { 
                            genome.gene(offset + slot, value);
                            bFNCNotSet = FALSE;
                        }
                    }
                }
            }
        }
    }
}

int CEACircuit::garng_mutator(GAGenome &g, float pmut) {
    GA1DArrayGenome<DWORD> &genome = (GA1DArrayGenome<DWORD>&) g;
    float   result = 0;
    
    for (int layer = 0; layer < 2 * pGACirc->numLayers; layer++) {
        int offset = layer * MAX_INTERNAL_LAYER_SIZE; 

        int numLayerInputs = 0;
        if (layer == 0) {
            // WHEN SECTORING IS USED THEN FIRST LAYER OBTAIN internalLayerSize INPUTS
            if (pGACirc->bSectorInputData) numLayerInputs = pGACirc->internalLayerSize; 
            else numLayerInputs = pGACirc->numInputs;
        }
        else numLayerInputs = pGACirc->internalLayerSize;

        int numFncInLayer = ((layer == 2 * pGACirc->numLayers - 1) || (layer == 2 * pGACirc->numLayers - 2)) ? pGACirc->outputLayerSize : pGACirc->internalLayerSize;

        for (int slot = 0; slot < numFncInLayer; slot++) {
            DWORD   value;
            if (layer % 2 == 0) {
                // CONNECTION SUB-LAYER
                
                // MUTATE CONNECTION SELECTOR (FLIP ONE SINGLE BIT == ONE CONNECTOR)
                if (GAFlipCoin(pmut)) {
                    DWORD   temp;
                    pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(numLayerInputs, &value);
                    temp = pGACirc->precompPow[value];
                    // SWITCH RANDOMLY GENERATED BIT
                    temp ^= genome.gene(offset + slot);
                    genome.gene(offset + slot, temp);
                }
            }
            else {
                // FUNCTION SUB-LAYER (NOP, OR, AND) 
                
                // MUTATE FUNCTION TYPE USING ONLY ALLOWED FNCs
                if (GAFlipCoin(pmut)) {
/* ALL ALLOWED FUNCTIONS ARE NOW POSSIBLE IN SELECTOR LAYER, NOT ONLY XOR               
                    if (layer / 2 < pGACirc->numSelectorLayers) {
                        // SELECTOR LAYER - ONLY SIMPLE COMPOSITION OF INPUTS IS ALLOWED
                        genome.gene(offset + slot, FNC_XOR);
                    }
                    else {
/**/                    
                        BOOL bFNCNotSet = TRUE;
                        while (bFNCNotSet) {
                            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(FNC_MAX, &value);
                            if (pGACirc->allowedFNC[value] != 0) { 
                                genome.gene(offset + slot, value);
                                bFNCNotSet = FALSE;
                            }
                        }
//                    }
                }
            }
        }
    }


    return result;
}
int CEACircuit::garng_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    GA1DArrayGenome<DWORD> &parent1 = (GA1DArrayGenome<DWORD>&) p1;
    GA1DArrayGenome<DWORD> &parent2 = (GA1DArrayGenome<DWORD>&) p2;
    GA1DArrayGenome<DWORD> &offspring1 = (GA1DArrayGenome<DWORD>&) *o1;
    GA1DArrayGenome<DWORD> &offspring2 = (GA1DArrayGenome<DWORD>&) *o2;
    
    // CROSS ONLY WHOLE LAYERS
    int cpoint = GARandomInt(1,pGACirc->numLayers) * 2; // bod ken (v mezch od,do)
    for (int layer = 0; layer < 2 * pGACirc->numLayers; layer++) {
        int offset = layer * MAX_INTERNAL_LAYER_SIZE; 

        if (layer <= cpoint) {
            for (int i = 0; i < MAX_INTERNAL_LAYER_SIZE; i++) {
                if (o1 != NULL) offspring1.gene(offset + i, parent1.gene(offset + i));
                if (o2 != NULL) offspring2.gene(offset + i, parent2.gene(offset + i));
            }
        }
        else {
            for (int i = 0; i < MAX_INTERNAL_LAYER_SIZE; i++) {
                if (o1 != NULL) offspring1.gene(offset + i, parent2.gene(offset + i));
                if (o2 != NULL) offspring2.gene(offset + i, parent1.gene(offset + i));
            }
        }
    }
    return 1;
}

int CEACircuit::ExecuteCircuit(GA_CIRCUIT* pCircuit, GA1DArrayGenome<DWORD>* pGenome, BYTE inputs[MAX_INPUTS], BYTE outputs[MAX_OUTPUTS]) {  
    int     status = STAT_OK;
    BYTE*   inputsBegin = inputs;
    int     numSectors = 1;
    int     sectorLength = pCircuit->numInputs;
    BYTE    localInputs[MAX_INPUTS];
    
    memset(outputs, 0, MAX_OUTPUTS);
    
    // CHECK WHETHER INPUT DATA SHOULD BE PROCCESSED AS SECTORED INPUT IN MULTIPLE RUNS
    if (pGACirc->bSectorInputData) {
        // MULTIPLE CALL OF CIRCUIT
        
        // CIRCUIT OUTPUT DATA (ACTUAL "STATE") ARE PASSED AS CIRCUIT INPUT 
        // FRESH INPUT DATA IS PASSED TO REMAINING FREE INPUTS 
        sectorLength = pCircuit->internalLayerSize - pCircuit->outputLayerSize;
        // ALL INPUT DATA ARE DIVIDED INTO SEVERAL SECTORS BASED ON sectorLength
        if (sectorLength > 0) {
            numSectors = pCircuit->numInputs / sectorLength;
        }
        else {
            LOG_INSERTSTRING("SECTORING ENABLED AND NO NODES LEFT FOR INPUT - INCREASE INTERNAL LAYER SIZE OR DECREASE OUTPUT LAYER SIZE"); 
            numSectors = 0;
            status = STAT_USERDATA_BAD;
        }
    }
    else {
        // ALL IN ONE RUN
        numSectors = 1;
        sectorLength = pCircuit->numInputs;
    }
    
    for (int sector = 0; sector < numSectors; sector++) { 
        // PREPARE INPUTS FOR ACTUAL RUN OF CIRCUIT
        if (numSectors == 1) {
            // ALL INPUT DATA AT ONCE
            memcpy(localInputs, inputs, MAX_INPUTS);
        }
        else {
            // USE STATE (OUTPUT) AS FIRST PART OF INPUT
            memcpy(localInputs, outputs, pCircuit->outputLayerSize);
            // ADD FRESH INPUT DATA
            memcpy(localInputs + pCircuit->outputLayerSize, inputs + sector * sectorLength, sectorLength);
        }
        
        // EVALUATE CIRCUIT
        for (int layer = 1; layer < 2 * pCircuit->numLayers; layer = layer + 2) {
            int offsetCON = (layer-1) * MAX_INTERNAL_LAYER_SIZE; 
            int offsetFNC = (layer) * MAX_INTERNAL_LAYER_SIZE; 
            memset(outputs, 0, MAX_INTERNAL_LAYER_SIZE);

            // actual number of inputs for this layer. For first layer equal to pCircuit->numInputs, for next layers equal to number of function in intermediate layer pCircuit->internalLayerSize
            int numLayerInputs = 0;
            if (layer == 1) {
                // WHEN SECTORING IS USED THEN FIRST LAYER OBTAIN internalLayerSize INPUTS
                if (pGACirc->bSectorInputData) numLayerInputs = pCircuit->internalLayerSize; 
                else numLayerInputs = pCircuit->numInputs;
            }
            else numLayerInputs = pCircuit->internalLayerSize;

            // actual number of functions in layer - different for the last "output" layer
            int numFncInLayer = (layer == (2 * pCircuit->numLayers - 1)) ? pCircuit->outputLayerSize : pCircuit->internalLayerSize; 

            // SELECTOR LAYERS HAVE FULL INTERCONNECTION (CONNECTORS), OTHER HAVE SPECIFIED NUMBER 
    	    int	numLayerConnectors = pCircuit->numLayerConnectors;
            if (layer / 2 < pCircuit->numSelectorLayers) {
                numLayerConnectors = numLayerInputs;    
            }
    	    int	halfConnectors = (numLayerConnectors - 1) / 2;

            for (int slot = 0; slot < numFncInLayer; slot++) {
                BYTE    result = 0;
                DWORD   connect = 0;
                int     connectOffset = 0;
                int     stopBit = 0;
                
                // COMPUTE RANGE OF INPUTS FOR PARTICULAR slot FUNCTION
                connect = pGenome->gene(offsetCON + slot);
    			
			    if (numLayerConnectors > numLayerInputs) {
				    // NUMBER OF CONNECTORS IS HIGHER THAN NUMBER OF FUNCTIONS IN LAYER - CUT ON BOTH SIDES			
				    connectOffset = 0;
				    stopBit = numLayerInputs;
			    }
			    else {
				    connectOffset = slot - halfConnectors;
				    stopBit = numLayerConnectors;
				    // NUMBER OF CONNECTORS FIT IN - BUT SOMETIMES CANNOT BE CENTERED ON slot
				    if ((slot - halfConnectors < 0)) {
					    // WE ARE TO CLOSE TO LEFT SIDE - ADD MORE INPUTS FROM RIGHT SIDE
					    connectOffset = 0;
				    }
				    if ((slot + halfConnectors + 1 >= numLayerInputs)) { // +1 AS WE ARE TAKING CENTRAL NODE AS BELONGING TO RIGHT HALF
					    // WE ARE TO CLOSE TO RIGHT SIDE - ADD MORE INPUTS FROM LEFT SIDE
					    connectOffset = numLayerInputs - numLayerConnectors;
				    }
			    }

                switch (pGenome->gene(offsetFNC + slot)) {
                    case FNC_NOP: {
                        // DO NOTHING, JUST PASS VALUE FROM PREVIOUS LAYER
                        result = localInputs[slot];
                        break;
                    }
                    case FNC_OR: {
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result |= localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_AND: {
                        result = ~0; // ASSIGN ALL ONES AS STARTING VALUE TO PERFORM result &= LATER
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION, THEN TAKE INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result &= localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_CONST: {
                        // SEND VALUE FROM CONNECTION LAYER DIRECTLY TO OUTPUT
                        result = pGenome->gene(offsetCON + slot) % UCHAR_MAX;
                        break;
                    }
                    case FNC_XOR: {
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result ^= localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_NOR: {
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result |= ~(localInputs[connectOffset + bit]);
                            }
                        }
                        break;
                    }
                    case FNC_NAND: {
                        result = ~0; // ASSIGN ALL ONES AS STARTING VALUE TO PERFORM result &= LATER
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result &= ~(localInputs[connectOffset + bit]);
                            }
                        }
                        break;
                    }
                    case FNC_ROTL: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS
                        // MAXIMIM SHIFT IS 7
                        result = localInputs[slot] << (pGenome->gene(offsetCON + slot) & 0x07);
                        break;
                    }
                    case FNC_ROTR: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS
                        // MAXIMIM SHIFT IS 7
                        result = localInputs[slot] >> (pGenome->gene(offsetCON + slot) & 0x07);
                        break;
                    }
                    case FNC_BITSELECTOR: {
                        // BIT SELECTOR
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS
                        // OUTPUT ONLY BITS ON POSITION GIVEN BY THE MASK FROM CONNECTION LAYER
                        result = localInputs[slot] & (pGenome->gene(offsetCON + slot) & 0x07);
                        break;
                    }
                    case FNC_SUM: {
                        // SUM ALL INPUTS
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result += localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_SUBS: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS ONE
                        // AND SUBSTRACT ALL VALUES FROM CONNECTED INPUTS
                        result = localInputs[slot];
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result -= localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_ADD: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS ONE
                        // AND ADD ALL VALUES FROM CONNECTED INPUTS
                        result = localInputs[slot];
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result += localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_MULT: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS ONE
                        // AND MULTIPLY IT BY ALL VALUES FROM CONNECTED INPUTS
                        result = localInputs[slot];
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                result *= localInputs[connectOffset + bit];
                            }
                        }
                        break;
                    }
                    case FNC_DIV: {
                        // TAKE INPUT FROM THE FUNCTION FROM PREVIOUS LAYER ON THE SAME POSITION AS THIS ONE
                        // AND DIVIDE IT BY ALL VALUES FROM CONNECTED INPUTS
                        result = localInputs[slot];
                        for (int bit = 0; bit < stopBit; bit++) {
                            // IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE NEGATION OF INPUT
                            if (connect & (DWORD) pCircuit->precompPow[bit]) {
                                if (localInputs[connectOffset + bit] != 0) {
                                    result /= localInputs[connectOffset + bit];
                                }
                            }
                        }
                        break;
                    }
                    
                    default: {
                        ASSERT(FALSE);
                        break;
                    }
                }
                
                outputs[slot] = result;
            }
            // PREPARE INPUTS FOR NEXT LAYER FROM OUTPUTS
            memcpy(localInputs, outputs, pCircuit->internalLayerSize);
        }
    }
    
    return status;
}

float CEACircuit::garng_fitness(GAGenome &g) {
    int                     status = STAT_OK;
    GA1DArrayGenome<DWORD>   &genome = (GA1DArrayGenome<DWORD>&) g;
    CString                 message;
    float                   fit = 0;
    BYTE                    inputs[MAX_INPUTS];    
    BYTE                    outputs[MAX_OUTPUTS];  
    BYTE                    usePredictorMask[MAX_OUTPUTS];  
    int                     predictorMatch[MAX_OUTPUTS];  
    BYTE                    correctOutputs[MAX_OUTPUTS];  
    int                     match = 0;
    int                     numPredictions = 0; 
	int						testVectOffset = pGACirc->testVectStart;	
	int                     remainingTestVectors = pGACirc->numTestVectors;

	// IF VECTOR SPLIT IS REQUIRED, THAN COMPUTE SUCCESS OVER FIRST HALF OF TEST VECTORS,
    // SELECT numBestPredictors PREDICTORS WITH BEST PERFORMANCE AND EVALUATE AGAIN 
    if (pGACirc->bSplitTestVectors) {
		match = 0;
		numPredictions = 0;
		memset(usePredictorMask, 1, sizeof(usePredictorMask));	// USE ALL PREDICTORS
		memset(predictorMatch, 0, sizeof(predictorMatch));
        memset(outputs, 0, MAX_OUTPUTS);
		for (int testSet = pGACirc->testVectStart; testSet < (pGACirc->testVectStart + pGACirc->numTestVectors / 2); testSet++) { 
			// LOAD TEST SET INTO inputs
			memcpy(inputs, pGACirc->testVectors[testSet], pGACirc->numInputs);
			// LOAD TEST SET INTO outputs
			memcpy(correctOutputs, pGACirc->testVectors[testSet] + MAX_INPUTS, MAX_OUTPUTS);
	        
			// EXECUTE CIRCUIT
			status = CEACircuit::ExecuteCircuit(pGACirc, &genome, inputs, outputs);

#ifdef STATIC_CIRCUIT
            BYTE    outputs2[MAX_OUTPUTS];  
            memset(outputs2, 0, MAX_OUTPUTS);
            circuit(inputs, outputs2);
            //ASSERT(outputs[1] == outputs2[1]);
            for (int i = 0; i < MAX_OUTPUTS; i++) ASSERT(outputs[i] == outputs2[i]);
#endif
			// EVALUATE SUCCESS OF CIRCUIT FOR THIST TEST VECTOR
			CEACircuit::EvaluateCircuitSuccess(outputs, correctOutputs, usePredictorMask, pGACirc->outputLayerSize, pGACirc->predictMethod, &match, &numPredictions, predictorMatch);

			testVectOffset = testSet;
			
			if (status != STAT_OK) break;
		}

		// SELECT PREDICTORS WITH BEST SUCCESS
		memset(usePredictorMask, 0, sizeof(usePredictorMask));	// USE NO PREDICTORS
		int maxMatch = 0;
		int maxMatchIndex = 0;
		for (int i = 0; i < pGACirc->numBestPredictors; i++) {
		    maxMatch = 0;
		    maxMatchIndex = 0;
		    for (int j = 0; j < pGACirc->outputLayerSize; j++) {
		        if (maxMatch < predictorMatch[j]) {
		            maxMatch = predictorMatch[j];
		            maxMatchIndex = j;
		        }
		    }
		    // SELECT BEST REMAINING PREDICTOR FOR USAGE
		    predictorMatch[maxMatchIndex] = 0;
		    usePredictorMask[maxMatchIndex] = 1;
		}
		
		remainingTestVectors = pGACirc->numTestVectors / 2;
	}
	else {
		memset(usePredictorMask, 1, sizeof(usePredictorMask));	// USE ALL PREDICTORS
		remainingTestVectors = pGACirc->numTestVectors;
	}

    if (status == STAT_OK) {
	    match = 0;
	    numPredictions = 0;
	    for (int testSet = testVectOffset; testSet < (testVectOffset + remainingTestVectors); testSet++) { 
            // LOAD TEST SET INTO inputs
            memcpy(inputs, pGACirc->testVectors[testSet], pGACirc->numInputs);
            // LOAD TEST SET INTO outputs
            memcpy(correctOutputs, pGACirc->testVectors[testSet] + MAX_INPUTS, MAX_OUTPUTS);
            
            // EXECUTE CIRCUIT
            status = CEACircuit::ExecuteCircuit(pGACirc, &genome, inputs, outputs);
/*
            BYTE    outputs2[MAX_OUTPUTS];  
            memset(outputs2, 0, MAX_OUTPUTS);
            circuit(inputs, outputs2);
            ASSERT(outputs[0] == outputs2[0])
/**/            
            // EVALUATE SUCCESS OF CIRCUIT FOR THIST TEST VECTOR
            CEACircuit::EvaluateCircuitSuccess(outputs, correctOutputs, usePredictorMask, pGACirc->outputLayerSize, pGACirc->predictMethod, &match, &numPredictions);

			if (status != STAT_OK) break;
        }
    }
    
    if (status == STAT_OK) {
        // EVALUATE SUCCESS OF CIRCUIT OVER WHOLE TEST SET
        fit = (numPredictions > 0) ? (match / ((float) numPredictions)) : 0;
        
        // include into average fitness of whole generation
        pGACirc->avgGenerFit += fit;
        pGACirc->numAvgGenerFit++;
        pGACirc->avgPredictions += numPredictions;

        if (fit > pGACirc->bestGenerFit) pGACirc->bestGenerFit = fit;

        if (pGACirc->maxFit < fit) {
            pGACirc->maxFit = fit;
            
            message.Format("NEW MAXIMUM FOUND: %f", fit); 
            LOG_INSERTSTRING(message); 
            
            // DISPLAY CURRENTLY BEST
		    CString filePath;
		    filePath.Format("circuit_%f", fit);
            CEACircuit::PrintCircuit(genome, filePath, usePredictorMask, FALSE);   // PRINT WITHOUT PRUNNING
		    filePath += "_prunned";
            CEACircuit::PrintCircuit(genome, filePath, usePredictorMask, TRUE);    // PRINT WITH PRUNNING
        }
    }
        
    return fit;
}

int CEACircuit::EvaluateCircuitSuccess(BYTE* outputs, BYTE* correctOutputs, BYTE* usePredictorsMask, int numOutputs, int predictType, int* pMatch, int* pTotalPredictCount, int* predictorMatch) {
    int     status = STAT_OK;
    
    switch (predictType) {
        case PREDICT_BIT: {
            // OUTPUT LAYER ENCODES DIRECTLY THE EXPECTED BITS
            // COUNT NUMBER OF CORRECT BITS 
            for (int out = 0; out < numOutputs; out++) {
                for (int bit = 0; bit < NUM_BITS; bit++) {
					if (usePredictorsMask[out] == 1) {
						// COMPARE VALUE ON bit-th POSITION
						if ((outputs[out] & (BYTE) pGACirc->precompPow[bit]) == (correctOutputs[out] & (BYTE) pGACirc->precompPow[bit])) {
							(*pMatch)++;
							if (predictorMatch != NULL) (predictorMatch[out])++;
						}
	                    
						(*pTotalPredictCount)++;                    
					}
                }
            }
            break;
        }
        case PREDICT_BITGROUP_PARITY: {
            // OUTPUT LAYER ENCODES THE GROUP OF BITS AND THEIR PARITY
            // SYNTAX: EACH BYTE B OF OUTPUT ENCODE POSITION OF ONE SINGLE BIT OF OUTPUT, IF B IS EVEN, THEN IS NOT USED 
            // PARITY OF FIRST BYTE OF OUTPUT IS PREDICTED PARITY OF GROUP OF BYTES 
            // COUNT NUMBER OF CORRECT BITS 
            int predictParity = 0;
            int correctParity = 0;
            int offsetBlock = 0;
            int offsetBit = 0;
            int numBitsPredicted = 0;
            
            BYTE usedBits[MAX_OUTPUTS * NUM_BITS];
            memset(usedBits, 0, sizeof(usedBits));
            
            // GET PREDICTION OF PARITY (parity of first output bit) 
            for (int bit = 0; bit < NUM_BITS; bit++) {
                if (outputs[0] & (BYTE) pGACirc->precompPow[bit]) predictParity++;
            }
            predictParity = (predictParity & 0x01) ? 1 : 0; 
            
            // COMPUTE REAL PAIRITY OF SIGNALIZED BITS
            for (int out = 1; out < numOutputs; out++) {
                if (outputs[out] & 0x80) { // mask out highest bit (used/not used flag)
                    // THIS OUTPUT WILL BE USED 
                    // OBTAIN VALUE OF ENCODED BIT
                    offsetBit = outputs[out] & 0x7f;  // take all bits except used/not used bit
                    
                    // CHECK IF THIS BIT WAS NOT ALREADY USED AND DO NOT EXCEED THE NUMBER OF BITS IN OUTPUT
                    if (usedBits[offsetBit] == 0 && (offsetBit < pGACirc->numOutputs * NUM_BITS)) {
                        // MARK THIS BIT AS USED TO PREVENT MULTIPLE SELECTION OF THE SAME BITS
                        usedBits[offsetBit] = 1;
                        
                        offsetBlock = (int) offsetBit / NUM_BITS;
                        offsetBit = offsetBit - (offsetBlock * NUM_BITS);
                        ASSERT(offsetBit < NUM_BITS);
                        ASSERT(offsetBlock < pGACirc->numOutputs);

                        if (correctOutputs[offsetBlock] & (BYTE) pGACirc->precompPow[offsetBit]) {
                            // SIGNALIZED BIT HAS VALUE 1
                            correctParity++;
                        }
                        
                        // AT LEAST ONE BIT WAS PREDICTED
                        numBitsPredicted++;
                    }
                }
                else {
                    // NOT USED
                }
            }
            correctParity = (correctParity & 0x01) ? 1 : 0; 

            // DO NOT ALLOW SOLUTIONS THAT DO PREDICT NO BITS
            if (numBitsPredicted >= 1) {
                //message.Format("%d", numBitsPredicted);
                //LOG_INSERTSTRING(message, FALSE); 
                
				if (predictParity == correctParity) {
					(*pMatch)++;      
				}
                (*pTotalPredictCount)++;                    
            }
            break;
        }
        case PREDICT_BYTES_PARITY: {
            // PARITY OF BYTES IN OUTPUT LAYER SHOULD MATCH PARITY IN CORRECT OUTPUT
            int predictParity = 0;
            int correctParity = 0;
            
            for (int out = 0; out < numOutputs; out++) {
				if (usePredictorsMask[out] == 1) {
					predictParity = 0;
					correctParity = 0;
					// GET PREDICTION OF PARITY, if hamming weight > NUM_BITS/2 then parity = 0, otherwise pariy = 1
					for (int bit = 0; bit < NUM_BITS; bit++) {
						if (outputs[out] & (BYTE) pGACirc->precompPow[bit]) predictParity++;
					}
					if (predictParity > (NUM_BITS / 2)) predictParity = 0;
					else predictParity = 1;
	                
					// GET REAL PARITY 
					for (int bit = 0; bit < NUM_BITS; bit++) {
						if (correctOutputs[out] & (BYTE) pGACirc->precompPow[bit]) correctParity++;
					}
					correctParity = (correctParity & 0x01) ? 1 : 0; 
	                
					// CHECK IF MATCH
					if (predictParity == correctParity) {
						(*pMatch)++;
						if (predictorMatch != NULL) (predictorMatch[out])++;
					}
					(*pTotalPredictCount)++;                    
				}
            }
            
            break;
        }
        case PREDICT_HAMMING_WEIGHT: {
            // HAMMIING WEIGHT OF BYTES IN OUTPUT LAYER SHOULD MATCH WEIGHT IN CORRECT OUTPUT
            int predictWeight = 0;
            int correctWeight = 0;
            
            for (int out = 0; out < numOutputs; out++) {
				if (usePredictorsMask[out] == 1) {
					predictWeight = 0;
					correctWeight = 0;
					// GET PREDICTION OF WEIGHT 
					for (int bit = 0; bit < NUM_BITS; bit++) {
						if (outputs[out] & (BYTE) pGACirc->precompPow[bit]) predictWeight++;
					}
	                
					// GET REAL WEIGHT 
					for (int bit = 0; bit < NUM_BITS; bit++) {
						if (correctOutputs[out] & (BYTE) pGACirc->precompPow[bit]) correctWeight++;
					}

					// OBTAIN POINTS FOR CORRECTNESS - IF COMPLETELY MATCH, THAN MAXIMUM POINTS ARE OBTAINED
					int points = NUM_BITS - abs(predictWeight - correctWeight);
					*pMatch += points;
					if (predictorMatch != NULL) (predictorMatch[out])+= points;
					(*pTotalPredictCount)++;         

/*	                
					// CHECK IF MATCH
					if (predictWeight == correctWeight) {
						(*pMatch)++;
						if (predictorMatch != NULL) (predictorMatch[out])++;
					}
					(*pTotalPredictCount)++;         
/**/
				}
            }
            
            break;
        }
        
        default: {
            status = STAT_USERDATA_BAD;
            break;
        }
    }
    
    return status;
}

int CEACircuit::GetFunctionLabel(DWORD functionID, DWORD connections, CString* pLabel) {
    int		status = STAT_OK;
    switch (functionID) {
        case FNC_NOP: *pLabel = "NOP"; break;
        case FNC_OR: *pLabel = "OR_"; break;
        case FNC_AND: *pLabel = "AND"; break;
//        case FNC_CONST: *pLabel = "CON"; break;
        case FNC_CONST: {
            pLabel->Format("CONST_%u", connections % UCHAR_MAX); 
            break;
        }
        case FNC_XOR: *pLabel = "XOR"; break;
        case FNC_NOR: *pLabel = "NOR"; break;
        case FNC_NAND: *pLabel = "NAN"; break;
//        case FNC_ROTL: *pLabel = "ROL"; break;
        case FNC_ROTL: {
            pLabel->Format("ROL_%u", connections & 0x07); 
            break;
        }
//        case FNC_ROTR: *pLabel = "ROR"; break;
        case FNC_ROTR: {
            pLabel->Format("ROR_%u", connections & 0x07); 
            break;
        }
//        case FNC_BITSELECTOR: *pLabel = "BSL"; break;
        case FNC_BITSELECTOR: {
            pLabel->Format("BSL_%u", connections & 0x07); 
            break;
        }
        case FNC_SUM: *pLabel = "SUM"; break;
        case FNC_SUBS: *pLabel = "SUB"; break;
        case FNC_ADD: *pLabel = "ADD"; break;
        case FNC_MULT: *pLabel = "MUL"; break;
        case FNC_DIV: *pLabel = "DIV"; break;
        default: {
            ASSERT(FALSE);
            *pLabel = "ERR";
            status = STAT_USERDATA_BAD;
        }
    }
    
    return status;
}

int CEACircuit::PruneCircuit(GAGenome &g, GAGenome &prunnedG) {
    int                     status = STAT_OK;
    GA1DArrayGenome<DWORD>  &genome = (GA1DArrayGenome<DWORD>&) g;
    GA1DArrayGenome<DWORD>  &prunnedGenome = (GA1DArrayGenome<DWORD>&) prunnedG;
    BOOL                    bChangeDetected = FALSE;
    
    // CREATE LOCAL COPY
    for (int i = 0; i < genome.size(); i++) {
        prunnedGenome.gene(i, genome.gene(i)); 
    }
    
    if (pGACirc->prunningInProgress) { 
        // WE ARE ALREDY PERFORMING PRUNING - DO NOT CONTINUE TO PREVENT OVERLAPS
    }
    else {
        //
        // METHOD - TRY TO TEMPORARY REMOVE CONNECTION/FUNCTION AND TEST FITNESS CHANGES
        //
        
        pGACirc->prunningInProgress = TRUE;        
        
        float origFit = garng_fitness(prunnedGenome);
        
        int prunneRepeat = 0; 
        bChangeDetected = TRUE;
        while (bChangeDetected && prunneRepeat < 10) {
            bChangeDetected = FALSE;
            prunneRepeat++;
            
            CString message;
            message.Format("PRUNNING ITERATION: '%d'", prunneRepeat);
            LOG_INSERTSTRING(message);
            
            // DISABLE GENES STARTING FROM END 
            for (int i = prunnedGenome.size() - 1; i >= 0; i--) {
                DWORD   origValue = prunnedGenome.gene(i);
                
                if (origValue != 0) {
                    // PRUNE FNC AND CONNECTION LAYER DIFFERENTLY
                    if (((i / MAX_INTERNAL_LAYER_SIZE) % 2) == 1) {
                        // FNCs LAYER - TRY TO SET AS NOP INSTRUCTION
                        prunnedGenome.gene(i, 0);
                        
                        ASSERT(origValue < FNC_MAX);
                        
                        float newFit = garng_fitness(prunnedGenome);
                        if (origFit > newFit) {
                            // GENE WAS IMPORTANT, SET BACK 
                            prunnedGenome.gene(i, origValue);
                        }
                        else {
                            bChangeDetected = TRUE;
                            if (origFit < newFit) {
                                // GENE WAS HARMING, KEEP REMOVED
                                CString message;
                                message.Format("Gene %d was harming, better fitness: %f", i, newFit);
                                //LOG_INSERTSTRING(message);
                            }
                            else {
                                // GENE WAS NOT IMPORTANT, KEEP REMOVED
                            }
                        }
                    }
                    else {
                        DWORD   tempOrigValue = origValue;  // WILL HOLD MASK OF INPORTANT CONNECTIONS
                        // CONNECTION LAYER - TRY TO REMOVE CONNECTIONS GRADUALLY
                        for (int conn = 0; conn < MAX_CONNECTORS; conn++) {
                            DWORD   newValue = tempOrigValue & (~pGACirc->precompPow[conn]);
                            
                            if (newValue != tempOrigValue) {
                                prunnedGenome.gene(i, newValue);
                                
                                float newFit = garng_fitness(prunnedGenome);
                                if (origFit > newFit) {
                                    // GENE WAS IMPORTANT, DO NOT REMOVE CONNECTION
                                }
                                else {
                                    bChangeDetected = TRUE;
                    
                                    // STORE NEW VALUE WITHOUT CONNECTION
                                    tempOrigValue = newValue;
                                    
                                    if (origFit < newFit) {
                                        // GENE WAS HARMING, REMOVED
                                        CString message;
                                        message.Format("Gene %d, connection %d was harming, better fitness: %f", i, conn, newFit);
                                        //LOG_INSERTSTRING(message);
                                    }
                                    else {
                                        // GENE WAS NOT IMPORTANT, REMOVE
                                    }
                                }
                            }
                        }
                        
                        // SET FINAL PRUNNED VALUE
                        prunnedGenome.gene(i, tempOrigValue);
                    }
                }
            }
        }
    
        pGACirc->prunningInProgress = FALSE;        
    }
    
    garng_fitness(prunnedGenome);    
    
    return status;
}

int CEACircuit::GetUsedNodes(GAGenome &g, BYTE usePredictorMask[MAX_OUTPUTS], BYTE displayNodes[MAX_GENOME_SIZE]) {
	int	status = STAT_OK;
    GA1DArrayGenome<DWORD>  &genome = (GA1DArrayGenome<DWORD>&) g;
	
	//
	// BUILD SET OF USED NODES FROM OUTPUT TO INPUT
	//
	
	// ADD OUTPUT NODES
    // VISUAL CIRC: CONNECT OUTPUT LAYER
    int offsetFNC = (2 * pGACirc->numLayers - 1) * MAX_INTERNAL_LAYER_SIZE; 
    for (int i = 0; i < pGACirc->outputLayerSize; i++) {
		if (usePredictorMask == NULL || usePredictorMask[i] == 1) {
			// ADD THIS ONE TO LIST OF USED NODES 
			displayNodes[offsetFNC + i] = 1;	
		}
    }
	
	
	// PROCESS ALL LAYERS FROM BACK
    for (int layer = 2 * pGACirc->numLayers - 1; layer > 0; layer = layer - 2) {
        int offsetCON = (layer-1) * MAX_INTERNAL_LAYER_SIZE; 
        int offsetFNC = (layer) * MAX_INTERNAL_LAYER_SIZE; 

        // actual number of inputs for this layer. For first layer equal to pCircuit->numInputs, for next layers equal to number of function in intermediate layer pCircuit->internalLayerSize
        int numLayerInputs = 0;
        if (layer == 1) {
            // WHEN SECTORING IS USED THEN FIRST LAYER OBTAIN internalLayerSize INPUTS
            if (pGACirc->bSectorInputData) numLayerInputs = pGACirc->internalLayerSize; 
            else numLayerInputs = pGACirc->numInputs;
        }
        else numLayerInputs = pGACirc->internalLayerSize;

        // actual number of functions in layer - different for the last "output" layer
        int numFncInLayer = (layer == (2 * pGACirc->numLayers - 1)) ? pGACirc->outputLayerSize : pGACirc->internalLayerSize; 

        // SELECTOR LAYERS HAVE FULL INTERCONNECTION (CONNECTORS), OTHER HAVE SPECIFIED NUMBER 
	    int	numLayerConnectors = pGACirc->numLayerConnectors;
        if (layer / 2 < pGACirc->numSelectorLayers) {
            numLayerConnectors = numLayerInputs;    
        }
	    int	halfConnectors = (numLayerConnectors - 1) / 2;

        for (int slot = 0; slot < numFncInLayer; slot++) {
            BYTE    result = 0;
            DWORD   connect = 0;
            int     connectOffset = 0;
            int     stopBit = 0;
            
            // ANALYZE ONLY SUCH NODES THAT ARE ALREADY IN USED SET
			if (displayNodes[offsetFNC + slot] == 1) {
				// COMPUTE RANGE OF INPUTS FOR PARTICULAR slot FUNCTION
				connect = genome.gene(offsetCON + slot);
				
				if (numLayerConnectors > numLayerInputs) {
					// NUMBER OF CONNECTORS IS HIGHER THAN NUMBER OF FUNCTIONS IN LAYER - CUT ON BOTH SIDES			
					connectOffset = 0;
					stopBit = numLayerInputs;
				}
				else {
					connectOffset = slot - halfConnectors;
					stopBit = numLayerConnectors;
					// NUMBER OF CONNECTORS FIT IN - BUT SOMETIMES CANNOT BE CENTERED ON slot
					if ((slot - halfConnectors < 0)) {
						// WE ARE TO CLOSE TO LEFT SIDE - ADD MORE INPUTS FROM RIGHT SIDE
						connectOffset = 0;
					}
					if ((slot + halfConnectors + 1 >= numLayerInputs)) { // +1 AS WE ARE TAKING CENTRAL NODE AS BELONGING TO RIGHT HALF
						// WE ARE TO CLOSE TO RIGHT SIDE - ADD MORE INPUTS FROM LEFT SIDE
						connectOffset = numLayerInputs - numLayerConnectors;
					}
				}

				for (int bit = 0; bit < stopBit; bit++) {
					BOOL	bImplicitConnection = FALSE;
					BOOL	bExplicitConnection = FALSE;
					bExplicitConnection = HasConnection(genome.gene(offsetFNC + slot), connect, slot, connectOffset, bit, &bImplicitConnection);
					if (bImplicitConnection || bExplicitConnection) {
						if (layer > 1) {
							int prevOffsetFNC = (layer - 2) * MAX_INTERNAL_LAYER_SIZE; 
							// ADD PREVIOUS NODE 	
							displayNodes[prevOffsetFNC + connectOffset + bit] = 1;
						}
					}		
				}
			}
		}
	}
	
	return status;
}        


BOOL CEACircuit::HasConnection(DWORD functionID, DWORD connectionMask, int fncSlot, int connectionOffset, int bit, BOOL* pbImplicitConnection) {
    BOOL    status = FALSE;
    
    // DEFAULT: IF SIGNALIZED IN MASK, THAN ALLOW CONNECTION
    // SELECTED INSTRUCTION MAY CHANGE LATER
    if (connectionMask & (DWORD) pGACirc->precompPow[bit]) status = TRUE;
    
    // IMPLICIT CONNECTION MAY EXIST
    switch (functionID) {
        case FNC_CONST: {
            // NO CONNECTION
            status = FALSE;
            break;
        }
        
        case FNC_SUBS: // no break
        case FNC_ADD:  // no break
        case FNC_MULT: // no break 
        case FNC_DIV: {
            // ALLOW *ALSO* CONNECTION FROM PREVIOUS LAYER ON SAME OFFSET
            if (fncSlot == connectionOffset + bit) {
                *pbImplicitConnection = TRUE;
            }
            break;
        }                    

        case FNC_NOP:  // no break
        case FNC_ROTL:  // no break
        case FNC_ROTR:  // no break
        case FNC_BITSELECTOR: { 
            // ALLOW *ONLY* CONNECTION FROM PREVIOUS LAYER ON SAME OFFSET
            if (fncSlot == connectionOffset + bit) {
                *pbImplicitConnection = TRUE;
                
                // NO EXPLICIT CONNECTION POSSIBLE  
                status = FALSE;
            }
            else status = FALSE;
            break;
        }                    
    }
    
    return status;
}

BOOL CEACircuit::HasImplicitConnection(DWORD functionID) {
    BOOL    status = FALSE;
    
    switch (functionID) {
        case FNC_NOP: // no break
        case FNC_SUBS: // no break
        case FNC_ADD:  // no break
        case FNC_MULT: // no break 
        case FNC_DIV:  // no break
        case FNC_ROTL:  // no break
        case FNC_ROTR:  // no break
        case FNC_BITSELECTOR: { 
            status = TRUE;
            break;
        }                    
    }
        
    return status;
}


BOOL CEACircuit::IsOperand(DWORD functionID, DWORD connectionMask, int fncSlot, int connectionOffset, int bit, CString* pOperand) {
    BOOL    status = FALSE;
    
    
    // DEFAULT: IF SIGNALIZED IN MASK, THAN ALLOW CONNECTION
    // SELECTED INSTRUCTION MAY CHANGE LATER
    if (connectionMask & (DWORD) pGACirc->precompPow[bit]) status = TRUE;
    
    // IMPLICIT CONNECTION MAY EXIST
    switch (functionID) {
        case FNC_NOP:  {
            *pOperand = ""; status = FALSE; break;
        }
        case FNC_OR: {
            pOperand->Format("|"); break;    
        }
        case FNC_AND: {
            pOperand->Format("&"); break;    
        }
        case FNC_CONST: {
            if (fncSlot == connectionOffset + bit) {
                pOperand->Format("%u", connectionMask % UCHAR_MAX);
                status = TRUE;
            }
            else status = FALSE;
            break;
        }
        case FNC_XOR: {
            pOperand->Format("^"); break;    
        }
        case FNC_NOR: {
            pOperand->Format("| ~"); break;    
        }
        case FNC_NAND: {
            pOperand->Format("& ~"); break;    
        }
        case FNC_ROTL: {
            if (fncSlot == connectionOffset + bit) {
                pOperand->Format("%u", connectionMask & 0x07);     
                status = TRUE;
            }
            else status = FALSE;
            break;
        }
        case FNC_ROTR: {
            if (fncSlot == connectionOffset + bit) {
                pOperand->Format("%u", connectionMask & 0x07);     
                status = TRUE;
            }
            else status = FALSE;
            break;
        }
        case FNC_BITSELECTOR: {
            if (fncSlot == connectionOffset + bit) {
                pOperand->Format("&");     
                status = TRUE;
            }
            else status = FALSE;
            break;
        }
        case FNC_SUM: {
            pOperand->Format("+"); break;
        }
        case FNC_SUBS: {
            pOperand->Format("-"); break;     
        }
        case FNC_ADD: {
            pOperand->Format("+"); break;    
        }  
        case FNC_MULT: {
            pOperand->Format("*"); break;    
        }
        case FNC_DIV: {
            pOperand->Format("/"); break;     
        }
        default: {
            *pOperand = "!!!";
            status = TRUE;
        }
    }
    
    return status;
}

int CEACircuit::GetNeutralValue(DWORD functionID, CString* pOperand) {
    int    status = STAT_OK;
    
    switch (functionID) {
        case FNC_OR: // no break
        case FNC_XOR: 
        case FNC_NAND: 
        case FNC_SUBS: 
        case FNC_SUM: 
        case FNC_ADD: {
            *pOperand = "0"; break;    
        }
        
        case FNC_MULT:
        case FNC_DIV: {
            *pOperand = "1"; break;    
        }

        case FNC_AND: 
        case FNC_NOR: { 
            *pOperand = "0xff"; break;    
        }

        case FNC_NOP: 
        case FNC_CONST: 
        case FNC_ROTL: 
        case FNC_BITSELECTOR: 
        case FNC_ROTR: {
            *pOperand = ""; break;    
        }

        default: {
            *pOperand = "!!!";
            status = TRUE;
        }
    }

    return status;
}


void CEACircuit::GenerateTestData(int testVectGenerMethod, int numTestVectors, int numRounds) {
	switch (testVectGenerMethod) {
        case TESTVECT_MD5INV: {
            pGACirc->numInputs = MD5_DIGEST_LENGTH;
            pGACirc->numOutputs = MD5_DIGEST_LENGTH;
            MD5_CTX mdContext;
            BYTE    temp[MD5_DIGEST_LENGTH];

            // Set TO DEFAULT ROUNDS IF NOT LIMITED
            if (numRounds == -1) numRounds = 64;

            // GENERATE NEW TEST SET
            for (int testSet = 0; testSet < numTestVectors; testSet++) {
                // MD5 HASH INVERSION
                // PSEUDO-RANDOM INPUT DATA
                for (int output = 0; output < MAX_OUTPUTS; output++) pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(pGACirc->testVectors[testSet][MAX_INPUTS + output]));                
                MD5_Init(&mdContext);
                for (int input = 0; input < pGACirc->numInputs; input++) temp[input] = pGACirc->testVectors[testSet][MAX_INPUTS + input];
                MD5_Update(&mdContext, temp, pGACirc->numInputs, numRounds);
                MD5_Final(&mdContext, numRounds);
                for (int input = 0; input < pGACirc->numOutputs; input++) pGACirc->testVectors[testSet][input] = mdContext.digest[input];
            }        
            break;
		}
		case TESTVECT_SHA1INV: {
            pGACirc->numInputs = SHA1_DIGEST_SIZE;
            pGACirc->numOutputs = SHA1_DIGEST_SIZE;

            sha1_ctx        shaCtx;

            BYTE    temp[SHA1_DIGEST_SIZE];

            // Set TO DEFAULT ROUNDS IF NOT LIMITED
            if (numRounds == -1) numRounds = SHA1_ROUNDS;

            // GENERATE NEW TEST SET
            for (int testSet = 0; testSet < numTestVectors; testSet++) {
                // SHA-1 HASH INVERSION
                // PSEUDO-RANDOM INPUT DATA
                for (int output = 0; output < MAX_OUTPUTS; output++) pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(pGACirc->testVectors[testSet][MAX_INPUTS + output]));                
				sha1_begin(&shaCtx);
                for (int input = 0; input < pGACirc->numInputs; input++) temp[input] = pGACirc->testVectors[testSet][MAX_INPUTS + input];

                sha1_hash(&shaCtx, temp, pGACirc->numInputs, numRounds);
	            BYTE message_digest_array[SHA1_DIGEST_SIZE];
                sha1_end(&shaCtx, message_digest_array, numRounds);
                for (int input = 0; input < SHA1_DIGEST_SIZE; input++) pGACirc->testVectors[testSet][input] = message_digest_array[input];
            }        
            break;
		}
		case TESTVECT_TEST: {
/*
		    pGACirc->numInputs = MAX_INPUTS;
			pGACirc->numOutputs = MAX_OUTPUTS;
/**/
		    pGACirc->numInputs = 16;
			pGACirc->numOutputs = 15;

			for (int testSet = 0; testSet < numTestVectors; testSet++) {
				// RANDOM DATA
				if (pGACirc->representBitAsBytes) for (int input = 0; input < pGACirc->numInputs; input++) pGACirc->testVectors[testSet][input] = rand() % 2;
				else for (int input = 0; input < pGACirc->numInputs; input++) pGACirc->testVectors[testSet][input] = rand() % 256;

				// XOR NEIGHBOURS 
				if (pGACirc->representBitAsBytes) {
					for (int output = 0; output < pGACirc->numOutputs; output++) {
						if (((pGACirc->testVectors[testSet][output] ^ pGACirc->testVectors[testSet][output + 1] ^ pGACirc->testVectors[testSet][output + 2]) & 0x01) == 1) pGACirc->testVectors[testSet][MAX_INPUTS + output] = 0xff;
						else pGACirc->testVectors[testSet][MAX_INPUTS + output] = 0x00;
					}
				}
				else for (int output = 0; output < pGACirc->numOutputs; output++) pGACirc->testVectors[testSet][MAX_INPUTS + output] = pGACirc->testVectors[testSet][output] ^ pGACirc->testVectors[testSet][output + 1];
			}

/**/            
/*
            pGACirc->numInputs = 16;
            pGACirc->numOutputs = 15;
            for (int testSet = 0; testSet < numTestVectors; testSet++) {
                // RANDOM DATA
                for (int input = 0; input < pGACirc->numInputs; input++) pGACirc->testVectors[testSet][input] = rand() % 256;
                // XOR NEIGHBOURS 
                for (int output = 0; output < pGACirc->numOutputs; output++) pGACirc->testVectors[testSet][MAX_INPUTS + output] = pGACirc->testVectors[testSet][output] ^ pGACirc->testVectors[testSet][output + 1];
            }
/**/            
            break;
       }
       case TESTVECT_MD5SHA_DISTINGUISH: {
            // DISTINGUISHER BETWEEN MD5 AND SHA1 OUTPUT STREAM
            // INPUTS: MD5 OR SHA1 HASH
            // OUTPUTS: 0x00 for MD5, 0xff for SHA1
            pGACirc->numInputs = MAX_INPUTS;
            pGACirc->numOutputs = 16;

            MD5_CTX         mdContext;
            sha1_ctx        shaCtx;
            BYTE            data[MD5_DIGEST_LENGTH];
            
            for (int testSet = 0; testSet < numTestVectors; testSet++) {
                // RANDOMLY DECIDE WHETHER MD5 OR SHA1 WILL BE USED
                BYTE    hashFnc = 0;    
                pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(2, &hashFnc);
                //hashFnc = rand() % 2;
                if (hashFnc == 0) {
                    // USE MD5
                    // SIGNALIZE MD5 BY SETTING OUTPUT AS 0x00
                    for (int i = 0; i < MAX_OUTPUTS; i++) pGACirc->testVectors[testSet][MAX_INPUTS + i] = 0x00;
                    // RANDOM DATA AS HASH INPUT (64 x 16B blocks)
                    int offset = 0;
                    for (int sector = 0; sector < MAX_INPUTS / MD5_DIGEST_LENGTH; sector++) { 
                        offset = sector * MD5_DIGEST_LENGTH; 
                        memset(data, 0, sizeof(data));
                        for (int input = 0; input < MD5_DIGEST_LENGTH; input++) pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(data[input]));
                        //for (int input = 0; input < pGACirc->numInputs; input++) data[input] = rand() % 256;
                        MD5Init(&mdContext);
                        MD5Update(&mdContext, data, MD5_DIGEST_LENGTH);
                        MD5Final(&mdContext);
                        for (int input = 0; input < MD5_DIGEST_LENGTH; input++) pGACirc->testVectors[testSet][offset + input] = mdContext.digest[input];
                    }
                }
                else {
                    // USE SHA1
                    // SIGNALIZE SHA1 BY SETTING OUTPUT AS 0xff
                    for (int i = 0; i < MAX_OUTPUTS; i++) pGACirc->testVectors[testSet][MAX_INPUTS + i] = 0xff;
                    int offset = 0;
                    for (int sector = 0; sector < MAX_INPUTS / MD5_DIGEST_LENGTH; sector++) { 
                        offset = sector * MD5_DIGEST_LENGTH; 
                        memset(data, 0, sizeof(data));
                        for (int input = 0; input < MD5_DIGEST_LENGTH; input++) pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(data[input]));
                        sha1_begin(&shaCtx);
                        sha1_hash(&shaCtx, data, MD5_DIGEST_LENGTH, numRounds);
			            BYTE message_digest_array[20];
                        sha1_end(&shaCtx, message_digest_array, numRounds);
                        for (int input = 0; input < MD5_DIGEST_LENGTH; input++) pGACirc->testVectors[testSet][offset + input] = message_digest_array[input];
                    }
                }
            }
            
            break;
       }
       case TESTVECT_SHA1_RAND_DISTINGUISH: // no break
       case TESTVECT_MD5_RAND_DISTINGUISH: {
            // DISTINGUISHER BETWEEN MD5 OUTPUT AND RANDOM STREAM
            // INPUTS: MD5/SHA1 HASH OR RANDOM STREAM
            // OUTPUTS: 0x00 for MD5/SHA1, 0xff for RANDOM
            
            //pGACirc->numInputs = MAX_INPUTS;
            
            int numInputs = (testVectGenerMethod == TESTVECT_SHA1_RAND_DISTINGUISH) ? SHA1_DIGEST_SIZE : MD5_DIGEST_LENGTH;
            
            pGACirc->numInputs = numInputs;
            pGACirc->numOutputs = numInputs;

            MD5_CTX         mdContext;
            BYTE            data[MD5_DIGEST_LENGTH];
            
			// RANDOM DATA ARE TAKEN EITHER FROM LOCAL RANDOM DATA STORAGE OF QRGBS (http://random.irb.hr/index.php) 
			// OR FROM BUILD-IN GENERATOR IF NOT AVAILABLE (!! NOT RECOMENDED FOR REAL USAGE AS IS ALSO BASED ON MD5)
			#define		QRGBS_PATH				"C:\\!RNG\\"
			#define		SINGLE_FILE_SIZE		10000000
			#define		SINGLE_FILE_INDEX_MAX	100
			
			CFile	file;
			BOOL	bQRGBS = FALSE;		// IS LOCAL RNG STORAGE AVAILABLE?
			int		offsetQRGBS = 0;	// OFFSET INSIDE RANDOM FILE
			BYTE*	randomData = new BYTE[SINGLE_FILE_SIZE];

            int		fileIndex;
            // GET RANDOMLY INDEX OF ONE FILE
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(SINGLE_FILE_INDEX_MAX, &fileIndex);
			CString fileName; fileName.Format("%s\\Random%.3d.bin", QRGBS_PATH, fileIndex);
			if (file.Open(fileName, CFile::modeRead)) {
				bQRGBS = TRUE;
				
				// READ CONTENT OF FILE
				if (SINGLE_FILE_SIZE > file.GetLength()) file.Read(randomData, file.GetLength());
				else file.Read(randomData, SINGLE_FILE_SIZE);
				
				// GET RANDOM OFFSET INSIDE FILE
                pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(SINGLE_FILE_SIZE - 2 * numInputs, &offsetQRGBS);
                //file.Seek(offsetQRGBS, CFile::begin); 
                file.Close();
			}
			else bQRGBS = FALSE;

			
            for (int testSet = 0; testSet < numTestVectors; testSet++) {
                // RANDOMLY DECIDE WHETHER MD5/SHA1 OR RANDOM STREAM WILL BE USED
                BYTE    hashFnc = 0;    

				// GET WHETHERT MD5/SHA1 OR RANDOM WILL BE USED
				if (bQRGBS) {
					hashFnc = randomData[++offsetQRGBS]; 
					//file.Read(&hashFnc, 1);
					hashFnc %= 2;
				}
				else pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(2, &hashFnc);

                if (hashFnc == 0) {
                    // USE MD5/SHA1
                    // SIGNALIZE MD5/SHA1 BY SETTING OUTPUT AS 0x00
                    for (int i = 0; i < MAX_OUTPUTS; i++) pGACirc->testVectors[testSet][MAX_INPUTS + i] = 0x00;
                    // RANDOM DATA AS HASH INPUT 
                    int offset = 0;
                    for (int sector = 0; sector < pGACirc->numInputs / numInputs; sector++) { 
                        offset = sector * numInputs; 
                        memset(data, 0, sizeof(data));
                        for (int input = 0; input < numInputs; input++) {
							if (bQRGBS) {
								//file.Read(&(data[input]), 1);
								data[input] = randomData[++offsetQRGBS]; 
							}
							else pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(data[input]));
						}
                        
                        // INTRODUCE REDUNDANCY INTO INPUT: REPEAT ONLY TWO BYTES                        
                        for (int input = 0; input < numInputs; input = input + 2) {
							data[input] = data[0];
							data[input + 1] = data[1];
						}	
                        
						if (testVectGenerMethod == TESTVECT_SHA1_RAND_DISTINGUISH) {
				            sha1_ctx        shaCtx;
							sha1_begin(&shaCtx);
							sha1_hash(&shaCtx, data, pGACirc->numInputs, numRounds);
							BYTE message_digest_array[SHA1_DIGEST_SIZE];
							sha1_end(&shaCtx, message_digest_array, numRounds);
							for (int input = 0; input < SHA1_DIGEST_SIZE; input++) pGACirc->testVectors[testSet][offset + input] = message_digest_array[input];
						}

						if (testVectGenerMethod == TESTVECT_MD5_RAND_DISTINGUISH) {
							MD5_Init(&mdContext);
							MD5_Update(&mdContext, data, MD5_DIGEST_LENGTH, numRounds);
							MD5_Final(&mdContext, numRounds);
							for (int input = 0; input < MD5_DIGEST_LENGTH; input++) pGACirc->testVectors[testSet][offset + input] = mdContext.digest[input];
						}
                    }
                }
                else {
                    // USE RANDOM
                    // SIGNALIZE RANDOM STREAM BY SETTING OUTPUT AS 0xff
                    for (int i = 0; i < MAX_OUTPUTS; i++) pGACirc->testVectors[testSet][MAX_INPUTS + i] = 0xff;
                    int offset = 0;
                    for (int sector = 0; sector < pGACirc->numInputs / MD5_DIGEST_LENGTH; sector++) { 
                        offset = sector * MD5_DIGEST_LENGTH; 
                        for (int input = 0; input < MD5_DIGEST_LENGTH; input++) {
							if (bQRGBS) {
								//file.Read(&(pGACirc->testVectors[testSet][offset + input]), 1);
								pGACirc->testVectors[testSet][offset + input] = randomData[++offsetQRGBS]; 
							}
							else pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(255, &(pGACirc->testVectors[testSet][offset + input]));
						}
                    }
                }
                
				// GET RANDOM OFFSET INSIDE FILE
                pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(SINGLE_FILE_SIZE - 2 * MD5_DIGEST_LENGTH, &offsetQRGBS);
                
                //if (bQRGBS) file.Close();
            }

			delete[] randomData;

            break;
       }
       default: {
            ASSERT(FALSE);
       }
   }
   
    // IF INPUT DATA ARE TOO LONG TO BE PROCESSED BY SINGLE LAYER, THAN ENABLE DATA SECTORING
    if (pGACirc->numInputs > MAX_INTERNAL_LAYER_SIZE) {
        pGACirc->bSectorInputData = TRUE;    
    }   
}

UINT CEACircuit::RunGACircuit(LPVOID pParam) {
    UINT                status = STAT_OK;
	TEST_THREAD_PARAM*	pThreadParam = (TEST_THREAD_PARAM*) pParam;

    DeleteFile("EAC_fitnessProgress.txt");

	// CREATE STRUCTURE OF CIRCUIT FROM BASIC SETTINGS
	pGACirc = &(pThreadParam->basicSettings.gaCircuitConfig);
    // ALLOCATE STRUCTURE FOR OCCASSINALY HIGHER PRECISION TEST (10x MORE TEST VECTORS)
    int     numTestVectors = pGACirc->numTestVectors;
    int     maxTestVectors = numTestVectors * 10;
    pGACirc->release();        
    pGACirc->numTestVectors = maxTestVectors;
    pGACirc->allocate();        
    // SET NUMBER OF TEST VECTORS TO STANDARD (LOWER) NUMBER
    pGACirc->numTestVectors = numTestVectors;

    if (status == STAT_OK) {
        srand((unsigned int) time(NULL));
        
        // SAVE USED VALUES
        CString                   message;  
		message.Format("\n\n###NumLayers: %d, InLayerSize: %d, OutLayerSize: %d, Num connectors: %d, NumInputs: %d, TestSetSize: %d\n\n",pGACirc->numLayers, pGACirc->internalLayerSize, pGACirc->outputLayerSize, pGACirc->numLayerConnectors, pGACirc->numInputs, pGACirc->numTestVectors); 
        LOG_INSERTSTRING(message); 

        
        // 
        // CREATE TEST DATA
        //
        if (!pThreadParam->basicSettings.gaCircuitConfig.limitAlgRounds) pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount = -1; 
        CEACircuit::GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount);
            
        //
        //  CREATE GA STRUCTS
        //
        GA1DArrayGenome<DWORD>    genom(pGACirc->genomeSize, garng_fitness);
        GA1DArrayGenome<DWORD>    genomeTemp(pGACirc->genomeSize, garng_fitness);

        genom.initializer(garng_initializer); 
        genom.mutator(garng_mutator); 
        genom.crossover(garng_crossover); 

        GASteadyStateGA ga(genom);
        ga.nReplacement(2 * pThreadParam->basicSettings.gaConfig.popSize / 3);  
        ga.populationSize(pThreadParam->basicSettings.gaConfig.popSize); 
        ga.nGenerations(pThreadParam->basicSettings.gaConfig.nGeners);
        ga.pCrossover(pThreadParam->basicSettings.gaConfig.pCross);
        ga.pMutation(pThreadParam->basicSettings.gaConfig.pMutt); 

        ga.initialize();

        int     actGener = 1;
        
        pGACirc->clearFitnessStats();
        
        //while (actGener < pThreadParam->basicSettings.gaConfig.nGeners) {
        while (TRUE) {
            // CHECk FOR SOFT INTERUPT
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                CString value;
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                // DELETE THREAD PARAM STRUCT
                delete pThreadParam;
                // SET ACTIVE THREAD PTR TO NULL
                pSensorSim->m_pActualThread = NULL;
                
                return 0;
            }        
        
            // CHOOSE RANDOM TEST SUBSET ON WHICH INDIVIDIALS WILL BE TESTED
            // pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(TEST_SET_SIZE - pGACircuit->numTestVectors - 1, &testSetStart);
            
            pGACirc->testVectStart = 0;        
        
            if ((pGACirc->testVectorChangeGener == 0) || (actGener % pGACirc->testVectorChangeGener == 0)) {
		        if (!pThreadParam->basicSettings.gaCircuitConfig.limitAlgRounds) pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount = -1; 
                GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount);
            }
            
            // RESET EVALUTION FOR ALL GENOMS
            ga.pop->flushEvalution();
            ga.step(); // GA evolution step

            if (actGener % 100 == 0) {
                // GENERATE FRESH SET AND EVALUATE ONLY THE BEST ONE
                // SET NUMBER OF TEST VECTORS TO MAXIMUM NUMBER
                pGACirc->numTestVectors = maxTestVectors;
		        if (!pThreadParam->basicSettings.gaCircuitConfig.limitAlgRounds) pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount = -1; 
                GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount);
                genomeTemp = (GA1DArrayGenome<DWORD>&) ga.statistics().bestIndividual(); 
                float bestFit = garng_fitness(genomeTemp);
                
                // REVERT NUMBER OF TEST VECTORS TO STANDARD NUMBER
                pGACirc->numTestVectors = numTestVectors;
                
//                PrintCircuit(genomeTemp);
                
                message.Format("(%d gen.): %f avg, %f best, avgPredict: %d, totalBest: %f", actGener, pGACirc->avgGenerFit / pGACirc->numAvgGenerFit, bestFit, pGACirc->avgPredictions / pGACirc->numAvgGenerFit, pGACirc->maxFit); 
                LOG_INSERTSTRING(message); 

				// SAVE FITNESS PROGRESS
				message.Format("%d %f\r\n", actGener, bestFit); 
				CCommonFnc::File_AppendString("EAC_fitnessProgress.txt", message);


				// SAVE ACTUAL STATUS INTO RUNNING INSTANCES STRING
                CSensorEnv::SaveInstanceLog(message);

                pGACirc->bestGenerFit = 0;
                pGACirc->avgGenerFit = 0;
                pGACirc->numAvgGenerFit = 0;
                pGACirc->avgPredictions = 0;
            }
            
            actGener++;
        }    
        
        // GENERATE FRESH NEW SET
        if (!pThreadParam->basicSettings.gaCircuitConfig.limitAlgRounds) pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount = -1; 
        GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, pThreadParam->basicSettings.gaCircuitConfig.limitAlgRoundsCount);
        
        genomeTemp = (GA1DArrayGenome<DWORD>&) ga.statistics().bestIndividual(); 
        float fit = garng_fitness(genomeTemp);
        message.Format("Total best: (%d gen.): %f", actGener, fit); 
        LOG_INSERTSTRING(message); 
    }    
    
    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}

int CEACircuit::ParseCircuit(CString textCircuit, DWORD* circuit, int* numLayers, int* intLayerSize, int* outLayerSize) {
    int         status = STAT_OK;
    int         pos = 0;
    int         pos2 = 0;
    
    // CHECK IF CIRCUIT IS GIVEN IN FORM OF GENOM ONLY OR TEXT REPRESENTATION
    if (textCircuit.Find("[") != -1) {
		// TEXT REPRESENTATION
		*numLayers = 0;
		while ((pos2 = textCircuit.Find(";", pos)) != -1) {
			CString line = textCircuit.Mid(pos, pos2 - pos + 1);
			line.TrimLeft();line.TrimRight(); 
			if (line.Find("[") != -1) {
				// AT LEAST ONE FNC ELEMENT PRESENT => NEW layer
				int offsetCON = (*numLayers * 2) * MAX_INTERNAL_LAYER_SIZE; 
				int offsetFNC = (*numLayers * 2 + 1) * MAX_INTERNAL_LAYER_SIZE; 
				int pos3 = 0;
				int pos4 = 0;
				int slot = 0;
				while ((pos4 = line.Find("]", pos3)) != -1) {
					// PARSE ELEMENTS
					CString elem = line.Mid(pos3, pos4 - pos3 + 1);
					elem.TrimLeft();elem.TrimRight(); 
	                
					// CONNECTOR LAYER
					//DWORD conn1 = atol(elem);
					DWORD conn = (DWORD) atof(elem);	// atol fail to correctly convert large number 2147483648 (convert to 214748364*7*) instead 
					//ASSERT(conn1 == conn);
	                
					// FUNCTION
					DWORD fnc = FNC_NOP;
					CString fncStr = elem.Right(5);
					if (fncStr.CompareNoCase("[NOP]") == 0) fnc = FNC_NOP;
					if (fncStr.CompareNoCase("[OR_]") == 0) fnc = FNC_OR;
					if (fncStr.CompareNoCase("[AND]") == 0) fnc = FNC_AND;
					if (fncStr.CompareNoCase("[CON]") == 0) fnc = FNC_CONST;
					if (fncStr.CompareNoCase("[XOR]") == 0) fnc = FNC_XOR;
					if (fncStr.CompareNoCase("[NOR]") == 0) fnc = FNC_NOR;
					if (fncStr.CompareNoCase("[NAN]") == 0) fnc = FNC_NAND;
					if (fncStr.CompareNoCase("[ROL]") == 0) fnc = FNC_ROTL;
					if (fncStr.CompareNoCase("[ROR]") == 0) fnc = FNC_ROTR;
					if (fncStr.CompareNoCase("[BSL]") == 0) fnc = FNC_BITSELECTOR;
					if (fncStr.CompareNoCase("[SUM]") == 0) fnc = FNC_SUM;
					if (fncStr.CompareNoCase("[SUB]") == 0) fnc = FNC_SUBS;
					if (fncStr.CompareNoCase("[ADD]") == 0) fnc = FNC_ADD;
					if (fncStr.CompareNoCase("[MUL]") == 0) fnc = FNC_MULT;
					if (fncStr.CompareNoCase("[DIV]") == 0) fnc = FNC_DIV;
	                
					circuit[offsetCON + slot] = conn;
					circuit[offsetFNC + slot] = fnc;
	                
					slot++; 
					pos3 = pos4 + 2;
				}
	            
				if (*numLayers == 0) *intLayerSize = slot; // NUMBER OF INTERNAL ELEMENTS IS SAME AS FOR FIRST LAYER
				*outLayerSize = slot; // WILL CONTAIN NUMBER OF ELEMENTS IN LAST LAYER
	            
				*numLayers = (*numLayers) + 1;
			}      
			pos = pos2 + 1;
		}
    }
    else {
		// TODO: OBTAIN CORRECT NUMBER OF LAYERS ETC - NOW TAKEN FROM USER INPUT

		// GENOM ONLY 
		int offset = 0;
		while ((pos2 = textCircuit.Find(" ", pos)) != -1) {
			CString value = textCircuit.Mid(pos, pos2 - pos + 1);
			value.TrimLeft();value.TrimRight(); 
			
			//DWORD temp = atol(value);
			circuit[offset] = (DWORD) atof(value);	// atol fail to correctly convert large number 2147483648 (convert to 214748364*7*) instead 
//			ASSERT(temp == circuit[offset]);
			
			offset++;
			pos = pos2 + 1;
		}
    }
    
    return status;
}

int CEACircuit::PrintCircuit(GAGenome &g, CString filePath, BYTE usePredictorMask[MAX_OUTPUTS], BOOL bPruneCircuit) {
    int                     status = STAT_OK;
    GA1DArrayGenome<DWORD>  &inputGenome = (GA1DArrayGenome<DWORD>&) g;
    GA1DArrayGenome<DWORD>  genome(MAX_GENOME_SIZE, CEACircuit::garng_fitness);
    CString                 message;
    CString                 value;
    CString                 value2;
    CString					visualCirc = "";
    CString					visualCircLabel = "";
    CString					codeCirc = "";
	CString 				actualSlotID; 
	CString 				previousSlotID; 
	BOOL                    bCodeCircuit = TRUE;
	BYTE					displayNodes[MAX_GENOME_SIZE];
	
	
    //
    // PRUNE CIRCUIT IF REQUIRED
    //
    if (bPruneCircuit) {
        // PRUNE
        status = PruneCircuit(inputGenome, genome);    
        bCodeCircuit = TRUE;
        
		// COMPUTE NODES TO DISPLAY 
		memset(displayNodes, 0, sizeof(displayNodes));
		status = GetUsedNodes(genome, usePredictorMask, displayNodes);        
    }
    else {
        // JUST COPY
        for (int i = 0; i < genome.size(); i++) {
			genome.gene(i, inputGenome.gene(i)); 
		}
		// ALL NODES WILL BE DISPLAYED
		memset(displayNodes, 1, sizeof(displayNodes));
    }
    
    
    // VISUAL CIRC: INPUTS 
    visualCirc += "digraph EACircuit {\r\n\
rankdir=BT;\r\n\
edge [dir=none];\r\n\
size=\"6,6\";\r\n\
ordering=out;\r\n\
node [color=lightblue2, style=filled];\r\n";

    // CODE CIRCUIT: 
    if (bCodeCircuit) codeCirc += "static void circuit(BYTE inputs[MAX_INPUTS], BYTE outputs[MAX_OUTPUTS]) {\r\n";

    message += "\r\n";
    for (int layer = 1; layer < 2 * pGACirc->numLayers; layer = layer + 2) {
        int offsetCON = (layer-1) * MAX_INTERNAL_LAYER_SIZE; 
        int offsetFNC = (layer) * MAX_INTERNAL_LAYER_SIZE; 

        int numLayerInputs = 0;
        if (layer == 1) {
            // WHEN SECTORING IS USED THEN FIRST LAYER OBTAIN internalLayerSize INPUTS
            if (pGACirc->bSectorInputData) numLayerInputs = pGACirc->internalLayerSize; 
            else numLayerInputs = pGACirc->numInputs;
            
			// VISUAL CIRC: INPUTS 
			for (int i = 0; i < numLayerInputs; i++) {
				actualSlotID.Format("IN_%d", i);
				value2.Format("\"%s\";\r\n", actualSlotID); 
				visualCirc += value2;
				
                if (bCodeCircuit) {
				    value2.Format("    BYTE VAR_%s = inputs[%d];\r\n", actualSlotID, i); 
				    codeCirc += value2;
				}
			}
    		codeCirc += "\r\n";
			
        }
        else numLayerInputs = pGACirc->internalLayerSize;

        int numFncs = pGACirc->internalLayerSize;
        // IF DISPLAYING THE LAST LAYER, THEN DISPLAY ONLY 'INTERNAL_LAYER_SIZE' FNC (OTHERS ARE UNUSED)
        if (layer == (2 * pGACirc->numLayers - 1)) numFncs = pGACirc->outputLayerSize;

		//
		// VISUAL CIRC: PUT ALL NODES FROM SAME LAYER INTO SAME RANK
		//
        value2 = "{ rank=same; ";
        for (int slot = 0; slot < numFncs; slot++) {
            // USE ONLY GENES THAT WERE NOT PRUNNED OUT
            //if (inputGenome.gene(offsetFNC + slot) == genome.gene(offsetFNC + slot)) {
            if (displayNodes[offsetFNC + slot] == 1) {
			    GetFunctionLabel(genome.gene(offsetFNC + slot), genome.gene(offsetCON + slot), &value);
			    actualSlotID.Format("\"%d_%d_%s\"; ", layer / 2 + 1, slot, value);
                value2 += actualSlotID; 
            }
		}
		value2 += "}\r\n";
        visualCirc += value2;

        // DISCOVER AND DRAW CONNECTIONS
        for (int slot = 0; slot < numFncs; slot++) {
			GetFunctionLabel(genome.gene(offsetFNC + slot), genome.gene(offsetCON + slot), &value);
            
			// SELECTOR LAYERS HAVE FULL INTERCONNECTION (CONNECTORS), OTHER HAVE SPECIFIED NUMBER 
			int	numLayerConnectors = pGACirc->numLayerConnectors;
			if (layer / 2 < pGACirc->numSelectorLayers) {
				numLayerConnectors = numLayerInputs;    
			}
			int	halfConnectors = (numLayerConnectors - 1) / 2;

			DWORD effectiveCon = genome.gene(offsetCON + slot);
			value2.Format("%.10u[%s]  ", effectiveCon, value);
			message += value2;
            
			// 
			// VISUAL CIRC: CREATE CONNECTION BETWEEN LAYERS
			//
			// USE ONLY GENES THAT WERE NOT PRUNNED OUT
//			if (inputGenome.gene(offsetFNC + slot) == genome.gene(offsetFNC + slot)) {
            if (displayNodes[offsetFNC + slot] == 1) {
				actualSlotID.Format("%d_%d_%s", layer / 2 + 1, slot, value);

				int connectOffset = 0;
				int stopBit = 0;
				if (numLayerConnectors > numLayerInputs) {
					// NUMBER OF CONNECTORS IS HIGHER THAN NUMBER OF FUNCTIONS IN LAYER - CUT ON BOTH SIDES			
					connectOffset = 0;
					stopBit = numLayerInputs;
				}
				else {
					connectOffset = slot - halfConnectors;
					stopBit = numLayerConnectors;
					// NUMBER OF CONNECTORS FIT IN - BUT SOMETIMES CANNOT BE CENTERED ON slot
					if ((slot - halfConnectors < 0)) {
						// WE ARE TO CLOSE TO LEFT SIDE - ADD MORE INPUTS FROM RIGHT SIDE
						connectOffset = 0;
					}
					if ((slot + halfConnectors + 1 >= numLayerInputs)) { // +1 AS WE ARE TAKING CENTRAL NODE AS BELONGING TO RIGHT HALF
						// WE ARE TO CLOSE TO RIGHT SIDE - ADD MORE INPUTS FROM LEFT SIDE
						connectOffset = numLayerInputs - numLayerConnectors;
					}
				}
			 
				BOOL    bFirstArgument = TRUE;
				BOOL    bAtLeastOneConnection = FALSE;
			    
				// CODE CIRCUIT: 
				if (bCodeCircuit) {
					switch (genome.gene(offsetFNC + slot)) {
						case FNC_CONST: {
							value2.Format("    BYTE VAR_%s = %u", actualSlotID, effectiveCon % UCHAR_MAX); 
							codeCirc += value2;
							bFirstArgument = FALSE;
							bAtLeastOneConnection = TRUE;
							break;
						}
						case FNC_NOR: {
							value2.Format("    BYTE VAR_%s = 0 | ~", actualSlotID); 
							codeCirc += value2;
							bFirstArgument = FALSE;
							bAtLeastOneConnection = TRUE;
							break;
						}
						case FNC_NAND: {
							value2.Format("    BYTE VAR_%s = 0xff & ~ ", actualSlotID); 
							codeCirc += value2;
							bFirstArgument = FALSE;
							bAtLeastOneConnection = TRUE;
							break;
						}
		                
					}
				}
			    
				if (bCodeCircuit) {
					// DIV AND SUBST HAVE PREVIOUS LAYER SLOT AS BASIC INPUT
					// USE ONLY GENES THAT WERE NOT PRUNNED OUT
					if (HasImplicitConnection(genome.gene(offsetFNC + slot))) {
						if (layer > 1) {
							int prevOffsetFNC = (layer - 2) * MAX_INTERNAL_LAYER_SIZE; 
							int prevOffsetCON = (layer - 3) * MAX_INTERNAL_LAYER_SIZE; 
							GetFunctionLabel(genome.gene(prevOffsetFNC + slot), genome.gene(prevOffsetCON + slot), &value);
							previousSlotID.Format("%d_%d_%s", layer / 2, slot, value);
						}
						else {
							previousSlotID.Format("IN_%d", slot);
						}
					    
						value2.Format("    BYTE VAR_%s =", actualSlotID); 
						codeCirc += value2;
	                    					    
						switch (genome.gene(offsetFNC + slot)) {
							case FNC_NOP: value2.Format(" VAR_%s", previousSlotID); break; 
							case FNC_SUBS: value2.Format(" VAR_%s - ", previousSlotID); break; 
							case FNC_ADD: value2.Format(" VAR_%s + ", previousSlotID); break; 
							case FNC_MULT: value2.Format(" VAR_%s * ", previousSlotID); break; 
							case FNC_DIV: value2.Format(" VAR_%s / ", previousSlotID); break; 
							case FNC_ROTL: value2.Format(" VAR_%s << %u", previousSlotID, effectiveCon & 0x07); break; 
							case FNC_ROTR: value2.Format(" VAR_%s >> %u", previousSlotID, effectiveCon & 0x07); break; 
							case FNC_BITSELECTOR: value2.Format(" VAR_%s & %u", previousSlotID, effectiveCon & 0x07); break; 
							default: value2 = "!!!";
						}
						codeCirc += value2;

						bFirstArgument = FALSE;
						bAtLeastOneConnection = FALSE;
					}
				}
			    
				for (int bit = 0; bit < stopBit; bit++) {
					// IF 1 IS ON bit-th POSITION (CONNECTION LAYER), THEN TAKE INPUT
					BOOL    bImplicitConnection = FALSE;
					BOOL    bExplicitConnection = FALSE;
	                
					bExplicitConnection = HasConnection(genome.gene(offsetFNC + slot), effectiveCon, slot, connectOffset, bit, &bImplicitConnection);
					if (bImplicitConnection || bExplicitConnection) {
						bAtLeastOneConnection = TRUE;
	                    
						if (layer > 1) {
							int prevOffsetFNC = (layer - 2) * MAX_INTERNAL_LAYER_SIZE; 
							int prevOffsetCON = (layer - 3) * MAX_INTERNAL_LAYER_SIZE; 
							GetFunctionLabel(genome.gene(prevOffsetFNC + connectOffset + bit), genome.gene(prevOffsetCON + connectOffset + bit), &value);
							previousSlotID.Format("%d_%d_%s", layer / 2, connectOffset + bit, value);
						}
						else {
							previousSlotID.Format("IN_%d", connectOffset + bit);
						}
						if (bExplicitConnection) {
							value2.Format("\"%s\" -> \"%s\";\r\n", actualSlotID, previousSlotID); 
							visualCirc += value2;
						}
						if (bImplicitConnection) {
							// USE ONLY GENES THAT WERE NOT PRUNNED OUT
							value2.Format("\"%s\" -> \"%s\" [color=red];\r\n", actualSlotID, previousSlotID); 
							visualCirc += value2;
						}
	                    
						if (bCodeCircuit) {
							CString operand;
							if (bFirstArgument) {
								value2.Format("    BYTE VAR_%s =", actualSlotID); 
								codeCirc += value2;
								bFirstArgument = FALSE;
							}
							if (bExplicitConnection) {
								if (IsOperand(genome.gene(offsetFNC + slot), effectiveCon, slot, connectOffset, bit, &operand)) {
									if (genome.gene(offsetFNC + slot) == FNC_DIV) {
										// SPECIAL FORMATING TO PREVENT ZERO DIVISION
										value2.Format(" ((VAR_%s != 0) ? VAR_%s : 1) %s", previousSlotID, previousSlotID, operand); 
									}
									else {
										// NORMAL FORMATING
										value2.Format(" VAR_%s %s", previousSlotID, operand); 
									}
									codeCirc += value2;
								}
							}
						}
					}
				}
	            
				if (bCodeCircuit) {
					if (bAtLeastOneConnection) {
						// ADD FINALIZING VALUE THAT IS NEUTRAL TO GIVEN OPERATION (E.G. 0 FOR XOR)
						CString operand;
						GetNeutralValue(genome.gene(offsetFNC + slot), &operand);
						value2.Format(" %s", operand); 
						codeCirc += value2;
						codeCirc += ";\r\n";
					}
					else {
						// NO CONNECTION TO GIVEN FUNCTION, OUTPUT ZERO
						value2.Format("    BYTE VAR_%s = 0", actualSlotID); 
						codeCirc += value2;
						codeCirc += ";\r\n";
					}
				}
			}
        }
        message += ";\r\n";
    }
    
    if (bCodeCircuit) codeCirc += "\r\n";
    
    // VISUAL CIRC: CONNECT OUTPUT LAYER
    for (int i = 0; i < pGACirc->outputLayerSize; i++) {
		if (usePredictorMask == NULL || usePredictorMask[i] == 1) {
			value2.Format("node [color=red];\r\n"); 
			visualCirc += value2;
		}
		else {
			value2.Format("node [color=lightblue2];\r\n"); 
			visualCirc += value2;
		}
		
		if (!bPruneCircuit || usePredictorMask == NULL || usePredictorMask[i] == 1) {
		    actualSlotID.Format("%d_OUT", i);
		    int prevOffsetFNC = (2 * pGACirc->numLayers - 1) * MAX_INTERNAL_LAYER_SIZE; 
		    int prevOffsetCON = (2 * pGACirc->numLayers - 2) * MAX_INTERNAL_LAYER_SIZE; 
		    CString		value;
		    GetFunctionLabel(genome.gene(prevOffsetFNC + i), genome.gene(prevOffsetCON + i), &value);
		    previousSlotID.Format("%d_%d_%s", pGACirc->numLayers, i, value);
		    value2.Format("\"%s\" -> \"%s\";\r\n", actualSlotID, previousSlotID); 
		    visualCirc += value2;
		    
            if (bCodeCircuit) {
			    value2.Format("    outputs[%d] = VAR_%s;\r\n", i, previousSlotID); 
			    codeCirc += value2;
            }
		}
    }
    
    if (bCodeCircuit) codeCirc += "\r\n}";
    
    message += "\r\n";
    LOG_INSERTSTRING_NOTIME(message); 

    visualCirc += "}";

	//
	//	STORE IN SPECIFIED FILES
	//
	if (filePath == "") filePath = "EAC_circuit";

	CFile	file;
	CString	newFilePath;

	// TEXT CIRCUIT
	newFilePath.Format("%s.txt", filePath); 
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
		file.Write((LPCTSTR) message, message.GetLength() + 1);	
		file.Close();
	}

	// BINARY GENOM
	newFilePath.Format("%s.bin", filePath); 
	message = "";
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
	    for (int i = 0; i < genome.length(); i++) {
			value.Format("%u ", genome.gene(i));
			file.Write((LPCTSTR) value, value.GetLength());	
			message += value;
	    }
		file.Close();
	}
    message += "\r\n";
    LOG_INSERTSTRING_NOTIME(message); 

	// GRAPHVIZ FILE
	newFilePath.Format("%s.dot", filePath); 
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
		file.Write((LPCTSTR) visualCirc, visualCirc.GetLength() + 1);	
		file.Close();
	}
    // DRAW CIRCUIT, IF DOT IS INSTALLED
    CString cmdLine;
    CString pngFilePath; 
	pngFilePath.Format("%s.png", filePath); 
    cmdLine.Format("dot -Tpng %s -o %s -Gsize=1000", newFilePath, pngFilePath);
	WinExec(cmdLine, 0);


	// C FILE
	if (bCodeCircuit) {
	    newFilePath.Format("%s.c", filePath); 
	    if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
		    file.Write((LPCTSTR) codeCirc, codeCirc.GetLength() + 1);	
		    file.Close();
	    }
    }


	//
	//	STORE IN DEFAULT FILE
	//
	filePath = "EAC_circuit";
	
	// TEXT CIRCUIT
	newFilePath.Format("%s.txt", filePath); 
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
		file.Write((LPCTSTR) message, message.GetLength() + 1);	
		file.Close();
	}

	// BINARY GENOM
	newFilePath.Format("%s.bin", filePath); 
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
	    for (int i = 0; i < genome.length(); i++) {
			value.Format("%u ", genome.gene(i));
			file.Write((LPCTSTR) value, value.GetLength());	
	    }
		file.Close();
	}

	// GRAPHVIZ FILE
	newFilePath.Format("%s.dot", filePath); 
	if (file.Open(newFilePath, CFile::modeReadWrite | CFile::modeCreate)) {
		file.Write((LPCTSTR) visualCirc, visualCirc.GetLength() + 1);	
		file.Close();
	}
    
    
    
    return status;
}

int	CEACircuit::ExecuteForInput(CString textCircuit, CString textInput, GA_CIRCUIT* gaCircuitConfig) { 
    int                         status = STAT_OK;
    DWORD                       circuit[MAX_GENOME_SIZE];    

    pGACirc = gaCircuitConfig;

    pGACirc->allocate();
    memset(circuit, 0, sizeof(circuit));
	CEACircuit::ParseCircuit(textCircuit, circuit, &(pGACirc->numLayers), &(pGACirc->internalLayerSize), &(pGACirc->outputLayerSize));
    
    CString     message;
    message.Format("\n\n###NumLayers: %d, InLayerSize: %d, OutLayerSize: %d, NumInputs: %d, NumOutputs: %d, NumTestSets: %d, TestSetSize: %d\n\n",pGACirc->numLayers, pGACirc->internalLayerSize, pGACirc->outputLayerSize, pGACirc->numInputs, pGACirc->numOutputs, NUM_TEST_SETS, pGACirc->numTestVectors); 
    LOG_INSERTSTRING(message); 
    
    // ACTUAL SIZE OF GENOM
    pGACirc->genomeSize = (pGACirc->numLayers * 2) * MAX_INTERNAL_LAYER_SIZE;
    GA1DArrayGenome<DWORD>      genome(MAX_GENOME_SIZE, CEACircuit::garng_fitness);
    
    for (int i = 0; i < pGACirc->genomeSize; i++) genome.gene(i, circuit[i]);    
    
	CEACircuit::PrintCircuit(genome);

    // COPY INPUT FOR EXECUTION
    BYTE    circInput[MAX_INPUTS];
    BYTE    circOutput[MAX_OUTPUTS];
    DWORD   numInputs = MAX_INPUTS;

	//CEACircuit::GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, limitRounds);
	CCommonFnc::BYTE_ConvertFromHexStringToArray(textInput, circInput, &numInputs);

	memset(circOutput, 0, MAX_OUTPUTS);

    // EXECUTE CIRCUIT
    status = CEACircuit::ExecuteCircuit(pGACirc, &genome, circInput, circOutput);


    CCommonFnc::BYTE_ConvertFromArrayToHexString(circOutput, pGACirc->outputLayerSize, &message);
    LOG_INSERTSTRING(message); 
    
    CCommonFnc::BYTE_ConvertFromHexStringToArray(textInput, circInput, &numInputs);
    
    // PRINT HASH OF INPUT
    MD5_CTX mdContext;
    MD5_Init(&mdContext);
    MD5_Update(&mdContext, circInput, numInputs);
    MD5_Final(&mdContext);
    CCommonFnc::BYTE_ConvertFromArrayToHexString(mdContext.digest, MD5_DIGEST_LENGTH, &message);
    LOG_INSERTSTRING(message); 

    CSHA1 sha1;
    // SIGNALIZE SHA1 BY SETTING OUTPUT AS 0xff
    sha1.Input(circInput, numInputs);
    BYTE message_digest_array[20];
    sha1.Result(message_digest_array);
    CCommonFnc::BYTE_ConvertFromArrayToHexString(message_digest_array, 20, &message);
    LOG_INSERTSTRING(message); 

	return status;
}

int CEACircuit::ExecuteFromText(CString textCircuit, GA_CIRCUIT* gaCircuitConfig) {
    int				status = STAT_OK;
    DWORD           circuit[MAX_GENOME_SIZE];    
    CString         message;

    pGACirc = gaCircuitConfig;

    pGACirc->allocate();
    memset(circuit, 0, sizeof(circuit));
	CEACircuit::ParseCircuit(textCircuit, circuit, &(pGACirc->numLayers), &(pGACirc->internalLayerSize), &(pGACirc->outputLayerSize));

    
	CEACircuit::GenerateTestData(pGACirc->testVectorGenerMethod, pGACirc->numTestVectors, pGACirc->limitAlgRoundsCount);
    
    
    memset(circuit, 0, sizeof(circuit));
    
    CEACircuit::ParseCircuit(textCircuit, circuit, &(pGACirc->numLayers), &(pGACirc->internalLayerSize), &(pGACirc->outputLayerSize));
    
    message.Format("\n\n###NumLayers: %d, InLayerSize: %d, OutLayerSize: %d, NumInputs: %d, NumOutputs: %d, NumTestSets: %d, TestSetSize: %d\n\n",pGACirc->numLayers, pGACirc->internalLayerSize, pGACirc->outputLayerSize, pGACirc->numInputs, pGACirc->numOutputs, NUM_TEST_SETS, pGACirc->numTestVectors); 
    LOG_INSERTSTRING(message); 
    
    BYTE    inputs[MAX_INPUTS];
    for (int i = 0; i < pGACirc->numInputs; i++) inputs[i] = pGACirc->testVectors[0][i];
    CCommonFnc::BYTE_ConvertFromArrayToHexString(inputs, pGACirc->numInputs, &message);
    LOG_INSERTSTRING(message); 
    message.Format("%d", pGACirc->testVectors[0][MAX_INPUTS]);
    LOG_INSERTSTRING(message); 
    for (int i = 0; i < pGACirc->numInputs; i++) inputs[i] = pGACirc->testVectors[1][i];
    CCommonFnc::BYTE_ConvertFromArrayToHexString(inputs, pGACirc->numInputs, &message);
    LOG_INSERTSTRING(message); 
    message.Format("%d", pGACirc->testVectors[1][MAX_INPUTS]);
    LOG_INSERTSTRING(message); 

    
    // ACTUAL SIZE OF GENOM
    pGACirc->genomeSize = (pGACirc->numLayers * 2) * MAX_INTERNAL_LAYER_SIZE;
    GA1DArrayGenome<DWORD>      genome(MAX_GENOME_SIZE, CEACircuit::garng_fitness);
    
    for (int i = 0; i < pGACirc->genomeSize; i++) genome.gene(i, circuit[i]);    
    
    CEACircuit::PrintCircuit(genome);
    
    pGACirc->clearFitnessStats();

    float fit = CEACircuit::garng_fitness(genome);
    message.Format("Loaded fitness: %f", fit); 
    LOG_INSERTSTRING(message); 

    message.Format("best %f, avg: %f, avgPredict: %d, totalBest: %f", pGACirc->bestGenerFit, pGACirc->avgGenerFit / pGACirc->numAvgGenerFit, pGACirc->avgPredictions / pGACirc->numAvgGenerFit, pGACirc->maxFit); 
    LOG_INSERTSTRING(message); 

	pGACirc = NULL;

	return status;
} 


int CEACircuit::TestEAC(GA_CIRCUIT* gaCircuitConfig) {
	int				status = STAT_OK;
	CString			message;
	
    pGACirc = gaCircuitConfig;
    pGACirc->allocate();

	// 
	// SAVE/LOAD TEST
	// GENERATE RANDOM CIRCUIT, SAVE, LOAD AND COMPARE
	LOG_INSERTSTRING("############# SAVE/LOAD TEST BEGIN");

    GA1DArrayGenome<DWORD>    genome(pGACirc->genomeSize, garng_fitness);
    // RANDOM CIRCUIT
    CEACircuit::garng_initializer(genome);
    CString filePath; filePath = "EAC_test";
    // SAVE
    status = CEACircuit::PrintCircuit(genome, filePath);
	message.Format("CEACircuit::PrintCircuit finish with %s(%d)", ERROR_TO_STRING(status), status);
	LOG_INSERTSTRING(message);
	
	// LOAD AND PARSE 
	CFile	file;
	CString textCircuit;
	CString	newFilePath;
	
	newFilePath.Format("%s.txt", filePath);
	
	if (file.Open(newFilePath, CFile::modeRead)) {
		file.Read(textCircuit.GetBuffer(file.GetLength() + 1), file.GetLength() + 1);
		textCircuit.ReleaseBuffer();
		file.Close();
		
		DWORD           circuit[MAX_GENOME_SIZE];    
		memset(circuit, 0, sizeof(circuit));
		status = CEACircuit::ParseCircuit(textCircuit, circuit, &(pGACirc->numLayers), &(pGACirc->internalLayerSize), &(pGACirc->outputLayerSize));
		message.Format("CEACircuit::ParseCircuit finish with %s(%d)", ERROR_TO_STRING(status), status);
		LOG_INSERTSTRING(message);
		
		// COMPARE WITH ORIGINAL
		for (int i = 0; i < genome.size(); i++) {
			if (circuit[i] != genome.gene(i)) {
				message.Format("Gene %d from text incorrect", i);
				LOG_INSERTSTRING(message);
				
				status = STAT_DATA_CORRUPTED;
			}
		}
		
		if (status == STAT_OK) {
			LOG_INSERTSTRING("\nOK: TEXT SAVE/RESTORE WORKING\n");
		}
	}
	else {
		message.Format("Fail to open %s", newFilePath);
		LOG_INSERTSTRING(message);
	}
/**/
	newFilePath.Format("%s.bin", filePath);
	
	if (file.Open(newFilePath, CFile::modeRead)) {
		char*	data = new char[file.GetLength() + 1];
		memset(data, 0, file.GetLength() + 1);
		file.Read(data, file.GetLength() + 1);
		textCircuit = data;
		file.Close();
		
		DWORD           circuit[MAX_GENOME_SIZE];    
		memset(circuit, 0, sizeof(circuit));
		status = CEACircuit::ParseCircuit(textCircuit, circuit, &(pGACirc->numLayers), &(pGACirc->internalLayerSize), &(pGACirc->outputLayerSize));
		message.Format("CEACircuit::ParseCircuit finish with %s(%d)", ERROR_TO_STRING(status), status);
		LOG_INSERTSTRING(message);
		
		// COMPARE WITH ORIGINAL
		for (int i = 0; i < genome.size(); i++) {
			DWORD	gen = genome.gene(i);
			if (circuit[i] != gen) {
				message.Format("Gene %d from binary incorrect", i);
				LOG_INSERTSTRING(message);
				status = STAT_DATA_CORRUPTED;
			}
		}
	
		if (status == STAT_OK) {
			LOG_INSERTSTRING("\nOK: TEXT SAVE/RESTORE WORKING\n");
		}
		
		delete[] data;
	}
	else {
		message.Format("Fail to open %s", newFilePath);
		LOG_INSERTSTRING(message);
	}

	LOG_INSERTSTRING("############# SAVE/LOAD TEST END");

	return status;	
} 

/*
 **********************************************************************
 ** md5.c                                                            **
 ** RSA Data Security, Inc. MD5 Message Digest Algorithm             **
 ** Created: 2/17/90 RLR                                             **
 ** Revised: 1/91 SRD,AJ,BSK,JT Reference C Version                  **
 **********************************************************************
 */

/*
 **********************************************************************
 ** Copyright (C) 1990, RSA Data Security, Inc. All rights reserved. **
 **                                                                  **
 ** License to copy and use this software is granted provided that   **
 ** it is identified as the "RSA Data Security, Inc. MD5 Message     **
 ** Digest Algorithm" in all material mentioning or referencing this **
 ** software or this function.                                       **
 **                                                                  **
 ** License is also granted to make and use derivative works         **
 ** provided that such works are identified as "derived from the RSA **
 ** Data Security, Inc. MD5 Message Digest Algorithm" in all         **
 ** material mentioning or referencing the derived work.             **
 **                                                                  **
 ** RSA Data Security, Inc. makes no representations concerning      **
 ** either the merchantability of this software or the suitability   **
 ** of this software for any particular purpose.  It is provided "as **
 ** is" without express or implied warranty of any kind.             **
 **                                                                  **
 ** These notices must be retained in any copies of any part of this **
 ** documentation and/or software.                                   **
 **********************************************************************
 */


static unsigned char PADDING[64] = {
  0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

/* F, G and H are basic MD5 functions: selection, majority, parity */
#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z))) 

/* ROTATE_LEFT rotates x left n bits */
#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4 */
/* Rotation is separate from addition to prevent recomputation */
#define FF(a, b, c, d, x, s, ac) \
  {(a) += F ((b), (c), (d)) + (x) + (UINT4)(ac); \
   (a) = ROTATE_LEFT ((a), (s)); \
   (a) += (b); \
  }
#define GG(a, b, c, d, x, s, ac) \
  {(a) += G ((b), (c), (d)) + (x) + (UINT4)(ac); \
   (a) = ROTATE_LEFT ((a), (s)); \
   (a) += (b); \
  }
#define HH(a, b, c, d, x, s, ac) \
  {(a) += H ((b), (c), (d)) + (x) + (UINT4)(ac); \
   (a) = ROTATE_LEFT ((a), (s)); \
   (a) += (b); \
  }
#define II(a, b, c, d, x, s, ac) \
  {(a) += I ((b), (c), (d)) + (x) + (UINT4)(ac); \
   (a) = ROTATE_LEFT ((a), (s)); \
   (a) += (b); \
  }

void CEACircuit::MD5_Init(MD5_CTX *mdContext) {
  mdContext->i[0] = mdContext->i[1] = (UINT4)0;

  /* Load magic initialization constants.
   */
  mdContext->buf[0] = (UINT4)0x67452301;
  mdContext->buf[1] = (UINT4)0xefcdab89;
  mdContext->buf[2] = (UINT4)0x98badcfe;
  mdContext->buf[3] = (UINT4)0x10325476;
}

void CEACircuit::MD5_Update(
MD5_CTX *mdContext,
unsigned char *inBuf,
unsigned int inLen, 
int numRounds)
{
  UINT4 in[16];
  int mdi;
  unsigned int i, ii;

  /* compute number of bytes mod 64 */
  mdi = (int)((mdContext->i[0] >> 3) & 0x3F);

  /* update number of bits */
  if ((mdContext->i[0] + ((UINT4)inLen << 3)) < mdContext->i[0])
    mdContext->i[1]++;
  mdContext->i[0] += ((UINT4)inLen << 3);
  mdContext->i[1] += ((UINT4)inLen >> 29);

  while (inLen--) {
    /* add new character to buffer, increment mdi */
    mdContext->in[mdi++] = *inBuf++;

    /* transform if necessary */
    if (mdi == 0x40) {
      for (i = 0, ii = 0; i < 16; i++, ii += 4)
        in[i] = (((UINT4)mdContext->in[ii+3]) << 24) |
                (((UINT4)mdContext->in[ii+2]) << 16) |
                (((UINT4)mdContext->in[ii+1]) << 8) |
                ((UINT4)mdContext->in[ii]);
      MD5_Transform (mdContext->buf, in, numRounds);
      mdi = 0;
    }
  }
}

void CEACircuit::MD5_Final(MD5_CTX *mdContext, int numRounds)
{
  UINT4 in[16];
  int mdi;
  unsigned int i, ii;
  unsigned int padLen;

  /* save number of bits */
  in[14] = mdContext->i[0];
  in[15] = mdContext->i[1];

  /* compute number of bytes mod 64 */
  mdi = (int)((mdContext->i[0] >> 3) & 0x3F);

  /* pad out to 56 mod 64 */
  padLen = (mdi < 56) ? (56 - mdi) : (120 - mdi);
  MD5Update (mdContext, PADDING, padLen);

  /* append length in bits and transform */
  for (i = 0, ii = 0; i < 14; i++, ii += 4)
    in[i] = (((UINT4)mdContext->in[ii+3]) << 24) |
            (((UINT4)mdContext->in[ii+2]) << 16) |
            (((UINT4)mdContext->in[ii+1]) << 8) |
            ((UINT4)mdContext->in[ii]);
  MD5_Transform (mdContext->buf, in, numRounds);

  /* store buffer in digest */
  for (i = 0, ii = 0; i < 4; i++, ii += 4) {
    mdContext->digest[ii] = (unsigned char)(mdContext->buf[i] & 0xFF);
    mdContext->digest[ii+1] =
      (unsigned char)((mdContext->buf[i] >> 8) & 0xFF);
    mdContext->digest[ii+2] =
      (unsigned char)((mdContext->buf[i] >> 16) & 0xFF);
    mdContext->digest[ii+3] =
      (unsigned char)((mdContext->buf[i] >> 24) & 0xFF);
  }
}

/* Basic MD5 step. Transform buf based on in.
 */
void CEACircuit::MD5_Transform (UINT4 *buf, UINT4 *in, int numRounds)
{
  UINT4 a = buf[0], b = buf[1], c = buf[2], d = buf[3];

  /* Round 1 */
#define S11 7
#define S12 12
#define S13 17
#define S14 22
 
if (numRounds > 0)  FF ( a, b, c, d, in[ 0], S11, 3614090360); /* 1 */
if (numRounds > 1)  FF ( d, a, b, c, in[ 1], S12, 3905402710); /* 2 */
if (numRounds > 2)  FF ( c, d, a, b, in[ 2], S13,  606105819); /* 3 */
if (numRounds > 3)  FF ( b, c, d, a, in[ 3], S14, 3250441966); /* 4 */
if (numRounds > 4)  FF ( a, b, c, d, in[ 4], S11, 4118548399); /* 5 */
if (numRounds > 5)  FF ( d, a, b, c, in[ 5], S12, 1200080426); /* 6 */
if (numRounds > 6)  FF ( c, d, a, b, in[ 6], S13, 2821735955); /* 7 */
if (numRounds > 7)  FF ( b, c, d, a, in[ 7], S14, 4249261313); /* 8 */
if (numRounds > 8)  FF ( a, b, c, d, in[ 8], S11, 1770035416); /* 9 */
if (numRounds > 9)  FF ( d, a, b, c, in[ 9], S12, 2336552879); /* 10 */
if (numRounds > 10)  FF ( c, d, a, b, in[10], S13, 4294925233); /* 11 */
if (numRounds > 11)  FF ( b, c, d, a, in[11], S14, 2304563134); /* 12 */
if (numRounds > 12)  FF ( a, b, c, d, in[12], S11, 1804603682); /* 13 */
if (numRounds > 13)  FF ( d, a, b, c, in[13], S12, 4254626195); /* 14 */
if (numRounds > 14)  FF ( c, d, a, b, in[14], S13, 2792965006); /* 15 */
if (numRounds > 15)  FF ( b, c, d, a, in[15], S14, 1236535329); /* 16 */

  /* Round 2 */
#define S21 5
#define S22 9
#define S23 14
#define S24 20
if (numRounds > 16)    GG ( a, b, c, d, in[ 1], S21, 4129170786); /* 17 */
if (numRounds > 17)    GG ( d, a, b, c, in[ 6], S22, 3225465664); /* 18 */
if (numRounds > 18)    GG ( c, d, a, b, in[11], S23,  643717713); /* 19 */
if (numRounds > 19)    GG ( b, c, d, a, in[ 0], S24, 3921069994); /* 20 */
if (numRounds > 20)    GG ( a, b, c, d, in[ 5], S21, 3593408605); /* 21 */
if (numRounds > 21)    GG ( d, a, b, c, in[10], S22,   38016083); /* 22 */
if (numRounds > 22)    GG ( c, d, a, b, in[15], S23, 3634488961); /* 23 */
if (numRounds > 23)    GG ( b, c, d, a, in[ 4], S24, 3889429448); /* 24 */
if (numRounds > 24)    GG ( a, b, c, d, in[ 9], S21,  568446438); /* 25 */
if (numRounds > 25)    GG ( d, a, b, c, in[14], S22, 3275163606); /* 26 */
if (numRounds > 26)    GG ( c, d, a, b, in[ 3], S23, 4107603335); /* 27 */
if (numRounds > 27)    GG ( b, c, d, a, in[ 8], S24, 1163531501); /* 28 */
if (numRounds > 28)    GG ( a, b, c, d, in[13], S21, 2850285829); /* 29 */
if (numRounds > 29)    GG ( d, a, b, c, in[ 2], S22, 4243563512); /* 30 */
if (numRounds > 30)    GG ( c, d, a, b, in[ 7], S23, 1735328473); /* 31 */
if (numRounds > 31)    GG ( b, c, d, a, in[12], S24, 2368359562); /* 32 */

  /* Round 3 */
#define S31 4
#define S32 11
#define S33 16
#define S34 23
if (numRounds > 32)      HH ( a, b, c, d, in[ 5], S31, 4294588738); /* 33 */
if (numRounds > 33)      HH ( d, a, b, c, in[ 8], S32, 2272392833); /* 34 */
if (numRounds > 34)      HH ( c, d, a, b, in[11], S33, 1839030562); /* 35 */
if (numRounds > 35)      HH ( b, c, d, a, in[14], S34, 4259657740); /* 36 */
if (numRounds > 36)      HH ( a, b, c, d, in[ 1], S31, 2763975236); /* 37 */
if (numRounds > 37)      HH ( d, a, b, c, in[ 4], S32, 1272893353); /* 38 */
if (numRounds > 38)      HH ( c, d, a, b, in[ 7], S33, 4139469664); /* 39 */
if (numRounds > 39)      HH ( b, c, d, a, in[10], S34, 3200236656); /* 40 */
if (numRounds > 40)      HH ( a, b, c, d, in[13], S31,  681279174); /* 41 */
if (numRounds > 41)      HH ( d, a, b, c, in[ 0], S32, 3936430074); /* 42 */
if (numRounds > 42)      HH ( c, d, a, b, in[ 3], S33, 3572445317); /* 43 */
if (numRounds > 43)      HH ( b, c, d, a, in[ 6], S34,   76029189); /* 44 */
if (numRounds > 44)      HH ( a, b, c, d, in[ 9], S31, 3654602809); /* 45 */
if (numRounds > 45)      HH ( d, a, b, c, in[12], S32, 3873151461); /* 46 */
if (numRounds > 46)      HH ( c, d, a, b, in[15], S33,  530742520); /* 47 */
if (numRounds > 47)      HH ( b, c, d, a, in[ 2], S34, 3299628645); /* 48 */

  /* Round 4 */
#define S41 6
#define S42 10
#define S43 15
#define S44 21
if (numRounds > 48)        II ( a, b, c, d, in[ 0], S41, 4096336452); /* 49 */
if (numRounds > 49)        II ( d, a, b, c, in[ 7], S42, 1126891415); /* 50 */
if (numRounds > 50)        II ( c, d, a, b, in[14], S43, 2878612391); /* 51 */
if (numRounds > 51)       II ( b, c, d, a, in[ 5], S44, 4237533241); /* 52 */
if (numRounds > 52)        II ( a, b, c, d, in[12], S41, 1700485571); /* 53 */
if (numRounds > 53)        II ( d, a, b, c, in[ 3], S42, 2399980690); /* 54 */
if (numRounds > 54)        II ( c, d, a, b, in[10], S43, 4293915773); /* 55 */
if (numRounds > 55)        II ( b, c, d, a, in[ 1], S44, 2240044497); /* 56 */
if (numRounds > 56)        II ( a, b, c, d, in[ 8], S41, 1873313359); /* 57 */
if (numRounds > 57)        II ( d, a, b, c, in[15], S42, 4264355552); /* 58 */
if (numRounds > 58)        II ( c, d, a, b, in[ 6], S43, 2734768916); /* 59 */
if (numRounds > 59)        II ( b, c, d, a, in[13], S44, 1309151649); /* 60 */
if (numRounds > 60)        II ( a, b, c, d, in[ 4], S41, 4149444226); /* 61 */
if (numRounds > 61)        II ( d, a, b, c, in[11], S42, 3174756917); /* 62 */
if (numRounds > 62)        II ( c, d, a, b, in[ 2], S43,  718787259); /* 63 */
if (numRounds > 63)        II ( b, c, d, a, in[ 9], S44, 3951481745); /* 64 */

  buf[0] += a;
  buf[1] += b;
  buf[2] += c;
  buf[3] += d;
}

//
// SHA-1
//
CSHA1::CSHA1() {
	Reset();
}

CSHA1::~CSHA1() {}

void CSHA1::Reset()
{

	Length_Low			= 0;
	Length_High			= 0;
	Message_Block_Index	= 0;

	H[0]		= 0x67452301;
	H[1]		= 0xEFCDAB89;
	H[2]		= 0x98BADCFE;
	H[3]		= 0x10325476;
	H[4]		= 0xC3D2E1F0;

	Computed	= false;
	Corrupted	= false;
}

bool CSHA1::Result(BYTE *message_digest_array)
{

	int i;									// Counter

	if (Corrupted)
	{
		return false;
	}

	if (!Computed)
	{
		PadMessage();
		Computed = true;
	}

    /* extract the hash value as bytes in case the hash buffer is   */
    /* misaligned for 32-bit words                                  */
	for(i = 0; i < 20; i++)
	{
        message_digest_array[i] = (unsigned char)(H[i >> 2] >> (8 * (~i & 3)));
	}

	return true;
}

void CSHA1::Input(	const unsigned char	*message_array,
					unsigned 			length)
{

	if (!length)
	{
		return;
	}

	if (Computed || Corrupted)
	{
		Corrupted = true;
		return;
	}

	while(length-- && !Corrupted)
	{
		Message_Block[Message_Block_Index++] = (*message_array & 0xFF);

		Length_Low += 8;
		Length_Low &= 0xFFFFFFFF;				// Force it to 32 bits
		if (Length_Low == 0)
		{
			Length_High++;
			Length_High &= 0xFFFFFFFF;			// Force it to 32 bits
			if (Length_High == 0)
			{
				Corrupted = true;				// Message is too long
			}
		}

		if (Message_Block_Index == 64)
		{
			ProcessMessageBlock();
		}

		message_array++;
	}
}


void CSHA1::Input(	const char	*message_array,
					unsigned 	length)
{

	Input((unsigned char *) message_array, length);
}

void CSHA1::Input(unsigned char message_element)
{

	Input(&message_element, 1);
}

void CSHA1::Input(char message_element)
{

	Input((unsigned char *) &message_element, 1);
}

CSHA1& CSHA1::operator<<(const char *message_array)
{

	const char *p = message_array;

	while(*p)
	{
		Input(*p);
		p++;
	}

	return *this;
}


CSHA1& CSHA1::operator<<(const unsigned char *message_array)
{

	const unsigned char *p = message_array;

	while(*p)
	{
		Input(*p);
		p++;
	}

	return *this;
}

CSHA1& CSHA1::operator<<(const char message_element)
{

	Input((unsigned char *) &message_element, 1);

	return *this;
}

CSHA1& CSHA1::operator<<(const unsigned char message_element)
{

	Input(&message_element, 1);

	return *this;
}

void CSHA1::ProcessMessageBlock()
{

	const unsigned K[] = 	{ 				// Constants defined for SHA-1
								0x5A827999,
								0x6ED9EBA1,
								0x8F1BBCDC,
								0xCA62C1D6
							};
	int 		t;							// Loop counter
	unsigned 	temp;						// Temporary word value
	unsigned	W[80];						// Word sequence
	unsigned	A, B, C, D, E;				// Word buffers

	/*
	 *	Initialize the first 16 words in the array W
	 */
	for(t = 0; t < 16; t++)
	{
		W[t] = ((unsigned) Message_Block[t * 4]) << 24;
		W[t] |= ((unsigned) Message_Block[t * 4 + 1]) << 16;
		W[t] |= ((unsigned) Message_Block[t * 4 + 2]) << 8;
		W[t] |= ((unsigned) Message_Block[t * 4 + 3]);
	}

	for(t = 16; t < 80; t++)
	{
	   W[t] = CircularShift(1,W[t-3] ^ W[t-8] ^ W[t-14] ^ W[t-16]);
	}

	A = H[0];
	B = H[1];
	C = H[2];
	D = H[3];
	E = H[4];

	for(t = 0; t < 20; t++)
	{
		temp = CircularShift(5,A) + ((B & C) | ((~B) & D)) + E + W[t] + K[0];
		temp &= 0xFFFFFFFF;
		E = D;
		D = C;
		C = CircularShift(30,B);
		B = A;
		A = temp;
	}

	for(t = 20; t < 40; t++)
	{
		temp = CircularShift(5,A) + (B ^ C ^ D) + E + W[t] + K[1];
		temp &= 0xFFFFFFFF;
		E = D;
		D = C;
		C = CircularShift(30,B);
		B = A;
		A = temp;
	}

	for(t = 40; t < 60; t++)
	{
		temp = CircularShift(5,A) +
		 	   ((B & C) | (B & D) | (C & D)) + E + W[t] + K[2];
		temp &= 0xFFFFFFFF;
		E = D;
		D = C;
		C = CircularShift(30,B);
		B = A;
		A = temp;
	}

	for(t = 60; t < 80; t++)
	{
		temp = CircularShift(5,A) + (B ^ C ^ D) + E + W[t] + K[3];
		temp &= 0xFFFFFFFF;
		E = D;
		D = C;
		C = CircularShift(30,B);
		B = A;
		A = temp;
	}

	H[0] = (H[0] + A) & 0xFFFFFFFF;
	H[1] = (H[1] + B) & 0xFFFFFFFF;
	H[2] = (H[2] + C) & 0xFFFFFFFF;
	H[3] = (H[3] + D) & 0xFFFFFFFF;
	H[4] = (H[4] + E) & 0xFFFFFFFF;

	Message_Block_Index = 0;
}

void CSHA1::PadMessage()
{

	/*
	 *	Check to see if the current message block is too small to hold
	 *	the initial padding bits and length.  If so, we will pad the
	 *	block, process it, and then continue padding into a second block.
	 */
	if (Message_Block_Index > 55)
	{
		Message_Block[Message_Block_Index++] = 0x80;
		while(Message_Block_Index < 64)
		{
			Message_Block[Message_Block_Index++] = 0;
		}

		ProcessMessageBlock();

		while(Message_Block_Index < 56)
		{
			Message_Block[Message_Block_Index++] = 0;
		}
	}
	else
	{
		Message_Block[Message_Block_Index++] = 0x80;
		while(Message_Block_Index < 56)
		{
			Message_Block[Message_Block_Index++] = 0;
		}

	}

	/*
	 *	Store the message length as the last 8 octets
	 */
	Message_Block[56] = (Length_High >> 24) & 0xFF;
	Message_Block[57] = (Length_High >> 16) & 0xFF;
	Message_Block[58] = (Length_High >> 8) & 0xFF;
	Message_Block[59] = (Length_High) & 0xFF;
	Message_Block[60] = (Length_Low >> 24) & 0xFF;
	Message_Block[61] = (Length_Low >> 16) & 0xFF;
	Message_Block[62] = (Length_Low >> 8) & 0xFF;
	Message_Block[63] = (Length_Low) & 0xFF;

	ProcessMessageBlock();
}


unsigned CSHA1::CircularShift(int bits, unsigned word) {
    return ((word << bits) & 0xFFFFFFFF) | ((word & 0xFFFFFFFF) >> (32-bits));
}

