// KeyEstablishTab.cpp : implementation file
// jen takova zmena

#include "stdafx.h"
#include "SensorSim.h"
#include "KeyEstablishTab.h"
#include "SensorSimCore.h"
#include "SensorSimDlg.h"
#include "TestQueueMngDlg.h"
#include "SavedCharsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimCore*  pSensorSim;
extern CSensorSimDlg*  pSensorSimDlg;

/////////////////////////////////////////////////////////////////////////////
// CKeyEstablishTab dialog


CKeyEstablishTab::CKeyEstablishTab(CWnd* pParent /*=NULL*/)
	: CDialog(CKeyEstablishTab::IDD, pParent) 
{
	//{{AFX_DATA_INIT(CKeyEstablishTab)
	m_batchPath = MATLAB_BASE_PATH;
	m_batchRepeat = _T("1");
	m_rangeFrom = _T("0.0");
	m_rangeStep = _T("0.1");
	m_rangeTo = _T("1");
	m_useKeyInfect = FALSE;
	m_bindBlackRatio = FALSE;
	m_useMMD = FALSE;
	m_testQueueing = FALSE;
	m_note = _T("");
	m_useFOE = FALSE;
	m_useNLeash = TRUE;
//    m_whatToSaveFlags = FLAG_WHATSAVE_AMPLIFICATIONIMPACT | FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE | FLAG_WHATSAVE_KEYSDISTRIBUTION | FLAG_WHATSAVE_SENSORPOSITION | FLAG_WHATSAVE_COMPRATIOALLMETHODS | FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY;
    m_whatToSaveFlags = FLAG_WHATSAVE_NEIGHBLEASH;
	m_optimalPoolSize = FALSE;
	//}}AFX_DATA_INIT
}


void CKeyEstablishTab::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CKeyEstablishTab)
    DDX_Text(pDX, IDC_BATCH_PATH, m_batchPath);
    DDX_Text(pDX, IDC_BATCH_REPEAT, m_batchRepeat);
    DDX_Text(pDX, IDC_RANGE_FROM, m_rangeFrom);
    DDX_Text(pDX, IDC_RANGE_STEP, m_rangeStep);
    DDX_Text(pDX, IDC_RANGE_TO, m_rangeTo);
    DDX_Check(pDX, IDC_USE_KEYINFECT, m_useKeyInfect);
    DDX_Check(pDX, IDC_BIND_BLACK_RATIO, m_bindBlackRatio);
    DDX_Check(pDX, IDC_USE_MMD, m_useMMD);
    DDX_Check(pDX, IDC_TEST_QUEUEING, m_testQueueing);
    DDX_Text(pDX, IDC_NOTE, m_note);
    DDX_Check(pDX, IDC_USE_FOE, m_useFOE);
    DDX_Check(pDX, IDC_OPTIM_POOL_SIZE, m_optimalPoolSize);
    //}}AFX_DATA_MAP
    DDX_Check(pDX, IDC_USE_NLEASH, m_useNLeash);
}


BEGIN_MESSAGE_MAP(CKeyEstablishTab, CDialog)
	//{{AFX_MSG_MAP(CKeyEstablishTab)
	ON_BN_CLICKED(IDC_BROWSE_BATCH_PATH, OnBrowseBatchPath)
	ON_BN_CLICKED(IDC_EXECUTE_FIXEDTEST, OnExecuteFixedtest)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERANGETESTS, OnExecuteVariablerangetests)
	ON_BN_CLICKED(IDC_STOP_CURRENT_OPERATIONS, OnStopCurrentOperations)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERATIOTESTS, OnExecuteVariableratiotests)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERECEIVERATIOTESTS, OnExecuteVariablereceiveratiotests)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLEBSNUMBERTEST, OnExecuteVariabletotalbscounttest)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLEACTIVEBSCOUNTTEST, OnExecuteVariableactivebscounttest)
	ON_BN_CLICKED(IDC_BATCH_QUEUE_MNG, OnBatchQueueMng)
	ON_BN_CLICKED(IDC_SAVED_CHARACTERISTICS, OnSavedCharacteristics)
	ON_BN_CLICKED(IDC_VARIABLE_COMPR_NODES, OnExecuteVariableComprNodesTest)
	ON_BN_CLICKED(IDC_VARIABLE_WHITE_NODES_COUNT, OnVariableWhiteNodesCount)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLETOTALBSCOUNTTEST, OnExecuteVariabletotalbscounttest)
	ON_BN_CLICKED(IDC_VARIABLE_POOLSIZE, OnVariablePoolsize)
    ON_BN_CLICKED(IDC_VARIABLE_DROPCHECK_PROB, OnBnClickedVariableDropcheckProb)
    ON_BN_CLICKED(IDC_VARIABLE_COMPRNODESFRACT_PROB, OnBnClickedVariableComprnodesfractProb)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKeyEstablishTab message handlers

void CKeyEstablishTab::OnBrowseBatchPath() 
{
    int         status = STAT_OK;
    CFileDialog fileDialog(TRUE);
    CString     fileName;

    UpdateData(TRUE);

    if (fileDialog.DoModal() == IDOK) {
        fileName = fileDialog.GetPathName();
        fileName = fileName.Left(fileName.GetLength() - fileDialog.GetFileName().GetLength());
        m_batchPath = fileName;
    }

	UpdateData(FALSE);
}

int CKeyEstablishTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
    
    UpdateData(TRUE);

    // VALUES STORED FROM CSavedCharsDlg
    pBasicSettings->batchTest.whatSaveFlags = m_whatToSaveFlags;

    if (pBasicSettings->batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
        // MAINTAING OF COMPROMISSED_BY LIST MUST BE ALLOWED
        pBasicSettings->maintainCompromissedBy = TRUE;
    }

    if (m_useKeyInfect) pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEKEYINFECT;
    if (m_useMMD) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEMULTIPATHMESSAGEDELIVERY;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_MMD;
    }
    if (m_useFOE) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEFORWARDONIONENCRYPTION;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_FOE;
    }
    if (m_useNLeash) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USENEIGHBLEASH;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_NEIGHBLEASH;
    }
    
    if (m_bindBlackRatio) pBasicSettings->distribFlags |= FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO;
    if (m_optimalPoolSize) pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEOPTIMALPOOLSIZE;
 
 
    pBasicSettings->batchTest.batchRepeat = atoi(m_batchRepeat);
    pBasicSettings->batchTest.rangeFrom = (float) atof(m_rangeFrom);
    pBasicSettings->batchTest.rangeTo = (float) atof(m_rangeTo);
    pBasicSettings->batchTest.rangeStep = (float) atof(m_rangeStep);
    pBasicSettings->batchTest.outputPath = m_batchPath;
    pBasicSettings->batchTest.note = m_note;

    return status;
}

int CKeyEstablishTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;

    // VALUES DISPLAYED IN CSavedCharsDlg
    m_whatToSaveFlags = pBasicSettings->batchTest.whatSaveFlags;

    m_useKeyInfect = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEKEYINFECT) ? TRUE : FALSE;
    m_useMMD = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEMULTIPATHMESSAGEDELIVERY) ? TRUE : FALSE;
    m_useFOE = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEFORWARDONIONENCRYPTION) ? TRUE : FALSE;
    m_useNLeash = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USENEIGHBLEASH) ? TRUE : FALSE;
    
    
    m_bindBlackRatio = (pBasicSettings->distribFlags & FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO) ? TRUE : FALSE;
    m_optimalPoolSize = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEOPTIMALPOOLSIZE) ? TRUE : FALSE;
    
    m_batchRepeat.Format("%d", pBasicSettings->batchTest.batchRepeat);
    m_rangeFrom.Format("%.2f", pBasicSettings->batchTest.rangeFrom);
    m_rangeTo.Format("%.2f", pBasicSettings->batchTest.rangeTo);
    m_rangeStep.Format("%.2f", pBasicSettings->batchTest.rangeStep);
    m_batchPath = pBasicSettings->batchTest.outputPath;
    m_note = pBasicSettings->batchTest.note;


    UpdateData(FALSE);

    return status;
}

int CKeyEstablishTab::ExecuteTestThread(DWORD testType) {
    int                     status = STAT_OK;
    BASIC_INIT_DATA         basicSettings;
    
    UpdateData(TRUE);

    if ((status = pSensorSimDlg->GetBasicSettings(&basicSettings)) == STAT_OK) {
		status = pSensorSim->m_testQueue.ExecuteTestThread(&basicSettings, testType);
    }	

    return status;
}

void CKeyEstablishTab::OnStopCurrentOperations() 
{
    // SET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = TRUE;

/*
// SIMPLE LOAD AND SAVE TEST
    BASIC_INIT_DATA     basicSettings1;
    BASIC_INIT_DATA     basicSettings2;
    BASIC_INIT_DATA     basicSettings3;

    pSensorSim->GetBasicSettings(&basicSettings1);
    pSensorSim->SaveConfigScript("C:\\cfg.ini", &basicSettings1);
    pSensorSim->LoadConfigScript("C:\\cfg.ini", &basicSettings2);
    
    ASSERT(memcmp(&basicSettings1, &basicSettings2, sizeof(BASIC_INIT_DATA)) == 0);

    pSensorSim->SetBasicSettings(&basicSettings2);
    pSensorSim->GetBasicSettings(&basicSettings3);

    ASSERT(memcmp(&basicSettings2, &basicSettings3, sizeof(BASIC_INIT_DATA)) == 0);
/**/
}


void CKeyEstablishTab::OnExecuteFixedtest() 
{
    //
    //  RATIO OF COMPROMISSED KEYS BY DISTANCE FROM CENTRAL NODE UP TO MAX. RANGE 
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_FIXEDTRANSMRANGE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_FIXEDTRANSMRANGE);
    }
}

void CKeyEstablishTab::OnExecuteVariablerangetests() 
{
    //
    //  SECURED KEYS TO NUMBER OF NEIGHBOURS DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLETRANSMRANGE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLETRANSMRANGE);
    }
}


void CKeyEstablishTab::OnExecuteVariableratiotests() 
{
    //
    //  SECURED KEYS TO NUMBER OF NEIGHBOURS DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO);
    }
}

void CKeyEstablishTab::OnExecuteVariablereceiveratiotests() 
{
    //
    //  SECURED KEYS TO BLACK SENSITIVITY DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY);
    }
}

void CKeyEstablishTab::OnExecuteVariabletotalbscounttest() 
{
    //
    //  SECURED MESSAGES TO TOTAL COUNT ASE STATIONS WITH 1 ACTIVE BS
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT);
    }
}

void CKeyEstablishTab::OnExecuteVariableactivebscounttest() 
{
    //
    //  SECURED MESSAGES TO TOTAL COUNT ASE STATIONS WITH 1 ACTIVE BS
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT);
    }
}


void CKeyEstablishTab::OnExecuteVariableComprNodesTest() 
{
    //
    //  IMPACT OF VARIABLE NUMBER OF COMPROMISSED NODES  
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT);
    }
}

void CKeyEstablishTab::OnBatchQueueMng() 
{
    int                 status = STAT_OK;
    CTestQueueMngDlg	mngDlg;

    if (mngDlg.DoModal() == IDOK) {
        switch (mngDlg.m_actionType) {
            case TESTQUEUE_RUN: {
                // RUN ALL TESTS
                pSensorSim->m_testQueue.Run();
                break;
            }  
            case TESTQUEUE_CLEAR: {
                // CLEAR QUEUE
                pSensorSim->m_testQueue.Clear();
                break;
            }
        }
    }
}

int CKeyEstablishTab::TestQueue_Add(DWORD testType) {
    int              status = STAT_OK;
    BASIC_INIT_DATA* pBasicSettings;
    
    pBasicSettings = new BASIC_INIT_DATA;

    // GET TEST SETTINGS
    if ((status = pSensorSimDlg->GetBasicSettings(pBasicSettings)) == STAT_OK) {
        // SET TEST TYPE
        pBasicSettings->testType = testType;

        status = pSensorSim->m_testQueue.Add(pBasicSettings);
    }
    
    return status;
}

void CKeyEstablishTab::OnSavedCharacteristics() 
{
    CSavedCharsDlg  charsDlg(this); 
    
    charsDlg.m_whatToSaveFlags = m_whatToSaveFlags;
    if (charsDlg.DoModal() == IDOK) {
        m_whatToSaveFlags = charsDlg.m_whatToSaveFlags;
    }
}


void CKeyEstablishTab::OnVariableWhiteNodesCount() 
{
    //
    //  CHANGING NUMBER OF WHITE NODES
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT);
    }
}

void CKeyEstablishTab::OnVariablePoolsize() 
{
    //
    //  CHANGING NUMBER OF KEYS IN POOL
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE);
    }
}

void CKeyEstablishTab::OnBnClickedVariableDropcheckProb()
{
    //
    //  CHANGING PROBABILITY OF CHECK FROM MESSAGE DROPPING
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEDROPCHECKPROB);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEDROPCHECKPROB);
    }
}

void CKeyEstablishTab::OnBnClickedVariableComprnodesfractProb()
{
    //
    //  CHANGING FRACTION OF COMPROMISSED NODES
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLECOMPRNODES);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLECOMPRNODES);
    }
}
