// SensorSimDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SensorSim.h"
#include "SensorSimCore.h"
#include "SensorSimDlg.h"
#include "VersionInfo.h"
#include "CommonFnc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CSensorSimCore*     pSensorSim = NULL;
extern CSensorSimDlg*  pSensorSimDlg;

CSensorSimCore::CSensorSimCore()
{
	m_logs = _T("");
    m_logFile = "s3.log";
	m_bStopThread = FALSE;

	pSensorSim = this;

    m_pEACircuit = NULL;
	
    m_pActualThread = NULL;
    m_stopThreadSoft = FALSE;
    m_visible = TRUE;
	m_runTests = FALSE;
	m_runProxy = FALSE;
	m_proxyPort = 0;
	m_numRepetitions = 1;
	m_numRepetitionsElapsed = 0;
    
    //
    //  ALLOCATE STRATEGY RESOURCES
    //
    //m_pGAStrategy = new GA_STRATEGY;
	m_pGAStrategy = &(pSensorSim->m_sensorEnv.m_basicSettings.gaConfig);
    CSensorEnv::GAS_AllocateDefaultStrategyStructs(m_pGAStrategy);
    CSensorEnv::GAS_AllocateDefaultStrategyStructs(&m_tempStrategy);
	 
	// STARTUP PATH
    GetCurrentDirectory(MAX_PATH, m_basePath.GetBuffer(MAX_PATH));
    m_basePath.ReleaseBuffer();
	m_defaultCfgFile.Format("%s\\default_cfg.ini", pSensorSim->m_basePath);
	m_defaultPopulationFile = _T("");

    m_pEACircuit = new CEACircuit();

#ifdef PERFTIME
	DeleteFile("perfTime.txt");
    ClearPerformance();
#endif
}

CSensorSimCore::~CSensorSimCore() {

#ifdef PERFTIME
	ShowPerformance();
#endif

    pSensorSim->m_sensorEnv.Release();

	if (pSensorSim->m_pGAStrategy != NULL)
        pSensorSim->m_pGAStrategy->release();

	pSensorSim->m_tempStrategy.release();
	
    pSensorSim->m_bStopThread = TRUE; 
	
	if (m_pEACircuit != NULL) {
		m_pEACircuit->m_gaCirc.release();
		delete m_pEACircuit;
		m_pEACircuit = NULL;
	}
}

void CSensorSimCore::HandleCommandLine() {
	//
    // OBTAIN AND PARSE COMMAND LINE PARAMETERS
    //
    CString tempCmdLine;
    CString path;
    int     pos;
    int     pos2;
    tempCmdLine = GetCommandLine();
    tempCmdLine.TrimRight();
    tempCmdLine += " ";

	CString message;
	message.Format("Application command line: '%s'", tempCmdLine);
	LOG_INSERTSTRING(message);

    // EXTERNAL CONFIGURATION SCRIPT
    if ((pos = tempCmdLine.Find("/s:\"")) != -1) {
        pos += (int) strlen("/s:\""); 
        pos2 = tempCmdLine.Find("\"", pos);
        path = tempCmdLine.Mid(pos, pos2 - pos);

		m_defaultCfgFile.Format("%s", path);
    }
	
	// DEFAULT POPULATION FILE
    if ((pos = tempCmdLine.Find("/p:\"")) != -1) {
        pos += (int) strlen("/p:\""); 
        pos2 = tempCmdLine.Find("\"", pos);
        path = tempCmdLine.Mid(pos, pos2 - pos);

		m_defaultPopulationFile = path;
    }

    // RUN IN BACKGROUND
    if ((pos = tempCmdLine.Find("/b ")) != -1) {
		m_visible = FALSE;
    }
		
    // EXIT AFTER TESTS IF REQUIRED
    if ((pos = tempCmdLine.Find("/e ")) != -1) {
		m_testQueue.ExitAfterTests(TRUE);
    }

    // RUN TESTS
    if ((pos = tempCmdLine.Find("/r ")) != -1) {
		m_runTests = TRUE;
    }

	if ((pos = tempCmdLine.Find("/x:\"")) != -1) {
		m_runProxy = TRUE;
        pos += (int) strlen("/x:\""); 
        pos2 = tempCmdLine.Find("\"", pos);
        m_proxyPort = atoi(tempCmdLine.Mid(pos, pos2 - pos));        
    }

	m_numRepetitions = 0;
	if ((pos = tempCmdLine.Find("/rep:\"")) != -1) {
        pos += (int) strlen("/rep:\""); 
        pos2 = tempCmdLine.Find("\"", pos);
        m_numRepetitions = atoi(tempCmdLine.Mid(pos, pos2 - pos));        
    }
	if (m_numRepetitions <= 0) m_numRepetitions = 1;
	message.Format("Num repetitions: '%d'", m_numRepetitions);
	LOG_INSERTSTRING(message);


}

void CSensorSimCore::Run() {
	int     status = STAT_OK;

	if (m_numRepetitionsElapsed < m_numRepetitions) {

		// Do init only once
		if (m_numRepetitionsElapsed == 0) {
			// DELETE OLD FILES
			DeleteFile(pSensorSim->m_logFile);
			DeleteFile("fraction_done.txt");
    
			// INIT RANDOM GENERATOR
			DWORD seed = (unsigned int) time(NULL);
			m_sensorEnv.m_rndGen.InitRandomGenerator(seed);
			pSensorSim->PostMessage(WM_RNG_UPDATESEED, (WPARAM) seed);
	
			//
			// DISPLAY COMMAND LINE HELP
			//
			CString message;
			CVersionInfo    ver;    
			ver.GetVersionInfo("SensorSim.exe");

			message.Format("S3 - Sensor Security Simulator, version %s\n", ver.m_strFixedProductVersion);
			LOG_INSERTSTRING_NOTIME(message);

			message.Format("*****************************************************************************************\n\
COMMAND LINE SWITCHES\n\
	/s:\"filepath.ini\"\t\t... configuration script to be loaded (do not ommit \"\")\n\
	/p:\"population.ini\"\t... population to be loaded\n\
	/r\t\t\t... run test required by configuration script (/s: switch)\n\
	/rep:\"rep_number\"\t\t... number of repetitions of SensorSim run for given (same) configuration. Basic configuration remains the same, .\n\
	\t\t\t\tOriginal configuration script is ised (/s: switch) and intermediate save of population (file \"population_intermediate.ini\") except first repeatition where\n\
	\t\t\t\t/p: switch is used. The reason is to be able to deterministically restart evolution and reach same intermediate population without runing whole evolution from initial configuration files.\n\
	\t\t\t\tAttempts to consistently recover whole internal GALib state from file failed so far due to high complexity of internal GaLib implementation.\n\
	\t\t\t\tFinal number of evolution generations simulated is equal to rep_number * NUM_GENERATIONS value from configuration script.\n\
	/b\t\t\t... run in background\n\
	/e\t\t\t... exit application after executed tests\n\
	/x:\"port\"\t\t... run remote control proxy (e.g., port 4001)\n\
	*****************************************************************************************");
			LOG_INSERTSTRING_NOTIME(message);
		}
		CString message;
		// LOAD SUPPLIED CONFIGURATION SCRIPT
		if ((status = pSensorSim->m_sensorEnv.LoadConfigScript(m_defaultCfgFile, &pSensorSim->m_sensorEnv.m_basicSettings)) == STAT_OK) {
			if (pSensorSimDlg != NULL)
				pSensorSimDlg->SetBasicSettings(&pSensorSim->m_sensorEnv.m_basicSettings);	// save to GUI
		}

		if (status == STAT_OK) {
			message.Format("Load config. script '%s' OK", m_defaultCfgFile);
			LOG_INSERTSTRING(message);
		} else {        
			message.Format("Load config. script '%s' fail with %s(%d)", m_defaultCfgFile, ErrorToString(status), status);
			LOG_INSERTSTRING(message);
		}

		if (m_runTests){
			m_testQueue.Add(&pSensorSim->m_sensorEnv.m_basicSettings);
			m_testQueue.Run();
		}

		// OPEN PROXY ON GIVEN PORT AND LISTEN FOR COMMANDS
		if (m_runProxy) 
			m_proxy.RunProxy(m_proxyPort);
	
		CFile   file;
		if (file.Open(pSensorSim->m_logFile, CFile::modeWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
			file.SeekToEnd();
			file.Write("**********", (int) strlen("**********"));    
			file.Close();
		}

		// DECREASE PRIORITY OF THIS PROCESS
		SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
/*
		// WAIT FOR FINISH OF STARTED THREADS

		DWORD dwRet;
		MSG msg;
		DWORD dwTimeout = 1000;
		while(true) {
			dwRet = MsgWaitForMultipleObjects(1, &(pSensorSim->m_pActualThread->m_hThread), FALSE, 1000, QS_ALLINPUT);
			if (dwRet == WAIT_OBJECT_0) break;
			if (dwRet == WAIT_OBJECT_0 + 1) {
				if (PeekMessage(&msg,NULL,NULL,NULL,PM_REMOVE)) {
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
		}

		/*
		DWORD dwRet = WaitForSingleObject(&(pSensorSim->m_pActualThread->m_hThread), INFINITE);
		switch (dwRet) {
			case WAIT_OBJECT_0 : break; //OK
			default: return;	// problem
		}
		*/
	}
}

int CSensorSimCore::Log_InsertString(CString message, BOOL bShowTime, BOOL bToFile) {
    int             status = STAT_OK;
    char            timeStr[MAX_PATH];
	
    message.Replace("\n", "\r\n");

    // TIME STAMP
    if (bShowTime) {
        message.Insert(0, "  ");
        _strtime(timeStr);
        message.Insert(0, timeStr); 
    }
    
    // ADD ENDING "\r\n" IF NOT ALREADY PRESENT
    if (message.Right(2) != "\r\n") message += "\r\n";
    
	if (pSensorSimDlg != NULL) 
		pSensorSimDlg->Log_InsertString(message);

    // WRITE INTO LOG FILE 
    if (bToFile) Log_InsertToFile(pSensorSim->m_logFile, message);

    return status;
}

int CSensorSimCore::Log_AppendString(CString message) {
    int             status = STAT_OK;
	
    message.Replace("\n", "\r\n");

	if (pSensorSimDlg != NULL) 
		pSensorSimDlg->Log_AppendString(message);

    return status;
}

int CSensorSimCore::Log_InsertClearLine() {
    int             status = STAT_OK;
	
    // WRITE INTO LOG FILE 
    //TODO: CCommonFnc::File_AppendString(pSensorSim->m_logFile, "\n");
	
	if (pSensorSimDlg != NULL) 
		pSensorSimDlg->Log_InsertClearLine();

    return status;
}

int CSensorSimCore::Log_InsertSeparator() {
    int             status = STAT_OK;
	
    // WRITE INTO LOG FILE 
    //TODO: CCommonFnc::File_AppendString(pSensorSim->m_logFile, "==================================================================\r\n");
	
	if (pSensorSimDlg != NULL) 
		pSensorSimDlg->Log_InsertSeparator();

    return status;
}

int CSensorSimCore::Log_InsertToFile(CString fileName, CString message) {
    int     status = STAT_OK;
    // WRITE INTO LOG FILE 
    CFile   file;
    if (file.Open(fileName, CFile::modeWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
        file.SeekToEnd();
        file.Write(message, message.GetLength());    
        file.Close();
    }
    else status = STAT_FILE_OPEN_FAIL;
    
    return status;
}

void CSensorSimCore::ClearPerformance() {
	pSensorSim->perfNestLevel = 0;	
	pSensorSim->perfCallTime.clear();
    pSensorSim->perfCallCounter.clear();
}

void CSensorSimCore::ShowPerformance() {
    lcs::iterator	iter;
    msd::iterator   iter2;
    msd::iterator   iter3;
    int             level = 0;
    CString         message;
    
    Log_InsertToFile("perfTime.txt", "\r\nFUNCTIONS STATISTICS\r\n");
    for (iter3 = pSensorSim->perfCallTime.begin(); iter3 != pSensorSim->perfCallTime.end(); iter3++) {
        double min = ULLONG_MAX;
        for (iter2 = pSensorSim->perfCallCounter.begin(); iter2 != pSensorSim->perfCallCounter.end(); iter2++) {
            if (iter2->second < min && iter2->second != -1) {
                min = iter2->second;
            }
        } 
        
        if (min != ULLONG_MAX) {
            for (iter2 = pSensorSim->perfCallCounter.begin(); iter2 != pSensorSim->perfCallCounter.end(); iter2++) {
                if (iter2->second == min && iter2->second != -1) {
	                message.Format("totalTime: %18.0Lf | total calls: %8.0Lf | time per call : %12.0Lf (%s)\r\n", pSensorSim->perfCallTime[(LPCTSTR)iter2->first], iter2->second, pSensorSim->perfCallTime[(LPCTSTR)iter2->first] / iter2->second, iter2->first);
	                Log_InsertToFile("perfTime.txt", message);

                    iter2->second = -1;
                    break;
	            }
	        }
	    }
    }
}

BOOL CSensorSimCore::PostMessageA(UINT Msg, WPARAM wParam, LPARAM lParam) {
	// ALWAYS FORWARD TO APPLICATION
	AfxGetApp()->PostThreadMessage(Msg, wParam, lParam);	
	return 0;
}

LRESULT CSensorSimCore::OnLogInsertString(WPARAM wParam, LPARAM lParam) {
	// INSERT INTO LOG
    switch (lParam) { 
        case 1: {
            Log_InsertString((LPCTSTR) ((char*) wParam), TRUE);  
            break;
        }
        case 2: {
            Log_InsertString((LPCTSTR) ((char*) wParam), FALSE);  
            break;
        }
        case 3: {
            Log_AppendString((LPCTSTR) ((char*) wParam));  
            break;
        }
        case 4: {
            Log_InsertClearLine();  
            break;
        }
        case 5: {
            Log_InsertSeparator();  
            break;
        }
    }
    
    // CLEAR LOG STRING    
    if (wParam) delete[] (char*) wParam;

    return 0;
}
LRESULT CSensorSimCore::OnUpdateRNGSeed(WPARAM wParam, LPARAM lParam) {
	if (pSensorSimDlg != NULL){
		// UPDATE RNG SEED FIELD
		pSensorSimDlg->m_tabBasic.UpdateData(TRUE);
		pSensorSimDlg->m_tabBasic.m_randomSeed.Format("%d", (DWORD) wParam);
		pSensorSimDlg->m_tabBasic.UpdateData(FALSE);
	}
	return 0;
}

int CSensorSimCore::RunRemoteExecution(CString host, CString configFilePath, CString instanceSubdir) {
	//
	// RUN ON SELECTED REMOTE COMPUTER
	//
	int                 status = STAT_OK;
	SocketClient*		pSocket = NULL;
	REMOTE_HOST_INFO	remoteInfo;
	CString				response;
	CString				request; 

	CString IP = host.Left(host.Find(":"));
    CString port = host.Right(host.GetLength() - host.Find(":") - 1);

	// try to connect to proxy on remote computer
	try {
		if ((pSocket = new SocketClient((LPCTSTR) IP, atoi(port))) != NULL) {
			CString				retStat;
			
			// COPY SCRIPT WITH SETTINGS
			CFile	file;
			CString	settings;
			if (file.Open(configFilePath, CFile::modeRead)) {
			    int	len = file.GetLength() * 3;
			    TCHAR*	buffer = new TCHAR[len];
			    memset(buffer, 0, len * sizeof(TCHAR));
			    file.Read(buffer, file.GetLength());
			    settings = buffer;
			    delete[] buffer;
			    file.Close();
            }
            else status = STAT_FILE_OPEN_FAIL;
            
            if (status == STAT_OK) {
			    CString command;

			    command.Format("%s#SensorSim.exe#%s\\SensorSim.exe#", PROXY_COPYFILE, instanceSubdir);
			    pSocket->SendLine((LPCTSTR) command);
			    string l = pSocket->ReceiveResponse(SCSAT_SOCKET_ENDSEQ, SCSAT_SOCKET_TIMEOUT);
			    CString data = l.c_str();

			    command.Format("%s#%s\\run_cfg.ini#%d#\n%s", PROXY_SETFILECONTENT, instanceSubdir, settings.GetLength(), settings);
			    pSocket->SendLine((LPCTSTR) command);
			    l = pSocket->ReceiveResponse(SCSAT_SOCKET_ENDSEQ, SCSAT_SOCKET_TIMEOUT);
			    data = l.c_str();

			    command.Format("%s#%s\\SensorSim.exe#/s:\"run_cfg.ini\" /r #", PROXY_RUNAPPLICATION, instanceSubdir);
			    pSocket->SendLine((LPCTSTR) command);
			    l = pSocket->ReceiveResponse(SCSAT_SOCKET_ENDSEQ, SCSAT_SOCKET_TIMEOUT);
			    data = l.c_str();
		    }
		}
		else {
		}

		// disconnect
		request.Format("%s%s", PROXY_DISCONNECT, PROXY_SEPARATOR);
		m_proxy.SocketSendLine(pSocket, request, &response);
		delete pSocket;
	}
	catch (...) {
		status = STAT_CHANNEL_NOT_EXISTS;	
	}
	
	return status;
}

CString     lastSuccessApplicationChannel = "";
