// SensorSimCode.h : header file
//

#if !defined(__SENSORSIMCORE_H__)
#define __SENSORSIMCORE_H__

#include "StdAfx.h"
#include "SensorEnv.h"
#include "socket.h"
#include "EACircuit.h"
#include "TestQueue.h"
#include "Proxy.h"

/////////////////////////////////////////////////////////////////////////////
// CSensorSimCore

class CSensorSimCore
{
// Construction
public:
	CSensorSimCore();
    ~CSensorSimCore();

    GA_STRATEGY*     m_pGAStrategy;   
    GA_STRATEGY		 m_tempStrategy; // temporary strategy structure

    int              perfNestLevel;      // actual nesting level of performance monitor
    msd              perfCallTime;       // map that counts total time in given function of function ivocations (by fnc name)
    msd              perfCallCounter;    // map that can count number of function ivocations (by fnc name)
    
public:
//    BOOL        m_pActualDeployment->sensorsDeployed;
    BOOL        m_stopThreadSoft;
    CWinThread* m_pActualThread;
	
	CString	    m_logs;
	CString     m_basePath;

// Implementation
public:
	void Run();
	void HandleCommandLine();

    //
    //  LOGGING FUNCTIONS
    //
    int Log_InsertString(CString message, BOOL bShowTime = TRUE, BOOL bToFile = TRUE);
    int Log_AppendString(CString message);
    int Log_InsertClearLine();
    int Log_InsertSeparator();
    int Log_EnsureVisibleLastLine();
    
    int Log_InsertToFile(CString fileName, CString message);
    
    void ClearPerformance();
    void ShowPerformance();

	int RunRemoteExecution(CString host, CString configFilePath, CString instanceSubdir);
	
    CEACircuit*         m_pEACircuit;       // class with structures and functions for evolution curcuit    
    CSensorEnv          m_sensorEnv;        // class with deployed sensors structures
	CProxy				m_proxy;
	
	// CORE OPTIONS
    CString				m_defaultPopulationFile;
	CString				m_defaultCfgFile;
    CString             m_logFile;
	BOOL				m_runTests;
	BOOL				m_runProxy;
	int					m_proxyPort; 
	int					m_numRepetitions;	// number of repetitions, search /rep: switch for more details
	int					m_numRepetitionsElapsed;

public:  
    BOOL                m_bStopThread;	

	BOOL				PostMessageA(UINT Msg, WPARAM wParam = 0, LPARAM lParam = 0);
	LRESULT				OnLogInsertString(WPARAM wParam, LPARAM lParam);	
	LRESULT				OnUpdateRNGSeed(WPARAM wParam, LPARAM lParam);

public:
	BOOL				m_visible;
	HWND				m_hWnd;

	CTestQueue			m_testQueue;    
};

#endif