#include "stdafx.h"
#include "SensorSim.h"
#include "SensorSimCore.h"
#include "KeyInfectTab.h"
#include "SensorEnv.h"
#include "Action.h"
#include "CommonFnc.h"
#include "ga/GASStateGA.h"
#include "ga/garandom.h"
#include "GAS_Genome.h"

extern CSensorSimCore*  pSensorSim;

//#define _LOG_GA_SEED_	{ char line[1024]; sprintf(line, "RNG=%u (line %d, file %s)", GARandomInt(0, INT_MAX), __LINE__, __FILE__); LOG_INSERTSTRING_NOTIME(line); }
#define _LOG_GA_SEED_ 

//JK
GA_ROUTE gaRoute;

CSensorEnv::CSensorEnv() {
    m_ssNextKeyID = 1;
	// allocate possible deployments structs
	for (int i = 0; i < MAX_DEPLOYMENTS; i++) {
		m_deploymentList[i] = new DEPLOYMENT_INFO;
	}
	m_pActualDeployment = m_deploymentList[0];
	m_pActualDeployment->sensorsDeployed = FALSE;
    m_lastLoadedGenom = -1;
    m_pLoadedPop = NULL;
    
    memset(bestNC, 0, sizeof(bestNC));
}
CSensorEnv::~CSensorEnv() {
	for (int i = 0; i < MAX_DEPLOYMENTS; i++) {
		if (m_deploymentList[i] != NULL) {
			m_deploymentList[i]->release();
			delete m_deploymentList[i]; 
			m_deploymentList[i] = NULL;
		}
	}
	if (m_pLoadedPop != NULL){
		delete m_pLoadedPop;
		m_pLoadedPop = NULL;
	}
}

int CSensorEnv::DeploySensors(BASIC_INIT_DATA* pBasicSettings) {
    int             status = STAT_OK;
    DWORD           i;
    DWORD           j;
    DWORD           random;
//    int             k;
//    int             l;
//    int             blackUp;
//    int             whiteUp;
//    DWORD           flags;
//    int             maxCell;
    SENSOR_NODE*    pNode;
    DWORD           approx_count_X = (DWORD) (pBasicSettings->planeX * APPROX_STEP);
    DWORD           approx_count_Y = (DWORD) (pBasicSettings->planeY * APPROX_STEP);
    float           help_X = pBasicSettings->planeX / approx_count_X;
    float           help_Y = pBasicSettings->planeY / approx_count_Y;
    float           nextBS_X = 0;
    float           nextBS_Y = 0;
    float           nextBlack_X = 0;
    float           nextBlack_Y = 0;
    CString         message;
    int             totalWhiteNodes = 0;

    // RELEASE PREVIOUS STRUCTS
    m_pActualDeployment->release();
    m_pActualDeployment->sensorsDeployed = FALSE;


    // GENERATE PLANE AND GEOGRID
    m_pActualDeployment->geoGrid.cellSize = pBasicSettings->whiteTransmRange;
    m_pActualDeployment->geoGrid.numCellX = (DWORD) (pBasicSettings->planeX / m_pActualDeployment->geoGrid.cellSize) + 1;
    m_pActualDeployment->geoGrid.numCellY = (DWORD) (pBasicSettings->planeY / m_pActualDeployment->geoGrid.cellSize) + 1;
    m_pActualDeployment->geoGrid.grid = new NEIGH_LIST_NODE**[m_pActualDeployment->geoGrid.numCellX];
    for (i = 0; i < m_pActualDeployment->geoGrid.numCellX; i++) {
        m_pActualDeployment->geoGrid.grid[i] = new NEIGH_LIST_NODE*[m_pActualDeployment->geoGrid.numCellY];
        for (j = 0; j < m_pActualDeployment->geoGrid.numCellY; j++) m_pActualDeployment->geoGrid.grid[i][j] = NULL; 
    }

    //
    //  WHITE SENSORS
    //
    totalWhiteNodes = pBasicSettings->whiteNum + pBasicSettings->bsConfig.bsCount + pBasicSettings->undeployedWhiteNum;

    m_pActualDeployment->whiteSensorList.numSensors = pBasicSettings->whiteNum + pBasicSettings->bsConfig.bsCount;
    m_pActualDeployment->whiteSensorList.sensors = new SENSOR_NODE*[totalWhiteNodes]; 
    for (i = 0; i < totalWhiteNodes; i++) m_pActualDeployment->whiteSensorList.sensors[i] = NULL;
    
    m_pActualDeployment->BSList.itemCount = pBasicSettings->bsConfig.bsCount;
    m_pActualDeployment->BSList.bs = new SENSOR_NODE*[m_pActualDeployment->BSList.itemCount]; 
    for (i = 0; i < m_pActualDeployment->BSList.itemCount; i++) m_pActualDeployment->BSList.bs[i] = NULL;

    // PROCUMPTATION FOR BS DISTRIBUTED IN GRID (BOX AREA ASSUMED)
    float BSperRow = sqrt((float)pBasicSettings->bsConfig.bsCount);
    float distrBS2BS = pBasicSettings->planeX / BSperRow;
    nextBS_X = nextBS_Y = distrBS2BS / 2;

    // CREATE WHITE SENSOR STRUCTS AND BASE STATIONS AND UNDEPLOYED NODES
    for (i = 0; i < totalWhiteNodes; i++) {
        // DISTRIBUTE WHITE SENSORS AND CREATE SEARCHING STRUCTRURES
        pNode = new SENSOR_NODE;
        pNode->id = i;
        pNode->flags = FLAG_SENSOR_WHITE;


        // BS ROUTING INFO STRUCTURES
        pNode->routeTableBS = new BS_ROUTE_STRUCT[pBasicSettings->bsConfig.bsCount];
        memset(pNode->routeTableBS, 0, sizeof(BS_ROUTE_STRUCT) * pBasicSettings->bsConfig.bsCount);
        
        // LAST pBasicSettings->bsConfig.bsCount ARE BASE STATIONS
        if (i >= pBasicSettings->whiteNum && i < pBasicSettings->whiteNum + pBasicSettings->bsConfig.bsCount) {
            pNode->flags |= FLAG_SENSOR_BASE_STATION;

            switch (pBasicSettings->bsConfig.bsPlacement) {
                case 1: {
                    // BS DISTRIBUTED ONTO CIRCLE
                    float measure= pBasicSettings->planeX + pBasicSettings->planeY;
                    pNode->cX= (float) (cos(2*3.14*(i-pBasicSettings->whiteNum)/pBasicSettings->bsConfig.bsCount)
                                    *pBasicSettings->planeX/2+pBasicSettings->planeX)/2;
                    pNode->cY= (float) (sin(2*3.14*(i-pBasicSettings->whiteNum)/pBasicSettings->bsConfig.bsCount)
                                    *pBasicSettings->planeY/2+pBasicSettings->planeY)/2;
                    break;
                }
                case 2: {
                    // BS DISTRIBUTED IN GRID (BOX AREA ASSUMED)
                    
                    // ASSIGN ACTUAL COORDINATES
                    pNode->cX = nextBS_X;
                    pNode->cY = nextBS_Y;

                    //message.Format("BS (%f,%f)", pNode->cX, pNode->cY);
                    //LOG_INSERTSTRING(message);

                    // PREPARE COORDINATES FOR NEXT BS
                    if ((nextBS_X + distrBS2BS) > pBasicSettings->planeX) {
                        if ((nextBS_Y + distrBS2BS) > pBasicSettings->planeY) {
                            // all grid positions filled, distribute randomly
                            m_rndGen.GetRandomFromInterval(approx_count_X, &random);
                            nextBS_X = random * help_X;
                            m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
                            nextBS_Y = random * help_Y;
                        }
                        else {
                            // move to begin of next row
                            nextBS_X = distrBS2BS / 2; 
                            nextBS_Y = nextBS_Y + distrBS2BS;
                        }
                    }  
                    else {
                        // ASSIGN NEXT FREE POSITION IN ROW, DO NOT CHANGE nextBS_X
                        nextBS_X = nextBS_X + distrBS2BS; 
                    }
                    break;
                }
                case 0: 
                default: {
                    // BS PLACED RANDOMLY
                    m_rndGen.GetRandomFromInterval(approx_count_X, &random);
                    pNode->cX = random * help_X;
                    m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
                    pNode->cY = random * help_Y;
                    break;
                }
            }
        
/**/
        } else {
            m_rndGen.GetRandomFromInterval(approx_count_X, &random);
            pNode->cX = random * help_X;
            m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
            pNode->cY = random * help_Y;
        }
        
        pNode->transmRange = pBasicSettings->whiteTransmRange;
        pNode->receiveRangeRatio = 1;   // SAME TRANSMISSION AND RECEIVING RANGE IS ASSUMED FOR WHITE

        // INSERT INTO ALL SENSORS LIST
        m_pActualDeployment->whiteSensorList.sensors[pNode->id] = pNode;

        // INSERT INTO APPRIPRIATE GRID CELL
        if (!(pNode->flags & FLAG_SENSOR_UNDEPLOYED)) {
            m_pActualDeployment->geoGrid.insertNode(pNode);
        }

        // LAST pBasicSettings->bsConfig.bsCount ARE BASE STATIONS
        if (i >= pBasicSettings->whiteNum && i < pBasicSettings->whiteNum + pBasicSettings->bsConfig.bsCount) m_pActualDeployment->BSList.bs[i - pBasicSettings->whiteNum] = pNode;

    }

    for (;i < pBasicSettings->undeployedWhiteNum; i++) {
        // DISTRIBUTE WHITE SENSORS AND CREATE SEARCHING STRUCTRURES
        pNode = new SENSOR_NODE;
        pNode->id = i;
        pNode->flags = FLAG_SENSOR_WHITE | FLAG_SENSOR_UNDEPLOYED;
        
        // BS ROUTING INFO STRUCTURES
        pNode->routeTableBS = new BS_ROUTE_STRUCT[pBasicSettings->bsConfig.bsCount];
        memset(pNode->routeTableBS, 0, sizeof(BS_ROUTE_STRUCT) * pBasicSettings->bsConfig.bsCount);

        // ASSIGN FUTURE DEPLOYMENT POSITION, BUT NOT INSERT INTO GEOGRID    
        m_rndGen.GetRandomFromInterval(approx_count_X, &random);
        pNode->cX = random * help_X;
        m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
        pNode->cY = random * help_Y;
    
        pNode->transmRange = pBasicSettings->whiteTransmRange;
        pNode->receiveRangeRatio = 1;   // SAME TRANSMISSION AND RECEIVING RANGE IS ASSUMED FOR WHITE

        // INSERT INTO ALL SENSORS LIST
        //m_pActualDeployment->undeployedWhiteSensorList.sensors[pNode->id] = pNode;

        // INSERT INTO APPRIPRIATE GRID CELL
        m_pActualDeployment->geoGrid.insertNode(pNode);

        // LAST pBasicSettings->bsConfig.bsCount ARE BASE STATIONS
        if (i >= pBasicSettings->whiteNum) m_pActualDeployment->BSList.bs[i - pBasicSettings->whiteNum] = pNode;

    }



    // PREDISTRIBUTE KEYS IF REQUIRED
    if (pBasicSettings->egScheme.useIt) {
        status = KeyPredistrib_EGScheme(pBasicSettings);
    }

    //
    //  BLACK SENSORS
    //
    m_pActualDeployment->blackSensorList.numSensors = pBasicSettings->blackNum;

    m_pActualDeployment->blackSensorList.sensors = new SENSOR_NODE*[m_pActualDeployment->blackSensorList.numSensors]; 
    for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) m_pActualDeployment->blackSensorList.sensors[i] = NULL;

    // PROCUMPTATION FOR BLACK DISTRIBUTED IN GRID (BOX AREA ASSUMED)
    float blackPerRow = sqrt((float)pBasicSettings->blackNum);
    float distrBlack2Black = pBasicSettings->planeX / blackPerRow;
    nextBlack_X = nextBlack_Y = distrBlack2Black / 2;

    // CREATE BLACK SENSOR STRUCTS
    for (i = 0; i < pBasicSettings->blackNum; i++) {
        // DISTRIBUTE BLACK SENSORS AND CREATE SEARCHING STRUCTRURES
        pNode = new SENSOR_NODE;
        pNode->id = i;
        pNode->flags = FLAG_SENSOR_BLACK;

        if (pBasicSettings->distribFlags & FLAG_DISTRIB_BLACKINGRID) {
            // PLACE IN REGULAR GRID 
            // ASSIGN ACTUAL COORDINATES
            pNode->cX = nextBlack_X;
            pNode->cY = nextBlack_Y;

            // PREPARE COORDINATES FOR NEXT BS
            if ((nextBlack_X + distrBlack2Black) > pBasicSettings->planeX) {
                if ((nextBlack_Y + distrBlack2Black) > pBasicSettings->planeY) {
                    // all grid positions filled, distribute randomly
                    m_rndGen.GetRandomFromInterval(approx_count_X, &random);
                    nextBlack_X = random * help_X;
                    m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
                    nextBlack_Y = random * help_Y;
                }
                else {
                    // move to begin of next row
                    nextBlack_X = distrBlack2Black / 2; 
                    nextBlack_Y = nextBlack_Y + distrBlack2Black;
                }
            }  
            else {
                // ASSIGN NEXT FREE POSITION IN ROW, DO NOT CHANGE nextBlack_Y
                nextBlack_X = nextBlack_X + distrBlack2Black; 
            }
        }
        else {
            // PLACE RANDOMLY
            m_rndGen.GetRandomFromInterval(approx_count_X, &random);
            pNode->cX = random * help_X;
            m_rndGen.GetRandomFromInterval(approx_count_Y, &random);
            pNode->cY = random * help_Y;
        }
        
        pNode->transmRange = pBasicSettings->blackTransmRange;
        pNode->receiveRangeRatio = pBasicSettings->blackRatioReceiveRange;

        // INSERT INTO ALL SENSORS LIST
        m_pActualDeployment->blackSensorList.sensors[pNode->id] = pNode;

        // INSERT INTO APPRIPRIATE GRID CELL
        m_pActualDeployment->geoGrid.insertNode(pNode);
    }


    status = DiscoverNeighbours(pBasicSettings, FLAG_SENSOR_WHITE | FLAG_SENSOR_BLACK);

    // OPTIONAL TESTS INTEGRITY OF DISTRIBUTION AGAINST FAULTS
    status = TestDeployIntegrity();

    if (status == STAT_OK) m_pActualDeployment->sensorsDeployed = TRUE; 

    return status;
}

int CSensorEnv::RemoveDeployedNodes(DWORD flags) {
    int             status = STAT_OK;
    SENSOR_LIST*    pList = NULL;

    if (flags & FLAG_SENSOR_WHITE) { pList = &(pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList); }
    if (flags & FLAG_SENSOR_BLACK) { pList = &(pSensorSim->m_sensorEnv.m_pActualDeployment->blackSensorList); }
    
    // REMOVE ALL NODES (BLACK EITHER WHITE) FROM GEOGRID
    for (int i = 0; i < pList->numSensors; i++) {
        if (pList->sensors[i] != NULL) { 
            pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.removeNode(pList->sensors[i]);
        }
    }
    
    // REMOVE BLACKS FROM ALL WHITE NEIGHB STRUCTS, IF NEEDED
    if (flags & FLAG_SENSOR_BLACK) {
        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            if (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i] != NULL) { 
                (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i])->releaseBlackNeigh();
            }
        }
    }
        
    // RELEASE NODES LIST
    pList->release();

    return status;
}
    
int CSensorEnv::DeployBlackNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern) {
    return DeployNodesByPattern(pBasicSettings, pDeployPattern, FLAG_SENSOR_BLACK);
}

int CSensorEnv::DeployWhiteNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern) {
    return DeployNodesByPattern(pBasicSettings, pDeployPattern, FLAG_SENSOR_WHITE);
}

int CSensorEnv::DeployNodesByPattern(BASIC_INIT_DATA* pBasicSettings, NODES_DEPLOY_PATTERN* pDeployPattern, DWORD flags) {
    int             status = STAT_OK;
    SENSOR_LIST*    pList = NULL;
    
    if (flags & FLAG_SENSOR_WHITE) {
        pList = &(pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList);
        pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors = pBasicSettings->whiteNum;
    }    
    if (flags & FLAG_SENSOR_BLACK) {
        pList = &(pSensorSim->m_sensorEnv.m_pActualDeployment->blackSensorList);
        pSensorSim->m_sensorEnv.m_pActualDeployment->blackSensorList.numSensors = pBasicSettings->blackNum;
    }    
    
    //
    //  ALLOCATE LIST
    //
    pList->sensors = new SENSOR_NODE*[pList->numSensors]; 
    for (int i = 0; i < pList->numSensors; i++) pList->sensors[i] = NULL;

    // 
    float x_offset = 0;
    float y_offset = 0;
    float x_offset_step = pBasicSettings->planeX / (float) pDeployPattern->numCellPerAxis;
    float y_offset_step = pBasicSettings->planeY / (float) pDeployPattern->numCellPerAxis;
    int nodesPerCell = 0;
    if (flags & FLAG_SENSOR_BLACK) nodesPerCell = (int) pBasicSettings->blackNum  / (float) (pow((float) pDeployPattern->numCellPerAxis, 2));   
    if (flags & FLAG_SENSOR_WHITE) nodesPerCell = (int) pBasicSettings->whiteNum  / (float) (pow((float) pDeployPattern->numCellPerAxis, 2));   
    
    // CREATE SENSORS STRUCTS
    int k = 0;
    for (int i = 0; i < pList->numSensors; i++) {
        // DISTRIBUTE BLACK SENSORS AND CREATE SEARCHING STRUCTRURES
        SENSOR_NODE* pNode = new SENSOR_NODE;
        pNode->id = i;
        if (flags & FLAG_SENSOR_WHITE) {pNode->flags = FLAG_SENSOR_WHITE;}
        if (flags & FLAG_SENSOR_BLACK) {pNode->flags = FLAG_SENSOR_BLACK;}

        // PLACE IN ACTUAL CELL BASED ON PATERN
        float xPos = (pDeployPattern->pattern[k * 2] / (float) INT_MAX);
        float yPos = (pDeployPattern->pattern[k * 2 + 1] / (float) INT_MAX);
        ASSERT(xPos >= 0 && xPos <= 1 && yPos >= 0 && yPos <= 1);
        
        pNode->cX = x_offset + (x_offset_step * xPos);
        pNode->cY = y_offset + (y_offset_step * yPos);
        if (flags & FLAG_SENSOR_WHITE) {
            pNode->transmRange = pBasicSettings->whiteTransmRange;
            pNode->receiveRangeRatio = 1;
        }
        if (flags & FLAG_SENSOR_BLACK) {
            pNode->transmRange = pBasicSettings->blackTransmRange;
            pNode->receiveRangeRatio = pBasicSettings->blackRatioReceiveRange;
        }

        // INSERT INTO ALL SENSORS LIST
        pList->sensors[pNode->id] = pNode;
        
        // GO TO NEXT PATTERN NODE
        k++;
        if (k >= nodesPerCell) {
            // INCREASE OFFSETS
            if ((x_offset + x_offset_step) >= pBasicSettings->planeX) {
                // GO TO BEGIN OF NEXT ROW
                y_offset += y_offset_step;
                x_offset = 0;
            }
            else {
                // GO TO NEXT CELL IN ROW
                x_offset += x_offset_step;
            }
            
            // FIRST PATTERN NODE
            k = 0;
        }
    }

    // INSERT NODES INTO APPRIPRIATE GEO GRID CELL
    for (int i = 0; i < pList->numSensors; i++) {
        pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.insertNode(pList->sensors[i]);
    }

    // DISCOVER NEIGHBOURS
    if (flags & FLAG_SENSOR_WHITE) {
        status = DiscoverNeighbours(pBasicSettings, FLAG_SENSOR_WHITE | FLAG_SENSOR_BLACK);
    }
    if (flags & FLAG_SENSOR_BLACK) {
        status = DiscoverNeighbours(pBasicSettings, FLAG_SENSOR_BLACK);
    }
    
    return status;
}


int CSensorEnv::TestDeployIntegrity() {
    int                 status = STAT_OK;
    DWORD               i;
    SENSOR_NODE*        pSensor;
    SENSOR_NODE*        pSensor2;
    NEIGH_LIST_NODE*    pListNode;
    BYTE*               pWhiteID;  
    BYTE*               pBlackID;  
    CString             message;            

    LOG_APPENDSTRING("--- TestDeployIntegrity start ---");

    pWhiteID = new BYTE[m_pActualDeployment->whiteSensorList.numSensors];
    memset(pWhiteID, 0, m_pActualDeployment->whiteSensorList.numSensors);
    pBlackID = new BYTE[m_pActualDeployment->blackSensorList.numSensors];
    memset(pBlackID, 0, m_pActualDeployment->blackSensorList.numSensors);

    //
    //  TEST1: NO MULTIPLE INTANCES OF SAME NODE AS NEIGHBOUR 
    //
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
        
        // WHITE NEIGHBOURS
        pListNode = pSensor->whiteNeigh;
        memset(pWhiteID, 0, m_pActualDeployment->whiteSensorList.numSensors);
        
        while (pListNode != NULL) {
            pSensor2 = (SENSOR_NODE*) pListNode->pItem;

            if (!(pSensor2->flags & FLAG_SENSOR_WHITE)) {
                message.Format("ERROR: Node %d: FLAG_SENSOR_WHITE missing for %d neighbour", pSensor->id, pSensor2->id);
                LOG_APPENDSTRING(message);
                ASSERT(FALSE);
                status = STAT_DEPLOY_INTEGRITY_BAD;
            }
            
            if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                pWhiteID[pSensor2->id]++;
                if (pWhiteID[pSensor2->id] > 1) {
                    message.Format("ERROR: Node %d: Multiple neighbour %d", pSensor->id, pSensor2->id);
                    LOG_APPENDSTRING(message);
                    ASSERT(FALSE);
                    status = STAT_DEPLOY_INTEGRITY_BAD;
                }
            }

            pListNode = pListNode->next;
        }

        // BLACK NEIGBOURS
        pListNode = pSensor->blackNeigh;
        memset(pBlackID, 0, m_pActualDeployment->blackSensorList.numSensors);
        
        while (pListNode != NULL) {
            pSensor2 = (SENSOR_NODE*) pListNode->pItem;

            if (!(pSensor2->flags & FLAG_SENSOR_BLACK)) {
                message.Format("ERROR: Node %d: FLAG_SENSOR_BLACK missing for %d neighbour", pSensor->id, pSensor2->id);
                LOG_APPENDSTRING(message);
                ASSERT(FALSE);
                status = STAT_DEPLOY_INTEGRITY_BAD;
            }
            
            if (pSensor2->flags & FLAG_SENSOR_BLACK) {
                pBlackID[pSensor2->id]++;
                if (pBlackID[pSensor2->id] > 1) {
                    message.Format("ERROR: Node %d: Multiple neighbour %d", pSensor->id, pSensor2->id);
                    LOG_APPENDSTRING(message);
                    ASSERT(FALSE);
                    status = STAT_DEPLOY_INTEGRITY_BAD;
                }
            }

            pListNode = pListNode->next;
        }

    }

    delete pWhiteID;
    delete pBlackID;
    
    if (status == STAT_OK) {
        LOG_APPENDSTRING(" OK");
    }
    else {
        LOG_APPENDSTRING(" ERRORS OCCURRED!");
    }

    return status; 
}


int CSensorEnv::DiscoverNeighbours(NEIGH_LIST_NODE* pList, DWORD discoverFlags) {
    int             status = STAT_OK;
    SENSOR_NODE*    pSensor;
    SENSOR_NODE*    pSensor2;
    NEIGH_LIST_NODE*      pNode;
    NEIGH_LIST_NODE*      pNode2;

    if (pList != NULL) {
        // ALL TO ALL DISCOVERY 
        pNode = pList;    
        while (pNode != NULL) {
            pNode2 = pNode->next;    
            while (pNode2 != NULL) {
                pSensor = (SENSOR_NODE*) pNode->pItem;
                pSensor2 = (SENSOR_NODE*) pNode2->pItem;
            
                // DISCOVER RELATION BETWEEN pSensor AND pSensor2
                status = DiscoverNeighbours(pSensor, pSensor2, discoverFlags);

                pNode2 = pNode2->next;
            }        
        
            pNode = pNode->next;
        }
    }

    return status;
}

int CSensorEnv::DiscoverNeighbours(NEIGH_LIST_NODE* first, NEIGH_LIST_NODE* second, DWORD discoverFlags) {
    int             status = STAT_OK;
    SENSOR_NODE*    pSensor;
    SENSOR_NODE*    pSensor2;
    NEIGH_LIST_NODE*      pNode;
    NEIGH_LIST_NODE*      pNode2;

    if (first != NULL && second != NULL) {
        // ALL TO ALL DISCOVERY BETWEEN LISTS
        pNode = first;    
        while (pNode != NULL) {
            pNode2 = second;    
            while (pNode2 != NULL) {
                pSensor = (SENSOR_NODE*) pNode->pItem;
                pSensor2 = (SENSOR_NODE*) pNode2->pItem;

                // TRY TO CATCH NON-INITILISED SENSORS (No MORE THAN 1000000 NODES ASSUMED)
                ASSERT(pSensor->id < 1000000);
                ASSERT(pSensor2->id < 1000000);
                
                
                // DISCOVER RELATION BETWEEN pSensor AND pSensor2
                status = DiscoverNeighbours(pSensor, pSensor2, discoverFlags);

                pNode2 = pNode2->next;
            }        
        
            pNode = pNode->next;
        }
    }
    
    return status;
}

int CSensorEnv::DiscoverNeighbours(BASIC_INIT_DATA* pBasicSettings, DWORD discoverFlags) {
    int     status = STAT_OK;
    DWORD   flags = 0;
    
    // ROUND UP NODES RECEVE RATIO TO OBTAIN MAXIMUM DISTANT CELL(S) IN WHICH NEIGHBOURS CAN BE LOCATED 
    int blackUp = (pBasicSettings->blackRatioReceiveRange != ((DWORD) pBasicSettings->blackRatioReceiveRange)) ? ((DWORD) (pBasicSettings->blackRatioReceiveRange) + 1) : (DWORD) (pBasicSettings->blackRatioReceiveRange);                 
    int whiteUp = (pBasicSettings->whiteRatioReceiveRange != ((DWORD) pBasicSettings->whiteRatioReceiveRange)) ? ((DWORD) (pBasicSettings->blackRatioReceiveRange) + 1) : (DWORD) (pBasicSettings->whiteRatioReceiveRange);                 
    int maxCell = (blackUp > whiteUp) ? blackUp : whiteUp;

    // PROCESS ALL GRID CELLS AND DISCOVER NEIGHBOUR RELATIONS
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.numCellX; i++) {
        for (int j = 0; j < pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.numCellY; j++) {
            if (pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i][j] != NULL) {
                // DISCOVER NEIGHBOURS (BOTH WHITE AND BLACK) IN GIVEN CENTER CELL 
                DiscoverNeighbours(pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i][j], discoverFlags);
            
                // TRY TO DISCOVER NEIGHBOURS IN SOURROUNDING CELLS. HOW FAR SHOULD WE SEARCH DEPENDS ON RECEIVE RATIO
                // NOTE: IF RECEIVE RATIO FOR GIVEN NODE IS n,24432424 THAN NEIGHBOUR CAN BE AT MOST (n+1) CELLS FROM CENTER CELL 
                
                // CHECK NEIGHBOUR RELATION (BOTH WHITE AND BLACK) FOR NOT_INSPECTED_BEFORE SURROUNDING CELLS (NON-ASTERIX CELLS) 
                // EXAMPLE CASE FOR RECEIVE RATIO <= 1
                //  * * *
                //  * x 1
                //  2 3 4
            
                // INSPECT ALL CELLS WHICH ARE EXATLY k CELLS AWAY AND NOT INSPECTED YET 
                
                // SAME ROW AS CENTER CELL
                for (int k = 1; k <= maxCell; k++) {
                    // OPTIMIZATION OF NEIGBOUR SEARCHING (E.G. ONLY BLACK NODES CAN BE NEIGHBOURS)
                    flags = 0;
                    if (discoverFlags & FLAG_SENSOR_WHITE) if (k <= whiteUp) flags |= FLAG_SENSOR_WHITE;
                    if (discoverFlags & FLAG_SENSOR_BLACK) if (k <= blackUp) flags |= FLAG_SENSOR_BLACK;

                    if ((j + k) < pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.numCellY) DiscoverNeighbours(pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i][j], pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i][j + k], flags);
                }

                // RECTANGLE UNDER CENTER CELL x ((2*maxCell + 1) X (maxCell) RECTANGLE)
                for (int l = 1; l <= maxCell; l++) {
                    for (int k = -maxCell; k <= maxCell; k++) {
                        // OPTIMIZATION OF NEIGBOUR SEARCHING (E.G. ONLY BLACK NODES CAN BE NEIGHBOURS)
                        flags = discoverFlags;
                        if (discoverFlags & FLAG_SENSOR_WHITE) if (abs(k) > whiteUp ||  l > whiteUp) flags &= ~FLAG_SENSOR_WHITE;
                        if (discoverFlags & FLAG_SENSOR_BLACK) if (abs(k) > blackUp ||  l > blackUp) flags &= ~FLAG_SENSOR_BLACK;

                        if (((j + k) >= 0) && ((j + k) < pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.numCellY)) { // TEST FOR ROW BOUNDARY
                            if ((i + l) < pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.numCellX) {               // TEST FOR COLUMN BOUNDARY
                                DiscoverNeighbours(pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i][j], pSensorSim->m_sensorEnv.m_pActualDeployment->geoGrid.grid[i + l][j + k], flags);
                            }
                        }
                    }
                }
            }
        }
    }
    
    return status;
}

int CSensorEnv::DiscoverNeighbours(SENSOR_NODE* pSensor, SENSOR_NODE* pSensor2, DWORD discoverFlags) {
    int         status = STAT_OK;
    float       dist;

    if (pSensor != NULL && pSensor2 != NULL) {
        //
        //  WHITE TO WHITE DISCOVERY
        //
        if ((pSensor->flags & FLAG_SENSOR_WHITE) && (pSensor2->flags & FLAG_SENSOR_WHITE)) {
            if (discoverFlags & FLAG_SENSOR_WHITE) {
                // CALCULATE DISTANCE 
                dist = (float) sqrt(pow(pSensor->cX - pSensor2->cX, 2) + pow(pSensor->cY - pSensor2->cY, 2));

                // pSensor2 CAN HEAR pSensor TRANSMISSION
                if (dist <= (pSensor->transmRange * pSensor2->receiveRangeRatio)) pSensor->insertNeigh(pSensor2, dist);

                // pSensor CAN HEAR pSensor2 TRANSMISSION
                if (dist <= (pSensor2->transmRange * pSensor->receiveRangeRatio)) pSensor2->insertNeigh(pSensor, dist);
            }
        }
        else {
        //
        //  WHITE TO BLACK DISCOVERY
        //
        if ((pSensor->flags & FLAG_SENSOR_WHITE) && (pSensor2->flags & FLAG_SENSOR_BLACK)) {
            if (discoverFlags & FLAG_SENSOR_BLACK) {
                // CALCULATE DISTANCE 
                dist = (float) sqrt(pow(pSensor->cX - pSensor2->cX, 2) + pow(pSensor->cY - pSensor2->cY, 2));

                // pSensor2 CAN HEAR pSensor TRANSMISSION
                if (dist <= (pSensor->transmRange * pSensor2->receiveRangeRatio)) pSensor->insertNeigh(pSensor2, dist);

                // pSensor CAN HEAR pSensor2 TRANSMISSION
                // NOTE: THIS DIRRECTION IS NOT USED 
                // if (dist <= (pSensor2->transmRange * pSensor->receiveRangeRatio)) pSensor2->insertNeigh(pSensor, dist);
            }
        }
        else {
        //
        //  BLACK TO WHITE DISCOVERY
        //
        if ((pSensor->flags & FLAG_SENSOR_BLACK) && (pSensor2->flags & FLAG_SENSOR_WHITE)) {
            if (discoverFlags & FLAG_SENSOR_BLACK) {
                // CALCULATE DISTANCE 
                dist = (float) sqrt(pow(pSensor->cX - pSensor2->cX, 2) + pow(pSensor->cY - pSensor2->cY, 2));

                // pSensor2 CAN HEAR pSensor TRANSMISSION
                // NOTE: THIS DIRRECTION IS NOT USED (BLACK->WHITE) 
                //if (dist <= (pSensor->transmRange * pSensor2->receiveRangeRatio)) pSensor->insertNeigh(pSensor2, dist);

                // pSensor CAN HEAR pSensor2 TRANSMISSION
                if (dist <= (pSensor2->transmRange * pSensor->receiveRangeRatio)) pSensor2->insertNeigh(pSensor, dist);
            }
        }
        else {
        //
        //  BLACK TO BLACK DISCOVERY
        //
        if ((pSensor->flags & FLAG_SENSOR_BLACK) && (pSensor2->flags & FLAG_SENSOR_BLACK)) {
            // CALCULATE DISTANCE 
            dist = (float) sqrt(pow(pSensor->cX - pSensor2->cX, 2) + pow(pSensor->cY - pSensor2->cY, 2));

            // pSensor2 CAN HEAR pSensor TRANSMISSION
            if (dist <= (pSensor->transmRange * pSensor2->receiveRangeRatio)) pSensor->insertNeigh(pSensor2, dist);

            // pSensor CAN HEAR pSensor2 TRANSMISSION
            if (dist <= (pSensor2->transmRange * pSensor->receiveRangeRatio)) pSensor2->insertNeigh(pSensor, dist);
        }
        }
        }
        }
    }

    return status;
}


void CSensorEnv::Clear() {
    m_pActualDeployment->release();
    m_pActualDeployment->sensorsDeployed = FALSE;
}

void CSensorEnv::ClearLinkKeysStatus() {
    m_pActualDeployment->whiteSensorList.clearLinkKeysStatus();
    m_pActualDeployment->blackSensorList.clearLinkKeysStatus();
}

void CSensorEnv::ResetNetwork() {
    ClearLinkKeysStatus();
    m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
}

void CSensorEnv::Release() {
	for (int i = 0; i < MAX_DEPLOYMENTS; i++) {
		if (m_deploymentList[i] != NULL) {
			m_deploymentList[i]->release();
		}
	}

    SS_ReleaseStructs();
}


int CSensorEnv::KeyPredistrib_EGScheme(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    BYTE*       keyPool;
    DWORD       keyID;
	DWORD		key2DID; // USED ONLY IF HASH CHAIN IN USE
    DWORD       keysAssigned;
    DWORD       i;
    SENSOR_NODE* pSensorA;
    DWORD       maskIndex;
    KEYS_MASK   maskKey;
    
    //
    //  PERFORM KEY PRE-DISTRIBUTION
    // 
    keyPool = new BYTE[pBasicSettings->egScheme.keyPoolSize];
    memset(keyPool, 1, pBasicSettings->egScheme.keyPoolSize);

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        
        // FILL KEY POOL WITH KEYS (KEY ID == ARRAY OFFSET, KEY PRESENCE SIGNALIZED BY 1
        memset(keyPool, 1, pBasicSettings->egScheme.keyPoolSize);

        if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {

            // #KEYRINGMASK ALLOCATE NODE'S KEY RING
            if (pSensorA->keyRingMask == NULL) {
                pSensorA->keyRingMaskSize = pBasicSettings->egScheme.keyPoolSize / BITS_KEYS_MASK + 1;
                pSensorA->keyRingMask = new KEYS_MASK[pSensorA->keyRingMaskSize]; 
                memset(pSensorA->keyRingMask, 0, sizeof(KEYS_MASK) * pSensorA->keyRingMaskSize);
            }
        }
        else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
            // ALLOCATE NODE'S KEY RING
            if (pSensorA->keyRing == NULL) {
                pSensorA->keyRing = new KEY_ID[pBasicSettings->egScheme.keyRingSize]; 
                pSensorA->keyRingSize = pBasicSettings->egScheme.keyRingSize;
            }
			// HASH CHAIN IN USE
			if (pBasicSettings->egScheme.useHashChain)
			{
				// ALLOCATE NODE'S KEY RING CHAIN POSITIONS
				if (pSensorA->keyRing2D == NULL) {
					pSensorA->keyRing2D = new KEY_2DID[pBasicSettings->egScheme.keyRingSize]; 
					//pSensorA->keyRingSize = pBasicSettings->egScheme.keyRingSize;
				}
			}
        }
        
        // PICK RANDOMLY KEYS FROM KEY POOL
        keysAssigned = 0;
        while (keysAssigned < pBasicSettings->egScheme.keyRingSize) {
            m_rndGen.GetRandomFromInterval(pBasicSettings->egScheme.keyPoolSize, &keyID); 
            
            if (keyPool[keyID] == 1) {
				//JK modify here


                // KEY WITH GIVEN ID STILL IN POOL, ASSIGN TO NODE
                if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
                    maskIndex = keyID / BITS_KEYS_MASK;
                    maskKey = 1 << (keyID - maskIndex * BITS_KEYS_MASK);
                    pSensorA->keyRingMask[maskIndex] |= maskKey;
                }
                else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                    pSensorA->keyRing[keysAssigned] = keyID;

					//IF WE USE INTERMEDIATE KEYS, WE HAVE TO REMOVE ALSO RELATED KEY FROM THE KEY POOL
					if (pBasicSettings->egScheme.useIntermKeys)
					{
						if (keyID&1)
								keyPool[keyID-1]=0;
							else
								keyPool[keyID+1]=0;
					}
					// HASH CHAIN IN USE
					if (pBasicSettings->egScheme.useHashChain)
					{
						// SELECT RANDOMLY POSITION IN HASH CHAIN TO BE ASSIGNED
						m_rndGen.GetRandomFromInterval(pBasicSettings->egScheme.hashChainLen, &key2DID);
						pSensorA->keyRing2D[keysAssigned] = key2DID;

					}

                }
                
                keyPool[keyID] = 0;
                keysAssigned++;
            }      
            else {
                // KEY ALREADY ASSIGNED, REPEAT
            }     
        }  
    }   
    
    delete[] keyPool;
    
    return status;
}

int CSensorEnv::CalcKeyChainUse(BASIC_INIT_DATA* pBasicSettings, float* avg, int* min, int* max) {
	int             status = STAT_OK;
    SENSOR_NODE*    pSensorA;
	
	BYTE** keyPool; 
	
	keyPool = new BYTE*[pBasicSettings->egScheme.keyPoolSize];
    for(int i=0; i<pBasicSettings->egScheme.keyPoolSize;i++)
	{
		keyPool[i] = new BYTE[pBasicSettings->egScheme.hashChainLen];
	}
	for(int j=0;j<pBasicSettings->egScheme.keyPoolSize;j++)
		for(int i=0;i< pBasicSettings->egScheme.hashChainLen;i++)
		{
			keyPool[j][i]=0;
		}  

    for (int i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];

		for(int j=0;j<pBasicSettings->egScheme.keyRingSize;j++)
		{
			int keyChain = pSensorA->keyRing[j];
			int keyChain2D = pSensorA->keyRing2D[j];
			keyPool[keyChain][keyChain2D]=1;
		}
	}

	(*min) = 100000;
	(*max) = 0;
	for(int i=0;i<pBasicSettings->egScheme.keyPoolSize;i++)
	{
		int count = 0;
		for(int j=0;j< pBasicSettings->egScheme.hashChainLen;j++)
		{
			count+=keyPool[i][j];
		}
		(*avg)+=count;
		if ((*min)>count)
			(*min)=count;

		if ((*max) < count)
			(*max)=count;
	}

	(*avg) = (*avg)/(float)(pBasicSettings->egScheme.keyPoolSize); 

	for(int i=0; i<pBasicSettings->egScheme.keyPoolSize;i++)
	{
		delete[] keyPool[i];
	}
	delete[] keyPool;

	return status;

}

int CSensorEnv::KeyPredistrib_ClearKeys(BASIC_INIT_DATA* pBasicSettings) {
    int             status = STAT_OK;
    DWORD           i;
    SENSOR_NODE*    pSensorA;

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];

        if (pSensorA->keyRing != NULL) {
            delete[] pSensorA->keyRing;
            pSensorA->keyRing = NULL;
            pSensorA->keyRingSize = 0;
        }
        if (pSensorA->keyRingMask) {
            delete[] pSensorA->keyRingMask;
            pSensorA->keyRingMask = NULL;
            pSensorA->keyRingMaskSize = 0;
        }
    }

    return status;
}

void CSensorEnv::CalcWhiteNeighStats(DWORD* pMax, DWORD* pMin, float* pAvg, float* pAvgWhiteBeforeBlack, float* pAvgDistNeigh) {
    //
    // CALCULATE STATISTICS FOR WHITE SENSORS
    //
    DWORD           avg_count = 0;
    CString         message;
    DWORD           i;
    SENSOR_NODE*    pNode;
    NEIGH_LIST_NODE*    pNeigh;
    NEIGH_LIST_NODE*    pNearestBlack;
    DWORD           avgWhiteBeforeBlack = 0;

    *pMax = 0;
    *pMin = ULONG_MAX;
    *pAvg = 0;

	//CVRCEK - just for debugging purposes
	int   totalBNo=0;
    float totalBDist=0;
    #define MAX_NUM_NEIGH   1000
    float distWhite[MAX_NUM_NEIGH];
    DWORD distWhiteCount[MAX_NUM_NEIGH];
    memset(distWhite, 0, MAX_NUM_NEIGH * sizeof(float));
    memset(distWhiteCount, 0, MAX_NUM_NEIGH * sizeof(DWORD));

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        if (m_pActualDeployment->whiteSensorList.sensors[i] != NULL) {
            pNode = m_pActualDeployment->whiteSensorList.sensors[i];
            // NUMBER OF WHITE NEIGHBOURS
            if (pNode->numWhiteNeigh > *pMax) *pMax = pNode->numWhiteNeigh; 
            if (pNode->numWhiteNeigh < *pMin) *pMin = pNode->numWhiteNeigh; 
            *pAvg += pNode->numWhiteNeigh; 
            avg_count++;

            // NUMBER OF WHITE NEIGHBOURS BEFORE FIRST BLACK NODE
            if ((pNearestBlack = pNode->blackNeigh) == NULL) {
                // AL WHITE ARE BEFORE BLACK
                avgWhiteBeforeBlack += pNode->numWhiteNeigh;
            } 
            else {
				totalBDist+=pNearestBlack->dist;
				totalBNo++;
                // FIND ALL WHITE BEFORE BLACK
                pNeigh = pNode->whiteNeigh;
                while ((pNeigh != NULL) && (pNeigh->dist < pNearestBlack->dist)) {
                    avgWhiteBeforeBlack++;
                    pNeigh = pNeigh->next;
                }
            }

            // DEBUG: average distance to i-th white neighbour
            pNeigh = pNode->whiteNeigh;
            int neighCounter = 1;
            while (pNeigh != NULL) {
                distWhite[neighCounter] += pNeigh->dist;
                distWhiteCount[neighCounter]++; 
                pNeigh = pNeigh->next;
                neighCounter++;
            }

        }
    }
    *pAvg /= (float) avg_count;

    if (pAvgDistNeigh) {
        for (i = 1; i < MAX_NUM_NEIGH; i++) {
            pAvgDistNeigh[i] = distWhite[i] / distWhiteCount[i];
        }
    }

    if (pAvgWhiteBeforeBlack) *pAvgWhiteBeforeBlack = avgWhiteBeforeBlack / (float) avg_count;
}

void CSensorEnv::CalcBlackNeighStats(DWORD* pMax, DWORD* pMin, float* pAvg) {
    //
    // CALCULATE STATISTICS FOR BLACK SENSORS
    //
    DWORD           min = ULONG_MAX;
    DWORD           max = 0;
    DWORD           avg_count = 0;
    float           avg = 0;
    DWORD           i;
    SENSOR_NODE*    pNode;

    *pMax = 0;
    *pMin = ULONG_MAX;
    *pAvg = 0;

    for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
        if (m_pActualDeployment->blackSensorList.sensors[i] != NULL) {
            pNode = m_pActualDeployment->blackSensorList.sensors[i];
            if (pNode->numBlackNeigh > *pMax) *pMax = pNode->numBlackNeigh; 
            if (pNode->numBlackNeigh < *pMin) *pMin = pNode->numBlackNeigh; 
            *pAvg += pNode->numBlackNeigh; 
            avg_count++;
        }
    }
    *pAvg /= (float) avg_count;
}

void CSensorEnv::CalcBlackAreaStats(BASIC_INIT_DATA* pBasicSettings, float* pBlackArea) {
    //
    // CALCULATE AREA COVERED BY BLACK NODES
    //
    float           x;
    float           y;
    DWORD           i;
    SENSOR_NODE*    pNode;
    float           step = (float) 0.1;
    float           dist;
    float           blackDist;
    DWORD           coverCount = 0;
    
    for (x = 0; x < pBasicSettings->planeX; x += step) {
        for (y = 0; y < pBasicSettings->planeY; y += step) {
            for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
                if (m_pActualDeployment->blackSensorList.sensors[i] != NULL) {
                    pNode = m_pActualDeployment->blackSensorList.sensors[i];

                    // CHECK IF THIS BLACK NODE COVER THIS POINT
                    blackDist = pBasicSettings->whiteTransmRange * pBasicSettings->blackRatioReceiveRange;
                    if ((abs(pNode->cX - x) < blackDist) || (abs(pNode->cY - y) < blackDist)) {
                        dist = (float) sqrt(pow(pNode->cX - x, 2) + pow(pNode->cY - y, 2));
                        if (dist < blackDist) {
                            coverCount++;
                            break;
                        }
                    }
                }
            }
        }
    }

    *pBlackArea = coverCount * step * step; 

}


int CSensorEnv::PlaintextMaxScream(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc ) {
    //
    //  KEY IS SCREAMED IN PLAINTEXT FORM AT MAXIMUM TRANSMISSION RANGE
    //
    int             status = STAT_OK;
    DWORD           i;
    DWORD           usedNeigh = 0;
    DWORD           maxUsedNeigh;
    SENSOR_NODE*    pSensor;
    SENSOR_NODE*    pSensor2;
    SENSOR_NODE*    pComprSensor;
    NEIGH_LIST_NODE*      pListNode;   
    NEIGH_LIST_NODE*      pListNode2;   

    
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY) {
        // SKIP TYPE SPECIFIC DISTRIBUTION AND JUST DISTRIBUTE STATUS OF LINK KEYS AS GIVEN BY pBasicSettings->distribKeysSecured and distribKeysCompr
        DistributeKeyStatusRandomly(pBasicSettings, pSecured, pCompr, 0, FALSE);
    }
    else {

        // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
        // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
        maxUsedNeigh = ULONG_MAX;
        if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
            maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
        }
    

        // MAX. SCREAM, BUT ESTABLISH KEY ONLY FOR pBasicSettings->useOnlyXNeigh WHITE NEIGHBOURS
        // IF BLACK SENSOR IN NEIGHBOUR LIST, THAN ALL KEYS TO ALL NEIGHBOURS ARE COMPROMISED
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensor->whiteNeigh;

            usedNeigh = 0;
            if (m_pActualDeployment->whiteSensorList.sensors[i]->numBlackNeigh == 0) {
                // NO BLACK SENSOR IN RANGE => ALL KEYS SECURE
                while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
                    pSensor2 = (SENSOR_NODE*) pListNode->pItem;

                    if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                        (*pSecured)++;
                        usedNeigh++;

                        pListNode->keyFlags |= FLAG_KEY_SECURE;
                        pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                    }

                    pListNode = pListNode->next;
                }
            }
            else {
                // BLACK SENSOR IN RANGE => ALL KEYS COMPROMISED
                while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
                    pSensor2 = (SENSOR_NODE*) pListNode->pItem;

                    if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                        (*pCompr)++;
                        usedNeigh++;

                        pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
                        pListNode->keyFlags &= ~FLAG_KEY_SECURE; 

                        // MAINTAING OF BLACK NODES WHICH HAS COMPROMISSED LINK KEY IS REQUIRED
                        if (pBasicSettings->maintainCompromissedBy) {
                            // INSERT ALL BLACK NEIGHBOUR INTO COMPROMISSED_BY LIST
                            pListNode2 = pSensor->blackNeigh;
                            while (pListNode2 != NULL) {
                                pComprSensor = (SENSOR_NODE*) pListNode2->pItem;
                                if ((pComprSensor->flags & FLAG_SENSOR_BLACK) && !(pComprSensor->flags & FLAG_SENSOR_BLACK_INACTIVENEIGH)) {
                                    // ADD THIS SENSOR AS COMPROMISSING ONE OF THIS LINK KEY 
                                    pListNode->insertCompromising(pComprSensor);
                                }
                                pListNode2 = pListNode2->next;
                            }  
                        }
                    }

                    pListNode = pListNode->next;
                }
            }
        }

        // IF REQUIRED, DISTRIBUTE RANDOMLY COMPROMISSED KEYS IN SAME AMOUNT AS FOR ORIGINAL DISTRIBUTION METHOD 
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_MARKEQUALRANDOMLY) {
            status = DistributeEqualKeyStatusRandomly(pBasicSettings, pSecured, pCompr);
        }
    }
       
	//
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = PlaintextMaxScream(pBasicSettings, pSecured, pCompr, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
 	    }
    }

    return status;
}

int CSensorEnv::BasicWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc) {
    //
    //  KEY IS SCREAMED IN PLAINTEXT FORM, BUT TRANSMISSION RANGE STARTS AT 0 AND IS CONTINUOSLY INCREASED UNTIL TARGET NEIGHBOUR IS REACHED  
    //
    int         status = STAT_OK;
    DWORD       i;
    DWORD       usedNeigh = 0;
    DWORD       maxUsedNeigh;
    DWORD       egSecure;
    DWORD       egMissing;
    SENSOR_NODE* pSensor;
    SENSOR_NODE* pSensor2;
    SENSOR_NODE* pComprSensor;
    NEIGH_LIST_NODE*  pListNode;   
    NEIGH_LIST_NODE*  pListNode2;   
    NEIGH_LIST_NODE*  pComprListNode;   
    BOOL        blackCanListen;


    if (pBasicSettings->distribFlags & FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY) {
        // SKIP TYPE SPECIFIC DISTRIBUTION AND JUST DISTRIBUTE STATUS OF LINK KEYS AS GIVEN BY pBasicSettings->distribKeysSecured and distribKeysCompr
        DistributeKeyStatusRandomly(pBasicSettings, pSecured, pCompr, 0, FALSE);
    }
    else {
        // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
        // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
        maxUsedNeigh = ULONG_MAX;
        if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
            maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
        }

        // NEIGHBOUR ARE SORTED BY DISTANCE, SO ALL KEYS TO NEIGHBOURS BEFORE FIRST BLACK NODE WHICH CAN LISTEN ARE SECURE
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensor->whiteNeigh;

            blackCanListen = FALSE;
            usedNeigh = 0;
            pComprSensor = NULL;
            pComprListNode = NULL;

            // OBTAIN NEAREST ACTIVE BLACK NODE, WHICH IS NEIGHBOUR OF pSensor
            pListNode2 = pSensor->blackNeigh;
            while (pListNode2 != NULL) {
                pSensor2 = (SENSOR_NODE*) pListNode2->pItem;
                if ((pSensor2->flags & FLAG_SENSOR_BLACK) && !(pSensor2->flags & FLAG_SENSOR_BLACK_INACTIVENEIGH)) {
                    // FIRST BLACK FOUND
                    pComprSensor = pSensor2;
                    pComprListNode = pListNode2;
                    break;    
                }
                pListNode2 = pListNode2->next;
            }  
            

            while (pListNode != NULL && (usedNeigh < maxUsedNeigh)) {
                pSensor2 = (SENSOR_NODE*) pListNode->pItem;
                if (!blackCanListen) {
                    // pSensor2 IS STILL CLOSER THAN NEAREST ACTIVE BLACK NODE CAN HEAR

                    if (pComprSensor != NULL && pComprListNode != NULL) {
                        // COMPARE IF SIGNAL TRANSMITTED BY pSensor TO pSensor2 CAN BE HEART BY SOME BLACK NODE
                        if ((pListNode->dist * pComprSensor->receiveRangeRatio) >= pComprListNode->dist) { 
                            
                            blackCanListen = TRUE;
                        }
                    }
                    
                    // CHECK IF ALREADY EXIST BLACK NODE, WHICH CAN LISTEN
                    // IF YES, MARK KEY COMPROMISSED, OTHERWISE SECURE
                    if (blackCanListen) {
                        if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                            (*pCompr)++;
                            pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
                            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                        }
                    }
                    else {
                        if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                            // ANOTHER ONE SECURE KEY
                            (*pSecured)++;
                            pListNode->keyFlags |= FLAG_KEY_SECURE;
                            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                        }
                    }
                }
                else {
                    // BLACK NODE CAN LISTEN, ALL FOLLOWING WHITE NODES IN LIST ARE COMPROMISED
                    if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                        (*pCompr)++;
                        pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
                        pListNode->keyFlags &= ~FLAG_KEY_SECURE; 

                        // MAINTAING OF BLACK NODES WHICH HAS COMPROMISED LINK KEY IS REQUIRED
                        if (pBasicSettings->maintainCompromissedBy) {
                            // INSERT ALL BLACK NEIGHBOUR WHICH ARE CLOSER TO pSensor THAN pSensor2 INTO COMPROMISED_BY LIST
                            pListNode2 = pSensor->blackNeigh;
                            while (pListNode2 != NULL) {
                                pComprSensor = (SENSOR_NODE*) pListNode2->pItem;
                                if (pListNode2->dist > (pListNode->dist * pComprSensor->receiveRangeRatio)) {
                                    // STOP LOOKING FOR COMPROMISING NODE WHEN FIRST BLACK NODE WHICH CAN'T HEAR TRANSMISSION IS FOUND
                                    // BECAUSE BLACK NODES ARE SORTED INCREASINGLY BY DISTANCE
                                    break;
                                }

                                if ((pComprSensor->flags & FLAG_SENSOR_BLACK) && !(pComprSensor->flags & FLAG_SENSOR_BLACK_INACTIVENEIGH)) {
                                    // ADD THIS SENSOR AS COMPROMISING ONE OF THIS LINK KEY 
                                    pListNode->insertCompromising(pComprSensor);
                                }
                                pListNode2 = pListNode2->next;
                            }  
                        }

                    }
                }

                if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                    // INCREASE COUNTER OF CONTACTED WHITE NEIGHBOURS
                    usedNeigh++;
                }
        
                pListNode = pListNode->next;
            }

        }
        
        //
        // IF REQUIRED, DISTRIBUTE RANDOMLY COMPROMISED KEYS IN SAME AMOUNT AS FOR ORIGINAL DISTRIBUTION METHOD 
        //
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_MARKEQUALRANDOMLY) {
            status = DistributeEqualKeyStatusRandomly(pBasicSettings, pSecured, pCompr);
        }

        //
        // IF REQUIRED, EG KEY PRE-DISTRIBUTION METHOD IS USED 
        //
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_USEEGSCHEME) {
            status = EGScheme(pBasicSettings, &egSecure, &egMissing);
        }

        //
        //  COUNT SECURE AND COMPROMISED KEYS 
        //
        *pSecured = *pCompr = 0;
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensor->whiteNeigh;

            while (pListNode != NULL) {
                pSensor2 = (SENSOR_NODE*) pListNode->pItem;

                if (pSensor2->flags & FLAG_SENSOR_WHITE) {
                    if (pListNode->keyFlags & FLAG_KEY_SECURE) {
                        // KEY IS SECURE 
                        (*pSecured)++;
                    }
                    if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                        // KEY IS COMPROMISED 
                        (*pCompr)++;
                    }
                }

                pListNode = pListNode->next;
            }
        }
    }

	//
	//	IF REQUIRED, MOST COMPROMISING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            
            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = BasicWhisper(pBasicSettings, pSecured, pCompr, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}

int CSensorEnv::MutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc) {
    //
    //  SAME AS BASIC WHISPERING, BUT KEY FOR A<->B IS CONSTRUCTED AS [A<->B = A->B xor A<-B]
    //

    int         status = STAT_OK;
    DWORD       i;
    DWORD       usedNeigh = 0;
    SENSOR_NODE* pSensorA;
    SENSOR_NODE* pSensorB;
    SENSOR_NODE* pSensorC;
    NEIGH_LIST_NODE*  pListNode;   
    NEIGH_LIST_NODE*  pListNode2;   
    
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY) {
        // SKIP TYPE SPECIFIC DISTRIBUTION AND JUST DISTRIBUTE STATUS OF LINK KEYS AS GIVEN BY pBasicSettings->distribKeysSecured and distribKeysCompr
        DistributeKeyStatusRandomly(pBasicSettings, pSecured, pCompr, 0, FALSE);
    }
    else  {
        // IF REQUIRED, DISTRIBUTE RANDOMLY COMPROMISSED KEYS IN SAME AMOUNTH AS FOR ORIGINAL DISTRIBUTION METHOD 
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_MARKEQUALRANDOMLY) {
            BASIC_INIT_DATA     tempBasicSetting = *pBasicSettings;
            // REMOVE RANDOM DISTRIB. FLAG
            tempBasicSetting.distribFlags &= ~FLAG_DISTRIB_MARKEQUALRANDOMLY;
            // PROCESS MUTUAL WHISPER AS NORMAL FIRST
            *pSecured = *pCompr = 0;
            status = MutualWhisper(&tempBasicSetting, pSecured, pCompr);

            // DISTRIBUTE RANDOMLY 
            DistributeEqualKeyStatusRandomly(pBasicSettings, pSecured, pCompr, FLAG_RANDOMDISTRIB_BIDIRECTIONAL);
        }
        else {
            // DO BASIC WHISPERING WITH KEY STATUS MARKING
            ClearLinkKeysStatus();
            if ((status = BasicWhisper(pBasicSettings, pSecured, pCompr)) == STAT_OK) {
                // MARK COMPROMISED KEY A->B AS SECURE IF AND ONLY IF B->A IS SECURE
                for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
                    pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
                    pListNode = pSensorA->whiteNeigh;

                    while (pListNode != NULL) {
                        pSensorB = (SENSOR_NODE*) pListNode->pItem;

                        if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                            if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                                // CHECK IF REVERSE LINK KEY IS SECURE
                        
                                pListNode2 = pSensorB->whiteNeigh;
                                while (pListNode2 != NULL) {
                                    pSensorC = (SENSOR_NODE*) pListNode2->pItem;

                                    if ((pSensorC->flags & FLAG_SENSOR_WHITE) && (pSensorC->id == pSensorA->id)) {
                                        // SENSOR A FOUND
                                
                                        if (pListNode2->keyFlags & FLAG_KEY_SECURE) {
                                            // A->B CAN BE SECURED
                                            pListNode->keyFlags |= FLAG_KEY_SECURE;
                                            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED;

                                            (*pSecured)++;
                                            (*pCompr)--; 

                                            // CLEAR LIST OF COMPROMISSING BLACK NODES (IF USED)
                                            pListNode->releaseComprBy();
                                        }

                                        break;
                                    }

                                    pListNode2 = pListNode2->next;
                                } 
                            }
                        }

                        pListNode = pListNode->next;
                    }
                }
            }

            //
            //  COUNT SECURE AND COMPROMISED KEYS 
            //
            *pSecured = *pCompr = 0;
            for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
                pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
                pListNode = pSensorA->whiteNeigh;

                while (pListNode != NULL) {
                    pSensorB = (SENSOR_NODE*) pListNode->pItem;

                    if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                        if (pListNode->keyFlags & FLAG_KEY_SECURE) {
                            // KEY IS SECURE 
                            (*pSecured)++;
                        }
                        if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                            // KEY IS COMPROMISED 
                            (*pCompr)++;
                        }
                    }

                    pListNode = pListNode->next;
                }
            }
        }
    }
      
    //
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = MutualWhisper(pBasicSettings, pSecured, pCompr, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}

int CSensorEnv::RossAmplifMaxScream(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BOOL bSkipRecalc) {
    int     status = STAT_OK;


    return status;
}

int CSensorEnv::RossAmplifWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc) {
    //
    //  PERFORM BASIC WHISPERING AND THEN 
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();

    // MAKE INITIAL WHISPERING AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = BasicWhisper(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = RossAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

	//
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = RossAmplifWhisper(pBasicSettings, pSecured, pCompr, repeat, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}

int CSensorEnv::RossAmplifMutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc) {
    //
    //  PERFORM BASIC WHISPERING AND THEN 
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();
    // MAKE INITIAL WHISPERING AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = MutualWhisper(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = RossAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

	//
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = RossAmplifMutualWhisper(pBasicSettings, pSecured, pCompr, repeat, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}

int CSensorEnv::DanAmplifWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc) {
    //
    //  PERFORM BASIC WHISPERING AND THEN DAN'S AMPLIFICATION
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();
    // MAKE INITIAL WHISPERING AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = BasicWhisper(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        if (repeat == 0) repeat = 1;
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = DanAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

	//
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = DanAmplifWhisper(pBasicSettings, pSecured, pCompr, repeat, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}


int CSensorEnv::PushAmplifEGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat) {
    //
    //  PERFORM BASIC WHISPERING AND THEN DAN'S AMPLIFICATION
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();
    // MAKE INITIAL KEY DISCOVERY AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = EGScheme(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        if (repeat == 0) repeat = 1;
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = RossAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

    return status;
}

int CSensorEnv::PullAmplifEGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat) {
    //
    //  PERFORM BASIC WHISPERING AND THEN DAN'S AMPLIFICATION
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();
    // MAKE INITIAL KEY DISCOVERY AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = EGScheme(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        if (repeat == 0) repeat = 1;
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = DanAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

    return status;
}

int CSensorEnv::DanAmplifMutualWhisper(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD repeat, BOOL bSkipRecalc) {
    //
    //  PERFORM MUTUAL WHISPERING AND THEN DAN'S AMPLIFICATION
    //
    int         status = STAT_OK;
    DWORD       i;
    
    // CLEAR PREVIOUS KEYS FLAGS
    ClearLinkKeysStatus();
    // MAKE INITIAL WHISPERING AND STORE KEY STATUS (SECURE/COMPROMISED)
    if ((status = MutualWhisper(pBasicSettings, pSecured, pCompr, TRUE)) == STAT_OK) {
        // PERFORM AMPLIFICATION
        if (repeat == 0) repeat = 1;
        for (i = 0; i < repeat; i++) {
            *pSecured = 0;
            *pCompr = 0;
            status = DanAmplif(pBasicSettings, pSecured, pCompr);
        }
    }

	//
	//	IF REQUIRED, MOST COMPROMMISSING BLACK NODES ARE REMOVED AND COMPUTATION IS PERFORMED AGAIN 
	//
	if (!bSkipRecalc) {
        if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) {
            // SET AS INACTIVE NEIGHBOURS NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            SetMostComprAsInactiveNeigh(pBasicSettings);
            

            // PERFORM DISTRIBUTION AGAIN 
            *pSecured = *pCompr = 0;
            ClearLinkKeysStatus();
            status = DanAmplifMutualWhisper(pBasicSettings, pSecured, pCompr, repeat, TRUE);

            // CLEAR INACTIVE NEIGBOUR STATUS FROM BLACK NODES
            m_pActualDeployment->blackSensorList.clearInactiveNeighFlags();
	    }
    }

    return status;
}

int CSensorEnv::RossAmplif(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr) {
    int         status = STAT_OK;
    DWORD       i;
    DWORD       usedNeigh;
    DWORD       maxUsedNeigh;
    SENSOR_NODE* pSensorA;
    SENSOR_NODE* pSensorB;
    SENSOR_NODE* pSensorC;
    SENSOR_NODE* pSensorHelp;
    NEIGH_LIST_NODE*  pListNode;   
    NEIGH_LIST_NODE*  pListNode2;   
    NEIGH_LIST_NODE*  pListNode3;   
    BOOL        bNewSecureKey = FALSE;


    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    maxUsedNeigh = ULONG_MAX;
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_AMPLIFICATION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        while (pListNode != NULL) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                if (pListNode->keyFlags & FLAG_KEY_SECURE) {
                    // KEY IS ALREADY SECURE 
                    (*pSecured)++;
                }
                
                if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                    bNewSecureKey = FALSE;

                    // KEY IS NOT SECURE FROM PLAINTEXT EXCHANGE, TRY NEIGHBOURS NODES TO ASSIST WITH ESTABLISHING NEW KEY
                    // NEW KEY FOR A->B WILL BE SECURE ONLY IF: 
                    // EXIST A-NEIGHBOUR C, A->C IS SECURE && B IS C-NEIGHBOUR, C->B IS SECURE 
                    pListNode2 = pSensorA->whiteNeigh;

                    // FIND ALL A-NEIGHBOURS WITH SECURE KEY
                    usedNeigh = 0;
                    while ((pListNode2 != NULL) && (usedNeigh < maxUsedNeigh)) {
                        pSensorC = (SENSOR_NODE*) pListNode2->pItem;
                        if ((pSensorC->flags & FLAG_SENSOR_WHITE) && (pListNode2->keyFlags & FLAG_KEY_SECURE)) {
                            // THIS CAN BE POTENTIONALLY NODE 'C'
                            // VERIFY, IF pSensorC IS NEIGBOUR OF pSensorB AND HAVE SECURE KEY 

                            // NOTE: VERIFYING IF pSensorC IS B-NEIGHBOUR IS NOT LIMITED BY maxUsedNeigh
                            pListNode3 = pSensorC->whiteNeigh;
                            while (pListNode3 != NULL) {
                                pSensorHelp = (SENSOR_NODE*) pListNode3->pItem;
                                if ((pSensorHelp->id == pSensorB->id) && (pSensorHelp->flags & FLAG_SENSOR_WHITE)) {
                                    // pSensorC IS B-NEIGHBOUR

                                    if (pListNode3->keyFlags & FLAG_KEY_SECURE) {
                                        // NEW SECURE KEY ESTABLISHED  
                                        bNewSecureKey = TRUE;
                                    }
                                    else {
                                        // pSensorC CANNOT BE USED TO ESTABLISH NEW KEY 
                                    }

                                    break;
                                }
                        
                                pListNode3 = pListNode3->next;
                            }
                        }

                        // COUNT THIS SENSOR AS USED NEIGHBOUR
                        if (pSensorC->flags & FLAG_SENSOR_WHITE) usedNeigh++;
                
                        pListNode2 = pListNode2->next;
                    }

                    if (bNewSecureKey) {
                        // USABLE NODE C FOUND => KEY A->B CAN BE SECURED
                        (*pSecured)++;
                        // SET KEY AS NEWLY SECURE (THIS STATUS IS NOT USED DURING THIS AMPLIFICATION
                        pListNode->keyFlags |= FLAG_KEY_NEWLY_SECURE;
                    }
                    else {
                        // NO USABLE NODE C FOUND => KEY A->B REMAIN COMPROMISED
                        (*pCompr)++;
                    }
                }
            }
            
            pListNode = pListNode->next;
        }
    }

    // 
    //  IF REQUIRED, THEN PERFORM FOE PROTOCOL OVER ACTUAL STATE OF LINKS
    //                    
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_USEFORWARDONIONENCRYPTION) {
        status = PEGA_ExecuteOverPartiallySecureLinks(pBasicSettings);  
    }

    // SET ALL NEWLY SECURE KEYS AS SECURE
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        while (pListNode != NULL) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                if (pListNode->keyFlags & FLAG_KEY_NEWLY_SECURE) {
                    // MARK AS ORDINARY SECURE
                    pListNode->keyFlags |= FLAG_KEY_SECURE;
                    pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                    pListNode->keyFlags &= ~FLAG_KEY_NEWLY_SECURE; 


                    // CLEAR LIST OF COMPROMISSING BLACK NODES (IF USED)
                    pListNode->releaseComprBy();
                }
            }

            pListNode = pListNode->next;
        }
    }

    return status;
}

int CSensorEnv::DanAmplif(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr) {
    int         status = STAT_OK;
    DWORD       i;
    DWORD       usedNeigh;
    DWORD       usedNeigh2;
    DWORD       maxUsedNeigh;
    SENSOR_NODE* pSensorA;
    SENSOR_NODE* pSensorB;
    SENSOR_NODE* pSensorC;
    NEIGH_LIST_NODE*  pListNode;   
    NEIGH_LIST_NODE*  pListNode2;   
    BOOL        bNewSecureKey = FALSE;

    *pSecured = *pCompr = 0;

    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    maxUsedNeigh = ULONG_MAX;
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_AMPLIFICATION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }

    // EACH NODE A TRY TO HELP ESTABLISH NEW KEY FOR ALL PAIRS OF ITS NEIGBOURS (B, C)
    // NEW KEY IS ESTABLISHED ONLY IF B AND C ARE ALSO NEIGHBOURS
    // NEW KEY IS NOT COMPROMITED, ONLY IF:
    // KEY B<->C WAS ALREADY SECURE BEFORE OR
    // A->B IS SECURE AND A->C IS SECURE 
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;
        
        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if ((pSensorB->flags & FLAG_SENSOR_WHITE) && (pListNode->keyFlags & FLAG_KEY_SECURE) && !(pListNode->keyFlags & FLAG_KEY_MISSING)) {
                pListNode2 = pListNode->next;
                
                usedNeigh2 = usedNeigh;     // usedNeigh2 SHOULD NOT EXCEED maxUsedNeigh SAME AS usedNeigh
                while ((pListNode2 != NULL) && (usedNeigh2 < maxUsedNeigh)) {
                    pSensorC = (SENSOR_NODE*) pListNode2->pItem;

                    if ((pSensorC->flags & FLAG_SENSOR_WHITE) && (pListNode2->keyFlags & FLAG_KEY_SECURE) && !(pListNode->keyFlags & FLAG_KEY_MISSING)) {
                        // IF B & C ARE NEIGHBOURS, b<->C CAN BE SECURED EVEN IF NOT BEFORE
                        // CHECK IF B & C ARE NEIGHBOURS

                        if (pSensorC->isWhiteNeigh(pSensorB, maxUsedNeigh)) pSensorC->markKeySecure(pSensorB, TRUE);
                        if (pSensorB->isWhiteNeigh(pSensorC, maxUsedNeigh)) pSensorB->markKeySecure(pSensorC, TRUE);
                    }
                    
                    // COUNT THIS SENSOR AS USED NEIGHBOUR
                    if (pSensorC->flags & FLAG_SENSOR_WHITE) usedNeigh2++;

                    pListNode2 = pListNode2->next;
                }
            }

            // COUNT THIS SENSOR AS USED NEIGHBOUR
            if (pSensorB->flags & FLAG_SENSOR_WHITE) usedNeigh++;
            
            pListNode = pListNode->next;
        }
    }

    // 
    //  IF REQUIRED, THEN PERFORM FOE PROTOCOL OVER ACTUAL STATE OF LINKS
    //                    
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_USEFORWARDONIONENCRYPTION) {
        status = PEGA_ExecuteOverPartiallySecureLinks(pBasicSettings);  
    }

    //
    //  COUNT SECURE AND COMPROMISED KEYS 
    //
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        while (pListNode != NULL) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                if ((pListNode->keyFlags & FLAG_KEY_SECURE) || (pListNode->keyFlags & FLAG_KEY_NEWLY_SECURE)) {
                    // KEY IS SECURE 
                    (*pSecured)++;

                    // MAR AS ORDINARY SECURE
                    pListNode->keyFlags |= FLAG_KEY_SECURE;
                    pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                    pListNode->keyFlags &= ~FLAG_KEY_NEWLY_SECURE; 

                    // CLEAR LIST OF COMPROMISSING BLACK NODES (IF USED)
                    pListNode->releaseComprBy();
                }
                if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                    // KEY IS COMPROMISED 
                    (*pCompr)++;
                }
            }

            pListNode = pListNode->next;
        }
    }


    return status;
}

int CSensorEnv::EGScheme(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pMissing, BOOL bMarkMissingAsCompr) {
    //
    //  RANDOM KEY PREDISTRIBUTION SCHEME BY ESCHENAUER AND GLIGOR
    //  EACH NODE IS ASSIGNED WITH m KEYS FROM LARGE KEY POOL WITH SIZE S WITHOUT REPLACEMENT
    //
    int         status = STAT_OK;
    BYTE*       keyPool;
	KEY_2DID*	keyPool2D;
    SENSOR_NODE* pSensorA;
    SENSOR_NODE* pSensorB;
    DWORD       i;
    DWORD       j;  
    DWORD       maxUsedNeigh;
    DWORD       usedNeigh;
    NEIGH_LIST_NODE*  pListNode; 
    BYTE        sharedCount;
	BYTE		sharedIntermCount;
    KEYS_MASK   sharedKeysMask;
      


    //
    //  SET LINK KEY STATUS FOR NEIGBOURS  
    //

    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    maxUsedNeigh = ULONG_MAX;
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }

    keyPool = new BYTE[pBasicSettings->egScheme.keyPoolSize];
    memset(keyPool, 1, pBasicSettings->egScheme.keyPoolSize);

	keyPool2D = new KEY_2DID[pBasicSettings->egScheme.keyPoolSize];
    memset(keyPool2D, 1, sizeof(KEY_2DID)*pBasicSettings->egScheme.keyPoolSize);

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

			sharedIntermCount = 0;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {

                if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
                    // #KEYRINGMASK
                    // AND BIT MASKS, KEY IS SHARED IF RESULT IS NONZERO
                    sharedCount = 0;
                    for (j = 0; j < pSensorB->keyRingMaskSize; j++) {
                        // BUGBUG: ONLY ONE KEY PER CELL IS COMPUTED TO sharedCount
                        if ((sharedKeysMask = pSensorA->keyRingMask[j] & pSensorB->keyRingMask[j]) != 0) { 
                            sharedCount++;
                        } 
                    }
                }
                else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                    // TRY TO FIND INTERSECTION KEY(S)
                    memset(keyPool, 0, pBasicSettings->egScheme.keyPoolSize);
					memset(keyPool2D, 0, sizeof(KEY_2DID)*pBasicSettings->egScheme.keyPoolSize);
                    
					if (pBasicSettings->egScheme.useIntermKeys)
					{
						 // RADIX SORT METHOD
						for (j = 0; j < pSensorB->keyRingSize; j++) {
							(keyPool[pSensorA->keyRing[j]])|=0x01;  //KEY PRESENT IN KEY RING A IF LSB BIT SET TO 1
							(keyPool[pSensorB->keyRing[j]])|=0x02;  //KEY PRESENT IN KEY RING B IF SECOND LSB BIT SET TO 1
							
							// HASH CHAIN IN USE
							if (pBasicSettings->egScheme.useHashChain)
							{
								//SET THE HIGHEST KEY FROM HASH CHAIN AS SHARED 
								if (pSensorA->keyRing2D[j] > keyPool2D[pSensorA->keyRing[j]])
									 keyPool2D[pSensorA->keyRing[j]]=pSensorA->keyRing2D[j];

								if (pSensorB->keyRing2D[j] > keyPool2D[pSensorB->keyRing[j]])
									 keyPool2D[pSensorB->keyRing[j]]=pSensorB->keyRing2D[j];
							}

						}
						// LOOK FOR SHARED KEYS	
						sharedCount = 0;
						sharedIntermCount = 0;
						for (j = 0; j < pBasicSettings->egScheme.keyPoolSize; j++) {
							if (keyPool[j] == 3) {
								// ADD THIS SHARED KEY INTO SHARED KEY LIST
								if (sharedCount < MAX_SHARED_KEYS) {
									pListNode->EG_sharedKeys[sharedCount].id = j;
									//ADD POSITION IN HASH CHAIN
									if (pBasicSettings->egScheme.useHashChain)
									{
										pListNode->EG_sharedKeys[sharedCount].id2D = keyPool2D[j];
									}
									sharedCount++;   
								}
								else {
									// MAX_SHARED_KEYS NEDDS TO BE INCREASED
									status = STAT_NOT_ENOUGHT_DATA_TYPE;
									break;
								}
							}
							if ((j&1)==0)							
								if ((keyPool[j] != 3) 
									&& (keyPool[j] != 0) 
									&& (((keyPool[j])^(keyPool[j+1]))==3))
								{
									// ADD THIS INTERMEDIATE KEY INTO SHARED INTERMEDIATE KEY LIST
									if (sharedIntermCount < MAX_SHARED_KEYS) {
										pListNode->EG_sharedIntermKeys[sharedIntermCount] = j;
										sharedIntermCount++;   
									}
									//JK modif
									else {
										// MAX_SHARED_KEYS NEDDS TO BE INCREASED
										status = STAT_NOT_ENOUGHT_DATA_TYPE;
										break;
									}
								}		     
						}
					}
					else
					{
						 // RADIX SORT METHOD
						for (j = 0; j < pSensorB->keyRingSize; j++) {
							(keyPool[pSensorA->keyRing[j]])++;  
							(keyPool[pSensorB->keyRing[j]])++;  

							// HASH CHAIN IN USE
							if (pBasicSettings->egScheme.useHashChain)
							{
								//SET THE HIGHEST KEY FROM HASH CHAIN AS SHARED 
								if (pSensorA->keyRing2D[j] > keyPool2D[pSensorA->keyRing[j]])
									 keyPool2D[pSensorA->keyRing[j]]=pSensorA->keyRing2D[j];

								if (pSensorB->keyRing2D[j] > keyPool2D[pSensorB->keyRing[j]])
									 keyPool2D[pSensorB->keyRing[j]]=pSensorB->keyRing2D[j];
							}

						}
						// LOOK FOR SHARED KEYS
						sharedCount = 0;
						for (j = 0; j < pBasicSettings->egScheme.keyPoolSize; j++) {
							if (keyPool[j] == 2) {
								// ADD THIS SHARED KEY INTO SHARED KEY LIST
								if (sharedCount < MAX_SHARED_KEYS) {
									pListNode->EG_sharedKeys[sharedCount].id = j;

									//ADD POSITION IN HASH CHAIN
									if (pBasicSettings->egScheme.useHashChain)
									{
										pListNode->EG_sharedKeys[sharedCount].id2D = keyPool2D[j];
									}
								}
								else {
									// MAX_SHARED_KEYS NEDDS TO BE INCREASED
									status = STAT_NOT_ENOUGHT_DATA_TYPE;
									break;
								}

								sharedCount++;        
							}
						}
					}
                }
				if (pBasicSettings->egScheme.useIntermKeys)
				{
					// SET LINK KEY STATUS BASED ON SHARED PRE-DISTRIB KEY INTERSECTION
					if (pBasicSettings->egScheme.normalKeys == 0 && pBasicSettings->egScheme.intermKeys == 0)
					{
						if (sharedCount+sharedIntermCount >= pBasicSettings->egScheme.qIntersect)
						{
							pListNode->keyFlags |= FLAG_KEY_SECURE;
							pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

							(*pSecured)++;
						}
						else {
							if (bMarkMissingAsCompr) {
								pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
								pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
							}

							(*pMissing)++;
						}
					}
					else
					{
						if ((sharedCount>= pBasicSettings->egScheme.normalKeys)
							&&
							(sharedIntermCount >=pBasicSettings->egScheme.intermKeys)) {
							pListNode->keyFlags |= FLAG_KEY_SECURE;
							pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

							(*pSecured)++;
						}
						else {
							if (bMarkMissingAsCompr) {
								pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
								pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
							}

							(*pMissing)++;
						}
					}
				}
				else
				{
					// SET LINK KEY STATUS BASED ON SHARED PRE-DISTRIB KEY INTERSECTION
					if (sharedCount >= pBasicSettings->egScheme.qIntersect) {
						pListNode->keyFlags |= FLAG_KEY_SECURE;
						pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

						(*pSecured)++;
					}
					else {
						if (bMarkMissingAsCompr) {
							pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
							pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
						}

						(*pMissing)++;
					}
				}

                usedNeigh++;
            }

            pListNode = pListNode->next;
        }
    }
    
    
    delete[] keyPool;

    return status;
} 

int CSensorEnv::GetKeysFromNodes(SENSOR_NODE** pNodes, DWORD numNodes, KEY_ID* pKeysList, DWORD* pKeysListSize) {
    int             status = STAT_OK;
    DWORD           keysCount = 0;
    SENSOR_NODE*    pSensor;
    
    // EXTRACT ITS KEYS WITHOUT DUPLICITIES
    if (status == STAT_OK) {
        keysCount = 0;
        for (int i = 0; i < numNodes; i++) {
            pSensor = pNodes[i];    
            
            if (pSensor != NULL) {
                // PROCESS KEYS FROM GIVEN NODE
                for (int j = 0; j < pSensor->keyRingSize; j++) {
                    // INSERT IF NOT INSERTED BEFORE
                    BOOL notInserted = TRUE;
                    for (int k = 0; k < keysCount; k++) {
                        if (pKeysList[k] == pSensor->keyRing[j]) {
                            // ALREADY INSERTTED 
                            notInserted = FALSE;
                            break;
                        }
                    }

                    if (notInserted) {
                        // CHECK IF SUPPLIED BUFFER IS LARGE ENOUGHT AND INSERT NEW KEY
                        if (keysCount < *pKeysListSize) {
                            pKeysList[keysCount] = pSensor->keyRing[j];
                        }
                        else {
                            // SET ERROR FLAG, BUT CONTINUE WITH COUNTING OF KEYS
                            status = STAT_DATA_TOO_LONG;
                        }
                        
                        keysCount++;
                    }
                }
            }
        }
    }

    // SET ACTUAL NUMBER OF EXTRACTED KEYS
    *pKeysListSize = keysCount;
    
    return status;
}

int CSensorEnv::GetKeysFromNodes(SENSOR_NODE** pNodes, DWORD numNodes, BYTE* pKeysList) {
    int             status = STAT_OK;
    DWORD           keysCount = 0;
    SENSOR_NODE*    pSensor;
    
    // EXTRACT ITS KEYS WITHOUT DUPLICITIES
    if (status == STAT_OK) {
        keysCount = 0;
        for (int i = 0; i < numNodes; i++) {
            pSensor = pNodes[i];    
            
            if (pSensor != NULL) {
                // PROCESS KEYS FROM GIVEN NODE
                for (int j = 0; j < pSensor->keyRingSize; j++) {
                    pKeysList[pSensor->keyRing[j]] = 1;
                }
            }
        }
    }
    
    return status;
}
int CSensorEnv::GetKeysFromNodes2D(SENSOR_NODE** pNodes, DWORD numNodes, BYTE* pKeysList, KEY_2DID* pKeysList2D) {
    int             status = STAT_OK;
    DWORD           keysCount = 0;
    SENSOR_NODE*    pSensor;
    
    // EXTRACT ITS KEYS WITHOUT DUPLICITIES
    if (status == STAT_OK) {
        keysCount = 0;
        for (int i = 0; i < numNodes; i++) {
            pSensor = pNodes[i];    
            
            if (pSensor != NULL) {
                // PROCESS KEYS FROM GIVEN NODE
                for (int j = 0; j < pSensor->keyRingSize; j++) {
                    pKeysList[pSensor->keyRing[j]] = 1;
					//IF NEW KEY FOUND IS LOWER IN THE KEY CHAIN (LESS TIMES HASHED) SET THIS KEY AS COMPR.
					if (pSensor->keyRing2D != NULL)
						if (pSensor->keyRing2D[j]<pKeysList2D[pSensor->keyRing[j]])
							pKeysList2D[pSensor->keyRing[j]]=pSensor->keyRing2D[j];
                }
            }
        }
    }
    
    return status;
}

int CSensorEnv::GetKeysFromRandomNodesBitMask(DWORD numNodes, KEYS_MASK* pKeysMask, DWORD* pKeysMaskSize) {
    int             status = STAT_OK;
    DWORD           i;
    DWORD           j;
    SENSOR_NODE**   pSensorsList;
    SENSOR_NODE*    pSensor;

    pSensorsList = new SENSOR_NODE*[numNodes];

    // SELECT RANDOM NODES
    if (status == STAT_OK) {
        status = GetRandomNodes(pSensorsList, numNodes);
    }

    // CLEAR ARRAY FOR OBTAINED KEYS
    memset(pKeysMask, 0, sizeof(KEYS_MASK) * (*pKeysMaskSize));

    // EXTRACT ITS KEYS WITHOUT DUPLICITIES
    if (status == STAT_OK) {
        for (i = 0; i < numNodes; i++) {
            pSensor = pSensorsList[i];    
            
            if (*pKeysMaskSize < pSensor->keyRingMaskSize) {
                status = STAT_DATA_TOO_LONG;
                *pKeysMaskSize = pSensor->keyRingMaskSize;
                break;
            } 

            // PROCESS KEYS FROM GIVEN NODE AND ADD TO KEYS SET
            for (j = 0; j < pSensor->keyRingMaskSize; j++) {
                pKeysMask[j] |= pSensor->keyRingMask[j];    
            }
        }
    }

    // RELEASE TEMP LIST 
    delete[] pSensorsList;

    return status; 
}

int CSensorEnv::GetKeysFromRandomNodes(DWORD numNodes, KEY_ID* pKeysList, DWORD* pKeysListSize) {
    int             status = STAT_OK;
    SENSOR_NODE**   pSensorsList;

    pSensorsList = new SENSOR_NODE*[numNodes];

    // SELECT RANDOM NODES
    if (status == STAT_OK) {
        status = GetRandomNodes(pSensorsList, numNodes);
    }

    // EXTRACT KEYS
    if (status == STAT_OK) {
        status = GetKeysFromNodes(pSensorsList, numNodes, pKeysList, pKeysListSize);
    }
    
    // RELEASE TEMP LIST 
    delete[] pSensorsList;

    return status; 
}

int CSensorEnv::ComputeSharedKeysSet(SENSOR_NODE* pSensorNode1, SENSOR_NODE* pSensorNode2, KEY_ID* pSharedKeys, DWORD* pNumSharedKeys) {
    int     status = STAT_OK;
    int     i;
    int     j;    
    DWORD   sharedKeysCount = 0;    

    for (i = 0; i < pSensorNode1->keyRingSize; i++) {
        for (j = 0; j < pSensorNode2->keyRingSize; j++) {
            if (pSensorNode1->keyRing[i] == pSensorNode2->keyRing[j]) {
                // SHARED KEY FOUND
                if (sharedKeysCount < *pNumSharedKeys) {
                    pSharedKeys[sharedKeysCount] = pSensorNode1->keyRing[i];
                }
                else {
                    // SET ERROR FLAG, BUT CONTINUE WITH COUNTING OF KEYS
                    status = STAT_DATA_TOO_LONG;
                }

                sharedKeysCount++;
            }
        }
    }

    *pNumSharedKeys = sharedKeysCount;

    return status;
}           

int CSensorEnv::ComputeSharedKeysSetBitMask(SENSOR_NODE* pSensorNode1, SENSOR_NODE* pSensorNode2, KEYS_MASK* pSharedKeysMask, DWORD* pKeysMaskSize) {
    int     status = STAT_OK;
    int     i;
    DWORD   sharedKeysCount = 0;    

    // CLEAR ARRAY FOR OBTAINED KEYS
    memset(pSharedKeysMask, 0, sizeof(KEYS_MASK) * (*pKeysMaskSize));

    if (*pKeysMaskSize < pSensorNode1->keyRingMaskSize) {
        status = STAT_DATA_TOO_LONG;
        *pKeysMaskSize = pSensorNode1->keyRingMaskSize;
    } 


    if (status == STAT_OK) {
        for (i = 0; i < pSensorNode1->keyRingMaskSize; i++) {
            pSharedKeysMask[i] = pSensorNode1->keyRingMask[i] & pSensorNode2->keyRingMask[i];
        }
    }

    return status;
}           

int CSensorEnv::GetNumberKeysBitMask(KEYS_MASK* pKeysMask, DWORD keysMaskSize, DWORD* pNumKeys) {
    int             status = STAT_OK;
    DWORD           i;
    DWORD           p;
    DWORD           power = 1;

    *pNumKeys = 0;
    // GO OVER ALL POSSIBLE POWER OF 2 UP TO 2^32
    for (p = 0; p < BITS_KEYS_MASK; p++) {
        // SEARCH FOR OCCURENCE IN CELLS
        for (i = 0; i < keysMaskSize; i++) {
            if ((pKeysMask[i] & power) != 0) (*pNumKeys)++; 
        }

        power = power << 1;
    }

    return status;
}

int CSensorEnv::MMD_CreateRoutingPaths(BASIC_INIT_DATA* pBasicSettings, double* pAvgHopLen) {
    int             status = STAT_OK;
    DWORD           i;
    DWORD           j;
    SENSOR_NODE*    pSensor;
    SENSOR_NODE*    pSensor2;
    SENSOR_NODE*    pBS;
    NEIGH_LIST_NODE*    pNeigh;
    NEIGH_LIST_NODE*    pMinNeigh;
    float           min;
    float           suggestedDist;
	double			avgHopLen = 0;
	DWORD			avgHopCount = 0;

    //
    // CALCULATE DISTANCES FROM ACTUAL SENSOR TO ALL BS
    //
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

        for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
            pBS = m_pActualDeployment->BSList.bs[j];
            pSensor->routeTableBS[j].bsID = pBS->id;
            // CALCULATE DISTANCE
            pSensor->routeTableBS[j].dist = (float) sqrt(pow(pSensor->cX - pBS->cX, 2) + pow(pSensor->cY - pBS->cY, 2));
            
        }
    }


    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

        //
        //  ROUTING TABLES FILLING
        //
        // SET NEIGHBOUR FOR EACH BS X, WHICH WILL BE USED FOR MESSAGE FORWARING WITH TARGET BS X 
        // NEIGBOUR IS SELECTED USING ROUTING STRATEGY
        switch (pBasicSettings->mmdConfig.routingStrategy) {
            case MMD_ROUTESTRAT_NEARESTTOBS: {

                // SEARCH FOR NEIGBOUR, WHICH HAVE SHORTES DISTANCE TO TARGET BS
                for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
                    // SET ACTUAL SENSOR DISTANCE FROM BS AS MINIMUM
                    min = pSensor->routeTableBS[j].dist;
                    pMinNeigh = NULL;

                    pNeigh = pSensor->whiteNeigh;
                    while (pNeigh != NULL) {
                        pSensor2 = (SENSOR_NODE*) pNeigh->pItem;

                        if (pSensor2->flags & FLAG_SENSOR_BASE_STATION) {
                            if (pSensor2->id == m_pActualDeployment->BSList.bs[j]->id) {
                                // TARGET BS DIRECT NEIGBOUR, STOP SEARCHING 
                                pMinNeigh = pNeigh;
                                break;
                            }
                        }

                        if (pSensor2->routeTableBS[j].dist < min) {
                            // NEIGBOUR CLOSEST TO BS FOUND 
                            pMinNeigh = pNeigh;
                            min = pSensor2->routeTableBS[j].dist;
                        }
                        pNeigh = pNeigh->next;
                    }

                    // SET NEIGHBOUR WITH SHORTEST DISTANCE TO BS AS FORWARD NODE
                    pSensor->routeTableBS[j].forwardNode = pMinNeigh;
					if (pMinNeigh) {
						avgHopLen += pMinNeigh->dist;
						avgHopCount++;
					}
                }
                                                
                break;
            }
            case MMD_ROUTESTRAT_NEARESTTOBS_GIVENDIST: {
                // SEARCH FOR NEIGBOURS, WHICH ARE CLOSER TO BS THAN ACTUAL 
                // AND HAVE DISTANCE NEAR REQUIRED LIMIT
                // SEARCH FOR NEIGBOUR, WHICH HAVE SHORTES DISTANCE TO TARGET BS
                suggestedDist = pBasicSettings->mmdConfig.nextNeighDistance * pSensor->transmRange;
                for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
                    // SET MINIMUM DISTANCE FROM REQUIRED ONE
                    min = USHRT_MAX;
                    pMinNeigh = NULL;

                    pNeigh = pSensor->whiteNeigh;
                    while (pNeigh != NULL) {
                        pSensor2 = (SENSOR_NODE*) pNeigh->pItem;

                        if (pSensor2->flags & FLAG_SENSOR_BASE_STATION) {
                            if (pSensor2->id == m_pActualDeployment->BSList.bs[j]->id) {
                                // TARGET BS DIRECT NEIGBOUR, STOP SEARCHING 
                                pMinNeigh = pNeigh;
                                break;
                            }
                        }

                        if (pSensor2->routeTableBS[j].dist < pSensor->routeTableBS[j].dist) {
                            // NEIGBOUR CLOSER TO BS FOUND 
                            
                            // IF NO OTHER FOUND YET, USE THIS ONE, BUT DO NOT SET min VALUE (CAN BE UNDER SUGGESTED LIMIT)
                            // IF SOME NEIGBOUR ALREADY USED THIS WAY AND NEW ONE CLOSER TO LIMIT BUT FROM BOTTOM, USE THE NEW ONE
                            if (min == USHRT_MAX) {
                                if (pMinNeigh == NULL) pMinNeigh = pNeigh;
                                else {
                                    if ((suggestedDist - pMinNeigh->dist) > (suggestedDist - pNeigh->dist)) {
                                        pMinNeigh = pNeigh;
                                    } 
                                }
                            }

                            if (pNeigh->dist >= suggestedDist) {
                                // NEIGBOUR OVER SUGGESTED LIMIT FOUND
                                
                                if ((pNeigh->dist - suggestedDist) < min) {
                                    // THIS ONE IS CLOSER TO SUGGESTED LIMIT
                                    pMinNeigh = pNeigh;
                                    min = pNeigh->dist - suggestedDist;
                                }
                            } 
                        }
                        pNeigh = pNeigh->next;
                    }

                    // SET NEIGHBOUR WITH SHORTEST DISTANCE TO BS AS FORWARD NODE
                    pSensor->routeTableBS[j].forwardNode = pMinNeigh;
					if (pMinNeigh) {
						avgHopLen += pMinNeigh->dist;
						avgHopCount++;
					}
                }
                

                break;
            }
            default: status = STAT_BS_ROUTESTRATEGYUNKNOWN;
        }
    }

	if (pAvgHopLen != NULL) *pAvgHopLen = avgHopLen / avgHopCount;		

    return status;
}

int CSensorEnv::MMD_EstablishBSPaths(BASIC_INIT_DATA* pBasicSettings, DWORD* pathsFound, DWORD* pathsNotFound) {
    int                 status = STAT_OK;
    DWORD               i;
    DWORD               j;
    DWORD               k;
    SENSOR_NODE*        pSensor;
    SENSOR_NODE*        pSensor2;
    NEIGH_LIST_NODE*    pNeigh;
    DWORD               comprCount = 0;
    DWORD               min_index;
    float               min;
    float*              dists;

    if (pathsFound != NULL) *pathsFound = 0;
    if (pathsNotFound != NULL) *pathsNotFound = 0;

    // TRY TO FOLLOW ROUTING TABLES UNTIL BS REACHED
    dists = new float[m_pActualDeployment->BSList.itemCount];
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

        //
        //  ACTIVE BS SELECTION
        //
        
        for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
            // LOCAL COPY OF DISTANCES, WILL BE USED FOR ACTIVE BS SELECTION
            dists[j] = pSensor->routeTableBS[j].dist;
            
            // SET NOT ACTIVE AS DEFAULT    
            pSensor->routeTableBS[j].active = FALSE;
        }

        switch (pBasicSettings->bsConfig.activeBSSelect) {
            case ACTIVEBS_NEAREST: {

                // SEARCH FOR pBasicSettings->bsConfig.bsActive NEAREST BS
                for (k = 0; k < pBasicSettings->bsConfig.bsActive; k++) {
                    min = USHRT_MAX;
                    min_index = 0;
                    for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
                        if (dists[j] < min) {
                            min = dists[j];
                            min_index = j;
                        }       
                    }

                    // SET BS AS ONE FROM ACTIVE 
                    pSensor->routeTableBS[min_index].active = TRUE;
                    // DISABLE THIS BS FOR NEXT MIN SEARCH LOOP
                    dists[min_index] = USHRT_MAX;
                }
                                                
                break;
            }
            case ACTIVEBS_RANDOM: {

                // TODO: RANDOM BS SELECTION

                break;
            }
            default: status = STAT_BS_SELECTRULEUNKNOWN;
        }

        if (status == STAT_OK) {
            for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
                pSensor->routeTableBS[j].hopsCount = 0;
                pSensor->routeTableBS[j].pathFlags = FLAG_BSPATH_NOTFOUND;

                // ESTABLISH PATHS ONLY FOR ACTIVE BS
                if (pSensor->routeTableBS[j].active) {
                    pSensor->routeTableBS[j].hopsCount = 0;
                    pSensor->routeTableBS[j].pathFlags = FLAG_BSPATH_SECURE;    // PATH SECURE FROM DEFAULT UNTIL COMPROMISSED KEY IS FOUND
                
                    // CHECK IF NOT BS ITSELF
                    if ((pSensor->flags & FLAG_SENSOR_BASE_STATION) && (pSensor->id == pSensor->routeTableBS[j].bsID)) {
                        // BS ITSELF 
                    }
                    else {
                        pNeigh = pSensor->routeTableBS[j].forwardNode;
                        while (true) {
                            if (pNeigh == NULL) {
                                // BS NOT REACHABLE, PATH NOT FOUND
                                pSensor->routeTableBS[j].hopsCount = 0;
                                pSensor->routeTableBS[j].pathFlags |= FLAG_BSPATH_NOTFOUND;
                                break;
                            }
                            else {
                                // INCREASE HOPS COUNTER
                                pSensor->routeTableBS[j].hopsCount++;


                                if (pNeigh->keyFlags & FLAG_KEY_COMPROMISED) {
                                    // COMPROMISSED KEY USED, PATH IS COMPROMISSED
                                    pSensor->routeTableBS[j].pathFlags &= ~FLAG_BSPATH_SECURE;
                                    pSensor->routeTableBS[j].pathFlags |= FLAG_BSPATH_COMPROMISED;
                                    comprCount++;
                                }

                                pSensor2 = (SENSOR_NODE*) pNeigh->pItem;
                                if ((pSensor2->flags & FLAG_SENSOR_BASE_STATION) && (pSensor2->id == pSensor->routeTableBS[j].bsID)) {
                                    // TARGET BS REACHED 
                                    break;
                                } 
                          
                                // PERFORM NEXT FORWARDING
                                pNeigh = pSensor2->routeTableBS[j].forwardNode;
                            }
                        }
                    }

                    if (pSensor->routeTableBS[j].pathFlags & FLAG_BSPATH_NOTFOUND) {
                        if (pathsNotFound != NULL) (*pathsNotFound)++;   
                    }
                    else {
                        if (pathsFound != NULL) (*pathsFound)++;   
                    }
                }

            }
        }
    }

    delete[] dists;

    return status;
}

int CSensorEnv::MMD_Test(BASIC_INIT_DATA* pBasicSettings, MMD_TEST_RESULTS* pResults) {
    int                 status = STAT_OK;
    DWORD               i;
    DWORD               j;
    SENSOR_NODE*        pSensor;
    DWORD               secureMessages = 0;  
    DWORD               comprMessages = 0; 
    DWORD               lostMessages = 0;
    DWORD               totalHops = 0;
    DWORD               foundPaths = 0;
    DWORD               totalActiveBS = 0;
    DWORD               notFoundPaths = 0;
    DWORD               foundPathsToBSLocal = 0;
    DWORD               comprPathsToBSLocal = 0;
    BOOL                bLost = FALSE;
    CString             message;

    // ESTABLISH STATUS OF SENSOR->BS PATHS
    if (status == STAT_OK) { 
        status = this->MMD_EstablishBSPaths(pBasicSettings);
    }
    
    if (status == STAT_OK) {
        // CHECK PATH SECURITY STATUS ALL ACTIVE BASE STATIONS
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

        
            foundPathsToBSLocal = 0;
            comprPathsToBSLocal = 0;
            bLost = FALSE;
            for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
                // IF ALL PATHS EXIT AND AT LEAST ONE PATH TO BASE IS SECURE, MESSAGE IS SECURE
                if (pSensor->routeTableBS[j].active) {
                    totalActiveBS++;
                    if (pSensor->routeTableBS[j].pathFlags & FLAG_BSPATH_NOTFOUND) {
                        // PATH TO THIS BS NOT FOUND
                        notFoundPaths++;
                        bLost = TRUE;

                        // PATH NOT FOUND, CHECK IF COMPROMISED
                        if (pSensor->routeTableBS[j].pathFlags & FLAG_BSPATH_COMPROMISED) {
                            // PATH NOT FOUND, MESSAGE IS COMPROMISED DURING PART OF ROUTED PATH
/* IF ENABLED, MMD MESSAGES WHICH CANNOT BE RECONSTRUCTED ON BS (LESS THAN pBasicSettings->bsConfig.bsActive PATHS) 
   WILL BE COUNTED AS COMPROMISSED, IF LINK KEY IS COMPROMISSED
                            comprPathsToBSLocal++;
/**/
                        }
                    }
                    else {
                        // ONE ACTIVE AND SECURE PATH FOUND => MESSAGE WILL BE SECURE
                        if (pSensor->routeTableBS[j].pathFlags & FLAG_BSPATH_SECURE) {
                            foundPathsToBSLocal++; 
                        }
                        else {
                            // PATH FOUND, BUT NOT SECURE => MESSAGE WILL BE COMPROMISED, IF ALL PATHS TO ACTIVE BS FOUND AND ARE COMPROMISED
                            comprPathsToBSLocal++;
                        }

                        // HOPS STATISTICS
                        totalHops += pSensor->routeTableBS[j].hopsCount;
                        foundPaths++;
                    }
                }
            }  

            // UPDATE STATISTICS
            if (foundPathsToBSLocal == pBasicSettings->bsConfig.bsActive) {
                secureMessages++;
            }

            if (comprPathsToBSLocal == pBasicSettings->bsConfig.bsActive) {
                comprMessages++;
            }

            if (bLost) {
                lostMessages++;
            }

        }
    }

    if (status == STAT_OK) {
        pResults->foundPaths = foundPaths;
        pResults->notFoundPaths = notFoundPaths;
        pResults->totalMessages = secureMessages + comprMessages + lostMessages ;
/*
        pResults->secureMessages = (secureMessages / (float) (secureMessages + comprMessages + lostMessages)) * 100;
        pResults->comprMessages = (comprMessages / (float) (secureMessages + comprMessages + lostMessages)) * 100;
        pResults->lostMessages = (lostMessages / (float) (secureMessages + comprMessages + lostMessages)) * 100;
/**/
        pResults->secureMessages = secureMessages;
        pResults->comprMessages = comprMessages;
        pResults->lostMessages = lostMessages;
        pResults->avgHops = totalHops / (float) foundPaths;

        message.Format("MMD -- totalBS: %d | activeBS: %.1f | foundPaths: %d | !foundPaths %d | avg. hops: %.2f", m_pActualDeployment->BSList.itemCount, totalActiveBS / (float) m_pActualDeployment->whiteSensorList.numSensors, foundPaths, notFoundPaths, totalHops / (float) foundPaths); 
        LOG_INSERTSTRING(message);
        message.Format("         -- totalMsg: %d | secure: %.4f%s | compromised: %.4f%s|  lost: %.4f%s", secureMessages + comprMessages + lostMessages, (secureMessages / (float) (secureMessages + comprMessages)) * 100, "%", (comprMessages / (float) (secureMessages + comprMessages)) * 100, "%", (lostMessages / (float) (secureMessages + comprMessages + lostMessages)) * 100, "%");
        LOG_INSERTSTRING(message);
    }

    return status;
}

int CSensorEnv::FOE_RecordMessagesPath(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, INT_DATA_BLOB* pCountHopsToBS, INT_DATA_BLOB* pCountHopsToCapture) {
    int                      status = STAT_OK;
    FOE_MESSAGE_LIST_NODE*   pMessage;
    FOE_MESSAGE_LIST_NODE*   pLastMessage;
    int                      i;   
    DWORD                    j;
    DWORD                    localFOEBoostCount;   
    SENSOR_NODE*             pSensor;
    SENSOR_NODE*             pSensor2;
    SENSOR_NODE*             pLocalSensor;
    NEIGH_LIST_NODE*         pNeigh;
    NEIGH_LIST_NODE*         pLocalNeigh;

    //
    // FOR EACH WHITE SENSOR SEND MESSAGE ALONG ROUTING PATH TO ACTIVE BS 
    // BUILD SET OF NODES, WHICH OCCURES ON PATH BEFORE MESSAGE IS CAPTURED FOR FUTHER PROCESSING
    //

    pLastMessage = pMessageList;

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

        // PROCESS ALL ACTIVE BS
        for (j = 0; j < m_pActualDeployment->BSList.itemCount; j++) {
            // CHECK IF GIVEN BS IS USED BY GIVEN SENSOR
            if (pSensor->routeTableBS[j].active) {
                // ... AND MESSAGE PATH IS FOUND (FROM PREVIOUS MMD_EstablishBSPaths() )
                if (!(pSensor->routeTableBS[j].pathFlags & FLAG_BSPATH_NOTFOUND)) {
                
                    //
                    // FOLLOW ROUTING TABLE AND CREATE PROFILE OF MESSAGE (VISITED NODES ...) 
                    //
                    
                    // ADD NEW MESSAGE PROFILE (FIRST NODE IS ASSUMED)
                    pMessage = new FOE_MESSAGE_LIST_NODE;
                    pMessage->clear();

                    pLastMessage->next = pMessage;
                    pMessage->prev = pLastMessage;

                    // SET SENDER AND RECEIVER
                    pMessage->pSenderNode = pSensor;
                    pMessage->pTargetNode = m_pActualDeployment->BSList.bs[j];

                    // INSERT GIVEN SENSOR AS FIRST NODE ON PATH
                    pMessage->insertNode(pSensor);

                    // MOVE POINTER ON ACTUAL MESSAGE PROFILE FOR NEXT ITERATION
                    pLastMessage = pMessage;

                    // INIT COUNTER OF REMAINING NODES WHERE SHOULD BE PERFORMED LOCAL BOOSTING
                    if (pBasicSettings->foeConfig.useLocalBoost) {
                        localFOEBoostCount = pBasicSettings->foeConfig.boostedNodesCount;
                    }

                    // CHECK IF NOT BS ITSELF
                    if ((pSensor->flags & FLAG_SENSOR_BASE_STATION) && (pSensor->id == pSensor->routeTableBS[j].bsID)) {
                        // BS ITSELF, MESSAGE IS SECURE 
                        pMessage->flags |= FLAG_FOE_MESSAGE_NOT_CAPTURED;
                    }
                    else {
                        //
                        //  BASIC FOE ROUTING
                        //
                        
                        // OBTAIN NEXT FORWARD NODE ON ROUTING PATH
                        if ((pNeigh = pSensor->routeTableBS[j].forwardNode) != NULL) {
                            pSensor2 = (SENSOR_NODE*) pNeigh->pItem;
                        }

                        while (true) {
                            if (pNeigh == NULL) {
                                // BS NOT REACHABLE, PATH NOT FOUND
                                // SHOULD NOT OCCURE!
                                _ASSERT(FALSE);
                                break;
                            }
                            else {
                                pSensor2 = (SENSOR_NODE*) pNeigh->pItem;

                                //
                                //  LOCAL FOE BOOST
                                //
                                if (pBasicSettings->foeConfig.useLocalBoost && localFOEBoostCount > 0) {
                                    // INVOLVE ALL DIRECT NEIGBOURS CLOSER TO pSensor THAN NEXT FORWARD NODE
                                    pLocalNeigh = pSensor->whiteNeigh;
                                    while (pLocalNeigh != NULL) {
                                        // USE DIRECT NEIGHBOUR, IF IS CLOSER THAN NEXT FORWARD NODE (CONSERVATIVE BOOST) OR 
                                        // IF ALL DIRECT NEIGHBOURS SHOULD BE USED 
                                        if ((pLocalNeigh->dist < pNeigh->dist) || pBasicSettings->foeConfig.useBoostAllDirectNeigh) {
                                            pLocalSensor = (SENSOR_NODE*) pLocalNeigh->pItem;
                                    
                                            // CHECK IF TRANSMISSION TO THIS NEIGBOUR CANNOT BE EAVESDROPED BY ATTACKER
                                            if (!(pLocalNeigh->keyFlags & FLAG_KEY_COMPROMISED)) {
                                                // ADD THIS LOCAL NEIGBOUR AS NEXT NODE WHICH CONTRIBUTE BY ONION KEYS
                                                if (pBasicSettings->foeConfig.egAuth) {
                                                    // IF EGAUTH TEST, THAN ADD ONLY WHEN LIMIT FOR NODES IS NOT REACHED 
                                                    if (pMessage->numberNodesOnPath < pBasicSettings->foeConfig.nodesInGroupCount) {
                                                        pMessage->insertNode(pLocalSensor);
                                                        // INCREASE COUNTER OF USED LOCAL BOOST NEIGBOURS
                                                        pMessage->numberOfUsedLocalBoostNeigh++;
                                                    }
                                                }
                                                else {
                                                    pMessage->insertNode(pLocalSensor);
                                                    // INCREASE COUNTER OF USED LOCAL BOOST NEIGBOURS
                                                    pMessage->numberOfUsedLocalBoostNeigh++;
                                                }
                                            }
                                            else {
                                                // TRANSMISSION IS EAVESDROPPED
                                                // COMPROMISSED KEY USED, MESSAGE IS CAPTURED BY ATTACKER RIGHT NOW                                    pSensor->routeTableBS[j].pathFlags &= ~FLAG_BSPATH_SECURE;
                                                if (!(pMessage->flags & FLAG_FOE_MESSAGE_CAPTURED)) {
                                                    pMessage->flags |= FLAG_FOE_MESSAGE_CAPTURED;
                                                    pMessage->numberOfHopsToCapture++;
                                                }
                                            }                                               
                                        }
                                        pLocalNeigh = pLocalNeigh->next;
                                    }

                                    // DECREASE COUNTER OF NODES WITH ALLOWED LOCAL BOOST
                                    localFOEBoostCount--;
                                }       

                                // INCREASE HOPS COUNTER
                                pMessage->numberOfHopsToBS++;

                                if (pNeigh->keyFlags & FLAG_KEY_COMPROMISED) {
                                    // CHECK IF NOT ALREADY CAPTURED
                                    if (!(pMessage->flags & FLAG_FOE_MESSAGE_CAPTURED)) {
                                        // COMPROMISSED KEY USED, MESSAGE IS CAPTURED BY ATTACKER RIGHT NOW                                    pSensor->routeTableBS[j].pathFlags &= ~FLAG_BSPATH_SECURE;
                                        pMessage->flags |= FLAG_FOE_MESSAGE_CAPTURED;
                                        // HOPS TO CAPTURING
                                        pMessage->numberOfHopsToCapture++;
                                    }
                                    else {
                                        // DO NOTHING
                                    }
                                }
                                else {
                                    // ADD NODE TO LIST OF NODES WHICH CONTRIBUTE WITH KEYS UNTIL MESSAGE IS CAPTURED
                                    if (!(pMessage->flags & FLAG_FOE_MESSAGE_CAPTURED)) {
                                        // HOPS TO CAPTURING
                                        pMessage->numberOfHopsToCapture++;
        
                                        // INSERT ACTUAL SENSOR AS NEXT NODE ON PATH
                                        if (pBasicSettings->foeConfig.egAuth) {
                                            if (pMessage->numberNodesOnPath < pBasicSettings->foeConfig.nodesInGroupCount) {
                                                // IF EGAUTH TEST, THAN ADD ONLY WHEN LIMIT FOR NODES IS NOT REACHED 
                                                pMessage->insertNode(pSensor2);
                                            }
                                        }
                                        else {
                                            pMessage->insertNode(pSensor2);
                                        }

                                        if ((pSensor2->flags & FLAG_SENSOR_BASE_STATION) && (pSensor2->id == pSensor->routeTableBS[j].bsID)) {
                                            // TARGET BS REACHED 
                                            pMessage->flags |= FLAG_FOE_MESSAGE_NOT_CAPTURED;
                                        } 
                                    }
                                }

                                if ((pSensor2->flags & FLAG_SENSOR_BASE_STATION) && (pSensor2->id == pSensor->routeTableBS[j].bsID)) {
                                    // TARGET BS REACHED 
                                    break;
                                }

                                // PERFORM NEXT FORWARDING
                                pNeigh = pSensor2->routeTableBS[j].forwardNode;
                            }
                        }

                        // IF MESSAGE WAS NOT CAPTURED, CLEAR numberOfHopsToCapture COUNTER
                        if (pMessage->flags & FLAG_FOE_MESSAGE_NOT_CAPTURED) {
                            pMessage->numberOfHopsToCapture = 0;
                        }
                    }
                        
                }
                else {
                    // PATH NOT FOUND
                }
            }
            else {
                // BS IS NOT ACTIVE FOR GIVEN SENSOR
            }
        }
    }

    // STATISTICS ABOUT HOPS
    if (pCountHopsToBS || pCountHopsToCapture) {
        pMessage = pMessageList->next;
        while (pMessage) {
            // DISTRIBUTION OF NUMBER OF HOPS
            if (pCountHopsToBS) if (pMessage->numberOfHopsToBS < pCountHopsToBS->dwMaxLen) pCountHopsToBS->pData[pMessage->numberOfHopsToBS]++;
            if (pCountHopsToCapture) if (pMessage->numberOfHopsToCapture < pCountHopsToCapture->dwMaxLen) pCountHopsToCapture->pData[pMessage->numberOfHopsToCapture]++;

            pMessage = pMessage->next;
        }
    }
    
    return status;
}

int CSensorEnv::FOE_MessageComputeAppliedKeys(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList) {
    int         status = STAT_OK;
    FOE_MESSAGE_LIST_NODE*  pMessage;

    pMessage = pMessageList->next;
    while (pMessage) {
        if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
            // ALLOCATE ARRAY, IF NEEDED
            if (pMessage->appliedKeysMaskCount < pBasicSettings->egScheme.keyPoolSize / BITS_KEYS_MASK + 1) {
                if (pMessage->pAppliedKeysMask != NULL) {
					delete[] pMessage->pAppliedKeysMask;
					pMessage->pAppliedKeysMask = NULL;
				}
                pMessage->appliedKeysMaskCount = pBasicSettings->egScheme.keyPoolSize / BITS_KEYS_MASK + 1;
                pMessage->pAppliedKeysMask = new KEYS_MASK[pMessage->appliedKeysMaskCount];
            }
            else pMessage->appliedKeysMaskCount = pBasicSettings->egScheme.keyPoolSize / BITS_KEYS_MASK + 1;
            memset(pMessage->pAppliedKeysMask, 0, pMessage->appliedKeysMaskCount * sizeof(KEYS_MASK));

            // COMPUTE APPLIED KEYS
            if (pBasicSettings->foeConfig.onlySenderTarget) {
                if ((status = ComputeSharedKeysSetBitMask(pMessage->pSenderNode, pMessage->pTargetNode, pMessage->pAppliedKeysMask, &(pMessage->appliedKeysMaskCount))) == STAT_OK) {
                }
            }
            else {
                if ((status = FOE_ComputeSharedKeysSetBitMask(pMessage->pNodesOnPath, pMessage->pTargetNode, pMessage->pAppliedKeysMask, &(pMessage->appliedKeysMaskCount))) == STAT_OK) {
                }
            }

            // COUNT NUMBER OF APPLIED KEYS
            GetNumberKeysBitMask(pMessage->pAppliedKeysMask, pMessage->appliedKeysMaskCount, &(pMessage->numberOfKeys)); 
        }
        else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
            // ALLOCATE ARRAY, IF NEEDED
            // NO MORE THAN KEYRINGSIZE KEYS CAN BE APPLIED
            pMessage->appliedKeysIDCount = pBasicSettings->egScheme.keyRingSize;
            if (pMessage->pAppliedKeysID == NULL) {
                pMessage->pAppliedKeysID = new KEY_ID[pMessage->appliedKeysIDCount];
            }

            memset(pMessage->pAppliedKeysID, 0, sizeof(KEY_ID) * pMessage->appliedKeysIDCount); 

            // COMPUTE APPLIED KEYS
            if ((status = FOE_ComputeSharedKeysSet(pMessage->pNodesOnPath, pMessage->pTargetNode, pMessage->pAppliedKeysID, &pMessage->appliedKeysIDCount)) == STAT_OK) {
                pMessage->numberOfKeys = pMessage->appliedKeysIDCount;        
            }
        }
                
        pMessage = pMessage->next;
    }

    return status;
}

int CSensorEnv::FOE_PrepareMessagesPath(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD* pathsFound, DWORD* pathsNotFound, INT_DATA_BLOB* pCountHopsToBS, INT_DATA_BLOB* pCountHopsToCapture) {
    int     status = STAT_OK;

    // SELECT ACTIVE BS FOR EACH SENSOR AND CREATE ROUTING TABLES
    if (status == STAT_OK) {
        // USE ONLY ONE NEAREST BASE STATION
        status = pSensorSim->m_sensorEnv.MMD_CreateRoutingPaths(pBasicSettings);
    }

    // ESTABLISH ROUTING PATHS FROM NODE TO BS
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.MMD_EstablishBSPaths(pBasicSettings, pathsFound, pathsNotFound);
    }
    
    // TRY TO SEND MESSAGE FROM ALL NODES TO NEAREST BASE STATION USING FOE METHOD
    // COMPUTE AVERAGE PATH LENGTH BEFORE MESSAGE CAPTURE
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.FOE_RecordMessagesPath(pBasicSettings, pMessageList, pCountHopsToBS, pCountHopsToCapture);
    }

    return status;
}

int CSensorEnv::FOE_Test(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD numComprNodes, FOE_TEST_RESULTS* pResults) {
    int                     status = STAT_OK;
    int                     i;
    FOE_MESSAGE_LIST_NODE*  pMessage;   
    BOOL                    messageCompromissed = FALSE;
    BOOL                    keyCompromised = FALSE;
    DWORD                   hopsCaptureCount = 0;
    DWORD                   hopsBSCount = 0;
    KEYS_MASK*              pComprKeysMask = NULL;   
    DWORD                   comprKeysMaskSize = 0;
    DWORD                   numMsg = 0;
    DWORD                   comprKeysCount;
    KEY_ID*                 pComprKeysList = NULL;
    int                     j;
    BOOL                    processMessage = FALSE;
    BYTE*                   pComprPoolMask = NULL;    

    pResults->clear();

    //
    // SELECT RANDOMLY COMPROMISSED NODES UP TO comprNodes AND COLLUDE THEIR KEYS
    //
    if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
        if (status == STAT_OK) {
            comprKeysMaskSize = pBasicSettings->egScheme.keyPoolSize / BITS_KEYS_MASK + 1;
            pComprKeysMask = new KEYS_MASK[comprKeysMaskSize];
            memset(pComprKeysMask, 0, comprKeysMaskSize * sizeof(KEYS_MASK));
            status = pSensorSim->m_sensorEnv.GetKeysFromRandomNodesBitMask(numComprNodes, pComprKeysMask, &comprKeysMaskSize);
        }
        
        pResults->comprNodes = numComprNodes;
        GetNumberKeysBitMask(pComprKeysMask, comprKeysMaskSize, &(pResults->comprKeys)); 
    }
    else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
        // OBTAIN COMPROMISED KEYS
        if (status == STAT_OK) {
            comprKeysCount = numComprNodes * pBasicSettings->egScheme.keyRingSize;
            pComprKeysList = new KEY_ID[comprKeysCount];
            memset(pComprKeysList, 0, comprKeysCount * sizeof(KEY_ID));
            status = pSensorSim->m_sensorEnv.GetKeysFromRandomNodes(numComprNodes, pComprKeysList, &comprKeysCount);
        }
        
        if (status == STAT_OK) {
            // CREATE MASK OF COMPROMISED KEYS
            pComprPoolMask = new BYTE[pBasicSettings->egScheme.keyPoolSize + 1];
            memset(pComprPoolMask, 0, pBasicSettings->egScheme.keyPoolSize + 1);
            for (j = 0; j < comprKeysCount; j++) {
                pComprPoolMask[pComprKeysList[j]] = 1;
            }
        }

        pResults->comprNodes = numComprNodes;
        pResults->comprKeys = comprKeysCount;
    }        
        
        
    pResults->maxHopsToBS = 0;
    pMessage = pMessageList->next;
    while (pMessage) {
        processMessage = TRUE;
        
        if (pBasicSettings->foeConfig.egAuth) {
            // THIS IF STATEMENT CAUSE TO COUNT ONLY MESSAGE WITH APPROPRIATE NUMBER OF APPLIED KEYS
            processMessage = FALSE;
            if (pMessage->numberNodesOnPath == pBasicSettings->foeConfig.nodesInGroupCount && pMessage->numberOfKeys >= pBasicSettings->foeConfig.minKeysCount) {
                processMessage = TRUE;
                // ALL MESSAGES ARE SET AS CAPTURED NOW TO TEST THEIR NODE CAPTURE RESILIENCE 
                pMessage->flags &= ~FLAG_FOE_MESSAGE_NOT_CAPTURED; 
                pMessage->flags |= FLAG_FOE_MESSAGE_CAPTURED;
             }
        }
        
        
        if (processMessage) {
            numMsg++;
            
            // TEST IF MESSAGE WAS CAPTURED
            if (pMessage->flags & FLAG_FOE_MESSAGE_NOT_CAPTURED) {
                // MESSAGE NOT CAPTURED
                pResults->secureMsg++;

                pResults->notCapturedMsg++;
            }
            else {
                // MESSAGE WAS CAPTURED
                pResults->capturedMsg++;


                if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) { 
                    // TRY TO FIND AT LEAST ONE NON-COMPROMISSED KEY
                    messageCompromissed = TRUE;
                    for (i = 0; i < pMessage->appliedKeysMaskCount; i++) {
                        // IF SHARED KEYS BETWEEN COMPR.SET AND MESSAGE ARE DIFFERENT FROM MESSAGE SET, THAN AT LEAST ONE KEY IS NON-COMPROMISSED
                        if ((pMessage->pAppliedKeysMask[i] & pComprKeysMask[i]) != pMessage->pAppliedKeysMask[i]) { 
                            pResults->secureMsg++;
                            messageCompromissed = FALSE;
                            break;
                        }

                    }
                }
                else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                    // TRY TO FIND AT LEAST ONE NON-COMPOMISSED KEY
                    messageCompromissed = TRUE;
                    for (i = 0; i < pMessage->appliedKeysIDCount; i++) {
                        keyCompromised = FALSE;
                        
                        if (pComprPoolMask[pMessage->pAppliedKeysID[i]] != 0) { 
                            // KEY IS COMPROMISSED
                            keyCompromised = TRUE;
                        }

                        if (!keyCompromised) {
                            // SECURE KEY FOUND, MESSAGE IS SECURE
                            pResults->secureMsg++;
                            messageCompromissed = FALSE;
                            break;
                        }
                    }
                }    

                // IF NO SECURE KEY IS FOUND, MESSAGE IS COMPROMISSED
                if (messageCompromissed) {
                    pResults->comprMsg++;

                    // STATISTICS OF COMPROMISED MESSAGES WHEN CAPTURED ON Nth HOP 
                    if (pMessage->numberOfHopsToCapture < pResults->numComprOnHop.dwMaxLen) {
                        pResults->numComprOnHop.pData[pMessage->numberOfHopsToCapture]++;    
                    }
                    else {
                        // END OF ARAY REACHED
                        _ASSERT(FALSE);
                    }
                }
            }

            // AVERAGE NUMBER OF HOPS
            pResults->avgAppliedKeys+= pMessage->numberOfKeys;
            pResults->numAppliedKeysCounts.pData[pMessage->numberOfKeys]++;

            pResults->avgHopsToBS += pMessage->numberOfHopsToBS;
            hopsBSCount++;

            if (pMessage->flags & FLAG_FOE_MESSAGE_CAPTURED) {
                pResults->avgHopsToCapture += pMessage->numberOfHopsToCapture;
                hopsCaptureCount++;

                // STATISTICS OF NUMBER OF HOPS TO CAPTURE
                if (pMessage->numberOfHopsToCapture < pResults->hopsToCaptureCounts.dwMaxLen) {
                    pResults->hopsToCaptureCounts.pData[pMessage->numberOfHopsToCapture]++;    
                }
                else {
                    // END OF ARAY REACHED
                    _ASSERT(FALSE);
                }
            }

            // STATISTICS OF NUMBER OF HOPS TO BS
            if (pMessage->numberOfHopsToBS < pResults->hopsToBSCounts.dwMaxLen) {
                pResults->hopsToBSCounts.pData[pMessage->numberOfHopsToBS]++;    
            }
            else {
                // END OF ARAY REACHED
                _ASSERT(FALSE);
            }

            // MAXIMUM NUMBER OF HOPS
            if (pResults->maxHopsToBS < pMessage->numberOfHopsToBS) pResults->maxHopsToBS = pMessage->numberOfHopsToBS;
/**/            
        }
        
        pMessage = pMessage->next;
    }

    pResults->avgHopsToBS /= hopsBSCount;
    pResults->avgHopsToCapture /= hopsCaptureCount;
    pResults->avgAppliedKeys /= numMsg;

    if (pComprKeysMask != NULL) delete[] pComprKeysMask;
    if (pComprKeysList != NULL) delete[] pComprKeysList;
    if (pComprPoolMask != NULL) delete[] pComprPoolMask;

/*
    }
    else {
        // SELECT RANDOMLY COMPROMISSED NODES UP TO comprNodes AND COLLUDE THEIR KEYS
        if (status == STAT_OK) {
            comprKeysCount = numComprNodes * pBasicSettings->egScheme.keyRingSize;
            pComprKeysList = new KEY_ID[comprKeysCount];
            memset(pComprKeysList, 0, comprKeysCount * sizeof(KEY_ID));
            status = pSensorSim->m_sensorEnv.GetKeysFromRandomNodes(numComprNodes, pComprKeysList, &comprKeysCount);
        }

        pResults->comprNodes = numComprNodes;
        pResults->comprKeys = comprKeysCount;
        

        // COMPUTE SET OF APPLIED ONION KEYS FOR EACH MESSAGE. IF AT LEAST ONE IS NOT IN
        // SET OF COMPROMISSED KEYS, THAN MESSAGE IS SECURE 
        appliedKeysCount = pBasicSettings->egScheme.keyRingSize;
        pAppliedKeysList = new KEY_ID[appliedKeysCount];
        memset(pAppliedKeysList, 0, sizeof(KEY_ID) * appliedKeysCount); 

        pResults->maxHopsToBS = 0;
        pMessage = pMessageList->next;
        while (pMessage) {
            // TEST IF MESSAGE WAS CAPTURED
            if (pMessage->flags & FLAG_FOE_MESSAGE_NOT_CAPTURED) {
                // MESSAGE NOT CAPTURED
                pResults->secureMsg++;

                pResults->notCapturedMsg++;
            }
            else {
                // MESSAGE WAS CAPTURED
                pResults->capturedMsg++;

                // NO MORE THAN KEYRINGSIZE KEYS CAN BE APPLIED
                // SPEED OPTIMALISATION: BUFFER IS REALLOCATED ONNLY IF IS NOT LARGE ENOUGHT 
                if (pMessage->pTargetNode->keyRingSize > pBasicSettings->egScheme.keyRingSize) {
                    delete[] pAppliedKeysList;
                    appliedKeysCount = pBasicSettings->egScheme.keyRingSize;
                    pAppliedKeysList = new KEY_ID[appliedKeysCount];
                }
                else {
                    appliedKeysCount = pBasicSettings->egScheme.keyRingSize;
                }
                memset(pAppliedKeysList, 0, sizeof(KEY_ID) * appliedKeysCount); 

                // COMPUTE APPLIED KEYS
                if ((status = FOE_ComputeSharedKeysSet(pMessage->pNodesOnPath, pMessage->pTargetNode, pAppliedKeysList, &appliedKeysCount)) == STAT_OK) {
                    pMessage->numberOfKeys = appliedKeysCount;
                    // TRY TO FIND AT LEAST ONE NON-COMPOMISSED KEY
                    messageCompromissed = TRUE;
                    for (i = 0; i < appliedKeysCount; i++) {
                        keyCompromised = FALSE;
                        for (j = 0; j < comprKeysCount; j++) {
                            if (pAppliedKeysList[i] == pComprKeysList[j]) {
                                // KEY IS COMPROMISSED
                                keyCompromised = TRUE;
                                break;
                            }
                        }

                        if (!keyCompromised) {
                            // SECURE KEY FOUND, MESSAGE IS SECURE
                            pResults->secureMsg++;
                            messageCompromissed = FALSE;
                            break;
                        }
                    }    

                    // IF NO SECURE KEY IS FOUND, MESSAGE IS COMPROMISSED
                    if (messageCompromissed) {
                        pResults->comprMsg++;

                        // STATISTICS OF COMPROMISED MESSAGES WHEN CAPTURED ON Nth HOP 
                        if (pMessage->numberOfHopsToCapture < pResults->numComprOnHop.dwMaxLen) {
                            pResults->numComprOnHop.pData[pMessage->numberOfHopsToCapture]++;    
                        }
                        else {
                            // END OF ARAY REACHED
                            _ASSERT(FALSE);
                        }
                    }
                }
            }

            // AVERAGE NUMBER OF HOPS
            pResults->avgHopsToBS += pMessage->numberOfHopsToBS;
            hopsBSCount++;

            if (pMessage->flags & FLAG_FOE_MESSAGE_CAPTURED) {
                pResults->avgHopsToCapture += pMessage->numberOfHopsToCapture;
                hopsCaptureCount++;

                // STATISTICS OF NUMBER OF HOPS TO CAPTURE
                if (pMessage->numberOfHopsToCapture < pResults->hopsToCaptureCounts.dwMaxLen) {
                    pResults->hopsToCaptureCounts.pData[pMessage->numberOfHopsToCapture]++;    
                }
                else {
                    // END OF ARAY REACHED
                    _ASSERT(FALSE);
                }
            }

            // STATISTICS OF NUMBER OF HOPS TO BS
            if (pMessage->numberOfHopsToBS < pResults->hopsToBSCounts.dwMaxLen) {
                pResults->hopsToBSCounts.pData[pMessage->numberOfHopsToBS]++;    
            }
            else {
                // END OF ARAY REACHED
                _ASSERT(FALSE);
            }

            // MAXIMUM NUMBER OF HOPS
            if (pResults->maxHopsToBS < pMessage->numberOfHopsToBS) pResults->maxHopsToBS = pMessage->numberOfHopsToBS;

            pMessage = pMessage->next;
        }

        pResults->avgHopsToBS /= hopsBSCount;
        pResults->avgHopsToCapture /= hopsCaptureCount;

        delete[] pAppliedKeysList;
        delete[] pComprKeysList;
    }
/**/

    return status;
}


int CSensorEnv::FOE_ComputeSharedKeysSet(LIST_NODE* pNodesOnPath, SENSOR_NODE* pTargetNode, KEY_ID* pKeysList, DWORD* pKeysListSize) {
    int           status = STAT_OK;
    int           i; 
    int           j; 
    LIST_NODE*    pNode;
    SENSOR_NODE*  pSensorNode;
    KEY_ID*       pSharedKeys;    
    DWORD         numSharedKeys = 0; 
    DWORD         numAppliedKeys = 0;
    BOOL          keyNotApplied = TRUE;    

    // NO MORE THAN TARGET RING SIZE KEYS CAN BE SHARED
    numSharedKeys = pTargetNode->keyRingSize;
    pSharedKeys = new KEY_ID[numSharedKeys];

    pNode = pNodesOnPath;
    while(pNode) {
        // CLEAR SHARED KEYS ARRAY
        numSharedKeys = pTargetNode->keyRingSize;
        memset(pSharedKeys, 0, numSharedKeys * sizeof(KEY_ID));

        pSensorNode = (SENSOR_NODE*) pNode->pItem;
        
        if (pSensorNode->id != pTargetNode->id) {
            // COMPUTE SHARED KEYS BETWEEN pSensorNode AND pTargetNode 
            if ((status = ComputeSharedKeysSet(pSensorNode, pTargetNode, pSharedKeys, &numSharedKeys)) == STAT_OK) {
            
                // ADD ALL KEYS, WHICH WERE NOT USED BEFORE TO pKeysList
                for (i = 0; i < numSharedKeys; i++) {
                    keyNotApplied = TRUE;
                    for (j = 0; j < numAppliedKeys; j++) {
                        if (pSharedKeys[i] == pKeysList[j]) {
                            // KEY ALREADY APPLIED
                            keyNotApplied = FALSE;
                            break;
                        } 
                    }   

                    // INSERT KEY INTO LIST
                    if (keyNotApplied) {
                        pKeysList[numAppliedKeys] = pSharedKeys[i];
                        numAppliedKeys++;    
                    }
                }   
            }            
        }

        pNode = pNode->next;
    }

    *pKeysListSize = numAppliedKeys;

    delete[] pSharedKeys;
    return status;    
}

int CSensorEnv::FOE_ComputeSharedKeysSetBitMask(LIST_NODE* pNodesOnPath, SENSOR_NODE* pTargetNode, KEYS_MASK* pKeysMask, DWORD* pKeysMaskSize) {
    int           status = STAT_OK;
    int           i; 
    LIST_NODE*    pNode;
    SENSOR_NODE*  pSensorNode;

    // CLEAR ARRAY FOR OBTAINED KEYS
    memset(pKeysMask, 0, sizeof(KEYS_MASK) * (*pKeysMaskSize));

    pNode = pNodesOnPath;
    while(pNode) {
        pSensorNode = (SENSOR_NODE*) pNode->pItem;
        
        if (pSensorNode->id != pTargetNode->id) {
            // ADD SHARED KEYS BETWEEN pSensorNode AND pTargetNode TO TARGET pKeysMask
            for (i = 0; i < pSensorNode->keyRingMaskSize; i++) {
                pKeysMask[i] |= pSensorNode->keyRingMask[i] & pTargetNode->keyRingMask[i];
            }
        }

        pNode = pNode->next;
    }

    return status;    
}

int CSensorEnv::PEGA_ExecuteOverPartiallySecureLinks(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    SENSOR_NODE*    pSensorA;
    SENSOR_NODE*    pSensorB;
    SENSOR_NODE*    pSensorC;
    
    
    // IMPLEMENTED ONLY FOR FLAG_KEYRING_IDSET
    if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
        
        //
        // GO OVER ALL PAIRS OF NEIGHBOURS (CENTRAL A AND NEIGHBOUR B) AND EXECUTE PEGA PROTOCOL. ONLY SUCH NEIGHBOURS, THAT HAVE ALREADY SECURE LINK 
        // WITH CENTRAL NODE ARE INCLUDED INTO GROUP, THAT PERFORMS SHARED KEY DISCOVERY.
        // LINK BETWEEN A AND B IS SECURE, IF AT LEAST ONE OTHER NEIGHBOUR C CONNECTABLE FROM A OVER ALREADY SECURE LINK SHARE EG KEY WITH B 

        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
            
            NEIGH_LIST_NODE*    pListNode = pSensorA->whiteNeigh; 
            while (pListNode != NULL) {
                pSensorB = (SENSOR_NODE*) pListNode->pItem;

                if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                    if ((pListNode->keyFlags & FLAG_KEY_SECURE) || (pListNode->keyFlags & FLAG_KEY_NEWLY_SECURE)) {
                        // LINK ALREADY SECURE, NO NEED FOR PEGA 
                    }
                    else {
                        // LINK IS NOT SECURE, TRY PEGA
                        KEY_ID*         sharedKeys = NULL;       
                        DWORD           sharedKeysSize = 0;
                        sharedKeysSize = pBasicSettings->egScheme.keyRingSize;
                        sharedKeys = new KEY_ID[sharedKeysSize];
                        memset(sharedKeys, 0, sharedKeysSize * sizeof(KEY_ID));
                        
                        FOE_MESSAGE_LIST_NODE   nodesGroup;
                        nodesGroup.pTargetNode = pSensorB;
                        nodesGroup.pSenderNode = pSensorA;
                        nodesGroup.insertNode(pSensorA);
                        // GO OVER ALL A' NEIGHBOURS
                        NEIGH_LIST_NODE*    pListNode2 = pSensorA->whiteNeigh; 
                        while (pListNode2 != NULL) {
                            pSensorC = (SENSOR_NODE*) pListNode2->pItem;
                            if (pSensorC != pSensorB) {
                                if ((pListNode2->keyFlags & FLAG_KEY_SECURE) || (pListNode2->keyFlags & FLAG_KEY_NEWLY_SECURE)) {
                                    // THIS NEIGHBOUR C HAS SECURE LINK TO A AND CAN PARTICIPATE IN PEGA
                                    nodesGroup.insertNode(pSensorC);
                                }
                            }
                            pListNode2 = pListNode2->next;
                        }

                        if ((status = FOE_ComputeSharedKeysSet(nodesGroup.pNodesOnPath, pSensorB, sharedKeys, &sharedKeysSize)) == STAT_OK) {
                            if (sharedKeysSize >= pBasicSettings->foeConfig.minKeysCount) {
                                // IF PEGA DISCOVER AT LEAST REQUIRED NUMBER OF SHARED KEYS, THAN MARK LINK NEWLY SECURE
                                pSensorA->markKeySecure(pSensorB, TRUE);
                            }           
                        }
                        else {
                            ASSERT(FALSE);
                            break;
                        }
                              
                        if (sharedKeys != NULL) delete[] sharedKeys;
                        nodesGroup.release();
                    }
                }
            
                pListNode = pListNode->next;
            }
        }
    }
    else {
        ASSERT(FALSE);
        status = STAT_KEY_TYPE_BAD;
    }    
    
    
    return status;
}

int CSensorEnv::NLeash_Test(BASIC_INIT_DATA* pBasicSettings, BOOL verbose, CString* pResultString) {
    int                     status = STAT_OK;
    int                     comprNodes;
    FOE_MESSAGE_LIST_NODE*  pMessageProfile = NULL;
    DWORD                   pathsFound;
    DWORD                   pathsNotFound;
    CString                 message;
    CString                 value;
    NLEASH_TEST_RESULTS     nLeashResults;
    float                   avgComprRatio = 0;

    // CREATE ROUTING PATH OF MESSAGE FROM EVERY NODE TO BASE STATION
    if (status == STAT_OK) {

        // USE ONLY ONE BS PER NODE
        pBasicSettings->bsConfig.bsActive = 1;

        // PREPARE NODES ON PATH FOR EACH POSSIBLE MESSAGE FROM EACH NODE TO BS
        if (status == STAT_OK) {
            pMessageProfile = new FOE_MESSAGE_LIST_NODE;
            pBasicSettings->foeConfig.useLocalBoost = FALSE;
            pBasicSettings->foeConfig.egAuth = FALSE;
            status = pSensorSim->m_sensorEnv.FOE_PrepareMessagesPath(pBasicSettings, pMessageProfile, &pathsFound, &pathsNotFound); 
        }

        message.Format("NLEASH -- foundPaths: %d | notFoundPaths: %d", pathsFound, pathsNotFound);
        if (verbose) LOG_INSERTSTRING(message);
        if (pResultString) {
            value.Format("%d %d ", pathsFound, pathsNotFound);
            *pResultString += value;
        }
        
        
        comprNodes = (int) (pBasicSettings->whiteNum * pBasicSettings->neighLeash.fractComprNodes);
        if (pResultString) {
            value.Format("%d %f %f ", comprNodes, pBasicSettings->neighLeash.dropMessageProb, pBasicSettings->neighLeash.neighTransmitCheckProb);
            *pResultString += value;
        }
        
    }
        
    // FOR EACH HOP CHECK IF MESSAGE IS ROUTED CORRECTLY
    if (status == STAT_OK) {
        if (status == STAT_OK) {
            if ((status = pSensorSim->m_sensorEnv.NLeash_MaliciousRouteTest(pBasicSettings, pMessageProfile, comprNodes, &nLeashResults)) == STAT_OK) {
                message.Format("NLEASH -- avgHopsToTarget: %0.1f | maxHopsToTarget: %d | avgNeighBeforeNextRouting: %0.1f", nLeashResults.avgHopsToTarget, nLeashResults.maxHopsToTarget, nLeashResults.avgNeighBeforeNextRouting);
                if (verbose) LOG_INSERTSTRING(message);
                if (pResultString) {
                    value.Format("%0.1f %d ", nLeashResults.avgHopsToTarget, nLeashResults.maxHopsToTarget);
                    *pResultString += value;
                }

                message.Format("NLEASH -- comprRatio: %0.3f | dropDetected %d | dropNotDetected %d ", nLeashResults.droppMsgNotDetected / (float) (nLeashResults.droppMsgNotDetected + nLeashResults.droppMsgDetected), nLeashResults.droppMsgDetected, nLeashResults.droppMsgNotDetected);
                if (verbose) LOG_INSERTSTRING(message);
                message.Format("NLEASH -- avgNeighDetectingDrop %.2f | compr. nodes: %d", nLeashResults.avgNeighDetecting, comprNodes);
                if (verbose) LOG_INSERTSTRING(message);
                if (pResultString) {
                   if (nLeashResults.droppMsgNotDetected + nLeashResults.droppMsgDetected == 0) {
                       value = "0 "; 
                    }
                    else {
                       value.Format("%f ", nLeashResults.droppMsgNotDetected / (float) (nLeashResults.droppMsgNotDetected + nLeashResults.droppMsgDetected));
                    }
                    *pResultString += value;

                    value.Format("%d %d ", nLeashResults.droppMsgDetected, nLeashResults.droppMsgNotDetected);
                    *pResultString += value;
                }

            }
        }
    }

    // CLEAR PROFILES OF MESSAGES
    while (pMessageProfile != NULL) {
        if (pMessageProfile->prev) {
            pMessageProfile->prev->release();
            delete pMessageProfile->prev;
        }
        pMessageProfile = pMessageProfile->next;
    }
    
    return status;
}

int CSensorEnv::NLeash_MaliciousRouteTest(BASIC_INIT_DATA* pBasicSettings, FOE_MESSAGE_LIST_NODE* pMessageList, DWORD numComprNodes, NLEASH_TEST_RESULTS* pResults) {
    int                     status = STAT_OK;
    int                     i;
    LIST_NODE*              pListNode = NULL;
    NEIGH_LIST_NODE*        pNeighListNode = NULL;
    SENSOR_NODE*            pPrevSensorNode = NULL;
    SENSOR_NODE*            pActSensorNode = NULL;
    SENSOR_NODE*            pNextSensorNode = NULL;
    SENSOR_NODE*            pSensorNode = NULL;
    SENSOR_NODE**           pSensorsList = NULL;
    FOE_MESSAGE_LIST_NODE*  pMessage;   
    BOOL                    bMessageValid = TRUE;
    BOOL                    bDropDetected = TRUE;
    DWORD                   numMsg = 0;
    DWORD                   hopsTargetCount = 0;
    DWORD                   neighBeforeNextCount = 0;
    DWORD                   neighDetectingCount = 0;
    float                   floatRand;
    float                   prevNodeDist;
    float                   nextNodeDist;
    float                   checkProb = 0;
/*
    BOOL                    messageCompromissed = FALSE;
    BOOL                    keyCompromised = FALSE;
    DWORD                   hopsCaptureCount = 0;
    DWORD                   hopsBSCount = 0;
    KEYS_MASK*              pComprKeysMask = NULL;   
    DWORD                   comprKeysMaskSize = 0;
    DWORD                   comprKeysCount;
    KEY_ID*                 pComprKeysList = NULL;
    int                     j;
    BOOL                    processMessage = FALSE;
    BYTE*                   pComprPoolMask = NULL;    
/**/

    pResults->clear();

    //
    // SELECT RANDOMLY COMPROMISSED NODES UP TO comprNodes 
    //
    if (status == STAT_OK) {
        pSensorsList = new SENSOR_NODE*[numComprNodes];
        status = GetRandomNodes(pSensorsList, numComprNodes);
        
        // MARK THEM AS MALICIOUS
        for (i = 0; i < numComprNodes; i++) {
            if (pSensorsList[i] != NULL) {
                pSensorNode = (SENSOR_NODE*) pSensorsList[i];
                pSensorNode->flags |= FLAG_SENSOR_NLEASH_COMPROMISED;    
            }
        }
    }        
        
    pResults->maxHopsToTarget = 0;
    pResults->avgHopsToTarget = 0;
    pResults->avgNeighDetecting = 0;
         
    pMessage = pMessageList->next;
    while (pMessage) {
        bMessageValid = TRUE;
        numMsg++;
        
        // PROCESS ALL NODES ON PATH AND CHECK, IF MESSAGE IS ROUTED CORRETLY
        pListNode = pMessage->pNodesOnPath;
        pPrevSensorNode = NULL;
        pNextSensorNode = NULL;
        pActSensorNode = (pListNode->pItem != NULL) ? (SENSOR_NODE*) pListNode->pItem : NULL;;
        // START CHECK FOR SECOND NODE ON PATH
        if (pListNode != NULL) pListNode = pListNode->next;
                
        while (pListNode && bMessageValid) {
            pPrevSensorNode = pActSensorNode;
            pActSensorNode = (pListNode->pItem != NULL) ? (SENSOR_NODE*) pListNode->pItem : NULL;
            pNextSensorNode = (pListNode->next != NULL) ? (SENSOR_NODE*) (pListNode->next)->pItem : NULL;
            
            if (pActSensorNode->flags & FLAG_SENSOR_NLEASH_COMPROMISED) {
                // COMPROMISED NODE IS ROUTING, CHECK FOR MALICIOUS BEHAVIOUR
                // DETERMINE MALICIOUS BEHAVIOUR BY PROBABILITY

                //
                // DROP MESSAGE
                //
                m_rndGen.GetRandomFromInterval(1, &floatRand);
                if (floatRand <= pBasicSettings->neighLeash.dropMessageProb) {
                    // MALICIOUS NODE TRY TO DROP MESSAGE
                    bDropDetected = FALSE;
                    
                    // IF PREV NODE IS ALSO COMPROMISED, THAN DROP IS AUTOMATICALLY SUCCESSFUL
                    if (pPrevSensorNode != NULL && pPrevSensorNode->flags & FLAG_SENSOR_NLEASH_COMPROMISED) {
                        bDropDetected = FALSE;
                    }
                    else {
                    
                        //
                        // ONLY SENDER pPrevSensorNode IS CHECKING, WILL NOTICE IF IS CLOSER TO pActSensorNode THAN pNextSensorNode
                        // (pActSensorNode CANNOT SAY THAT HE ROUTED MESSAGE CORRECTLY AND pPrevSensorNode JUST DON'T HEARED)
                        if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_SENDERONLY) {
                            if (pActSensorNode->whiteNeigh != NULL) {
                                // GET DISTANCE FROM pActSensorNode TO pPrevSensorNode AND pNextSensorNode
                                pNeighListNode = pActSensorNode->whiteNeigh;
                                prevNodeDist = nextNodeDist = 0;
                                while (pNeighListNode) {
                                    pSensorNode = (SENSOR_NODE*) pNeighListNode->pItem;
                                    if (pSensorNode->id == pPrevSensorNode->id) {
                                        prevNodeDist = pNeighListNode->dist;
                                    }
                                    if (pSensorNode->id == pNextSensorNode->id) {
                                        nextNodeDist = pNeighListNode->dist;
                                    }
                                    
                                    // COUNT ALL NODES BEFORE NEXT ROUTING NODE
                                    if (nextNodeDist != NULL) {
                                        pResults->avgNeighBeforeNextRouting++;
                                    }
                                    
                                    // BREAK WHEN BOTH NODES FOUND
                                    if (prevNodeDist != 0 && nextNodeDist != 0) break;
                                    
                                    pNeighListNode = pNeighListNode->next;
                                }
                                
                                neighBeforeNextCount++;
                                neighDetectingCount++;
                                
                                if (prevNodeDist != 0 && nextNodeDist != 0) {
                                    // CHECK DROP DETECTION
                                    if (prevNodeDist <= nextNodeDist) {
                                        // DROP DETECTED   
                                        bDropDetected = TRUE;
                                        
                                        // SENDER ALWAYS CHECK DROP  
                                        pResults->avgNeighDetecting++;
                                    } 
                                    else {
                                        // DROP NOT DETECTED
                                    }
                                }
                                else {
                                    // ERROR OCCURED, PREVIOUS AND NEXT NODE MUST BE NEIGHBOUR OF pActSensorNode
                                    _ASSERT(FALSE);
                                    status = STAT_INVALID_PTR;
                                }
                            }
                            else {
                                _ASSERT(FALSE);
                                // NO NEIGHBOURS, SHOULD NOT OCCURE
                            }
                        }

                        //
                        // ALL NEIGHBOURS OF pActSensorNode ARE CHECKING, THEY WILL NOTICE IF AT LEAST ONE IS CLOSER TO pActSensorNode
                        if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_ALLNEIGHS || pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_NEIGHPROB) {
                            if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_ALLNEIGHS) {
                                // ALL NEIGHBOURS ARE CHECKING IS EQUAL TO CHECK WITH PROBABILITY 1
                                checkProb = 1;
                            }
                            
                            if (pBasicSettings->neighLeash.transmitCheckFlags & FLAG_TRANSMITCHECK_NEIGHPROB) {
                                checkProb = pBasicSettings->neighLeash.neighTransmitCheckProb;
                            }


                            if (pActSensorNode->whiteNeigh != NULL) {
                                
                                // GET DISTANCE FROM pActSensorNode TO pNextSensorNode
                                pNeighListNode = pActSensorNode->whiteNeigh;
                                while (pNeighListNode) {
                                    
                                    pSensorNode = (SENSOR_NODE*) pNeighListNode->pItem;
                                    if (pSensorNode->id == pNextSensorNode->id) {
                                        // NEXT ROUTING NODE REACHED
                                        break;
                                    }
                                    else {
                                        pResults->avgNeighBeforeNextRouting++;
                                    
                                        // NODE WITH DISTANCE BEFORE NEXT ROUTING NODE 
                                        // IF NON-COMPROMISED, THAN DROP IS DETECTED
                                        m_rndGen.GetRandomFromInterval(1, &floatRand);
                                        if (floatRand <= checkProb) { 
                                            if (!(pSensorNode->flags & FLAG_SENSOR_NLEASH_COMPROMISED)) {
                                                bDropDetected = TRUE;    
                                                pResults->avgNeighDetecting++;
                                                // break;
                                            }
                                            else {
                                                // THIS NODE IS COMPROMISED AND NOT TELLING OTHERS ABOUT DROP
                                            }
                                        }
                                    }
                                    
                                    pNeighListNode = pNeighListNode->next;
                                }
                                
                                neighBeforeNextCount++;
                            }
                            else {
                                _ASSERT(FALSE);
                                // NO NEIGHBOURS, SHOULD NOT OCCURE
                                status = STAT_INVALID_PTR;
                            }
                        }
                    }
                    
                    if (status == STAT_OK) {
                        // CHECK DROP DETECTION
                        if (bDropDetected) {
                            // DROP DETECTED    
                            pResults->droppMsgDetected++;
                        } 
                        else {
                            // DROP NOT DETECTED
                            pResults->droppMsgNotDetected++;
                        }
                    }
                    else break;
                    
                    // MESSAGE IS INVALID NOW. EITHER DROPPED OR DETECTED DROPPING
                    bMessageValid = FALSE;
                } 
            }
            else {
                // NON-COMPROMISED NODE IS ROUTING
            }
            
            pListNode = pListNode->next;
        }
        
        pResults->avgHopsToTarget += pMessage->numberOfHopsToBS;
        if (pMessage->numberOfHopsToBS > pResults->maxHopsToTarget) pResults->maxHopsToTarget = pMessage->numberOfHopsToBS;
        
        hopsTargetCount++;
    
        pMessage = pMessage->next;
    }
    
    if (hopsTargetCount > 0) pResults->avgHopsToTarget /= hopsTargetCount;
    else pResults->avgHopsToTarget = 0;
    if (neighBeforeNextCount > 0) pResults->avgNeighBeforeNextRouting /= neighBeforeNextCount;
    else pResults->avgNeighBeforeNextRouting = 0;
    if (neighDetectingCount > 0) pResults->avgNeighDetecting /= neighDetectingCount;
    else pResults->avgNeighDetecting = 0;
    
    if (pSensorsList) delete[] pSensorsList;

    return status;
}

int CSensorEnv::DistributeKeyStatusRandomly(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD distribFlags, BOOL bForceNewKeyDistribution) {
    int         status = STAT_OK;
    DWORD       i;
    DWORD       usedNeigh;
    DWORD       totalKeys;
    int         remainCompr;
    DWORD       maxUsedNeigh;
    DWORD       random;
    SENSOR_NODE* pSensorA;
    SENSOR_NODE* pSensorB;
    NEIGH_LIST_NODE*  pListNode;   
    BOOL        bNewSecureKey = FALSE;
    
    *pSecured = *pCompr = 0;

#ifdef _DEBUG
	if (bForceNewKeyDistribution) { 
		pSensorSim->Log_InsertString("INFO: DistributeKeyStatusRandomly() changed link's key status");    
		int a = 0;
	}
	else {
		int a = 0;
	}
#endif

    //distribFlags |= FLAG_RANDOMDISTRIB_BIDIRECTIONAL;

    //
    // COUNT TOTAL NUMBER OF KEYS WHICH CAN BE MARKET AS SECURE OR COMPROMISED 
    //
    
    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    maxUsedNeigh = ULONG_MAX;
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }

    totalKeys = 0;
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                totalKeys++;
                usedNeigh++;
            }

            pListNode = pListNode->next;
        }
    }
    

    //
    // MARK RANDOMLY SELECTED KEYS AS SECURE OR COMPRIOMISED BASED ON GIVEN PERCENTAGE pBasicSettings->distribKeysSecured ...
    //
    remainCompr = (int) (((float) totalKeys / 100) * pBasicSettings->distribKeysCompr);

    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;

        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

			if (pSensorB->flags & FLAG_SENSOR_WHITE) {
				if (!bForceNewKeyDistribution) { 
					// USE PRE-EXISTING ORIGINAL KEY STATUS
					pListNode->keyFlags = pListNode->keyFlags_orig;
					if (pListNode->keyFlags & FLAG_KEY_SECURE) {
						(*pSecured)++;
					}
					if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
						remainCompr--;
						(*pCompr)++;
					}
					totalKeys--;
			    }
				else {
					// FORCE NEW DISTRIBUTION
					if (!this->HasKeyDistribStatus(pListNode->keyFlags)) {
						// DO ONLY IF KEY STATUS IS NOT DETERMINED YET

						// RANDOMLY DECIDE, IF THIS KEY WILL BE MARKED AS SECURE OR COMPROMITED 
						this->m_rndGen.GetRandomFromInterval(totalKeys, &random);
						if ((int) random < remainCompr) {
							// MARK KEY COMPROMISED
							pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
							pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
							pListNode->keyFlags_orig = pListNode->keyFlags;

							remainCompr--;
							totalKeys--;
							(*pCompr)++;

							if (distribFlags & FLAG_RANDOMDISTRIB_BIDIRECTIONAL) {
								// IF BIDIRECTINAL KEY STATUS, THEN MARK ALSO LINK KEY 
								// FROM OPPOSITE DIRECTION AS COMPROMISSED
								//ASSERT(pSensorB->markKeyCompromised(pSensorA) != STAT_NO_SUCH_NEIGHBOUR);
								if (pSensorB->markKeyCompromised(pSensorA, TRUE) == STAT_NO_SUCH_NEIGHBOUR) {
									pSensorSim->Log_InsertString("markKeyCompromised problem");    
								}
                            
								remainCompr--;
								totalKeys--;
								(*pCompr)++;
							}
						}
						else {
							// MARK KEY SECURE
							pListNode->keyFlags |= FLAG_KEY_SECURE;
							pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
							pListNode->keyFlags_orig = pListNode->keyFlags;

							totalKeys--;
							(*pSecured)++;
                
							if (distribFlags & FLAG_RANDOMDISTRIB_BIDIRECTIONAL) {
								// IF BIDIRECTINAL KEY STATUS, THEN MARK ALSO LINK KEY 
								// FROM OPPOSITE DIRECTION AS SECURED
								//ASSERT(pSensorB->markKeySecure(pSensorA, FALSE) != STAT_NO_SUCH_NEIGHBOUR);
								if (pSensorB->markKeySecure(pSensorA, FALSE, TRUE) == STAT_NO_SUCH_NEIGHBOUR) {
									pSensorSim->Log_InsertString("markKeySecure problem");    
								}
								totalKeys--;
								(*pSecured)++;
							}
						}

						usedNeigh++;
					}
				}
			}
            pListNode = pListNode->next;
        }
    }



    return status;
}

int CSensorEnv::DistributeKeyStatusByComprList(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BYTE* pComprKeysStatusList, BOOL bMarkMissingAsCompr) {
//int CSensorEnv::DistributeKeyStatusByComprList(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, KEY_ID* pComprKeysList, DWORD comprKeysListSize) {
    int     status = STAT_OK;
    int     i = 0;

    //
    //  SET LINK KEY STATUS FOR NEIGBOURS  
    //

    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    DWORD maxUsedNeigh = ULONG_MAX;
    DWORD usedNeigh = 0;  
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }
    
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        SENSOR_NODE* pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        NEIGH_LIST_NODE* pListNode = pSensorA->whiteNeigh;

        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
            SENSOR_NODE* pSensorB = (SENSOR_NODE*) pListNode->pItem;

            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
                    ASSERT(FALSE);
                    // TODO: add support for bit mask style of key distribution
                }
                else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                    // INSPECT ALL KEYS USED TO SECURE THE LINK - IF ALL COMPROMISED, THAN MARK LINK COMPROMISED
                    BOOL    bAtLeastOneSecure = FALSE;
                    DWORD   numSharedKeys = 0;
                    for (int j = 0; j < MAX_SHARED_KEYS; j++) {
                        if (pListNode->EG_sharedKeys[j].id != 0) {
                            // INCREASE SHARED KEYS COUNTER
                            numSharedKeys++;
                            if (pComprKeysStatusList[pListNode->EG_sharedKeys[j].id] == 0) {
                                // KEY IS NOT COMPROMISED
                                bAtLeastOneSecure = TRUE;
                            }
                            else {
                                // KEY IS COMPROMISED
                            }
                        }
                        else {
                            // IF FIRST ZERO ENCOUNTERED THAN NO OTHER KEYS ARE SHARED 
                            break;
                        }
                    }

                    // SET LINK KEY STATUS BASED ON SHARED PRE-DISTRIB KEY INTERSECTION
                    if (numSharedKeys >= pBasicSettings->egScheme.qIntersect) {
                        if (bAtLeastOneSecure) {
                            pListNode->keyFlags |= FLAG_KEY_SECURE;
                            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

                            (*pSecured)++;
                        }
                        else {
                            pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
                            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
                            (*pCompr)++;
                        }
                    }
                    else {
                        // REQUIED NUMBER OF SHARED KEYS NOT FOUND 
                        if (bMarkMissingAsCompr) {
                            pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
                            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
                            (*pCompr)++;
                        }
                        else {
                            pListNode->keyFlags |= FLAG_KEY_MISSING;
                            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                        }
                    }
                }
    
                usedNeigh++;
            }

            pListNode = pListNode->next;
        }
    }
    
    return status;
} 

int CSensorEnv::GetSharedKeysStatus(NEIGH_LIST_NODE* pListNode, DWORD* pNumSharedKeys, DWORD* pNumSecuredKeys, BYTE* pComprKeysStatusList, KEY_2DID* pComprKeysStatusList2D)
{

	(*pNumSecuredKeys)=0;
	(*pNumSharedKeys)=0;

	for (int j = 0; j < MAX_SHARED_KEYS; j++) {
            if (pListNode->EG_sharedKeys[j].id != 0) {
                // INCREASE SHARED KEYS COUNTER
                (*pNumSharedKeys)++;
                if (pComprKeysStatusList[pListNode->EG_sharedKeys[j].id] == 0) {
                    // KEY IS NOT COMPROMISED
                    (*pNumSecuredKeys)++;
                }
                else {
					if (pComprKeysStatusList2D!=NULL)
					{
						//IF ATTACKER COMPROMISED KEY IN THE HASH CHAIN THAT IS HIGHER THAN SHARED KEY IS SECURED
						if (pComprKeysStatusList2D[pListNode->EG_sharedKeys[j].id]>pListNode->EG_sharedKeys[j].id2D)
							(*pNumSecuredKeys)++;
					}
                    // KEY IS COMPROMISED IF NOT SECURED
                }
            }
            else {
                // IF FIRST ZERO ENCOUNTERED THAN NO OTHER KEYS ARE SHARED 
                break;
            }
        }

	return STAT_OK;
}

int CSensorEnv::GetSharedIntermKeysStatus(NEIGH_LIST_NODE* pListNode, DWORD* pNumSharedIntermKeys, DWORD* pNumSecuredIntermKeys, BYTE* pComprKeysStatusList)
{

	(*pNumSecuredIntermKeys)=0;
	(*pNumSharedIntermKeys)=0;

	//GO THROUGH INTERMEDIATE KEYS
	for (int j = 0; j < MAX_SHARED_KEYS; j++) {
        if (pListNode->EG_sharedIntermKeys[j] != 0) {
            // INCREASE INTERMEDIATE SHARED KEYS COUNTER
            (*pNumSharedIntermKeys)++;
            if (pComprKeysStatusList[pListNode->EG_sharedIntermKeys[j]] == 0 &&
				pComprKeysStatusList[(pListNode->EG_sharedIntermKeys[j]) + 1] == 0) {
                // KEY IS NOT COMPROMISED
                (*pNumSecuredIntermKeys)++;
            }
            else {
                // KEY IS COMPROMISED
            }
        }
        else {
            // IF FIRST ZERO ENCOUNTERED THAN NO OTHER KEYS ARE SHARED 
            break;
        }
    }
	return STAT_OK;
}

int CSensorEnv::SetLinkKeyStatus(BASIC_INIT_DATA* pBasicSettings, NEIGH_LIST_NODE* pListNode, DWORD numSharedKeys, DWORD numSecuredKeys, BOOL bMarkMissingAsCompr)
{
	// SET LINK KEY STATUS BASED ON SHARED PRE-DISTRIB KEY INTERSECTION
    if (numSharedKeys >= pBasicSettings->egScheme.qIntersect) {
        if (numSecuredKeys>0) {
            pListNode->keyFlags |= FLAG_KEY_SECURE;
            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

            //(*pSecured)++;
        }
        else {
            pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
            //(*pCompr)++;
        }
    }
    else {
        // REQUIED NUMBER OF SHARED KEYS NOT FOUND 
        if (bMarkMissingAsCompr) {
            pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
            //(*pCompr)++;
        }
        else {
            pListNode->keyFlags |= FLAG_KEY_MISSING;
            pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
        }
    }
	return STAT_OK;
}
int CSensorEnv::SetLinkKeyStatusIntermediate(BASIC_INIT_DATA* pBasicSettings, NEIGH_LIST_NODE* pListNode, DWORD numSharedKeys,  DWORD numSharedIntermKeys,DWORD numSecuredKeys, BOOL bMarkMissingAsCompr)
{
	
	
	if (pBasicSettings->egScheme.normalKeys == 0 && pBasicSettings->egScheme.intermKeys == 0)
	{
		// USE ONLY SUM OF KEYS
		if (numSharedKeys+numSharedIntermKeys >= pBasicSettings->egScheme.qIntersect)
		{
			if (numSecuredKeys>0) {
				pListNode->keyFlags |= FLAG_KEY_SECURE;
				pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

				//(*pSecured)++;
			}
			else {
				pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
				//(*pCompr)++;
			}
		}
		else {
			// REQUIED NUMBER OF SHARED KEYS NOT FOUND 
			if (bMarkMissingAsCompr) {
				pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
				//(*pCompr)++;
			}
			else {
				pListNode->keyFlags |= FLAG_KEY_MISSING;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
				pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
			}
		}
	}
	// USE AT LEAST X NORMAL KEYS AND Y INTERMEDIATE TO SECURE LINK
	else 
	{
		if ((numSharedKeys>= pBasicSettings->egScheme.normalKeys)
			&&
			(numSharedIntermKeys >=pBasicSettings->egScheme.intermKeys)) 
		{
			if (numSecuredKeys>0) {
				pListNode->keyFlags |= FLAG_KEY_SECURE;
				pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 

				//(*pSecured)++;
			}
			else {
				pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
				//(*pCompr)++;
			}
		}
		else {
			// REQUIED NUMBER OF SHARED KEYS NOT FOUND 
			if (bMarkMissingAsCompr) {
				pListNode->keyFlags |= FLAG_KEY_COMPROMISED;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
                            
				//(*pCompr)++;
			}
			else {
				pListNode->keyFlags |= FLAG_KEY_MISSING;
				pListNode->keyFlags &= ~FLAG_KEY_SECURE; 
				pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
			}
		}
	}

	return STAT_OK;
}

int CSensorEnv::DistributeKeyStatusByComprList2D(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, BYTE* pComprKeysStatusList, KEY_2DID* pComprKeysStatusList2D, SENSOR_NODE** pComprNodeList, DWORD numCaptured, BOOL bMarkMissingAsCompr) {
//int CSensorEnv::DistributeKeyStatusByComprList(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, KEY_ID* pComprKeysList, DWORD comprKeysListSize) {
    int     status = STAT_OK;
    int     i = 0;

    //
    //  SET LINK KEY STATUS FOR NEIGBOURS  
    //

    // IF RESTRICTION TO ONLY pBasicSettings->useOnlyXNeigh IS ENABLED, USE THAT
    // OTHERWISE, USE ULONG_MAX WHICH CANNOT BE REACHED (SO ALL POSSIBLE NEIGHBOURS WILL BE USED
    DWORD maxUsedNeigh = ULONG_MAX;
    DWORD usedNeigh = 0;  
    if ((pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) && (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION)) {
        maxUsedNeigh = pBasicSettings->useOnlyXNeigh;   
    }
    
	// FOR EVERY NODE DO FOLLOWING
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
		//GET PARTICULAR NODE A
        SENSOR_NODE* pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
        NEIGH_LIST_NODE* pListNode = pSensorA->whiteNeigh;

		// FOR EVERY NEIGHBOUR OF THE GIVEN NODE A
        usedNeigh = 0;
        while ((pListNode != NULL) && (usedNeigh < maxUsedNeigh)) {
			//GET NEIGHBOUR (B)
            SENSOR_NODE* pSensorB = (SENSOR_NODE*) pListNode->pItem;

			//ONLY FOR WHITE NODES
            if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
                    ASSERT(FALSE);
                    // TODO: add support for bit mask style of key distribution
                }
                else if (pBasicSettings->egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                    // INSPECT ALL KEYS USED TO SECURE THE LINK - IF ALL COMPROMISED, THAN MARK LINK COMPROMISED
                    BOOL    bAtLeastOneSecure = FALSE;
                    DWORD   numSharedKeys = 0;
					DWORD	numSecuredKeys = 0;
					DWORD	numSecuredIntermKeys=0;
					DWORD   numSharedIntermKeys = 0;
                    

					//IF pComprKeysStatusList2D == NULL, THEN HASH CHAIN IS NOT IN USE
					pSensorSim->m_sensorEnv.GetSharedKeysStatus(pListNode,&numSharedKeys,&numSecuredKeys,pComprKeysStatusList, pComprKeysStatusList2D);

					if (pBasicSettings->egScheme.useIntermKeys)
					{
						pSensorSim->m_sensorEnv.GetSharedIntermKeysStatus(pListNode,&numSharedIntermKeys,&numSecuredIntermKeys,pComprKeysStatusList);

						numSecuredKeys += numSecuredIntermKeys;
					}
					

                    // SET LINK KEY STATUS BASED ON SHARED PRE-DISTRIB KEY INTERSECTION
					if (pBasicSettings->egScheme.useIntermKeys)
					{
						pSensorSim->m_sensorEnv.SetLinkKeyStatusIntermediate(pBasicSettings, pListNode,numSharedKeys,numSharedIntermKeys,numSecuredKeys, bMarkMissingAsCompr);
					}
					else
					{
						pSensorSim->m_sensorEnv.SetLinkKeyStatus(pBasicSettings, pListNode,numSharedKeys,numSecuredKeys, bMarkMissingAsCompr);
					}
					// BASED ON STATUS MODIFY COUNTER 
					if (pBasicSettings->egScheme.countComprLinks) {
						if (pListNode->keyFlags & FLAG_KEY_COMPROMISED)	{
							(*pCompr)++;
						}
						if (pListNode->keyFlags & FLAG_KEY_SECURE) {
							(*pSecured)++;
						}
					}
					else {
						//IF NEITHER OF THE NODES IS COMPROMISED, CALCULATE STATS
						if ((!(CSensorEnv::IsNodeInList(pSensorA,pComprNodeList,numCaptured))) &&
							(!(CSensorEnv::IsNodeInList(pSensorB,pComprNodeList,numCaptured))))
						{
							if (pListNode->keyFlags & FLAG_KEY_COMPROMISED)	{
								(*pCompr)++;
							}
							if (pListNode->keyFlags & FLAG_KEY_SECURE) {
								(*pSecured)++;
							}
						}
					}
					
				}
    
                usedNeigh++;
            }

            pListNode = pListNode->next;
        }
    }
    
    return status;
} 
BOOL CSensorEnv::IsNodeInList(SENSOR_NODE* pSensorNode, SENSOR_NODE** pList,DWORD listLength)
{
	
	for (int i=0;i<listLength;i++)
	{
		if (pSensorNode->id == pList[i]->id)
			return TRUE;
	}
	return FALSE;
}



int CSensorEnv::ComputeNumberOfComprKeysByBlack() {
	int					status = STAT_OK;
	DWORD    			i;
    NEIGH_LIST_NODE*    pListNode;
    LIST_NODE*          pComprListNode;
    SENSOR_NODE*        pSensor;
    SENSOR_NODE*        pComprSensor;


    // CLEAR PREVIOUS STATISTICS FOR BLACK NODES
    for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
        m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter = 0;
    }
    
    // CHECK ALL LINK KEYS AND INCREASE COUNTER FOR THAT BLACK NODE, WHICH HAS COMPROMISSED GIVEN LINK KEY 
    for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensor->whiteNeigh;
        while(pListNode != NULL) {
            if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                // GO THROW ALL COMPROMISSING NODES
                pComprListNode = pListNode->compromisedBy;
                while (pComprListNode != NULL) { 
                    pComprSensor = (SENSOR_NODE*) pComprListNode->pItem;
                    
                    // INCREASE COUNTER
                    pComprSensor->comprKeysCounter++; 

                    pComprListNode = pComprListNode->next;
                }
            }

            pListNode = pListNode->next;
        }
    }

	return status;
}

int CSensorEnv::SetMostComprAsInactiveNeigh(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    DWORD   i;

    // COMPUTE NUMBER OF COMPROMISSED KEYS FOR EACH BLACK NODE
    status = ComputeNumberOfComprKeysByBlack();

    if (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACKWEIGHTED) {
        // CALCULATE TOTAL COUNT OF COMPROMISSED KEYS
        DWORD totalCompr = 0;
        for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
            totalCompr += m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter;
        }

        // REMOVE MOST COMPROMISSING NODES WHILE NUMBER OF COMPPROMISSED KEYS IS ABOVE EXPECTED 
        DWORD actualCompr = totalCompr;
        DWORD limitCompr = (DWORD) (totalCompr * (1 - (pBasicSettings->removeMostComprBlack/100)));
        DWORD maxCompr = 0;
        SENSOR_NODE* pMaxSensor = NULL;
        while (actualCompr >= limitCompr) {
            // SEARCH FOR BLACK NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            maxCompr = 0;
            for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
                if ((m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter > maxCompr) && !(m_pActualDeployment->blackSensorList.sensors[i]->flags & FLAG_SENSOR_BLACK_INACTIVENEIGH)) {
                    maxCompr = m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter;
                    pMaxSensor = m_pActualDeployment->blackSensorList.sensors[i];
                }
            }

            // SET AS INACTIVE 
            pMaxSensor->flags |= FLAG_SENSOR_BLACK_INACTIVENEIGH;
            actualCompr -= pMaxSensor->comprKeysCounter;
        }
    }
    else {
        // REMOVE MOST COMPROMISSING NODES 
        DWORD actualActive = m_pActualDeployment->blackSensorList.numSensors;
        DWORD limitActiv = (DWORD) (m_pActualDeployment->blackSensorList.numSensors * (1 -  (pBasicSettings->removeMostComprBlack/100)));
        DWORD maxCompr = 0;
        SENSOR_NODE* pMaxSensor = NULL;
        while (actualActive >= limitActiv) {
            // SEARCH FOR BLACK NODE WITH HIGHEST COMPROMISSED COUNT OF LINK KEYS
            maxCompr = 0;
            for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
                if ((m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter > maxCompr) && !(m_pActualDeployment->blackSensorList.sensors[i]->flags & FLAG_SENSOR_BLACK_INACTIVENEIGH)) {
                    maxCompr = m_pActualDeployment->blackSensorList.sensors[i]->comprKeysCounter;
                    pMaxSensor = m_pActualDeployment->blackSensorList.sensors[i];
                }
            }

            if (pMaxSensor) {
                // SET AS INACTIVE 
                pMaxSensor->flags |= FLAG_SENSOR_BLACK_INACTIVENEIGH;
                actualActive--;
            }
            else {
                // NO BLACK WITH ANY COMPROMISSED KEYS FOUND
                break;
            }
        }
    }
    
    return status;
}

BOOL CSensorEnv::HasKeyDistribStatus(DWORD keyFlags) {
    if ((keyFlags & FLAG_KEY_SECURE) || (keyFlags & FLAG_KEY_NEWLY_SECURE) || (keyFlags & FLAG_KEY_COMPROMISED)) {
        return TRUE;
    }
    else return FALSE; 
}

int CSensorEnv::InitRandomGenerator(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    
    // INIT RANDOM GENERATOR
    if (pBasicSettings->rndGen.useFixedSeed) {
        // USE GIVEN SEED
    }
    else {
        // GENERATE NEW SEED
		unsigned int seed;
		rand_s(&seed);
        pBasicSettings->rndGen.randomSeed = seed;
		
		CString message;
		message.Format("New deployment seed: '%u'", seed);
		LOG_INSERTSTRING(message);
    }
    // INIT
    this->m_rndGen.InitRandomGenerator(pBasicSettings->rndGen.randomSeed);
    // DISPLAY ASYNCHRONOUSLY USED SEED
    pSensorSim->PostMessage(WM_RNG_UPDATESEED, (WPARAM) pBasicSettings->rndGen.randomSeed);


	// GA RANDOM NUMBER GENERATOR
	unsigned int ga_seed = 0, old_ga_seed = 0;
	old_ga_seed = GAGetRandomSeed();
	// SET FIXED SEED
	if (pBasicSettings->rndGenGA.useFixedSeed && pBasicSettings->rndGenGA.randomSeed != 0)
		ga_seed = pBasicSettings->rndGenGA.randomSeed;
	// OR IF NOT SET BEFORE THEN GENERATE OWN (SHOULD BE BETTER THAN USING time())
	else if (old_ga_seed == 0)
		rand_s( &ga_seed);

	// SET NEW SEED
	GARandomSeed(ga_seed);

	// GET CURRENT SEED
	ga_seed = GAGetRandomSeed();

	// IF SEED WAS CHANGED, LOG IT
	if (ga_seed != old_ga_seed){
		CString message;
		message.Format("Current GA seed: '%u'", ga_seed);
		LOG_INSERTSTRING(message);
	}

    return status;
}

int CSensorEnv::Dir_Create(CString path) {
    int     status = STAT_OK;
    CString helpPath;    

    if (path.GetAt(path.GetLength() - 1) != '\\') path += "\\";

    helpPath = path;

    char* sHelpString = new char[helpPath.GetLength() + 1];
//    strncpy_s(sHelpString, helpPath.GetLength() + 1, (LPCTSTR) helpPath, helpPath.GetLength());
    strncpy(sHelpString, (LPCTSTR) helpPath, helpPath.GetLength());

    int i = 0;
    for (;;)
    {
        for (; (i < path.GetLength()) && (path.GetAt(i) != '\\'); i ++);

        if (i == path.GetLength())
        {
            break;
        }

        sHelpString[i] = 0;

        CreateDirectory(sHelpString, NULL);

        sHelpString[i] = '\\';

        i ++;
    }

    delete[] sHelpString;         
  
    return status;
}

int CSensorEnv::SaveWhitePositions(CString fileName) {
    int             status = STAT_OK;
    DWORD           i;
    CFile           file;
    SENSOR_NODE*    pSensor;
    CString         message;

    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // WHITE SENSORS
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];

            message.Format("%d %f %f %f\r\n", pSensor->id, pSensor->cX, pSensor->cY, pSensor->transmRange * pSensor->receiveRangeRatio);
            file.Write((LPCTSTR) message, message.GetLength());
        }
        file.Close();
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveBlackPositions(CString fileName) {
    int             status = STAT_OK;
    DWORD           i;
    CFile           file;
    SENSOR_NODE*    pSensor;
    CString         message;

    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // BLACK SENSORS
        for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->blackSensorList.sensors[i];

            message.Format("%d %f %f %f\r\n", pSensor->id, pSensor->cX, pSensor->cY, pSensor->transmRange * pSensor->receiveRangeRatio);
            file.Write((LPCTSTR) message, message.GetLength());
        }
        file.Close();
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveSecureKeyDistrib(CString fileName) {
    int             status = STAT_OK;
    DWORD           i;
    CFile           file;
    SENSOR_NODE*    pSensor;
    NEIGH_LIST_NODE*      pListNode;
    CString         message;
    DWORD           secured;
    DWORD           compr;

    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // WHITE SENSORS
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensor->whiteNeigh;

            secured = compr = 0;
            while(pListNode != NULL) {
                if (pListNode->keyFlags & FLAG_KEY_SECURE) secured++;
                if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) compr++;

                pListNode = pListNode->next;
            }

            message.Format("%f %f %d %d %d %d\r\n", pSensor->cX, pSensor->cY, pSensor->numWhiteNeigh, pSensor->numBlackNeigh, secured, compr);
            file.Write((LPCTSTR) message, message.GetLength());
        }
        file.Close();
        
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveCompromissedByBlack(CString fileName) {
    int                 status = STAT_OK;
    DWORD               i;
    CFile               file;
    CString             message;
    USHORT              comprCountRes[MAX_POSSIBLE_COMPROMISSED_KEYS_PER_BLACK];
    DWORD               maxComprCount = 0;
    SENSOR_NODE*        pSensor;

    memset(comprCountRes, 0, sizeof(comprCountRes));

    // PROCESS ALL LINK KEYS, IF COMPROMISSED THAN ADD POINT TO EACH COMPROMISSING BLACK SENSOR
    // THAN, SUMARIZE NUMBER OF BLACK SENSORS WHICH HAVE COMPRIMMISED EXACTLY i KEYS
    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // COMPUTE NUMBER OF COMPROMISSED KEYS FOR EACH BLACK NODE
        status = ComputeNumberOfComprKeysByBlack();

        // SAVE ID, POSITION AND NUMBER OF COMPROMISSED KEYS
        for (i = 0; i < m_pActualDeployment->blackSensorList.numSensors; i++) {
            // STORE MAXIMUM REACHED REACHED OF COMPROMISSED KEYS
            pSensor = m_pActualDeployment->blackSensorList.sensors[i];
            
            message.Format("%d %f %f %d\r\n", pSensor->id, pSensor->cX, pSensor->cY, pSensor->comprKeysCounter);
            file.Write((LPCTSTR) message, message.GetLength());
        }


        file.Close();
        
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveWhiteTriangleProperties(CString fileName) {
    int                 status = STAT_OK;
    DWORD               i;
    CFile               file;
    CString             message;
    SENSOR_NODE*        pSensorA;
    SENSOR_NODE*        pSensorB;
    SENSOR_NODE*        pSensorC;
    SENSOR_NODE*        pSensorT;
    BOOL                bIsNeigh;
    NEIGH_LIST_NODE*      pListNode;
    NEIGH_LIST_NODE*      pListNode2;
    NEIGH_LIST_NODE*      pListNode3;

    NEIGH_LIST_NODE*      pRevListNode;
    NEIGH_LIST_NODE*      pRevListNode2;
    NEIGH_LIST_NODE*      pRevListNode3;

    // PROCESS ALL NEIGHBORS WHICH FORMS TRIANGLE A->B->C->A (SUITABLE FOR AMPLIFICATION)
    // AND SAVE PROPER INFORMATIONS   
    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // CLEAR FLAG_SENSOR_NODE_INACTIVE BEFORE PROCESSING
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];
            pSensorA->flags &= ~FLAG_SENSOR_NODE_INACTIVE; 
        }
                
        // PROCESS ALL WHITE SENSORS
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensorA = m_pActualDeployment->whiteSensorList.sensors[i];

            // PROCESS ALL WHITE NEIGBOURS OF A
            pListNode = pSensorA->whiteNeigh;
            while (pListNode != NULL) {
                pSensorB = (SENSOR_NODE*) pListNode->pItem;

                // PROCESS ONLY WHITE NODES WHICH WAS NOT REMOVED FROM PROCESSING YET 
                if ((pSensorB->flags & FLAG_SENSOR_WHITE) && !(pSensorB->flags & FLAG_SENSOR_NODE_INACTIVE)) {

                    // PROCESS ALL REMAINING NEIGHBOURS OF A AND CHECK, IF ARE ALSO NEIGBOUR OF B  
                    pListNode2 = pListNode->next;
                    while (pListNode2 != NULL) {
                        pSensorC = (SENSOR_NODE*) pListNode2->pItem;

                        if ((pSensorC->flags & FLAG_SENSOR_WHITE) && !(pSensorC->flags & FLAG_SENSOR_NODE_INACTIVE)) {
                            // CHECK IF B & C ARE NEIGHBOURS
                            bIsNeigh = FALSE;
                            pListNode3 = pSensorC->whiteNeigh;
                            while (pListNode3 != NULL) {
                                pSensorT = (SENSOR_NODE*) pListNode3->pItem;
                                
                                if (pSensorT->flags & FLAG_SENSOR_WHITE) {
                                    if (pSensorT->id == pSensorB->id) {
                                        // B IS NEIGBOUR OF C
                                        bIsNeigh = TRUE;
                                        break;
                                    }
                                }

                                pListNode3 = pListNode3->next;
                            } 


                            if (bIsNeigh) {
                                // FIND STRUCTURES FO REVERSE LINK KEY 
                                pRevListNode = pSensorB->getNeighStructKeyFlags(pSensorA);
                                pRevListNode2 = pSensorC->getNeighStructKeyFlags(pSensorA);
                                pRevListNode3 = pSensorB->getNeighStructKeyFlags(pSensorC);
                                
                                // TRIANGLE FOUND
                                // | Ax, Ay, Bx, By, Cx, Cy, dist(A->B), dist(B->C), dist(C->A), cimcumference, Key_A->B, Key_B->A, Key_A->C, Key_C->A, Key_C->B, Key_B->C | 
                                message.Format("%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f %d %d %d %d %d %d\r\n", pSensorA->cX, pSensorA->cY, pSensorB->cX, pSensorB->cY, pSensorC->cX, pSensorC->cY, pListNode->dist, pListNode2->dist, pListNode3->dist, pListNode->dist + pListNode2->dist + pListNode3->dist, pListNode->keyFlags, pRevListNode->keyFlags, pListNode2->keyFlags, pRevListNode2->keyFlags, pListNode3->keyFlags, pRevListNode3->keyFlags);                                
                                file.Write((LPCTSTR) message, message.GetLength());
                            }

                        }

                        pListNode2 = pListNode2->next;

                    }
                }

                pListNode = pListNode->next;
            }

            // MARK SENSOR AS FLAG_SENSOR_NODE_INACTIVE (NO OTHER TRIANGLE WITH A POSSIBLE)
            pSensorA->flags |= FLAG_SENSOR_NODE_INACTIVE;
        }


        file.Close();
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveInstanceLog(CString message) {
	char    timeStr[MAX_PATH];
	CString	timeString;
	_strdate_s(timeStr, MAX_PATH); timeString += timeStr;
	_strtime_s(timeStr, MAX_PATH); timeString += timeStr;
	
	// TRY TO SAVE INTO C:
	WritePrivateProfileString(pSensorSim->m_basePath, "DATE", (LPCTSTR) timeString, S3_INSTANCES_LOGS_C);
	WritePrivateProfileString(pSensorSim->m_basePath, "STATUS", message, S3_INSTANCES_LOGS_C);
	// TRY TO SAVE INTO D:
	WritePrivateProfileString(pSensorSim->m_basePath, "DATE", (LPCTSTR) timeString, S3_INSTANCES_LOGS_D);
	WritePrivateProfileString(pSensorSim->m_basePath, "STATUS", message, S3_INSTANCES_LOGS_D);
	// TRY TO SAVE INTO E:
	WritePrivateProfileString(pSensorSim->m_basePath, "DATE", (LPCTSTR) timeString, S3_INSTANCES_LOGS_E);
	WritePrivateProfileString(pSensorSim->m_basePath, "STATUS", message, S3_INSTANCES_LOGS_E);
	
	return STAT_OK;
}


int CSensorEnv::SaveCompromissedByDistance(BASIC_INIT_DATA* pBasicSettings, CString fileName) {
    int                 status = STAT_OK;
    DWORD               i;
    CFile               file;
    CString             message;
    USHORT              comprCountRes[101];
    USHORT              totalCountRes[101];
    USHORT				dist;
	DWORD               maxComprCount = 0;
    NEIGH_LIST_NODE*    pListNode;
    SENSOR_NODE*        pSensor;


    memset(comprCountRes, 0, sizeof(comprCountRes));
    memset(totalCountRes, 0, sizeof(totalCountRes));

    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        // PROCESS ALL WHITE SENSORS AND COUNT NUMBER OF COMPROMISSED KEYS UP TO 
        for (i = 0; i < m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensor = m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensor->whiteNeigh;
            while (pListNode != NULL) {
                // CALCULATE % OF MAX. DISTANCE (pBasicSettings->whiteTransmRange == 100%) 
				dist = (USHORT) ((pListNode->dist / pBasicSettings->whiteTransmRange) * 100);
				if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
					// COMPR. KEY ON PARTICULAR DISTANCE FOUND
					(comprCountRes[dist])++;
					(totalCountRes[dist])++;
                }

                if (pListNode->keyFlags & FLAG_KEY_SECURE) {
					// COMPR. KEY ON PARTICULAR DISTANCE FOUND
					(totalCountRes[dist])++;
                }

                pListNode = pListNode->next;
            }
        }

        // WRITE COUNTS UP TO MAX OF COMPROMISSED KEYS INTO FILE
        for (i = 0; i <= 100; i++) {
            if (totalCountRes[i] > 0) message.Format("%d %d %f\r\n", comprCountRes[i], totalCountRes[i], ((float)comprCountRes[i])/totalCountRes[i]);
			else message.Format("0 0 0\r\n");
            file.Write((LPCTSTR) message, message.GetLength());
        }

        file.Close();
        
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}

int CSensorEnv::SaveCompromissedByNodeDensity(BASIC_INIT_DATA* pBasicSettings, CString fileName) {
    int                 status = STAT_OK;
    DWORD               i;
    DWORD               j;
    USHORT              whiteCount;
    USHORT              blackCount;
    USHORT              comprCount;
    USHORT              totalCount;


    CFile               file;
    CString             message;
    NEIGH_LIST_NODE*    pListNode;
    NEIGH_LIST_NODE*    pListNode2;
    SENSOR_NODE*        pSensor;


    // PROCESS ALL CELLS, COUNT NUMBER OF WHITE AND BLACK SENSOR WITHIN AND RATION OF COMPROMISSED KEYS 
    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {

        // FIRST ROW OF FILE IS SPECIAL AND CONTAINS NUMBER OF GRID CELLS
        message.Format("%d %d 0 0 0\r\n", m_pActualDeployment->geoGrid.numCellX, m_pActualDeployment->geoGrid.numCellY);
        file.Write((LPCTSTR) message, message.GetLength());
        
        
        for (i = 0; i < m_pActualDeployment->geoGrid.numCellX; i++) {
            for (j = 0; j < m_pActualDeployment->geoGrid.numCellY; j++) {
                whiteCount = 0;
                blackCount = 0;
                comprCount = 0;
                totalCount = 0;
                
                pListNode = m_pActualDeployment->geoGrid.grid[i][j];
                while (pListNode != NULL) {
                    pSensor = (SENSOR_NODE*) pListNode->pItem;

                    if (pSensor->flags & FLAG_SENSOR_BLACK) {
                        blackCount++;
                    }

                    if (pSensor->flags & FLAG_SENSOR_WHITE) {
                        whiteCount++;
                        
                        // PROCESS ALL ESTABLISHED LINK KEYS AND OBTAIN STATUS
                        pListNode2 = pSensor->whiteNeigh;
                        while (pListNode2 != NULL) {
                            
                            if (pListNode2->keyFlags & FLAG_KEY_COMPROMISED) {
                                totalCount++;
                                comprCount++;
                            }
                            if (pListNode2->keyFlags & FLAG_KEY_SECURE) {
                                totalCount++;
                            }

                            pListNode2 = pListNode2->next;
                        }
                    }

                    pListNode = pListNode->next;
                }


                // SUMARIZE NUMBER OF WHITE AND BLACK NODES INTO FILE
                // EACH ROW CORRESPOND TO ONE CELL, CELLS ARE TAKEN FROM LEFT TO RIGHT AND UP TO DOWN  
                message.Format("%d %d %d %d %f\r\n", whiteCount, blackCount, totalCount, comprCount, (totalCount > 0) ? ((float) comprCount) / totalCount : 0);
                file.Write((LPCTSTR) message, message.GetLength());
            }
        }

        file.Close();
        
    }
    else status = STAT_FILE_OPEN_FAIL;
    

    return status;
}
/*
int CSensorEnv::FormatSettingsString(BASIC_INIT_DATA* pBasicSettings, CString* pString) {
    int     status = STAT_OK;

    pString->Format("whiteNum=%d\r\n whiteRange=%.2f\r\n whiteFailureRate=%.2f\r\n blackNum=%d\r\n planeX=%.2f\r\n planeY=%.2f\r\n useOnlyXNeigh=%d\r\n useNeighFlags=%d\r\n amplifRepeat=%d\r\n", \
    pBasicSettings->whiteNum, pBasicSettings->whiteTransmRange, pBasicSettings->whiteFailureRate, pBasicSettings->blackNum, \
    pBasicSettings->planeX, pBasicSettings->planeY, (pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) ? pBasicSettings->useOnlyXNeigh : -1, pBasicSettings->useNeighFlags, pBasicSettings->amplifRepeat);

    return status;
}

int CSensorEnv::CreateTestHeader(CString testName, BASIC_INIT_DATA* pBasicSettings, CString basicPath, CString columns) {
    int     status = STAT_OK;
    CString message;
    CFile   file;   
    CString fileName; 

    // CREATE TARGET DIRECTORY IF NOT YET
    Dir_Create(basicPath);

    fileName.Format("%s\\!header.dat", basicPath);
    if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate)) {
        message.Format("testName=%s\r\n", testName);
        file.Write((LPCTSTR) message, message.GetLength());

        FormatSettingsString(pBasicSettings, &message);
        file.Write((LPCTSTR) message, message.GetLength());

        message.Format("columns=%s", columns);
        file.Write((LPCTSTR) message, message.GetLength());

        file.Close();
    }

    return status;
}
/**/
int CSensorEnv::SS_ExecuteStrategy(SCRIPT_STRATEGY* pStrategy) {
    int                 status = STAT_OK;
    lac::iterator       iter;   
    CAction*            pAction = NULL;
    int                 i;
    SENSOR_NODE*        pSensorA;    
    SENSOR_NODE*        pSensorB; 
    NEIGH_LIST_NODE*    pListNode;
    DATA_LIST           sharedKeys;
    CString             message;
    SENSITIVE_OBJECT*   pKeyABComposite = NULL;
    DWORD               compromised = 0;
    DWORD               secured = 0;

    // STEPS:
    // HOW MANY NODES
    // TYPE OF ITERATION
    // REQUIRED POSITION PROPERTIES OF SELECTED NODES
    // USED MESSAGES


    //
    // ALLOCATE CROSS-POINTERS TO MAINTAN VARIABLES USED IN MORE ACTIONS
    //
    if (status == STAT_OK) {
        status = SS_AllocateObjectsCrossPointers(pStrategy);
    }

    //
    //  SELECT CENTRAL NODE
    //
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

        // TRY TO ASSIGN pSensorA TO ACTIONS
        for (iter = pStrategy->actionsList.begin(); iter != pStrategy->actionsList.end(); iter++) {
            pAction = *iter;
            pAction->TryAssignPointer(SENSOR_NODE_Ptr, 1, (void*) pSensorA); // FIRST NODE
        }

        //
        //  SELECT OTHER NODE
        //
        switch (pStrategy->numNodes) {
            case 2: {
                // FOR EACH CENTRAL NODE SELECT OTHER NODES BASED ON STRATEGY
                pListNode = pSensorA->whiteNeigh;
                while (pListNode != NULL) {
                    pSensorB = (SENSOR_NODE*) pListNode->pItem;

                    for (iter = pStrategy->actionsList.begin(); iter != pStrategy->actionsList.end(); iter++) {
                        pAction = *iter;
                        // TRY TO ASSIGN pSensorB TO ACTUAL ACTION
                        pAction->TryAssignPointer(SENSOR_NODE_Ptr, 2, (void*) pSensorB);    // SECOND NODE

                        // CHECK CORRECTNESS OF CROSS-POINTER ALLOCATION FOR ACTUAL ACTION
                        if ((status = pAction->CheckPointerCorrectness()) == STAT_OK) {
                            // EXECUTE ACTION
                            pAction->Execute();
                        }
                        else {
                            // BAD POINTERS
                            ASSERT(FALSE);
                            break;
                        }

                    }

                    pListNode = pListNode->next;
                }
                break;
             }
            case 3: {
                // NOT IMPLEMENTED YET    
                break;
            }
        }
    }

    //
    //  EVALUATE ATTACKER SUCCESS - CHECK KEYS BETWEEN ALL DIRECT NEIGHBOURS
    //
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
        pListNode = pSensorA->whiteNeigh;
        while (pListNode != NULL) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;
            
            // get shared keys between A and B
            sharedKeys.release();
            status = SS_GetSharedKeys(pSensorA, pSensorB, &sharedKeys);
            // combine shared keys 
            status = SS_CombineKeys(pSensorA, pSensorB, &sharedKeys, &pKeyABComposite);
            
            if (pKeyABComposite->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED) compromised++; 
            else secured++;

            pListNode = pListNode->next;
        }
    }

    message.Format("Secured: %d (%.2f), Compromised: %d (%.2f) ", secured, (float) 100 * secured/((float) secured+compromised), compromised, (float) 100 * compromised/((float) secured+compromised));
    LOG_INSERTSTRING(message);

    return status;
}

int CSensorEnv::SS_ExecuteStrategies(lss* pStrategies) {
    int                 status = STAT_OK;
    LIST_NODE*          pStratNode = NULL;    
    CString             message;
    SCRIPT_STRATEGY*    pStrategy = NULL;
    lss::iterator       iter;
    
    for (iter = pStrategies->begin(); iter != pStrategies->end(); iter++) {
        // EXECUTE STRATEGY
        pStrategy = *iter;
        if ((status = SS_ExecuteStrategy(pStrategy)) != STAT_OK) {
            // ERROR OCCURED
            ASSERT(FALSE);
            break;
        }
        // TODO: EVALUATE ATTACKER SUCCESS
    }

    return status;
}



int CSensorEnv::SS_ExecutePUSHStrategy(SCRIPT_STRATEGY* pStrategy) {
    int                 status = STAT_OK;
/*
    int                 i;
    int                 nextKeyID = 1;
    SENSOR_NODE*        pSensorA = NULL;
    SENSOR_NODE*        pSensorB = NULL;
    SENSOR_NODE*        pSensorC = NULL;
    NEIGH_LIST_NODE*    pListNode;
    NEIGH_LIST_NODE*    pListNode2;
    SENSITIVE_OBJECT*   pKeyAB = NULL;
    SENSITIVE_OBJECT*   pMessage1;
    SENSITIVE_OBJECT*   pKeyAC;
    SENSITIVE_OBJECT*   pMessage2;
    SENSITIVE_OBJECT*   pKeyCB;
    SENSITIVE_OBJECT*   pMessage3;
    SENSITIVE_OBJECT*   pKeyR;
    SENSITIVE_OBJECT*   pMessage4;
    SENSITIVE_OBJECT*   pMessage5;
    SENSITIVE_OBJECT*   pKeyABComposite = NULL;
    DATA_LIST           sharedKeys;
    CString             message;
    

    // GO THROW ALL NODES
    // FOR EACH CENTRAL NODE SELECT OTHER NODES BASED ON STRATEGY
    // EXCHANGE KEYS, MESSAGE, ETC. => LIST OF SENSITIVE_OBJECT
    // COMPUTE ATTACKER SUCCESS

    LOG_INSERTSTRING("Script strategy begin");

    //
    //  PLAINTEXT KEY EXCHANGE PART
    //
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
        if (pSensorA != NULL) pSensorSim->Log_InsertString("pSensorA");
        else pSensorSim->Log_InsertString("pSensorA == NULL");

        // FOR EACH CENTRAL NODE SELECT OTHER NODES BASED ON STRATEGY
        pListNode = pSensorA->whiteNeigh;
        while (pListNode != NULL) {
            pSensorB = (SENSOR_NODE*) pListNode->pItem;
            if (pSensorB != NULL) pSensorSim->Log_InsertString("pSensorB");
            else pSensorSim->Log_InsertString("pSensorB == NULL");

            // exchange key A->B
            status = SS_NewKeyObject(&nextKeyID, FLAG_SENSITOBJECT_SIMPLEKEY, &pKeyAB);                       
            ASSERT(status == STAT_OK);
            pSensorA->ss_addKey(pKeyAB);
            status = SS_NewMessageObject(NULL, pKeyAB, &pMessage1);  // no encryption
            ASSERT(status == STAT_OK);
            status = SS_SendMessage(pSensorA, pSensorB, pMessage1);
            ASSERT(status == STAT_OK);
            pSensorB->ss_dispatchMessages();
            pListNode = pListNode->next;
        }
    }
    pSensorSim->Log_InsertString("Plaintext exchange finished");

    //
    //  SECRECY AMPLIFICATION PART
    // 

    //  *********** 
    //  MUTUAL WHISPERING
    //  *********** 
    int compromised = 0;
    int secured = 0;
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

        pListNode = pSensorA->whiteNeigh;
        while (pListNode != NULL) {
            // SELECT NEIGHBOUR NODE
            pSensorB = (SENSOR_NODE*) pListNode->pItem;

            // get shared keys between A and B
            sharedKeys.release();
            status = SS_GetSharedKeys(pSensorA, pSensorB, &sharedKeys);
            // combine shared keys 
            status = SS_CombineKeys(pSensorA, pSensorB, &sharedKeys, &nextKeyID, &pKeyABComposite);
            
            if (pKeyABComposite->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED) compromised++; 
            else secured++;

            pListNode = pListNode->next;
        }
    }

    message.Format("Secured: %d (%.2f), Compromised: %d (%.2f) ", secured, (float) 100 * secured/((float) secured+compromised), compromised, (float) 100 * compromised/((float) secured+compromised));
    LOG_INSERTSTRING(message);
/**/
/*
    // GO THROW ALL NODES
    for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

        //
        // PERFORM "PUSH" STRATEGY
        //

        // FOR EACH CENTRAL NODE SELECT OTHER NODES BASED ON STRATEGY
        pListNode = pSensorA->whiteNeigh;
        while (pListNode != NULL) {
            // SELECT AMPLIFICATION NODE
            pSensorC = (SENSOR_NODE*) pListNode->pItem;

            // exchange key A->C
            status = SS_NewKeyObject(&nextKeyID, FLAG_SENSITOBJECT_SIMPLEKEY, &pKeyAC);                       
            pSensorA->ss_addKey(pKeyAC);
            status = SS_NewMessageObject(NULL, pKeyAC, &pMessage2);  // no encryption
            status = SS_SendMessage(pSensorA, pSensorC, pMessage2);
            pSensorC->ss_dispatchMessages();

            pListNode2 = pSensorA->whiteNeigh;
            while (pListNode2 != NULL) {
                // SELECT SECOND NODE B, NODE C WILL TRY TO AMPLIFY KEY BETWEEN A AND B
                if ((pSensorB = (SENSOR_NODE*) pListNode2->pItem)) {
                    // CHOOSE DISTINCT TWO NODES
                    if (pSensorB->id != pSensorC->id) {

                        // CHECK IF pSensorC IS ALSO NEIGHBOUR OF pSensorB 
                        if (pSensorC->isWhiteNeigh(pSensorB)) {
                            
                            // exchange key A->B
                            status = SS_NewKeyObject(&nextKeyID, FLAG_SENSITOBJECT_SIMPLEKEY, &pKeyAB);                         
                            pSensorA->ss_addKey(pKeyAB);
                            
                            status = SS_NewMessageObject(NULL, pKeyAB, &pMessage1);  // no encryption
                            status = SS_SendMessage(pSensorA, pSensorB, pMessage1);

                            // exchange key C->B
                            status = SS_NewKeyObject(&nextKeyID, FLAG_SENSITOBJECT_SIMPLEKEY, &pKeyCB);                         
                            pSensorC->ss_addKey(pKeyCB);
                            status = SS_NewMessageObject(NULL, pKeyCB, &pMessage3);  // no encryption
                            status = SS_SendMessage(pSensorC, pSensorB, pMessage3);
                            
                            // DISPATCH INCOMING MESSAGES 
                            pSensorA->ss_dispatchMessages();
                            pSensorB->ss_dispatchMessages();

                            // send fresh random value R (~key) from A->C
                            status = SS_NewKeyObject(&nextKeyID, FLAG_SENSITOBJECT_SIMPLEKEY, &pKeyR);                         
                            pSensorA->ss_addKey(pKeyR);
                            status = SS_NewMessageObject(pKeyAB, pKeyR, &pMessage4);  // encryption with pKeyA->B
                            status = SS_SendMessage(pSensorA, pSensorC, pMessage4);

                            // re-send R C->B
                            status = SS_ResendMessage(pSensorC, pSensorB, pMessage4, pKeyCB, &pMessage5);

                            // combine original key1 and keyR
                            pSensorB->ss_dispatchMessages();
                            status = SS_CombineKeys2(pSensorA, pSensorB, pKeyAB, pKeyR, &nextKeyID, &pKeyABComposite);
                        }
                    }
                }

                pListNode2 = pListNode2->next;
            }

            pListNode = pListNode->next;
        }
    }
/**/    

    
    return status;
}



int CSensorEnv::SS_AllocateObjectsCrossPointers(SCRIPT_STRATEGY* pStrategy) {
    int                 status = STAT_OK;
    SENSITIVE_OBJECT**  ppSENSITIVE_OBJECT = NULL; 
    SENSOR_NODE**       ppSENSOR_NODE = NULL;
    CAction*            pAction = NULL;
    int                 i;
    lac::iterator       iter;
    
    // SENSITIVE_OBJECT POINTERS TO POINTERS
    for (i = 1; i <= pStrategy->ptrCount_SENSITIVE_OBJECT; i++) {
        ppSENSITIVE_OBJECT = new SENSITIVE_OBJECT*;
        SS_AddPointerToList(ppSENSITIVE_OBJECT);

        // TRY TO ASSIGN THIS POINTERS TO ACTIONS
        for (iter = pStrategy->actionsList.begin(); iter != pStrategy->actionsList.end(); iter++) {
            pAction = *iter;
            pAction->TryAssignPointer(SENSITIVE_OBJECT_Ptr, i, (void**) ppSENSITIVE_OBJECT);
        }
        
    }

    // SENSITIVE_OBJECT POINTERS TO POINTERS
    for (i = 1; i <= pStrategy->ptrCount_SENSOR_NODE; i++) {
        ppSENSOR_NODE = new SENSOR_NODE*;
        SS_AddPointerToList(ppSENSOR_NODE);

        // TRY TO ASSIGN THIS POINTERS TO ACTIONS
        for (iter = pStrategy->actionsList.begin(); iter != pStrategy->actionsList.end(); iter++) {
            pAction = *iter;
            pAction->TryAssignPointer(SENSOR_NODE_Ptr, i, (void**) ppSENSOR_NODE);
        }
        
    }

    return status;
}

int CSensorEnv::SS_SendMessage(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, SENSITIVE_OBJECT* pMessage) {
    int     status = STAT_OK;
    
    // CHECK IF pSensor2 IS NEIGBOUR OF pSensor1
    if (status == STAT_OK) {
        if (!pSensor1->isWhiteNeigh(pSensor2)) status = STAT_NOT_NEIGHBOUR;   
    }


    // CHECK FOR ATTACKER INTERCEPTION (BLACK NODE BETWEEN pSensor1 AND pSensor2
    if (status == STAT_OK) {
        if (pSensor1->isBlackBeforeWhite(pSensor2)) {
            // ATTACKER DID INTERCEPT MESSAGE
            status = SS_AttackerInterceptMessage(pMessage);
        }
    }

    // STORE MESSAGE ON pSensor2 NODE
    if (status == STAT_OK) {
        pSensor2->ss_addMessage(pMessage);
    }

    return status;
}

int CSensorEnv::SS_ProcessMessages() {
    int     status = STAT_OK;
    return status;
}

int CSensorEnv::SS_ProcessMessage(SENSOR_NODE* pSensor, SENSITIVE_OBJECT* pMessage) {
    int     status = STAT_OK;
    
    // TODO: DISPATCH MESSAGE

    return status;
}

int CSensorEnv::SS_ResendMessage(SENSOR_NODE* pSender, SENSOR_NODE* pTarget, SENSITIVE_OBJECT* pOrigMessage, SENSITIVE_OBJECT* pNewKey, SENSITIVE_OBJECT** ppNewMessage) {
    int			status = STAT_OK;
    DATA_LIST*	pContent = NULL;
    LIST_NODE*  pNode;
    SENSITIVE_OBJECT*   pMessage;

    // CHECK IF NODE HAVE THIS MESSAGE BUFFERED
    if (status == STAT_OK) {
        if (pSender->ssMessages != NULL) {
            pNode = pSender->ssMessages->pFrontNode;
            while (pNode != NULL) {
                if ((pMessage = (SENSITIVE_OBJECT*) pNode->pItem) != NULL) {
                    if (pMessage->equal(pOrigMessage)) {
                        // MESSAGE FOUND, REMOVE FROM NODE BUFFER
                        pSender->ssMessages->remove(pNode);
                        break;
                    }
                }

                pNode = pNode->next;
            }
        }
        else status = STAT_MESSAGE_MISSING;
    }

    // CHECK IF NODE HAVE ALL KEYS TO DECRYPT MESSAGE 
    if (status == STAT_OK) {
        if ((status = pSender->ss_haveAllKeysOfMessage(pOrigMessage)) == STAT_OK) {           
            // OBTAIN CONTENT AND CREATE NEW MESSAGE 
            if (pOrigMessage->content != NULL) {
                status = SS_DuplicateContent(pOrigMessage->content, &pContent);
                status = SS_NewMessageObject(pNewKey, pContent, ppNewMessage);            
                status = SS_SendMessage(pSender, pTarget, *ppNewMessage);
            }
            else {
                // NO CONTENT!
                ASSERT(FALSE);
            }
        }
        else {
            status = STAT_KEY_MISSING;
        }
    }

    return status;
}

int CSensorEnv::SS_ForwardMessage(SENSOR_NODE* pSender, SENSOR_NODE* pTarget, SENSITIVE_OBJECT* pMessage) {
    int     status = STAT_OK;
    
    // TODO: dodelat pripadne meziakce
    // NO DECRYPTION ON NODE

    return status;
}

int CSensorEnv::SS_CombineKeys(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, DATA_LIST* pKeys, SENSITIVE_OBJECT** ppKeyComposite) {
    int                 status = STAT_OK;
    LIST_NODE*          pKeyNode;
    SENSITIVE_OBJECT*   pKey;
    BOOL                compositeCompromised = FALSE;
    
    // CHECK IF BOTH NODES HAVE ALL KEYS
    if (status == STAT_OK) {
        pKeyNode = pKeys->pFrontNode;
        while (pKeyNode != NULL) {
            pKey = (SENSITIVE_OBJECT*) pKeyNode->pItem;
            if ((pSensor1 != NULL) && (!pSensor1->ss_haveKey(pKey)) 
                || (pSensor2 != NULL) && (!pSensor2->ss_haveKey(pKey))) { 
                // KEY IS MISSING ON SENSOR
                ASSERT(FALSE);
                status = STAT_KEY_MISSING;
                break;
            }

            pKeyNode = pKeyNode->next;
        }
    }


    if (status == STAT_OK) {
        // CREATE NEW COMPOSITE KEY
        status = SS_NewKeyObject(FLAG_SENSITOBJECT_COMPOSITEKEY, ppKeyComposite);
        // ADD ORIGINAL KEYS AS COMPOSITE PARTS 
        compositeCompromised = TRUE;
        (*ppKeyComposite)->components = new DATA_LIST;
        pKeyNode = pKeys->pFrontNode;
        while (pKeyNode != NULL) {
            pKey = (SENSITIVE_OBJECT*) pKeyNode->pItem;
            (*ppKeyComposite)->components->push_back(pKey);
            // IF AT LEAST ONE KEY IS NOT COMPROMISED, THAN COMPOSITE IS SAFE
            if (!(pKey->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED)) compositeCompromised = FALSE;
            pKeyNode = pKeyNode->next;
        }

        // SET COMPROMISATION OF COMPOSITE 
        if (compositeCompromised) {
            (*ppKeyComposite)->secFlags |= FLAG_SENSITOBJECTSEC_COMPROMISED;
        }
    }

    return status;
}

int CSensorEnv::SS_CombineKeys2(SENSITIVE_OBJECT* pKey1, SENSITIVE_OBJECT* pKey2, SENSITIVE_OBJECT** ppKeyComposite) {
    int     status = STAT_OK;

    if (status == STAT_OK) {
        // CREATE NEW COMPOSITE KEY
        status = SS_NewKeyObject(FLAG_SENSITOBJECT_COMPOSITEKEY, ppKeyComposite);
        // ADD ORIGINAL KEYS AS COMPOSITE PARTS 
        (*ppKeyComposite)->components = new DATA_LIST;
        (*ppKeyComposite)->components->push_back(pKey1);
        (*ppKeyComposite)->components->push_back(pKey2);

        // COMPUTE COMPROMISATION OF COMPOSITE 
        if ((pKey1->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED) && (pKey2->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED)) {
            (*ppKeyComposite)->secFlags |= FLAG_SENSITOBJECTSEC_COMPROMISED;
        }
    }

    return status;
}

int CSensorEnv::SS_GetSharedKeys(SENSOR_NODE* pSensor1, SENSOR_NODE* pSensor2, DATA_LIST* pKeys) {
    int     status = STAT_OK;
    LIST_NODE*          pKeyNode1;
    LIST_NODE*          pKeyNode2;
    SENSITIVE_OBJECT*   pKey1;
    SENSITIVE_OBJECT*   pKey2;
    
    if (pSensor1->ssKeys != NULL && pSensor2->ssKeys != NULL) {
        // FIRST NODE RING
        pKeyNode1 = pSensor1->ssKeys->pFrontNode;
        while (pKeyNode1 != NULL) {
            pKey1 = (SENSITIVE_OBJECT*) pKeyNode1->pItem;
            // SECOND NODE RING
            pKeyNode2 = pSensor2->ssKeys->pFrontNode;
            while (pKeyNode2 != NULL) {
                pKey2 = (SENSITIVE_OBJECT*) pKeyNode2->pItem;
                // TEST FOR SHARE KEY
                if (pKey1->equal(pKey2)) {
                    // SHARED KEY FOUND
                    pKeys->push_back(pKey1);
                    // GO TO NEXT ITERATION OF OUTER LOOP
                    break;
                }

                pKeyNode2 = pKeyNode2->next;
            }

            pKeyNode1 = pKeyNode1->next;
        }
    }
    
    return status;
}

int CSensorEnv::SS_AttackerInterceptMessage(SENSITIVE_OBJECT* pMessage) {
    int     status = STAT_OK;
    LIST_NODE*  pNode;
    SENSITIVE_OBJECT*   pKey;

    
    // STORE THIS MESSAGE
    m_ssAttackerMessagesList.push_back(pMessage);
    pMessage->secFlags |= FLAG_SENSITOBJECTSEC_INTERCEPTED;

    // CHECK IF ALL USED KEYS ARE KNOWN
    if (SS_AttackerHaveAllKeys(pMessage) == STAT_OK) {
        pMessage->secFlags |= FLAG_SENSITOBJECTSEC_COMPROMISED;
        // EXTRACT ALL KEYS FROM MESSAGE CONTENT
        if (pMessage->content) {
            pNode = pMessage->content->pFrontNode;
            while (pNode != NULL) {
                if ((pKey = (SENSITIVE_OBJECT*) pNode->pItem) != NULL) {
                    if (pKey->typeFlags & FLAG_SENSITOBJECT_KEY) {
                        status = SS_AttackerInterceptKey(pKey);
                    }
                }

                pNode = pNode->next;
            }
        }
    }

    return status;
}

int CSensorEnv::SS_AttackerInterceptKey(SENSITIVE_OBJECT* pKey) {
    int     status = STAT_OK;
    
    if (pKey->typeFlags & FLAG_SENSITOBJECT_KEY) {
        m_ssAttackerKeysList.push_back(pKey);
        pKey->secFlags |= FLAG_SENSITOBJECTSEC_COMPROMISED | FLAG_SENSITOBJECTSEC_INTERCEPTED;
    }
    
    return status;
}

int CSensorEnv::SS_AttackerHaveAllKeys(SENSITIVE_OBJECT* pMessage) {
    int                 status = STAT_OK;
    LIST_NODE*          pNode;
    SENSITIVE_OBJECT*   pKey;
    BOOL                keyMissing = FALSE;

    // CHECk ALL COMPONENTS OF MESSAGE
    if (pMessage->components) {
        pNode = pMessage->components->pFrontNode;
        while (pNode != NULL && keyMissing == FALSE) {
            if ((pKey = (SENSITIVE_OBJECT*) pNode->pItem) != NULL) {
                if (pKey->typeFlags & FLAG_SENSITOBJECT_KEY) {
                    // KEY FOUND, CHECK IF ATTACK HAVE THIS
                    if (pKey->secFlags & FLAG_SENSITOBJECTSEC_COMPROMISED) {
                        // ATTACKER HAVE THIS ONE
                    }
                    else {
                        // ATTACKER DO NOT HAVE THIS, STOP SEARCHHING    
                        keyMissing = TRUE;
                        status = STAT_KEY_MISSING;
                        break;
                    }
                }
            }

            pNode = pNode->next;
        }
    }
    else {
        // NO KEYS USED => HAVE ALL KEYS
    }
    

    return status;
}

int CSensorEnv::SS_NewKeyObject(DWORD typeFlags, SENSITIVE_OBJECT** ppKey) {
    int     status = STAT_OK;
    
    *ppKey = new SENSITIVE_OBJECT;
    (*ppKey)->id = m_ssNextKeyID;
    (*ppKey)->typeFlags = typeFlags;

    (*ppKey)->components = new DATA_LIST;
    m_ssNextKeyID++;

    // ADD INTO GLOBAL KEYS LIST
    m_ssKeysList.push_back(*ppKey);

    return status;
}

int CSensorEnv::SS_NewMessageObject(DATA_LIST* pComponents, DATA_LIST* pContent, SENSITIVE_OBJECT** ppMessage) {
    int     status = STAT_OK;
    
    *ppMessage = new SENSITIVE_OBJECT;
    (*ppMessage)->typeFlags = FLAG_SENSITOBJECT_MESSAGE;
    (*ppMessage)->components = pComponents;
    (*ppMessage)->content = pContent;

    // ADD INTO GLOBAL MESSAGES LIST
    m_ssMessagesList.push_back(*ppMessage);

    return status;
}

int CSensorEnv::SS_NewMessageObject(SENSITIVE_OBJECT* pKey, DATA_LIST* pContent, SENSITIVE_OBJECT** ppMessage) {
    int     status = STAT_OK;
    DATA_LIST*   pKeyCont = NULL;

    // ADD ENCRYPTION KEY AS COMPONENT (INSIDE CONTANIER)
    if (status == STAT_OK) { 
        if (pKey != NULL) {
            pKeyCont = new DATA_LIST;
            pKeyCont->push_back(pKey);
        }
    }

    // CREATE MESSAGE OBJECT
    if (status == STAT_OK) {
        status = SS_NewMessageObject(pKeyCont, pContent, ppMessage);    
    }    

    return status;
}


int CSensorEnv::SS_NewMessageObject(SENSITIVE_OBJECT* pKey, SENSITIVE_OBJECT* pContent, SENSITIVE_OBJECT** ppMessage) {
    int     status = STAT_OK;
    DATA_LIST*   pKeyCont = NULL;
    DATA_LIST*   pContentCont = NULL;

    // ADD ENCRYPTION KEY AS COMPONENT (INSIDE CONTANIER)
    if (status == STAT_OK) { 
        if (pKey != NULL) {
            pKeyCont = new DATA_LIST;
            pKeyCont->push_back(pKey);
        }
    }

    // ADD CONTENT OF MESSAGE 
    if (status == STAT_OK) { 
        if (pContent != NULL) {
            pContentCont = new DATA_LIST;
            pContentCont->push_back(pContent);
        }
    }

    // CREATE MESSAGE OBJECT
    if (status == STAT_OK) {
        status = SS_NewMessageObject(pKeyCont, pContentCont, ppMessage);    
    }    

    return status;
}

int CSensorEnv::SS_DuplicateContent(DATA_LIST* pOrigContent, DATA_LIST** ppDuplicateContent) {
    int         status = STAT_OK;
    LIST_NODE*  pNode;

    *ppDuplicateContent = new DATA_LIST;
    
    pNode = pOrigContent->pFrontNode;
    while (pNode != NULL) { 
        (*ppDuplicateContent)->push_back(pNode->pItem);
        pNode = pNode->next;
    }

    return status;
}

void CSensorEnv::SS_AddPointerToList(SENSITIVE_OBJECT* pPtr) {
    m_ssSENSITIVE_OBJECT_PtrList.push_back(pPtr);
}

void CSensorEnv::SS_AddPointerToList(SENSITIVE_OBJECT** pPtr) {
    m_ssSENSITIVE_OBJECT_PtrPtrList.push_back(pPtr);
}
void CSensorEnv::SS_AddPointerToList(SENSOR_NODE* pPtr) {
    m_ssSENSOR_NODE_PtrList.push_back(pPtr);
}

void CSensorEnv::SS_AddPointerToList(SENSOR_NODE** pPtr) {
    m_ssSENSOR_NODE_PtrPtrList.push_back(pPtr);
}

int CSensorEnv::SS_ReleaseStructs() {
    // RELEASE CONTENT OF m_ssKeysList
    LIST_NODE*   pNode;
    SENSITIVE_OBJECT*   pItem;
    pNode = m_ssKeysList.pFrontNode;
    while (pNode != NULL) {
        pItem = (SENSITIVE_OBJECT*) pNode->pItem;
        if (pItem != NULL) delete pItem;
        pNode = pNode->next;
    }
    m_ssKeysList.release();

    // RELEASE CONTENT OF m_ssMesagesList
    pNode = m_ssMessagesList.pFrontNode;
    while (pNode != NULL) {
        pItem = (SENSITIVE_OBJECT*) pNode->pItem;
        if (pItem != NULL) delete pItem;
        pNode = pNode->next;
    }
    m_ssMessagesList.release();

    // RELEASE m_ssSENSITIVE_OBJECT_PtrList;
    pNode = m_ssSENSITIVE_OBJECT_PtrList.pFrontNode;
    while (pNode != NULL) {
        if (pNode->pItem != NULL) delete (SENSITIVE_OBJECT*) pNode->pItem;
        pNode = pNode->next;
    }
    m_ssSENSITIVE_OBJECT_PtrList.release();

    // RELEASE m_ssSENSITIVE_OBJECT_PtrPtrList;
    pNode = m_ssSENSITIVE_OBJECT_PtrPtrList.pFrontNode;
    while (pNode != NULL) {
        if (pNode->pItem != NULL) delete (SENSITIVE_OBJECT**) pNode->pItem;
        pNode = pNode->next;
    }
    m_ssSENSITIVE_OBJECT_PtrPtrList.release();

    // RELEASE m_ssSENSOR_NODE_PtrList;
    pNode = m_ssSENSOR_NODE_PtrList.pFrontNode;
    while (pNode != NULL) {
        if (pNode->pItem != NULL) delete (SENSOR_NODE*) pNode->pItem;
        pNode = pNode->next;
    }
    m_ssSENSITIVE_OBJECT_PtrList.release();

    // RELEASE m_ssSENSOR_NODE_PtrPtrList;
    pNode = m_ssSENSOR_NODE_PtrPtrList.pFrontNode;
    while (pNode != NULL) {
        if (pNode->pItem != NULL) delete (SENSOR_NODE**) pNode->pItem;
        pNode = pNode->next;
    }
    m_ssSENSOR_NODE_PtrPtrList.release();

    return STAT_OK;
}


int CSensorEnv::GAS_Evolve(GA_STRATEGY* pStrategy) {
    _PERFTIME_BEGIN_
    
    int                 status = STAT_OK;
    CString             message;
    CString             value;
    CTimeSpan           ts(0, 3, 0, 0 ); 
    CTime               lastSave = CTime::GetCurrentTime();
    CTime               actualTime = CTime::GetCurrentTime();
    
    GAS_Genome<GAS_ELEMENT_TYPE> genomeTemp(pStrategy->gLength, CSensorEnv::GAS_fitness);

    pStrategy->maxFit = 0;
	pStrategy->totalFitEvals = 0;
    
    message.Format("GAS# Num. parties: %d, MaxInstr: %d, Memory size: %d, Protocol repeat: %d", pStrategy->maxParties, pStrategy->maxInstruct, pStrategy->partyMemorySize, pStrategy->numRepeat);    
    LOG_INSERTSTRING(message); 
    message.Format("GAS# PopSize: %d, NumGener: %d, MutProb: %f, CrossProb: %f", pStrategy->popSize, pStrategy->nGeners, pStrategy->pMutt, pStrategy->pCross);    
    LOG_INSERTSTRING(message); 
    message.Format("GAS# WeightLinks: %f, WeightMessages: %f, LinksLowerBound: %f, MessagesUpperBound: %f, MaxSndCount: %d", 
		pSensorSim->m_sensorEnv.m_basicSettings.fitnessWeightLinks,
		pSensorSim->m_sensorEnv.m_basicSettings.fitnessWeightMessages,
		pSensorSim->m_sensorEnv.m_basicSettings.fitnessLinksLowerBound,
		pSensorSim->m_sensorEnv.m_basicSettings.fitnessMessagesUpperBound,
		pSensorSim->m_sensorEnv.m_basicSettings.fitnessMaxSndCount
	);
    LOG_INSERTSTRING(message); 
    
    //
    //  EVOLVE STEP-BY-STEP
    //
    int actGener = 1;
    pStrategy->pGA->flushScores();
    pStrategy->pGA->pop->evaluated = gaFalse;	
    pStrategy->totalMaxMsg = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors * pSensorSim->m_sensorEnv.m_basicSettings.numDeployments * pSensorSim->m_pGAStrategy->maxInstruct * pSensorSim->m_pGAStrategy->numRepeat;
    while (/*pStrategy->pGA->statistics().maxEver() < 1 && */actGener <= pStrategy->nGeners) {
        pStrategy->actGeneration = actGener;

		// CHECK FOR SOFT INTERRUPT
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            CString value;
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);

            // SET ACTIVE THREAD PTR TO NULL
            pSensorSim->m_pActualThread = NULL;
            
            return 0;
        }
		
		// SET SEED FOR GALIB RNG FOR DETERMINISTIC EVOLUTION
		// use fixed GA seed only for first generation
		if (pSensorSim->m_sensorEnv.m_basicSettings.rndGenGA.useFixedSeed && pSensorSim->m_numRepetitionsElapsed == 0 && actGener == 1)
			GAResetRNG(GAGetRandomSeed());
		else if (pStrategy->nextSeed != 0)	// use seed saved in previous step
			GAResetRNG(pStrategy->nextSeed);
		else
			GAResetRNG(GAGetRandomSeed());	// use seed initialized in InitRandomGenerator()
		
        //pStrategy->pGA->pop->flushEvalution();
		
#ifdef _DEBUG
		if (actGener == 1) { 
			// Dump complete state of GASteadyState object
			std::string dump;
			pStrategy->pGA->Serialize(&dump);
			CString fileName;
			fileName.Format("GalibStateDump_gener_%d_%d_beforeFirstStep.ini", pSensorSim->m_numRepetitionsElapsed, actGener);
			DeleteFile(fileName);
			pSensorSim->Log_InsertToFile(fileName, dump.c_str());
		}
#endif
        pStrategy->pGA->step(); // krok GA
        
		// WRITE FRACTION OF COMPUTATION TO FILE
		float fraction_done = ((pSensorSim->m_numRepetitionsElapsed+1) * actGener) / (float) (pSensorSim->m_numRepetitions * pStrategy->nGeners);
		CFile   fractionFile;
		if (fractionFile.Open("fraction_done.txt", CFile::modeWrite | CFile::modeCreate)){
			message.Format("%lf", fraction_done);
			fractionFile.Write(message, message.GetLength());
			fractionFile.Close();
		}

        genomeTemp = (GAS_Genome<GAS_ELEMENT_TYPE>&) pStrategy->pGA->statistics().bestIndividual(); 
		message.Format("(%d gen.): %f best (seed %u, %0.4f%% secured, %0.2f messages)", 
			(pSensorSim->m_numRepetitionsElapsed+1) * actGener, 
			genomeTemp.score(), 
			GAGetRandomSeed(), 
			genomeTemp.stats.getSecuredRatio()*100,
			genomeTemp.stats.getAverageMessages()
		); 
        LOG_INSERTSTRING(message); 

		// SAVE ACTUAL STATUS INTO RUNNING INSTANCES STRING
        SaveInstanceLog(message);
        
		// save seed for next evolution step
		pStrategy->nextSeed = GARandomInt(1,INT_MAX);
	
		actualTime = CTime::GetCurrentTime();

#ifdef _DEBUG
		// SAVE POPULATION TO FILE EACH GENERATION
		if (true) {
#else
		// SAVE POPULATION TO FILE EACH 3 HOURS OR 100 GENERATION	
		if ((actGener % 100 == 1) || (actualTime - ts  > lastSave)) {
#endif
            lastSave = CTime::GetCurrentTime();

            CString fileName;
            fileName.Format("population_%d_%d.ini", pSensorSim->m_numRepetitionsElapsed, actGener);
            DeleteFile(fileName);
			GAS_SaveToFile(fileName, pStrategy);
		}

#ifdef _DEBUG
		// Dump complete state of GASteadyState object
		std::string dump;
		pStrategy->pGA->Serialize(&dump);
		CString fileName;
		fileName.Format("GalibStateDump_gener_%d_%d.ini", pSensorSim->m_numRepetitionsElapsed, actGener);
		DeleteFile(fileName);
		pSensorSim->Log_InsertToFile(fileName, dump.c_str());
#endif

        actGener++;
    }
    
    // Intermediate population usable for automatic restart of evolution to maintain suitable repeatability of experiments
	// Search codes for /rep: switch for more description
    CString fileName;
	fileName.Format("population.ini");
    DeleteFile(fileName);
	GAS_SaveToFile(fileName, pStrategy);

#ifdef _DEBUG
	std::string dump;
	pStrategy->pGA->Serialize(&dump);
	fileName.Format("GalibStateDump_population.ini");
	DeleteFile(fileName);
	pSensorSim->Log_InsertToFile(fileName, dump.c_str());
#endif

    genomeTemp = (GAS_Genome<GAS_ELEMENT_TYPE>&) pStrategy->pGA->statistics().bestIndividual(); 
	int fncSlots = (pStrategy->useSubFunct) ? pStrategy->sfncSlots : 0;
    for (int slot = 0; slot < pStrategy->maxInstruct + fncSlots; slot++) {
		pStrategy->toString(&(*genomeTemp)+ slot*GAS_INSLENGTH, &message, pStrategy->protocolTypeFlags);
        value.Format("%2d:", slot);
        value += message;
        LOG_INSERTSTRING_NOTIME(value); 
    }
    
	message.Format("Best fitness: %f (%0.4f%% secured, %0.2f messages, %d/%d SND ins)", 
		genomeTemp.score(), 
		genomeTemp.stats.getSecuredRatio()*100, 
		genomeTemp.stats.getAverageMessages(),
		genomeTemp.stats.insCount[GAS_INS_SEND],
		genomeTemp.stats.insTotalCount
	); 
    LOG_INSERTSTRING(message); 

	if (!pStrategy->prunneResults){
		GAS_ELEMENT_TYPE* prunnedGenom = new GAS_ELEMENT_TYPE[(pStrategy->maxInstruct + fncSlots) * GAS_INSLENGTH];
		memset(prunnedGenom, 0, (pStrategy->maxInstruct + fncSlots) * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
    
		float*  insFitnessGain = new float[pStrategy->maxInstruct + fncSlots]; 
		memset(insFitnessGain, 0, (pStrategy->maxInstruct + fncSlots) * sizeof(float));
                
		status = GAS_PruneProtocol(&(*genomeTemp), prunnedGenom, pStrategy->maxInstruct, genomeTemp.score(), 3, insFitnessGain, pStrategy->sfncSlots);
        
		delete[] insFitnessGain;
		delete[] prunnedGenom;
	}

	message.Format(
		"Global stats: %u fitness evaluations, %0.4f%% initially secured, %0.4f white neighbors", 
		pStrategy->totalFitEvals,
		genomeTemp.stats.getOrigSecuredRatio()*100,
		genomeTemp.stats.getAverageWhiteNeigh()
	); 
    LOG_INSERTSTRING(message); 

    _PERFTIME_END_
    
    return status;    
}   
void CSensorEnv::GAS_SaveToFile(CString filePath, GA_STRATEGY* pStrategy) {
	GAPopulation pop = pStrategy->pGA->population();
	GAS_SaveToFile(filePath, pStrategy, &pop);
}


void CSensorEnv::GAS_SaveToFile(CString filePath, GA_STRATEGY* pStrategy, GAPopulation* pop) {
	GAS_Genome<GAS_ELEMENT_TYPE> genomeTemp(pStrategy->gLength, CSensorEnv::GAS_fitness);
	CString message, value;
	
	// SAVE ACTUAL SETTINGS
	pSensorSim->Log_InsertToFile(filePath, "########## ACTUAL POPULATION #############\r\n"); 
	message.Format("%s\r\n", GAS_HEADER_BEGIN);
	pSensorSim->Log_InsertToFile(filePath, message); 
	message = "";
	pStrategy->settingsToString(&message);
	pSensorSim->Log_InsertToFile(filePath, message); 
	pSensorSim->Log_InsertToFile(filePath, GAS_HEADER_END); 
	pSensorSim->Log_InsertToFile(filePath, GAS_SEPARATOR); 
	
	for (int nGen = 0; nGen < pStrategy->popSize; nGen++) {
        message.Format("%s\r\n", GAS_GENOM_BEGIN);
        pSensorSim->Log_InsertToFile(filePath, message); 
		genomeTemp = (GAS_Genome<GAS_ELEMENT_TYPE>&) pop->individual(nGen);

		int fncSlots = (pStrategy->useSubFunct) ? pStrategy->sfncSlots : 0;
        for (int slot = 0; slot < pStrategy->maxInstruct + fncSlots; slot++) {
            pStrategy->toString(&(*genomeTemp)+ slot*GAS_INSLENGTH, &message, pStrategy->protocolTypeFlags, FALSE);
            value.Format("%2d:", slot);
            value += message;
            value += "\r\n";
            pSensorSim->Log_InsertToFile(filePath, value); 
        }
                
        pSensorSim->Log_InsertToFile(filePath, GAS_GENOM_END); 
        pSensorSim->Log_InsertToFile(filePath, GAS_SEPARATOR); 
    }

	// WRITE FITNESS DATA FOR EACH INDIVIDUAL
    message.Format("%s\r\n", GAS_FITNESS_BEGIN);
    pSensorSim->Log_InsertToFile(filePath, message); 
	for (int nGen = 0; nGen < pStrategy->popSize; nGen++) {
		message.Format("%2d:%f\r\n", nGen, pop->individual(nGen).score());
		pSensorSim->Log_InsertToFile(filePath, message); 
    }
    pSensorSim->Log_InsertToFile(filePath, GAS_FITNESS_END); 
    pSensorSim->Log_InsertToFile(filePath, GAS_SEPARATOR); 

    message.Format("%s\r\n", GAS_SECURED_LINKS_BEGIN);
    pSensorSim->Log_InsertToFile(filePath, message); 
	for (int nGen = 0; nGen < pStrategy->popSize; nGen++) {		
        GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) pop->individual(nGen); 
		message.Format("%2d:%f\r\n", nGen, genome.stats.getSecuredRatio());
		pSensorSim->Log_InsertToFile(filePath, message); 
    }
    pSensorSim->Log_InsertToFile(filePath, GAS_SECURED_LINKS_END); 
    pSensorSim->Log_InsertToFile(filePath, GAS_SEPARATOR); 
	
    message.Format("%s\r\n", GAS_MESSAGES_BEGIN);
    pSensorSim->Log_InsertToFile(filePath, message); 
	for (int nGen = 0; nGen < pStrategy->popSize; nGen++) {		
        GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) pop->individual(nGen); 
		message.Format("%2d:%f\r\n", nGen, genome.stats.getAverageMessages());
		pSensorSim->Log_InsertToFile(filePath, message); 
    }
    pSensorSim->Log_InsertToFile(filePath, GAS_MESSAGES_END); 
    pSensorSim->Log_InsertToFile(filePath, GAS_SEPARATOR); 
	   
	pSensorSim->Log_InsertToFile(filePath, "########## END ACTUAL POPULATION #############\r\n"); 
}

int CSensorEnv::GAS_LoadFromFile(CString filePath, GA_STRATEGY* pStrategy) {
    int         status = STAT_OK;
    CFile       file;
    CString     content;
    int         pos = 0;
    int         pos2 = 0;
    
    if (file.Open(filePath, CFile::modeRead)) {
        // READ CONTENT
        char*   buff;
		pos = file.GetLength();
        int buffSize = file.GetLength() + 1000;
        buff = new char[buffSize];
        memset(buff, 0, buffSize);
        file.Read(buff, file.GetLength());
        content = buff;
        
        delete[] buff;
        file.Close();
    }
    else status = STAT_FILE_OPEN_FAIL;

	// FILE IS EMPTY
	if (pos == 0) return STAT_DATA_INCORRECT_LENGTH;

    // READ AND PARSE HEADER
    if (status == STAT_OK) {
        CString     header;
        
        if ((pos = content.Find(GAS_HEADER_BEGIN)) != -1) {
            if ((pos2 = content.Find(GAS_HEADER_END)) != -1) {
                header = content.Mid(pos, pos2 - pos);   
                
                pStrategy->parseSettingsFromString(header);
            }                
        }
    }
    
    //  RELEASE PREVIOUS STRUCTS
    if (status == STAT_OK) {
        if (pStrategy->pGenom) {
            delete pStrategy->pGenom;
            pStrategy->pGenom = NULL;
        }       
        if (pStrategy->pGA) {
            delete pStrategy->pGA;
            pStrategy->pGA = NULL;
        }
    }
    
    // READ AND PARSE GENOMS
    if (status == STAT_OK) {
        CString                 value;
        GAS_Genome<GAS_ELEMENT_TYPE>*  pGenomeTemp = NULL;
        int                     nGen = 0;
        DWORD                   nIns = (DWORD) pStrategy->stratStringSize / GAS_INSLENGTH;

        if (m_pLoadedPop != NULL) delete m_pLoadedPop;
        m_pLoadedPop = new GAPopulation;
        
        while ((pos = content.Find(GAS_GENOM_BEGIN, pos)) != -1) {
            if ((pos2 = content.Find(GAS_GENOM_END, pos)) != -1) {
                int len = pos2 - pos - (int) strlen(GAS_GENOM_BEGIN);
                value = content.Mid(pos + (int) strlen(GAS_GENOM_BEGIN), len);   
                value.TrimLeft();value.TrimRight();
                // CLEAR TEMP STRATEGY STRING
                memset(pStrategy->stratString, 0, pStrategy->stratStringSize * sizeof(GAS_ELEMENT_TYPE));
                nIns = pStrategy->stratStringSize / GAS_INSLENGTH;
                // ALLOCATE NEW GENOM, DEALLOCATION IN POPULATION DESCTRUCTOR
                pGenomeTemp = new GAS_Genome<GAS_ELEMENT_TYPE> (pStrategy->gLength, CSensorEnv::GAS_fitness); 
                // PARSE AND LOAD GENOM
                if ((status = pSensorSim->m_sensorEnv.GAS_ParseProtocol(value, pStrategy->stratString, &nIns, NULL, pSensorSim->m_pGAStrategy)) == STAT_OK) {
                    // COPY READED VALUE TO GENOM
                    for (int i = 0; i < nIns; i++) {
						DWORD insBase = i * GAS_INSLENGTH;
						for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) pGenomeTemp->gene(insBase + insOffs, pStrategy->stratString[insBase + insOffs]);
                    }
                    // ADD TO POPULATION
                    //pStrategy->pGA->pop->add(pGenomeTemp);
                    m_pLoadedPop->add(pGenomeTemp);
                    
                    // OK, PROCESS NEXT GENOM
                    nGen++;
                }
                
                pos = pos2;
            }                
        }
		GAS_presort(m_pLoadedPop);
        // OBTAIN ACTUAL POPULATION SIZE AFTER LOADING
        pStrategy->popSize = m_pLoadedPop->size();
    }
    
    //
    //  PREPARE NEW STRUCTS
    //
    if (status == STAT_OK) {
		pStrategy->pGenom = new GAS_Genome<GAS_ELEMENT_TYPE> (pStrategy->gLength, CSensorEnv::GAS_fitness);
		
        if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_NODESORIENTED) { 
            pStrategy->pGenom->mutator(CSensorEnv::GAS_mutator); 
            pStrategy->pGenom->crossover(CSensorEnv::GAS_crossover); 
			pStrategy->pGenom->initializer(CSensorEnv::GAS_initializer_load); 		
        }
        if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED) { 
            pStrategy->pGenom->mutator(CSensorEnv::GAGS_mutator); 
            pStrategy->pGenom->crossover(CSensorEnv::GAGS_crossover); 
			pStrategy->pGenom->initializer(CSensorEnv::GAGS_initializer_load); 
        }
		
        pStrategy->pGA = new GASteadyStateGA(*(pStrategy->pGenom));
        // SET POPULATION SIZE TO 1 AND ADD LOADED GENOMS LATER
        // AT LEAST ONE GENOM MUST BE PRESENT OTHERWISE GALIB initialize() FAIL, WE WILL REMOVE THIS ONE LATER
        pStrategy->pGA->populationSize(pStrategy->popSize); 
        // SET REPLACEMENT FRACTION TO 2/3
        pStrategy->pGA->nReplacement(2 * (pStrategy->popSize / 3));  
        pStrategy->pGA->nGenerations(pStrategy->nGeners);
        pStrategy->pGA->pCrossover(pStrategy->pCross);
        pStrategy->pGA->pMutation(pStrategy->pMutt); 
		// IF LOADED, SET INTERNAL VALUES FOR REPLACEMENT (NEEDED FOR DETERMINISTIC RUN WHEN EVOLUTION SPLITTED INTO SUBRUNS) 
		if (pStrategy->GASteadyStateGA_pRepl != -1) pStrategy->pGA->pRepl = pStrategy->GASteadyStateGA_pRepl;
		if (pStrategy->GASteadyStateGA_nRepl != -1) pStrategy->pGA->nRepl = pStrategy->GASteadyStateGA_nRepl;
		if (pStrategy->GASteadyStateGA_which != -1) pStrategy->pGA->which = pStrategy->GASteadyStateGA_which;

        // DISABLE PRUNING FOR NOW
        BOOL prunneResults = pStrategy->prunneResults;
		pStrategy->prunneResults = FALSE;
        m_lastLoadedGenom = pStrategy->popSize;
		
		// INITIALIZE BY LOADED VALUES
        if (pSensorSim->m_sensorEnv.m_pLoadedPop->isSorted())
			pStrategy->pGA->initializeAlreadySorted();	
		else 
			pStrategy->pGA->initialize();	

		// KULDA: BUGBUG - I dosabled scale() - when population is saved in prevous generation, population is NOT scaled
		// pStrategy->pGA->pop->scale();


		// RESTORE ORIGINAL PRUNNING SETTING
		pStrategy->prunneResults = prunneResults;
    }
    
    //
    //  PRINT LOADED GENOMS TO OTHER FILE FOR CORRECTNESS CHECKING
    //
    if (status == STAT_OK) {
        CString fileName;
        
        fileName.Format("population_loaded.ini");
        DeleteFile(fileName);        
        // SAVE ACTUAL SETTINGS
		GAS_SaveToFile(fileName, pStrategy);

#ifdef _DEBUG
		std::string dump;
		pStrategy->pGA->Serialize(&dump);
		fileName.Format("GalibStateDump_population_loaded.ini");
		DeleteFile(fileName);
		pSensorSim->Log_InsertToFile(fileName, dump.c_str());
#endif

    }
    return status;
}

int CSensorEnv::GAS_ExecuteStrategy(GA_STRATEGY* pStrategy, GA_PROTOCOL_STATS* pStats) {
    _PERFTIME_BEGIN_
    
    int                 status = STAT_OK;
    lac::iterator       iter;   
    CAction*            pAction = NULL;
    int                 i;
    SENSOR_NODE*        pSensorA;    
    SENSOR_NODE*        pSensorB; 
    NEIGH_LIST_NODE*    pListNode;
    DATA_LIST           sharedKeys;
    CString             message;
    SENSITIVE_OBJECT*   pKeyABComposite = NULL;
    DWORD               compromised = 0;
    DWORD               secured = 0;
	GA_PROTOCOL_STATS	stats;
    
    //
    // STORE ORIGINAL STRATEGY TO TEMP ARRAY 
    //
    GAS_ELEMENT_TYPE*               origStratString = NULL;
    DWORD               origStratStringSize;
    origStratStringSize = pStrategy->stratStringSize;
    origStratString = new GAS_ELEMENT_TYPE[origStratStringSize];
    memset(origStratString, 0, origStratStringSize * sizeof(GAS_ELEMENT_TYPE));
    // COPY ORIGINAL STRATEGY STRING (PREVENT MODIFICATION INSIDE NESTED FUNCTION FOR DIFFERENT DEPLOYMENT)
    memcpy(origStratString, pStrategy->stratString, origStratStringSize * sizeof(GAS_ELEMENT_TYPE));
    
	stats.clear();
	stats.numDeployments = pSensorSim->m_sensorEnv.m_basicSettings.numDeployments;	
	
    //
    //  GO THROUGH ALL DEPLOYMENTS AND CALCULATE AVERAGE OF FITNESSES
    //
	for (int deployment = 0; deployment < pSensorSim->m_sensorEnv.m_basicSettings.numDeployments; deployment++) {
		// SET ACTUAL DEPLOYMENT
		pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[deployment];

        // COPY ORIGINAL STRATEGY STRING (PREVENT MODIFICATION INSIDE NESTED FUNCTION FOR DIFFERENT DEPLOYMENT)
        memset(pStrategy->stratString, 0, origStratStringSize * sizeof(GAS_ELEMENT_TYPE));
        memcpy(pStrategy->stratString, origStratString, origStratStringSize * sizeof(GAS_ELEMENT_TYPE));

        //
        // EXECUTE FOR GIVEN DEPLOYMENT
        //
        pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
        
        // FOR KEY INFECTION, COMMUNICATION FOR FIRST ITERATION IS NOT COMPLETELY EAVESDROPPED
        pStrategy->bAllEavesdropped = FALSE; 
        
        DWORD secured = 0, compromised = 0;

        // IF REQUIRED, SET LINK KEYS COMPROMITATION STATUS RANDONMLY 
        if (pSensorSim->m_sensorEnv.m_basicSettings.distribFlags & FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY) {
            DistributeKeyStatusRandomly(&(pSensorSim->m_sensorEnv.m_basicSettings), &secured, &compromised, FLAG_RANDOMDISTRIB_BIDIRECTIONAL, FALSE);

            // ALL COMMUNICATION IS EAVEDROPPED 
            pStrategy->bAllEavesdropped = TRUE; 
        } else {
            BasicWhisper(&(m_basicSettings), &secured, &compromised, FALSE);
		}
		
		if (secured + compromised > 0)
			stats.totalOrigSecuredRatio += secured / (float) (compromised + secured);

        // REPEAT n-TIMES
        for (int rep = 0; rep < pStrategy->numRepeat; rep++) {
        
            // ENABLE NEXT LINES, IF TURN TO EAVESDROPPED AREA SHOULD OCCURE AFTER FIRST ITERATION
            // if (rep == 0) pStrategy->bAllEavesdropped = FALSE; 
            // else pStrategy->bAllEavesdropped = TRUE; 
            
            //
            //  SELECT CENTRAL NODE
            //
            for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                // CLEAR PREVIOUS NODES
                pStrategy->clearNodes();

                pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
                
                //
                //  BASED ON STRATEGY NATURE (NODES vs GROUP ORIENTED), EXECUTE PROTOCOL
                //
                if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_NODESORIENTED) { 
                    // NODES ORIENTED
					status = GAS_ExecuteNodesOriented(pStrategy, pSensorA, &stats.totalMessages);
                }
                
                if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED) { 
                    // GROUP ORIENTED
                    status = GAS_ExecuteGroupOriented(pStrategy, pSensorA, &stats.totalMessages);
                }
            }
            
            //
            // TRANSFORM ALL NEWLY SECURE KEYS ( FLAG_KEY_NEWLY_SECURE) INTO SECURE (FLAG_KEY_SECURE)
            //
            for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
                pListNode = pSensorA->whiteNeigh;

                while (pListNode != NULL) {
                    pSensorB = (SENSOR_NODE*) pListNode->pItem;

                    if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                        if (pListNode->keyFlags & FLAG_KEY_NEWLY_SECURE) {
                            pListNode->keyFlags |= FLAG_KEY_SECURE;
                            pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                            pListNode->keyFlags &= ~FLAG_KEY_NEWLY_SECURE; 
                        }
                    }
                    pListNode = pListNode->next;
                }
            }
        }
        
        //
        //  COUNT SECURE AND COMPROMISED KEYS 
        //
        secured = 0;
        compromised = 0;
        for (i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
            pListNode = pSensorA->whiteNeigh;

            while (pListNode != NULL) {
                pSensorB = (SENSOR_NODE*) pListNode->pItem;

                if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                    if ((pListNode->keyFlags & FLAG_KEY_SECURE) || (pListNode->keyFlags & FLAG_KEY_NEWLY_SECURE)) {
                        // KEY IS SECURE 
                        secured++;

                        // MARK AS ORDINARY SECURE
                        pListNode->keyFlags |= FLAG_KEY_SECURE;
                        pListNode->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                        pListNode->keyFlags &= ~FLAG_KEY_NEWLY_SECURE; 

                        // CLEAR LIST OF COMPROMISSING BLACK NODES (IF USED)
                        pListNode->releaseComprBy();
                    }
                    else {
                        // LINK KEY IS NOT SECURE 
                        compromised++;
                    }
                }

                pListNode = pListNode->next;
            }
        }

		stats.totalWhiteNeigh += pSensorSim->m_sensorEnv.m_pActualDeployment->avgWhiteNeigh;
		stats.totalSecuredKeys += secured;
		stats.totalCompromisedKeys += compromised;
		if (compromised + secured > 0)
			stats.totalSecuredRatio += secured / (float) (compromised + secured);
    }
	
	// ANALYSE PROTOCOL INSTRUCTIONS
	for (i=0; i < pStrategy->stratStringSize; i+=GAS_INSLENGTH){
		if (GAS_ValidateInstructionCorrectness(pStrategy->stratString+i, pStrategy) == STAT_OK && 
			pStrategy->stratString[i + GAS_OFFSET_INSTAT] != GAS_INSINACTIVE){
				stats.insTotalCount++;
				// FREQUENCY ANALYSIS
				stats.insCount[pStrategy->stratString[i + GAS_OFFSET_INSTYPE]]++;	
		}
	}
    
	// set again first deployment struct
	pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[0];
    
	int maxSndCount = pSensorSim->m_sensorEnv.m_basicSettings.fitnessMaxSndCount;
	if (maxSndCount <= 0) maxSndCount = pStrategy->maxInstruct;
	
	// maximal number of messages
	unsigned int maxMessages = pStrategy->numRepeat * maxSndCount * 
		pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors * stats.getAverageWhiteNeigh();
	
	pStrategy->totalKeys = stats.getTotalKeys();

	// compute weighed fitness from secured links ratio and number of messages
	float numMessages = stats.getAverageMessages()<maxMessages?stats.getAverageMessages():maxMessages;
	
	pStrategy->fitness = 0;
	if (stats.getSecuredRatio() > 0 && stats.getAverageMessages() > 0) {
		// if links are below lower bound, don't allow improving fitness with messages and vice versa
		if (stats.getSecuredRatio() >= pSensorSim->m_sensorEnv.m_basicSettings.fitnessLinksLowerBound) {
			pStrategy->fitness += 
					(1 - numMessages/maxMessages) * pSensorSim->m_sensorEnv.m_basicSettings.fitnessWeightMessages;
		}
		if (pSensorSim->m_sensorEnv.m_basicSettings.fitnessMessagesUpperBound == 0 || 
			stats.getAverageMessages() < pSensorSim->m_sensorEnv.m_basicSettings.fitnessMessagesUpperBound) {
				pStrategy->fitness += 
					stats.getSecuredRatio() * pSensorSim->m_sensorEnv.m_basicSettings.fitnessWeightLinks;
		}
	}	

	if (pStats != NULL){
		pStats->clear();
		*pStats = stats;
	}

    if (origStratString != NULL) delete[] origStratString;
    
    _PERFTIME_END_
    
    return status;
}

int CSensorEnv::GAS_ExecuteProtocol(GA_STRATEGY* pStrategy, int* pNumMessages) {
    _PERFTIME_BEGIN_
    
    int             status = STAT_OK;
    SENSOR_NODE*    pNode = NULL;
    SENSOR_NODE*    pNode2 = NULL;
    DWORD           i = 0;
    DWORD           j = 0;
    DWORD           phase = GAS_KEYINFECT;
    BOOL            bDone = FALSE;
    BOOL            bPresent1 = FALSE;
    BOOL            bPresent2 = FALSE;
	BOOL			bIsValid = FALSE;
    DWORD           value = 0;
    DWORD           key = 0;
    DWORD           offset = 0;



    //
    // PARSE STRATEGY FROM INTEGER STRING
    // 
    i = 0;
    // INITIALLY, WE ARE AT THE 'KEY INFECTION' PHASE - PLAINTEXT KEY EXCHANGE
    phase = GAS_KEYINFECT;
    bDone = FALSE;
    while (i < pStrategy->stratStringSize && !bDone) {
/*
        // CHECK IF 'SECRECY AMPLIFICATION' PHASE SHOULD START - ENCRYPTED EXCHANGE
        if (i >= pStrategy->secAmplifOffset && phase != GAS_SECAMPLIF) {
            phase = GAS_SECAMPLIF;
        }
/**/    
		// CHECK IF INSTRUCTION IS VALID AND ACTIVE
		if ((bIsValid = (GAS_ValidateInstructionCorrectness(pStrategy->stratString+i, pStrategy) == STAT_OK)) && 
			pStrategy->stratString[i + GAS_OFFSET_INSTAT] != GAS_INSINACTIVE) {

            // OBTAIN ACTUAL NODE FOR INSTRUCTION
            if ((pNode = pStrategy->nodes[pStrategy->stratString[i + GAS_OFFSET_NODE]]) != NULL) {
                // PARSE INSTRUCTION
                switch (pStrategy->stratString[i + GAS_OFFSET_INSTYPE]) {
                    case GAS_INS_NOP: {
                        // NO OPERATION
                        //ASSERT(FALSE);
                        break;
                    }
                    case GAS_INS_RNG: {
                        // GENERATE NEW KEY ID 
                        pStrategy->newValue++;
                        ASSERT(pStrategy->newValue < pStrategy->valuesInfoSize);
                        pStrategy->valuesInfo[pStrategy->newValue].type = GAS_SIMPLEVALUE;
                        pStrategy->valuesInfo[pStrategy->newValue].valueID = pStrategy->newValue;
                        
                        // PUT NEW VALUE TO NODE MEMORY 
                        pNode->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM1], pStrategy->newValue);

                        break;
                    }
                    case GAS_INS_COMBINE: {
                        // CHECK IF BOTH COMPONENTS ARE AVAILABLE
                        if ((pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM1]) != GAS_NO_VALUE) &&
                            (pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM2]) != GAS_NO_VALUE)) {

                            // GENERATE NEW KEY ID 
                            pStrategy->newValue++;
                            ASSERT(pStrategy->newValue < pStrategy->valuesInfoSize);
                            pStrategy->valuesInfo[pStrategy->newValue].type = GAS_COMPOSITEVALUE;
                            pStrategy->valuesInfo[pStrategy->newValue].valueID = pStrategy->newValue;
                            pStrategy->valuesInfo[pStrategy->newValue].subValueID1 = pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM1]);
                            pStrategy->valuesInfo[pStrategy->newValue].subValueID2 = pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM2]);
                            ASSERT(pStrategy->valuesInfo[pStrategy->newValue].subValueID1 < pStrategy->valuesInfoSize);
                            ASSERT(pStrategy->valuesInfo[pStrategy->newValue].subValueID2 < pStrategy->valuesInfoSize);
                            // PUT NEW VALUE TO NODE MEMORY 
                            pNode->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM3], pStrategy->newValue);
                        }
                        else {
                            // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                            //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                            //ASSERT(FALSE);
                        }
                        
                        break;
                    }
                    case GAS_INS_REMOTECOMBINE: {
                        if ((pNode2 = pStrategy->nodes[pStrategy->stratString[i + GAS_OFFSET_PARAM1]]) != NULL) {
                            value = pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM2]);
                            if (value != GAS_NO_VALUE && pStrategy->valuesInfo[value].type == GAS_COMPOSITEVALUE) {
                                bPresent1 = bPresent2 = FALSE;
                                for (j = 0; j < pNode2->valueMemSize; j++) {
                                    if (pNode2->getValueMem(j) == pStrategy->valuesInfo[value].subValueID1) bPresent1 = TRUE;
                                    if (pNode2->getValueMem(j) == pStrategy->valuesInfo[value].subValueID2) bPresent2 = TRUE;
                                    if (bPresent1 && bPresent2) break;
                                }
                                
                                if (bPresent1 && bPresent2) {
                                    // OK, BOTH PRESENT, STORE COMBINED VALUE INTO MEMORY
                                    pNode2->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM3], value);
									// COUNT THIS MESSAGE IF REQUIRED
                                    if (pNumMessages != NULL) *pNumMessages += 1; 
                                }
                                else {
                                    // VALUES ARE NOT PRESENT, SET INSTRUCTION AS INACTIVE
                                    //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                                    //ASSERT(FALSE);
                                }
                            }
                            else {
                                // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                                //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                                //ASSERT(FALSE);
                            }
                        }
                        else {
                            // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                            //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                            //ASSERT(FALSE);
                        }
                                            
                        break;
                    }
                    case GAS_INS_SEND: {
                        // CHECK IF SENDER VALUE IS AVAILABLE
                        value = pNode->getValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM2]);
                        if (value != GAS_NO_VALUE) {
                            // PUT NEW VALUE TO TARGET NODE MEMORY 
                            if ((pNode2 = pStrategy->nodes[pStrategy->stratString[i + GAS_OFFSET_PARAM1]]) != NULL) {
                                // CHECK IF pNode AND pNode2 ARE NEIGHBOURS
                                if (pNode->isWhiteNeigh(pNode2) == TRUE) {
                                    pNode2->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM3], value);
                                    
                                    // COUNT THIS MESSAGE IF REQUIRED
                                    if (pNumMessages != NULL) *pNumMessages += 1; 
                                    
/* THIS PART ALLOWS TO TRANSPARENTLY USE LINK KEYS, IF AVAILABLE */
                                    // CHECK IF LINK IS NOT ALREADY SECURE
                                    NEIGH_LIST_NODE*    pNeigh = pNode->getNeighStructKeyFlags(pNode2);
                                    if (pNeigh->keyFlags & FLAG_KEY_SECURE) {
                                        // LINK IS ALREADY SECURE, SO SEND IS NOT EAVESDROPPED
                                    }
                                    else {
                                        // LINK IS NOT SECURE
                                        // CHECK, IF TRANSMISSION WAS COMPROMISED
                                        if (pStrategy->bAllEavesdropped || pNode->isBlackBeforeWhite(pNode2) == TRUE) {
                                            // TRANSMISSION EAVESDROPPED
                                            pStrategy->valuesInfo[value].flags |= FLAG_GAS_EAVESDROPPED;
                                        }
                                        else {
                                            // OK
                                        }
                                    }
/**/             
/* THIS PART DO NOT ASSUME TRANSPARENT LINK KEY USAGE - VALUE MUST BE EXPLICITLY ENCRYPTED                        
                                    // CHECK, IF TRANSMISSION WAS COMPROMISED
                                    if (pNode->isBlackBeforeWhite(pNode2) == TRUE || pStrategy->bAllEavesdropped) {
                                        // TRANSMISSION EAVESDROPPED
                                        pStrategy->valuesInfo[value].flags |= FLAG_GAS_EAVESDROPPED;
                                    }
                                    else {
                                        // OK
                                    }
/**/
                                }
                                else {
                                    // NODES ARE NOT NEIGBOURS, TRANSMISSION NOT POSSIBLE
                                    int x = 0;
                                }
                            }
                            else {
                                // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                                //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                                //ASSERT(FALSE);
                            }
                        }
                        else {
                            // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
							// EVEN IF THE VALUE IS NOT AVAILABLE IN CURRENT ROUND, 
							// THE INSTRUCTION CAN STILL BE USEFUL LATER
                            // pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                            //ASSERT(FALSE);
                        }

                        break;
                    }
                    case GAS_INS_ENCRYPT: {
                        // CHECK IF REQUIRED VALUES ARE AVAILABLE
                        int mem = pStrategy->stratString[i + GAS_OFFSET_PARAM1];
                        value = pNode->getValueMem(mem);
                        mem = pStrategy->stratString[i + GAS_OFFSET_PARAM2];
                        key = pNode->getValueMem(mem);
                        if (value != GAS_NO_VALUE && key != GAS_NO_VALUE) {
                            // GENERATE NEW VALUE ID 
                            pStrategy->newValue++;
                            ASSERT(pStrategy->newValue < pStrategy->valuesInfoSize);
                            pStrategy->valuesInfo[pStrategy->newValue].type = GAS_ENCRYPTEDVALUE;
                            pStrategy->valuesInfo[pStrategy->newValue].valueID = pStrategy->newValue;
                            pStrategy->valuesInfo[pStrategy->newValue].subValueID1 = value;      // ORIGINAL VALUE
                            pStrategy->valuesInfo[pStrategy->newValue].subValueID2 = key;        // ENCRYPTION KEY
                            ASSERT(pStrategy->valuesInfo[pStrategy->newValue].subValueID1 < pStrategy->valuesInfoSize);
                            ASSERT(pStrategy->valuesInfo[pStrategy->newValue].subValueID2 < pStrategy->valuesInfoSize);
                            
                            // PUT NEW VALUE TO NODE MEMORY 
                            pNode->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM3], pStrategy->newValue);
                        }
                        else {
                            // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                            //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                            //ASSERT(FALSE);
                        }
                        break;
                    }
                    case GAS_INS_DECRYPT: {
                        // CHECK IF REQUIRED VALUES ARE AVAILABLE
                        int mem = pStrategy->stratString[i + GAS_OFFSET_PARAM1];
                        value = pNode->getValueMem(mem);
                        mem = pStrategy->stratString[i + GAS_OFFSET_PARAM2];
                        key = pNode->getValueMem(mem);
                        if (value != GAS_NO_VALUE && key != GAS_NO_VALUE) {
                            // CHECK IF VALUE IS ENCRYPTED
                            if (pStrategy->valuesInfo[value].type == GAS_ENCRYPTEDVALUE) {
                                // CHECK IF CORRECT DECRYPTION KEY IS USED
                                if (pStrategy->valuesInfo[value].subValueID2 == key) { 
                                    // PUT DECRYPTED VALUE TO NODE MEMORY 
                                    pNode->setValueMem(pStrategy->stratString[i + GAS_OFFSET_PARAM3], pStrategy->valuesInfo[value].subValueID1);
                                }
                                else{ 
                                    // INVALID ENCRYPTION KEY, SET INSTRUCTION AS INACTIVE
                                    //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                                }
                            }
                            else {
                                // NOT THE ENCRYPTED VALUE, SET INSTRUCTION AS INACTIVE
                                //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                                //ASSERT(FALSE);
                            }
                        }
                        else {
                            // REQUIRED VALUES ARE NOT AVAILABLE, SET INSTRUCTION AS INACTIVE
                            //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                            //ASSERT(FALSE);
                        }

                        break;
                    }
                    case GAS_INS_SWITCH: {
                        switch (phase) {
                            case GAS_KEYINFECT: {
                                phase = GAS_SECAMPLIF;
                                i = pStrategy->secAmplifOffset - GAS_INSLENGTH; // GAS_INSLENGTH WILL BE AUTOMATICALLY ADDED AFTER THE CYCLE
                                break;
                            }
                            case GAS_SECAMPLIF: {
                                bDone = TRUE;
                                break;
                            }
                        }
                        break;
                    }
                    default: {
                        ASSERT(FALSE);
                        CString message;
                        message.Format("Unknown instruction '%d'!!!", pStrategy->stratString[i + GAS_OFFSET_INSTYPE]);
                        LOG_INSERTSTRING(message);
                    }
                }
            }            
        } 

		if (!bIsValid && pStrategy->stratString[i + GAS_OFFSET_INSTYPE] != GAS_INS_NOP){
			// INSTRUCTION IS INVALID
            //pStrategy->stratString[i + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
            ASSERT(FALSE);

			CString msg1, msg2;
			pStrategy->toString(i, &msg1, pStrategy->protocolTypeFlags, FALSE);
			msg2.Format("WARNING: invalid instruction %2d:", i);
			msg2.Append(msg1);
    
			LOG_INSERTSTRING(msg2);
		}
        
        i += GAS_INSLENGTH;
    }
    
    _PERFTIME_END_
    
    return status;
}   

int  CSensorEnv::GAS_EvaluateSuccess(GA_STRATEGY* pStrategy, BOOL* bSecure) {
    _PERFTIME_BEGIN_
    
    int     status = STAT_OK;
    int     i = 0;    
    int     j = 0;    
    int     k = 0;
    BOOL    changeFound = TRUE;
    BOOL    bSecureLink = TRUE;
    SENSOR_NODE*    pSensorA = NULL;
    SENSOR_NODE*    pSensorB = NULL;
    SENSOR_NODE*    pSensorN = NULL;
    NEIGH_LIST_NODE* pListNode = NULL;
    DWORD           valueID = 0;
    
    int             maxUsedValue = pStrategy->newValue + 1;
    
    // REPEAT UNTIL CHANGE IN COMPROMITATION WAS FOUND 
    while (changeFound) {
        changeFound = FALSE;
        
//        for (i = 0; i < pStrategy->valuesInfoSize; i++) {
        for (i = 0; i < maxUsedValue; i++) {
            if ((pStrategy->valuesInfo[i]).type != GAS_NO_VALUE) {
                switch ((pStrategy->valuesInfo[i]).type) {
                    case GAS_SIMPLEVALUE: {
                        if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_COMPROMISED) {
                            // ALREADY COMPROMISED
                        }
                        else {
                            // CHECK IF EAVESDROPPED DURING TRANSMISSION
                            if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_EAVESDROPPED) {
                                (pStrategy->valuesInfo[i]).flags |= FLAG_GAS_COMPROMISED;   
                                changeFound = TRUE;
                            }
                        }
                        break;
                    }
                    case GAS_COMPOSITEVALUE: {
                        if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_COMPROMISED) {
                            // ALREADY COMPROMISED
                        }
                        else {
                            // CHECK IF BOTH COMPONENTS ARE COMPROMISED, IF YES, THEN COMPROMISED AS WELL
                            if ((pStrategy->valuesInfo[(pStrategy->valuesInfo[i]).subValueID1].flags & FLAG_GAS_COMPROMISED) &&
                                (pStrategy->valuesInfo[(pStrategy->valuesInfo[i]).subValueID2].flags & FLAG_GAS_COMPROMISED)) {
                                
                                (pStrategy->valuesInfo[i]).flags |= FLAG_GAS_COMPROMISED;   
                                changeFound = TRUE;
                            }
                            // CHECK IF EAVESDROPPED DURING TRANSMISSION
                            if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_EAVESDROPPED) {
                                (pStrategy->valuesInfo[i]).flags |= FLAG_GAS_COMPROMISED;   
                                changeFound = TRUE;
                            }
                            
                        }
                        break;
                    }
                    case GAS_ENCRYPTEDVALUE: {
                        // CHECK IF EAVESDROPPED DURING TRANSMISSION
                        if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_EAVESDROPPED) {
                            if ((pStrategy->valuesInfo[i]).flags & FLAG_GAS_COMPROMISED) {
                                // ALREADY COMPROMISED
                            }
                            else {
                                // SET ENCRYPTED VALUE AS COMPROMISED (NOT INNER VALUE, JUST CIPHERTEXT
                                (pStrategy->valuesInfo[i]).flags |= FLAG_GAS_COMPROMISED;   
                                changeFound = TRUE;
                            }

                            // CHECK IF ATTACKER KNOWS APPLIED KEY, IF YES, THAN ALSO INNER VALUE IS EAVESDROPPED
                            if (pStrategy->valuesInfo[(pStrategy->valuesInfo[i]).subValueID2].flags & FLAG_GAS_COMPROMISED) {
                                if (pStrategy->valuesInfo[(pStrategy->valuesInfo[i]).subValueID1].flags & FLAG_GAS_EAVESDROPPED) {
                                    // INNER VALUE ALREADY EAVESDROPPED WITHOUT ENCRYPTION
                                }
                                else {
                                    // INNER VALUE EAVESDROPPED!
                                    pStrategy->valuesInfo[(pStrategy->valuesInfo[i]).subValueID1].flags |= FLAG_GAS_EAVESDROPPED;   
                                    changeFound = TRUE;
                                }
                            }
                        }
                        break;
                    }
                    default: {
                        //ASSERT(FALSE);
                    }
                }    
            }
        }
    }

    //
    //  EVALUATE ATTACKER SUCCESS - CHECK KEYS BETWEEN A AND B NEIGHBOURS 
    //
    pSensorA = pStrategy->nodes[1];
    pSensorB = pStrategy->nodes[2];
    ASSERT(pSensorA->isWhiteNeigh(pSensorB));
    if (GAS_UncompromisedKeyShared(pStrategy, pSensorA, pSensorB) == STAT_OK) {
        *bSecure = TRUE;
    }
    else {*bSecure = FALSE;}
    if (*bSecure) {
        // SET LINK AS SECURE
        NEIGH_LIST_NODE* pListNodeTemp = pSensorA->getNeighStructKeyFlags(pSensorB);
        if (pListNodeTemp != NULL) {
            pListNodeTemp->keyFlags |= FLAG_KEY_NEWLY_SECURE;
            pListNodeTemp->keyFlags &= ~FLAG_KEY_COMPROMISED; 
        }
        else {
            ASSERT(FALSE); // SHOULD NOT HAPPEN
        }
            
        // SET REVERSE LINK KEY SECURE AS WELL
        NEIGH_LIST_NODE* pListNodeTemp2 = pSensorB->getNeighStructKeyFlags(pSensorA);

        if (pListNodeTemp2 != NULL) {
            pListNodeTemp2->keyFlags |= FLAG_KEY_NEWLY_SECURE;
            pListNodeTemp2->keyFlags &= ~FLAG_KEY_COMPROMISED; 
        }
        else {
            ASSERT(FALSE); // SHOULD NOT HAPPEN
        }
    }
    // 
    //  CHECK ALSO, IF SOME LINK BETWEEN PARTICIPATING NEIGHBOURS CAN BE MARKED SECURE 
    //  
    //
    if (pStrategy->autoKeyDiscovery) {
        for (i = 1; i <= pStrategy->maxParties; i++) {
            if ((pSensorA = pStrategy->nodes[i]) != NULL) {
                // j=2 ALREADY CHECKED ABOVE
                for (int j = 3; j <= pStrategy->maxParties; j++) {
                    pSensorN = pStrategy->nodes[j];
                    if (pSensorN != NULL && pSensorA->isWhiteNeigh(pSensorN)) {
                        NEIGH_LIST_NODE* pListNodeTemp = pSensorA->getNeighStructKeyFlags(pSensorN);
                        if (pListNodeTemp != NULL) {
                            if (pListNodeTemp->keyFlags & FLAG_KEY_NEWLY_SECURE || pListNodeTemp->keyFlags & FLAG_KEY_SECURE) {
                                // LINK ALREADY SECURE
                            }
                            else {
                                if (GAS_UncompromisedKeyShared(pStrategy, pSensorA, pSensorN) == STAT_OK) {
                                    // UNCOMPROMISED KEY FOUND
                                    // SET LINK AS SECURE
                                    pListNodeTemp->keyFlags |= FLAG_KEY_NEWLY_SECURE;
                                    pListNodeTemp->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                                    
                                    // SET REVERSE LINK KEY SECURE AS WELL
                                    NEIGH_LIST_NODE* pListNodeTemp2 = pSensorN->getNeighStructKeyFlags(pSensorA);

                                    if (pListNodeTemp2 != NULL) {
                                        pListNodeTemp2->keyFlags |= FLAG_KEY_NEWLY_SECURE;
                                        pListNodeTemp2->keyFlags &= ~FLAG_KEY_COMPROMISED; 
                                    }
                                    else {
                                        ASSERT(FALSE); // SHOULD NOT OCCURE
                                    }
                                }
                            }
                        }
                        else {
                            ASSERT(FALSE); // SNO == SHOULD NOT OCCURE
                        }
                    }
                }
            }
        }
    }
        
    _PERFTIME_END_
    
    return status;
}    

/*
int CSensorEnv::GAS_ExecuteProtocolRecursive(GA_STRATEGY* pStrategy, int actualNodeCount, BOOL* bSecure) {
    int                 status = STAT_OK;
    int                 i;
    SENSOR_NODE*        pSensor; 
    NEIGH_LIST_NODE*    pListNode;
    
    // ALL REQUIRED NEIGHBOURS ARE SET, RUN PROTOCOL
    if (pStrategy->maxParties <= actualNodeCount) {
        // CLEAR PREVIOUS VALUES
        pStrategy->clearValuesInfo();
        for (i = 1; i <= pStrategy->maxParties; i++) {
            if (pStrategy->nodes[i] != NULL) {
                memset((pStrategy->nodes[i])->valueMem, 0, sizeof(DWORD) * (pStrategy->nodes[i])->valueMemSize);
            }
        }

        // EXECUTE PROTOCOL
        if ((status = GAS_ExecuteProtocol(pStrategy)) == STAT_OK) {
            // EVALUATE SUCCESS
            if ((status = GAS_EvaluateSuccess(pStrategy, bSecure)) != STAT_OK) {
                ASSERT(FALSE);
            }
        }    
        else {
            ASSERT(FALSE);
        }
    }
    else {
        // GATHER SUBSEQUNT NEXT NEIGHBOUR
        if (pStrategy->nodes[1] != NULL) {
            pListNode = (pStrategy->nodes[1])->whiteNeigh;
            while (pListNode != NULL) {
                if ((pSensor = (SENSOR_NODE*) pListNode->pItem) != NULL) {
                    
                    // CHECK IF THIS NODE IS NOT ALREADY IN SET
                    BOOL newNode = TRUE;
                    for (i = 2; i <= actualNodeCount; i++) {
                        if (pSensor == pStrategy->nodes[i]) {
                            newNode = FALSE;
                            break;
                        }
                    }
                    
                    if (newNode) {
                        actualNodeCount++;
                        pStrategy->nodes[actualNodeCount] = pSensor;
                        
                        // RUN NEXT RECURSIVE ITERATION
                        if ((status = GAS_ExecuteProtocolRecursive(pStrategy, actualNodeCount, bSecure)) != STAT_OK) {
                            ASSERT(FALSE);
                        }
                    }
                    
                }
                if (*bSecure) break;
                pListNode = pListNode->next;
            }
         }
         else {
            status = STAT_USERDATA_BAD;
         }
    }
    
    return status;
}
*/
int CSensorEnv::GAS_ExecuteNodesOriented(GA_STRATEGY* pStrategy, SENSOR_NODE* pCentralNode, int* pNumMessages) {
    int     status = STAT_OK;
    
    // SET CENTRAL NODE AS FIRST
    pStrategy->nodes[1] = pCentralNode;
    
    //
    //  SELECT OTHER NODES
    //
    // FOR EACH CENTRAL NODE SELECT OTHER NODES BASED ON STRATEGY
    NEIGH_LIST_NODE* pListNode = pCentralNode->whiteNeigh;

    // SELECT SECOND NODE pSensorB, THAT WILL BE TRYING TO ESTABLISH SECURE LINK KEY WITH pCentralNode
    while (pListNode != NULL) {
        SENSOR_NODE* pSensorB = (SENSOR_NODE*) pListNode->pItem;
        pStrategy->nodes[2] = pSensorB;

        // GATHER RECURSIVELY OTHER PARTIES AND EXECUTE
        BOOL bLinkSecure = FALSE;
        
        // !!! TODO: REKURZIVNI VYBER NEFUNGUJE, V RELEASE PODIVNE PADA  - poskozeny heap??? :(
//                status = GAS_ExecuteProtocolRecursive(pStrategy, 2, &bLinkSecure);

        if (pStrategy->maxParties > 2) {
            // SELECT ALL POSIBBLE 3TH NODES
            NEIGH_LIST_NODE*    pListNode2 = pCentralNode->whiteNeigh;
            while (pListNode2 != NULL) {
                SENSOR_NODE* pSensorC = (SENSOR_NODE*) pListNode2->pItem;
                if (pSensorC != pSensorB) {
                    pSensorC = (SENSOR_NODE*) pListNode2->pItem;
                    pStrategy->nodes[3] = pSensorC;

                    if (pStrategy->maxParties > 3) {
                        NEIGH_LIST_NODE* pListNode3 = pCentralNode->whiteNeigh;
                        while (pListNode3 != NULL) {
                            SENSOR_NODE* pSensorD = (SENSOR_NODE*) pListNode3->pItem;
                            if (pSensorD != pSensorB && pSensorD != pSensorC) {
                                pStrategy->nodes[4] = pSensorD;
                                
                                if (pStrategy->maxParties > 4) {
                                    NEIGH_LIST_NODE* pListNode4 = pCentralNode->whiteNeigh;
                                    while (pListNode4 != NULL) {
                                        SENSOR_NODE* pSensorE = (SENSOR_NODE*) pListNode4->pItem;
                                        if (pSensorE != pSensorB && pSensorE != pSensorC && pSensorE != pSensorD) {
                                            pStrategy->nodes[5] = pSensorE;
                                            
                                            // CLEAR PREVIOUS VALUES
                                            pStrategy->clearValuesInfo();
                                            memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
                                            memset(pSensorB->valueMem, 0, sizeof(DWORD) * pSensorB->valueMemSize);
                                            memset(pSensorC->valueMem, 0, sizeof(DWORD) * pSensorC->valueMemSize);
                                            memset(pSensorD->valueMem, 0, sizeof(DWORD) * pSensorD->valueMemSize);
                                            memset(pSensorE->valueMem, 0, sizeof(DWORD) * pSensorE->valueMemSize);
                                            
                                            // SET FIRST MEMORY SLOTS BY KEYS WITH SECURITY ACCORDING TO LINK STATUS
                                            GAS_SetLinkKeys(pStrategy);
                                            
                                            // EXECUTE PROTOCOL
                                            if ((status = GAS_ExecuteProtocol(pStrategy, pNumMessages)) == STAT_OK) {
                                                // EVALUATE SUCCESS
                                                if ((status = GAS_EvaluateSuccess(pStrategy, &bLinkSecure)) != STAT_OK) {
                                                    ASSERT(FALSE);
                                                }
                                            }    
                                            else {
                                                ASSERT(FALSE);
                                            }
                                        }
                                                                                        
                                        if (bLinkSecure) break;
                                        pListNode4 = pListNode4->next;
                                    }
                                }
                                else {
                                    //
                                    // 4 PARTIES ONLY
                                    //
                                            
                                    // CLEAR PREVIOUS VALUES
                                    pStrategy->clearValuesInfo();
                                    memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
                                    memset(pSensorB->valueMem, 0, sizeof(DWORD) * pSensorB->valueMemSize);
                                    memset(pSensorC->valueMem, 0, sizeof(DWORD) * pSensorC->valueMemSize);
                                    memset(pSensorD->valueMem, 0, sizeof(DWORD) * pSensorD->valueMemSize);

                                    // SET FIRST MEMORY SLOTS BY KEYS WITH SECURITY ACCORDING TO LINK STATUS
                                    GAS_SetLinkKeys(pStrategy);

                                    // EXECUTE PROTOCOL
                                    if ((status = GAS_ExecuteProtocol(pStrategy, pNumMessages)) == STAT_OK) {
                                        // EVALUATE SUCCESS
                                        if ((status = GAS_EvaluateSuccess(pStrategy, &bLinkSecure)) != STAT_OK) {
                                            ASSERT(FALSE);
                                        }
                                    }    
                                    else {
                                        ASSERT(FALSE);
                                    }
                                }    
                            }
                            if (bLinkSecure) break;
                            pListNode3 = pListNode3->next;
                        }
                    }
                    else {
                        //
                        // 3 PARTIES ONLY
                        //
                    
                        // CLEAR PREVIOUS VALUES
                        pStrategy->clearValuesInfo();
                        memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
                        memset(pSensorB->valueMem, 0, sizeof(DWORD) * pSensorB->valueMemSize);
                        memset(pSensorC->valueMem, 0, sizeof(DWORD) * pSensorC->valueMemSize);

                        // SET FIRST MEMORY SLOTS BY KEYS WITH SECURITY ACCORDING TO LINK STATUS
                        GAS_SetLinkKeys(pStrategy);

                        // EXECUTE PROTOCOL
                        if ((status = GAS_ExecuteProtocol(pStrategy, pNumMessages)) == STAT_OK) {
                            // EVALUATE SUCCESS
                            if ((status = GAS_EvaluateSuccess(pStrategy, &bLinkSecure)) != STAT_OK) {
                                ASSERT(FALSE);
                            }
                        }    
                        else {
                            ASSERT(FALSE);
                        }
                    }
                }
                
                if (bLinkSecure) break;

                pListNode2 = pListNode2->next;
            }
        }    
        else {
            //
            // 2 PARTIES ONLY
            //
            
            // CLEAR PREVIOUS VALUES
            pStrategy->clearValuesInfo();
            memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
            memset(pSensorB->valueMem, 0, sizeof(DWORD) * pSensorB->valueMemSize);

            // SET FIRST MEMORY SLOTS BY KEYS WITH SECURITY ACCORDING TO LINK STATUS
            GAS_SetLinkKeys(pStrategy);

            // EXECUTE PROTOCOL
            if ((status = GAS_ExecuteProtocol(pStrategy, pNumMessages)) == STAT_OK) {
                // EVALUATE SUCCESS
                if ((status = GAS_EvaluateSuccess(pStrategy, &bLinkSecure)) != STAT_OK) {
                    ASSERT(FALSE);
                }
            }    
            else {
                ASSERT(FALSE);
            }
        }
        pListNode = pListNode->next;
    }
    
    return  status;
}


int CSensorEnv::GAS_ExecuteGroupOriented(GA_STRATEGY* pStrategy, SENSOR_NODE* pCentralNode, int* pNumMessages) {
    _PERFTIME_BEGIN_
    
    int     status = STAT_OK;
//    GA_STRATEGY tempStrategy;
    
    
    // COPY CONTENT OF ORIGINAL ONE
	CSensorEnv::GAS_CopyStrategyStructs(pStrategy, &(pSensorSim->m_tempStrategy));
	// pSensorSim->m_tempStrategy = *pStrategy;
    // ALLOCATE TEMPORARY STRATEGY STRUCTS TO PREVENT GLOBAL MODIFICATIONS 
    // WILL CREATE NEW POINTERS TO DYNAMIC ARRAY STRUCTS
    CSensorEnv::GAS_ClearDefaultStrategyStructs(&(pSensorSim->m_tempStrategy));

    ASSERT(pSensorSim->m_tempStrategy.stratString != pStrategy->stratString);
    ASSERT(pSensorSim->m_tempStrategy.nodes != pStrategy->nodes);
    ASSERT(pSensorSim->m_tempStrategy.valuesInfo != pStrategy->valuesInfo);
    
    // CHANGE MAXIMUM OF PARTIES TO THE WHOLE GROUP
    pSensorSim->m_tempStrategy.maxParties = pCentralNode->numWhiteNeigh;
    
    // CLEAR INFO ABOUT TRANSMITTED VALUES
    pSensorSim->m_tempStrategy.clearValuesInfo();
    // CLEAR PREVIOUS VALUES OF pCentralNode AND ALL pCentralNode NEIGHBOURS
    memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
    NEIGH_LIST_NODE* pListNode2 = pCentralNode->whiteNeigh;
    while (pListNode2 != NULL) {
        SENSOR_NODE* pSensor = (SENSOR_NODE*) pListNode2->pItem;
        memset(pSensor->valueMem, 0, sizeof(DWORD) * pSensor->valueMemSize);
        pListNode2 = pListNode2->next;  
    }    

    //
    // EXECUTE GROUP ORIENTED PROTOCOL
    // FOR EACH NEIGHBOUR CHOOSE THE RIGHT SUBPROTOCOL BASED ON ITS RELATIVE DISTANCE
    NEIGH_LIST_NODE*    pListNode = pCentralNode->whiteNeigh;
    while (pListNode != NULL) {
        // IF NO OVERLAPPING BETWEEN RUNS OF PROTOCOL BETWEEN NODES IN GROUP IS ALLOWED, THEN CLEAR USED VALUES
        if (pSensorSim->m_tempStrategy.protocolTypeFlags & FLAG_PROTOCOL_GROUP_INDEPENDENTEXECUTION) {
            // CLEAR INFO ABOUT TRANSMITTED VALUES
            pSensorSim->m_tempStrategy.clearValuesInfo();
            // CLEAR PREVIOUS VALUES OF pCentralNode AND ALL pCentralNode NEIGHBOURS
            memset(pCentralNode->valueMem, 0, sizeof(DWORD) * pCentralNode->valueMemSize);
            NEIGH_LIST_NODE* pListNode2 = pCentralNode->whiteNeigh;
            while (pListNode2 != NULL) {
                SENSOR_NODE* pSensor = (SENSOR_NODE*) pListNode2->pItem;
                memset(pSensor->valueMem, 0, sizeof(DWORD) * pSensor->valueMemSize);
                pListNode2 = pListNode2->next;  
            }    
        }
    
    
        // CLEAR ASSIGNED NODES
        pSensorSim->m_tempStrategy.clearNodes();
    
        SENSOR_NODE* pNeighSensor = (SENSOR_NODE*) pListNode->pItem;
        // OBTAIN SLOT WITH SUBFUNCTION CALL CONTAINING PROTOCOL FOR THIS NEIGHBOUR (BASED ON DISTANCE)
        int slot = (int) ((pListNode->dist / pSensorSim->m_sensorEnv.m_basicSettings.whiteTransmRange) * pSensorSim->m_tempStrategy.sfncSlots);
        
        // SUBFUNCTION CALL ARE POSITIONED AT THE END OF INSTRUCTION POOL
        slot += pSensorSim->m_tempStrategy.maxInstruct;


        // SET FIRST NODE
        pSensorSim->m_tempStrategy.nodes[1] = pCentralNode;
        // SET GIVEN NEIGHBOUR AS A SECOND PARTY TO PROTOCOL        
        pSensorSim->m_tempStrategy.nodes[2] = pNeighSensor;
        
        
        int insCount = 0;
        // CONSTRUCT PROTOCOL FROM SUBFUNCTION CALL ONLY
        ASSERT(pStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] == GAS_INS_SUBFUNCTCALL);
        if (pStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] == GAS_INS_SUBFUNCTCALL) {
            // NOTE:ALL SUBFUNCTIONS ARE TAKEN, EVEN INACTIVE ONES
            
            // COMPUTE OFFSET OF SUBFUNCTION IN EXISTING STRATEGY
            int sfncOffset = pStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_NODE];
            ASSERT(sfncOffset < pStrategy->sfncCount);
            sfncOffset *= (pStrategy->sfncSize * GAS_INSLENGTH);
            ASSERT(sfncOffset < pStrategy->stratStringSize);
            // OBTAIN NUMBER OF INSTRUCTION FOR THIS SUBFUNCTION
            insCount = pStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1];
            ASSERT(insCount <= pStrategy->sfncSize);
            // COPY SUBFUNCTION CONTENT INTO TEMP STRATEGY STRING
            memset(pSensorSim->m_tempStrategy.stratString, 0, pSensorSim->m_tempStrategy.sfncSize * sizeof(GAS_ELEMENT_TYPE)); 
            pSensorSim->m_tempStrategy.stratStringSize = insCount * GAS_INSLENGTH;

//            memcpy(pSensorSim->m_tempStrategy.stratString, pStrategy->stratString + sfncOffset, insCount * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
            for (int i = 0; i < insCount; i++) {
				DWORD insBase = i * GAS_INSLENGTH;
				for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) pSensorSim->m_tempStrategy.stratString[insBase + insOffs] = pStrategy->stratString[sfncOffset + insBase + insOffs];
            }
        }
        else {
            // NOT A SUBFUNCTION CALL
            ASSERT(FALSE);
            status = STAT_DATA_INCORRECT_LENGTH;
        }

        // TRANSFORM NODES IDENTIFIED BY THE DISTANCE FROM CENTRAL BY ACTUAL NEIGHBOURS
        if (status == STAT_OK) {
            for (int i = 0; i < insCount; i++) {
                if (pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_INSTAT] == GAS_INSACTIVE) {
                    // RULE: 1. IF REL. NODE IDENTIFICATION IS BETWEEN <0, NODE_DIST_GRANULARITY-1>, THEN 
                    //       OBTAIN RELATIVE NODE IDENTIFICATION AND SCALE TO <0, white transm range>
                    // 2. IF MUTUAL RELATIVE DISTANCE REQUIRED, THEN FIND SUCH NODE, THAT HAVE LOWEST DISTANCE 
                    //    GIVEN BY RELATIVE IDENT. BOTH FROM CENTRAL AND ACTIVE NODE
                    // OTHERWISE IF WITHIN <NODE_DIST_GRANULARITY, 255> AND ODD => CENTRAL NODE, EVEN => ACTIVE NEIGHBOUR
                    // INITIALIZE NODE TO FIND BY CENTRAL NODE 
                    
                    //
                    // GAS_OFFSET_NODE
                    //
                    SENSOR_NODE*    pClosestSensor = pCentralNode;
                    int             closestSensorIndex = 1;
                    GAS_ELEMENT_TYPE            relNodeValue = pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_NODE];
                    if ((status = GAGS_GetClosestDistNeigh(pCentralNode, pNeighSensor, &pClosestSensor, &closestSensorIndex, relNodeValue, pSensorSim->m_tempStrategy.protocolTypeFlags)) == STAT_OK) {
                        // ACTUAL NEIGHBOUR CAN BE FOUND AS ORDINARY NODE, SET TO 2 FOR BETTER READABILITY
                        if (pClosestSensor == pNeighSensor) closestSensorIndex = 2;
                        // CHANGE NODE IDENTIFICATION INSIDE PROTOCOL
                        pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_NODE] = closestSensorIndex;
                        switch (closestSensorIndex) {
                            case 1: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pCentralNode; break;
                            case 2: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pNeighSensor; break;
                            default: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pClosestSensor; break;
                        }
                    }
                    else {
                        ASSERT(FALSE);
                    }
                    
                    //
                    // GAS_OFFSET_PARAM1 IF SND OR RMB INSTRUCTION 
                    //
                    if (pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] == GAS_INS_SEND ||
                        pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] == GAS_INS_REMOTECOMBINE) {
                     
                        relNodeValue = pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_PARAM1];
                        if ((status = GAGS_GetClosestDistNeigh(pCentralNode, pNeighSensor, &pClosestSensor, &closestSensorIndex, relNodeValue, pSensorSim->m_tempStrategy.protocolTypeFlags)) == STAT_OK) {
                            // ACTUAL NEIGHBOUR CAN BE FOUND AS ORDINARY NODE, SET TO 2 FOR BETTER READABILITY
                            if (pClosestSensor == pNeighSensor) closestSensorIndex = 2;
                            pSensorSim->m_tempStrategy.stratString[i * GAS_INSLENGTH + GAS_OFFSET_PARAM1] = closestSensorIndex;
                            switch (closestSensorIndex) {
                                case 1: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pCentralNode; break;
                                case 2: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pNeighSensor; break;
                                default: pSensorSim->m_tempStrategy.nodes[closestSensorIndex] = pClosestSensor; break;
                            }
                        }
                        else {
                            ASSERT(FALSE);
                        }
                    }
                }
            }
        } 

        // EXECUTE MODIFIED PROTOCOL
        if (status == STAT_OK) {
            if ((status = GAS_ExecuteProtocol(&pSensorSim->m_tempStrategy, pNumMessages)) == STAT_OK) {
                // EVALUATE SUCCESS
                BOOL    bLinkSecure = FALSE;
                if ((status = GAS_EvaluateSuccess(&pSensorSim->m_tempStrategy, &bLinkSecure)) != STAT_OK) {
                    ASSERT(FALSE);
                }
            }
        }

        pListNode = pListNode->next;  
    }    
    
    // pSensorSim->m_tempStrategy.release();

    _PERFTIME_END_

    return  status;
}

int CSensorEnv::GAS_SetLinkKeys(GA_STRATEGY* pStrategy) {
    int status = STAT_OK;
/*
    int i;
    int j;
    SENSOR_NODE*    pSensor1;
    SENSOR_NODE*    pSensor2;
    NEIGH_LIST_NODE* pListNode;

  disabled now, enable when explicit link encryption is required   
    // PROCESS ALL NODES AND CREATE NEW KEY WITH COMPROMISE FLAGS EQUAL TO ACTUAL STATE OF LINK
    for (i = 1; i <= pStrategy->maxParties; i++) {
        if ((pSensor1 = pStrategy->nodes[i]) != NULL) {
            for (j = i+1; j <= pStrategy->maxParties; j++) {
                if ((pSensor2 = pStrategy->nodes[j]) != NULL) {
                    // GET LINK KEY FLAGS
                    if ((pListNode = pSensor1->getNeighStructKeyFlags(pSensor2)) != NULL) {
                        // CHECK IF KEY EXISTS
                        if (pListNode->keyFlags & FLAG_KEY_COMPROMISED || pListNode->keyFlags & FLAG_KEY_SECURE) {
                            // CREATE NEW KEY
                            pStrategy->newValue++; ASSERT(pStrategy->newValue < pStrategy->valuesInfoSize);
                            pStrategy->valuesInfo[pStrategy->newValue].type = GAS_SIMPLEVALUE;pStrategy->valuesInfo[pStrategy->newValue].valueID = pStrategy->newValue;
                            // SET KEY COMPROMISE STATUS
                            if (pListNode->keyFlags & FLAG_KEY_COMPROMISED) {
                                pStrategy->valuesInfo[pStrategy->newValue].flags |= FLAG_GAS_COMPROMISED;
                            }
                            // PUT NEW KEY TO NODE MEMORY INTO REGISTER WITH NUMBER SAME AS RELATIVE NUMBER OF PARTY
                            pSensor1->setValueMem(j, pStrategy->newValue);
                            // AND ALSO INTO REGISTER OF SECOND NODE
                            pSensor2->setValueMem(i, pStrategy->newValue);
                        }
                    }
                }
            }              
        }
    }
/**/
    return status;
}

int  CSensorEnv::GAS_AllocateDefaultStrategyStructs(GA_STRATEGY* pStrategy) {
    pStrategy->stratStringSize = 1000 * GAS_INSLENGTH;
    pStrategy->stratString = new GAS_ELEMENT_TYPE[pStrategy->stratStringSize];
    memset(pStrategy->stratString, 0, pStrategy->stratStringSize * sizeof(GAS_ELEMENT_TYPE));
    pStrategy->secAmplifOffset = pStrategy->stratStringSize / 2;
    pStrategy->nodesSize = 100;
    pStrategy->nodes = new SENSOR_NODE*[pStrategy->nodesSize];
    memset(pStrategy->nodes, 0, sizeof(SENSOR_NODE*) * pStrategy->nodesSize);
    pStrategy->valuesInfoSize = 10000;
    pStrategy->valuesInfo = new VALUE_INFO[pStrategy->valuesInfoSize];
    memset(pStrategy->valuesInfo, 0, sizeof(VALUE_INFO) * pStrategy->valuesInfoSize);
    pStrategy->pGenom = NULL;
    pStrategy->pGA = NULL;
    
    return STAT_OK;
}

int CSensorEnv::GAS_ClearDefaultStrategyStructs(GA_STRATEGY* pStrategy) {
    memset(pStrategy->stratString, 0, pStrategy->stratStringSize * sizeof(GAS_ELEMENT_TYPE));
    memset(pStrategy->nodes, 0, sizeof(SENSOR_NODE*) * pStrategy->nodesSize);
    memset(pStrategy->valuesInfo, 0, sizeof(VALUE_INFO) * pStrategy->valuesInfoSize);
    pStrategy->pGenom = NULL;
    pStrategy->pGA = NULL;
    
    return STAT_OK;
}

int CSensorEnv::GAS_CopyStrategyStructs(GA_STRATEGY* pOriginal, GA_STRATEGY* pCopy) {
	// SAVE ORIGINAL POINTERS TO BUFFERS
	GAS_ELEMENT_TYPE* tempStratStringPtr = pCopy->stratString;
	SENSOR_NODE** tempNodesPtr = pCopy->nodes;
	VALUE_INFO* tempValuesInfoPtr = pCopy->valuesInfo;

	// COPY ALL VALUES (INCLUDING BUFFER POINTERS)
	*pCopy = *pOriginal;

	// RESTORE ORIGINAL BUFFERS
	pCopy->stratString = tempStratStringPtr;
	pCopy->nodes = tempNodesPtr;
	pCopy->valuesInfo = tempValuesInfoPtr;

	// COPY CONTENT OF BUFFERS
	ASSERT(pCopy->stratStringSize == pOriginal->stratStringSize);
	ASSERT(pCopy->nodesSize == pOriginal->nodesSize);
	ASSERT(pCopy->valuesInfoSize == pOriginal->valuesInfoSize);
	memcpy(pCopy->stratString, pOriginal->stratString, pOriginal->stratStringSize * sizeof(GAS_ELEMENT_TYPE)); 
	memcpy(pCopy->nodes, pOriginal->nodes, pOriginal->nodesSize * sizeof(SENSOR_NODE*)); 
	memcpy(pCopy->valuesInfo, pOriginal->valuesInfo, pOriginal->valuesInfoSize * sizeof(VALUE_INFO)); 

	return STAT_OK;
}

int CSensorEnv::GAGS_GetClosestDistNeigh(SENSOR_NODE* pCentralNode, SENSOR_NODE* pNeighNode, SENSOR_NODE** ppClosestSensor, int* pClosestSensorIndex, GAS_ELEMENT_TYPE relativeNodeIdentif, DWORD protocolTypeFlags) {
    _PERFTIME_BEGIN_
    
    int     status = STAT_OK;
    float   distMin = INT_MAX;

    *ppClosestSensor = pCentralNode;
    *pClosestSensorIndex = 1;

    // LOWEST 8bits OF relativeNodeIdentif IS RELATIVE DISTANCE FROM CENTRAL NODE pCentralNode
    // NEXT 8bits OF relativeNodeIdentif IS RELATIVE DISTANCE FROM ACTIVE NEIGHBOUR pNeighNode
    BYTE centralRelativeIdentif = relativeNodeIdentif & 0xff;
    BYTE neighRelativeIdentif = (relativeNodeIdentif & 0xff00) >> 8;
    
    float whiteTransmRangeStep = pSensorSim->m_sensorEnv.m_basicSettings.whiteTransmRange / NODE_DIST_GRANULARITY;

    if (centralRelativeIdentif < NODE_DIST_GRANULARITY && neighRelativeIdentif < NODE_DIST_GRANULARITY) { 
        // BOTH centralRelativeIdentif AND neighRelativeIdentif ARE INSIDE <0, NODE_DIST_GRANULARITY-1> RANGE, SEARCH FOR INTERMEDIATE NODE WILL BE PERFORMED
        float centralRelDist = centralRelativeIdentif * whiteTransmRangeStep;
        float neighRelDist = neighRelativeIdentif * whiteTransmRangeStep;
        
        // FIND NODE WITH DISTANCE NEAREST TO relDist (if protocolTypeFlags & FLAG_PROTOCOL_MUTUALRELATIVEDISTANCE, then also from active neighbour 
        NEIGH_LIST_NODE*    pNeighListNode = pCentralNode->whiteNeigh;
        int                 nodeIndex = 3;  // 1,2 already reserved for central node and active neighbour
        distMin = INT_MAX;
        while (pNeighListNode != NULL) {
            float tempDist = 0; // will contain distance of pNeighListNode from both central and active neighbour
            if (protocolTypeFlags & FLAG_PROTOCOL_MUTUALRELATIVEDISTANCE) {
                // cumulated difference from central and active neighbour
                // linear: tempDist = abs(pNeighListNode->dist - centralRelDist);
                tempDist = pow((pNeighListNode->dist - centralRelDist), 2);
                
                
                // check if pNeighNode is also neighbour of active neighbour
                NEIGH_LIST_NODE* pTempNeigh; 
                if ((pTempNeigh = pNeighNode->getNeighStructKeyFlags((SENSOR_NODE*) pNeighListNode->pItem)) != NULL) {
                    // cumulate also distance from this one one obtain 
                    // linear: tempDist += abs(pTempNeigh->dist - neighRelDist);
                    tempDist += pow((pTempNeigh->dist - neighRelDist), 2);
                }
                else {
                    // not a neighbour, set very high value to prevent selection of this neighbour
                    tempDist = INT_MAX;  
                }
            }
            else {
                // JUST DIFFERENCE IN DISTANCE FROM CENTRAL NODE
                // linear: tempDist = abs(pNeighListNode->dist - centralRelDist);
                tempDist = pow((pNeighListNode->dist - centralRelDist), 2);
            }
            
            if (tempDist < distMin) { 
                *ppClosestSensor = (SENSOR_NODE*) pNeighListNode->pItem;
                distMin = tempDist;
                *pClosestSensorIndex = nodeIndex;
            }
            nodeIndex++;
            pNeighListNode = pNeighListNode->next;  
        }    
    }
    else {
        // TARGET NODE IS CHOOSED FROM CENTRAL OR ACTIVE NEIGHBOUR ONLY
        if (relativeNodeIdentif % 2 == 1) {
            *pClosestSensorIndex = 1;
        }
        else {
            *pClosestSensorIndex = 2;
        }
    }
    
    _PERFTIME_END_
    
    return status;
}


int  CSensorEnv::GAS_PrepareMutualWhisperStrategy(GA_STRATEGY* pStrategy) {
    int     status = STAT_OK;
    DWORD   offset;
    
    // CREATE MUTUAL WHISPER STRATEGY
    offset = 0;
    // GENERATE KEYS - 1. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    // GENERATE KEYS - 2. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;

    // EXCHANGE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
  
    // COMBINE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 4;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
/*    
    // END BOTH PARTS
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;

    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
/**/
    pStrategy->maxParties = 2;
    pStrategy->partyMemorySize = 4;
    pStrategy->maxInstruct = offset / GAS_INSLENGTH;

    return status;
}

int  CSensorEnv::GAS_PrepareDanAmplifStrategy(GA_STRATEGY* pStrategy) {
    int     status = STAT_OK;
    DWORD   offset = 0;
    
    
    // CREATE DAN AMPLIF MUTUAL WHISPER
    offset = 0;
    
    // GENERATE KEYS - 3. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    
    // EXCHANGE KEYS 3-1
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    
    

    pStrategy->maxParties = 3;
    pStrategy->partyMemorySize = 4;
    pStrategy->maxInstruct = offset / GAS_INSLENGTH;
    
    
        
/*    
    // GENERATE KEYS - 1. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    // GENERATE KEYS - 2. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    // GENERATE KEYS - 3. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;

    // EXCHANGE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    
    // EXCHANGE KEYS 2-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
    
    // EXCHANGE KEYS 1-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
  
    // COMBINE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;

    // COMBINE KEYS 2-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 4;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    
    // COMBINE KEYS 1-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 4;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;

    // GENERATE AND EXCHANGE NEW VALUE ENCRYPTED BY LINK KEY
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_ENCRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1; // KEY FROM 3 TO 2
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 6;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 6; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_DECRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_ENCRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; // KEY FROM 3 TO 1
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 7;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 7; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_DECRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    // COMBINE NEW KEYS BETWEEN 1 AND 2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    
    

    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;

    pStrategy->maxParties = 3;
    pStrategy->partyMemorySize = 8;
    pStrategy->maxInstruct = offset / GAS_INSLENGTH;
/**/    
    return status;
}

int CSensorEnv::GAS_PrepareRossAmplifStrategy(GA_STRATEGY* pStrategy) {
    int     status = STAT_OK;
    DWORD   offset = 0;
    
    // CREATE ROSS AMPLIF MUTUAL WHISPER STRATEGY
    offset = 0;
    // GENERATE KEYS - 1. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    // GENERATE KEYS - 2. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    // GENERATE KEYS - 3. node
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;

    // EXCHANGE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    
    // EXCHANGE KEYS 2-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 3;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
    
    // EXCHANGE KEYS 1-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 4;
    offset += GAS_INSLENGTH;
  
    // COMBINE KEYS 1-2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;

    // COMBINE KEYS 2-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 4;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    
    // COMBINE KEYS 1-3
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 4;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 2;
    offset += GAS_INSLENGTH;
    
    // END INFECT PHASE
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    
    pStrategy->secAmplifOffset = offset;

    // GENERATE AND EXCHANGE NEW VALUE ENCRYPTED BY LINK KEY
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_ENCRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; // KEY FROM 1 TO 3
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 6;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 6; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_DECRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 6;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_ENCRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 6;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1; // KEY FROM 3 TO 2
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 7;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 3;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 7; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_DECRYPT;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 2; 
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 5;
    offset += GAS_INSLENGTH;
    // COMBINE NEW KEYS BETWEEN 1 AND 2
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 5;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 2;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 1;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 1;
    offset += GAS_INSLENGTH;
    
    

    pStrategy->stratString[offset + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
    pStrategy->stratString[offset + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
    pStrategy->stratString[offset + GAS_OFFSET_NODE] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM1] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM2] = 0;
    pStrategy->stratString[offset + GAS_OFFSET_PARAM3] = 0;
    offset += GAS_INSLENGTH;
    
    return status;
}

int CSensorEnv::GAS_PruneProtocol(GAS_ELEMENT_TYPE* orig, GAS_ELEMENT_TYPE* pruned, DWORD maxInstruct, float origFitness, int pruneRepeat, float* insFitnessGain, DWORD sfncSlots) {
    _PERFTIME_BEGIN_
    
    int         status = STAT_OK;    
    GAS_ELEMENT_TYPE*       pruneTemp = NULL;
    CString     message, value;
	GA_PROTOCOL_STATS stats, bestStats;

    pruneTemp = new GAS_ELEMENT_TYPE[maxInstruct * GAS_INSLENGTH];
    memcpy(pruneTemp, orig, maxInstruct * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
    
    if (insFitnessGain != NULL) memset(insFitnessGain, 0, maxInstruct * sizeof(float));
    
    for (int pruneRep = 0; pruneRep < pruneRepeat; pruneRep++) {
#ifdef _DEBUG
		message.Format("Prunning repeat %d", pruneRep + 1);
        LOG_INSERTSTRING(message);
#endif

        // TRY TO REMOVE ALL UNIMPORTANT INSTRUCTIONS
        for (int slot = 0; slot < maxInstruct; slot++) {
#ifdef _DEBUG
            if (slot % 10 == 0) {
                message.Format("Prunning %d-th instruction", slot);
                LOG_INSERTSTRING(message);
            }
#endif

            // RESTORE TAIL FROM ORIGINAL PROTOCOL
            for (int i = slot; i < maxInstruct; i++) {
				DWORD insBase = i * GAS_INSLENGTH;
				for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) pSensorSim->m_pGAStrategy->stratString[insBase + insOffs] = pruneTemp[insBase + insOffs];
            }
            // ADD SUBFUNCTION CALL INSTRUCTIONS IF REQUIRED
            for (int i = maxInstruct; i < (maxInstruct+sfncSlots); i++) {
				DWORD insBase = i * GAS_INSLENGTH;
				for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) pSensorSim->m_pGAStrategy->stratString[insBase + insOffs] = orig[insBase + insOffs]; 
            }
            
            if (pSensorSim->m_pGAStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT] == GAS_INSACTIVE) {
                // TRY TO REMOVE THIS ACTIVE INSTRUCTION, IF FITNESS REMAINS SAME, THAN REMOVE INSTRUCTION COMPLETELLY 
                pSensorSim->m_pGAStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT] = GAS_INSINACTIVE;
                pSensorSim->m_pGAStrategy->fitness = 0;
                if ((status = pSensorSim->m_sensorEnv.GAS_ExecuteStrategy(pSensorSim->m_pGAStrategy, &stats)) == STAT_OK) {
                    if (origFitness > pSensorSim->m_pGAStrategy->fitness) {
                        // INSTRUCTION WAS IMPORTANT!  
                        pSensorSim->m_pGAStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
                        // STORE THIS INSTRUCTION
                        memcpy(pruned + slot * GAS_INSLENGTH, pSensorSim->m_pGAStrategy->stratString + slot * GAS_INSLENGTH, GAS_INSLENGTH  * sizeof(GAS_ELEMENT_TYPE));
                    }
                    else {
                        if (origFitness < pSensorSim->m_pGAStrategy->fitness) {
                            // EVEN BETTER RESULT WITHOUT INSTRUCTION
                           message.Format("Better result %f with pruning! (when ins %d removed)", pSensorSim->m_pGAStrategy->fitness, slot);
                           LOG_INSERTSTRING(message);
                        }
                        else {
                            // NOT IMPORTANT, KEEP INSTRUCTION INACTIVE
                        }
                    }
                    
                    // IF REQUIRED, RETURN FITNESS GAIN OR DRAWBACK OF THIS INSTRUCTION 
                    if (insFitnessGain != NULL) {
                        insFitnessGain[slot] = origFitness - pSensorSim->m_pGAStrategy->fitness;    
                    }
                }
                else {
                    ASSERT(FALSE);
                }
            }
        }
        
        // CHECK IF PROTOCOL BEFORE PRUNNING IS EQUAL TO PROTOCOL AFTER PRUNNING
        if (memcmp(pruned, pruneTemp, maxInstruct * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE)) == 0) {
            message.Format("Prunned version found after %d prunning repeats", pruneRep);
            LOG_INSERTSTRING(message);

            // ADD SUBFUNCTION CALL INSTRUCTIONS IF REQUIRED
            for (int i = maxInstruct; i < (maxInstruct+sfncSlots); i++) {
				DWORD insBase = i * GAS_INSLENGTH;
				for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) 
					pruned[insBase + insOffs] = orig[insBase + insOffs]; 
            }

			memset(pSensorSim->m_pGAStrategy->stratString, 0, pSensorSim->m_pGAStrategy->stratStringSize); 
			memcpy(pSensorSim->m_pGAStrategy->stratString, pruned, (maxInstruct + sfncSlots) * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE)); 

			message.Format("PRUNNED PROTOCOL:");
            LOG_INSERTSTRING(message);

			for (int slot = 0; slot < maxInstruct + sfncSlots; slot++) {
				// PRINT ACTIVE INSTRUCTIONS
				if (pSensorSim->m_pGAStrategy->stratString[slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT] == GAS_INSACTIVE) {
					pSensorSim->m_pGAStrategy->toString(slot, &message, pSensorSim->m_pGAStrategy->protocolTypeFlags);
					value.Format("  [%.3f] %2d: ", insFitnessGain[slot], slot);
					value += message;
					LOG_INSERTSTRING_NOTIME(value); 
				}
			}

            pSensorSim->m_pGAStrategy->fitness = 0;
            pSensorSim->m_sensorEnv.GAS_ExecuteStrategy(pSensorSim->m_pGAStrategy, &stats);
            message.Format("Prunned fitness: %f (%0.4f%% secured, %0.2f messages, %d/%d SND ins)", 
				pSensorSim->m_pGAStrategy->fitness, 
				stats.getSecuredRatio()*100,
				stats.getAverageMessages(),
				stats.insCount[GAS_INS_SEND],
				stats.insTotalCount
			);
            LOG_INSERTSTRING(message);

            break; 
        }
        else {
            // PRUNNED VERSION DIFF FROM ORIGINAL, TRY ANOTHER LOOP
            memcpy(pruneTemp, pruned, maxInstruct * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
            memset(pruned, 0, maxInstruct * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
        }
    }
    
    if (pruneTemp) delete[] pruneTemp;

    _PERFTIME_END_
    
    return status;
}       

int CSensorEnv::GAS_ParseProtocol(CString textProtocol, GAS_ELEMENT_TYPE* protocol, DWORD* maxInstruct, DWORD* maxParties, GA_STRATEGY* pStrategy) {
    int     status = STAT_OK;
    int     pos = 0;
    int     pos2 = 0;
    int     pos3 = 0;
    int     pos4 = 0;
    DWORD   slot = 0;
    CString line;
    CString value;
    int     sfncOffset = 0;   
    int     fncSlot = 0;
    BOOL    bGroupProtocol = FALSE;    
    
	textProtocol.Replace("\r\n", "\n");
    textProtocol += "\n";
    
    if (maxParties != NULL) *maxParties = 0;
    
    if (pStrategy != NULL) pStrategy->sfncSlots = 0;	// @TODO WARNING!!! what if pStrategy == NULL ?
    
    memset(protocol, 0, (*maxInstruct) * GAS_INSLENGTH);
    while (pos < textProtocol.GetLength()) {
        if ((pos2 = textProtocol.Find('\n', pos)) != -1) {
            line = textProtocol.Mid(pos, pos2 - pos);        
            line.TrimLeft();
            line.TrimRight();
            if (line != "") {
                if (line.Find("---") != -1) {
                    // DELIMINATOR FOR GROUP ORIENTED FUNCTION SLOT FOUND
                    // OBTAIN FUNCTION SLOT NUMBER
                    int pos5 = line.Find("FNC:");
                    fncSlot = atoi(line.Mid(pos5 + (int) strlen("FNC:")));
                    // SET FUNCTION SLOT
                    protocol[(pStrategy->maxInstruct + fncSlot) * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_SUBFUNCTCALL;
                    protocol[(pStrategy->maxInstruct + fncSlot) * GAS_INSLENGTH + GAS_OFFSET_INSTAT] = GAS_INSACTIVE;
                    protocol[(pStrategy->maxInstruct + fncSlot) * GAS_INSLENGTH + GAS_OFFSET_NODE] = fncSlot;
                    protocol[(pStrategy->maxInstruct + fncSlot) * GAS_INSLENGTH + GAS_OFFSET_PARAM1] = pStrategy->sfncSize;
                    
                    // COMPUTE BASE INS OFFSET FOR GIVEN FNC BLOCK
                    sfncOffset = fncSlot * pStrategy->sfncSize * GAS_INSLENGTH;
                    ASSERT(sfncOffset < pStrategy->sfncCount * pStrategy->sfncSize * GAS_INSLENGTH);
                    
                    // RESET SLOT VALUE WITHIN ACTUAL FNC BLOCK
                    slot = 0;
                    
                    // SET FLAG SINALIZING GROUP ORIENTED PROTOCOL
                    bGroupProtocol = TRUE;
                }
                else {
                    // IGNORE EVERYTHING UNTIL LAST ':' FOUND
                    if ((pos3 = line.ReverseFind(':')) != -1) {
                        line = line.Mid(pos3 + 1);
                    }
                    line.TrimLeft();
                    line.TrimRight();
                    line += " ";
                    // INS TYPE
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        if (value.CompareNoCase("RNG") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_RNG;
                        if (value.CompareNoCase("CMB") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_COMBINE;
                        if (value.CompareNoCase("RCB") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_REMOTECOMBINE;
                        if (value.CompareNoCase("SND") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_SEND;
                        if (value.CompareNoCase("ENC") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_ENCRYPT;
                        if (value.CompareNoCase("DEC") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_DECRYPT;
                        if (value.CompareNoCase("SWC") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_SWITCH;
                        if (value.CompareNoCase("FNC") == 0) protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] = GAS_INS_SUBFUNCTCALL;
                    }
                    
                    // INS STATUS
                    line = line.Mid(pos3 + 1);
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT] = atoi(value);
                    }
                    // NODE INDEX
                    line = line.Mid(pos3 + 1);
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        if ((pos4 = value.FindOneOf("01234656789")) != -1) {
                            value = value.Mid(pos4);   
                            int pos5 = 0;
                            GAS_ELEMENT_TYPE relativeNodeIdentif = 0;
                            if ((pos5 = value.Find("_")) != -1) {
                                // RELATIVE IDENTIFICATION, GROUP ORIENT. PROTOCOL
                                GAS_ParseRelativeNodeIdentif(value, &relativeNodeIdentif);
                                protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_NODE] = relativeNodeIdentif;
                            }
                            else {
                                // ABSOLUTE IDENTIFICATION, NODE ORINENTED PROTOCOL 
                                protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_NODE] = atoi(value);
                                if (maxParties != NULL) {
                                    if (*maxParties < atoi(value)) *maxParties = atoi(value);
                                }
                            }
                        }
                    }
                    // PARAM1
                    line = line.Mid(pos3 + 1);
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        if ((pos4 = value.FindOneOf("01234656789")) != -1) {
                            value = value.Mid(pos4);   
                            int pos5 = 0;
                            GAS_ELEMENT_TYPE relativeNodeIdentif = 0;
                            if ((pos5 = value.Find("_")) != -1) {
                                // RELATIVE IDENTIFICATION, GROUP ORIENT. PROTOCOL
                                GAS_ParseRelativeNodeIdentif(value, &relativeNodeIdentif);
                                protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1] = relativeNodeIdentif;
                            }
                            else {
                                // ABSOLUTE IDENTIFICATION, NODE ORINENTED PROTOCOL 
                                protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1] = atoi(value);
                                if (maxParties != NULL) {
                                    if (*maxParties < atoi(value)) *maxParties = atoi(value);
                                }
                            }
                        }
                    }
                    // PARAM2
                    line = line.Mid(pos3 + 1);
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        if ((pos4 = value.FindOneOf("1234656789")) != -1) {
                            value = value.Mid(pos4);   
                            protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_PARAM2] = atoi(value);
                        }
                    }
                    // PARAM3
                    line = line.Mid(pos3 + 1);
                    if ((pos3 = line.Find(' ')) != -1) {
                        value = line.Left(pos3);
                        if ((pos4 = value.FindOneOf("1234656789")) != -1) {
                            value = value.Mid(pos4);   
                            protocol[sfncOffset + slot * GAS_INSLENGTH + GAS_OFFSET_PARAM3] = atoi(value);
                        }
                    }
                    slot++;     
                }           
            }
        }
        else {
            break;
        }    
        
        pos = pos2 + 1;
    }

	// Supress last unused instruction
	//slot--;

    if (maxInstruct != NULL) {
        if (bGroupProtocol) *maxInstruct = pStrategy->maxInstruct;
        else *maxInstruct = slot;
    }
    
    
    //
    //  obtain more detailed information about scripts
    // 
    if (pStrategy != NULL) {    
        // CHECK INSTRUCTION FROM END FOR FNC INS  
        int i = 0;
        if (bGroupProtocol) i = pStrategy->maxInstruct + fncSlot;
        else i = slot - 1;
        
        while (i >= 0) {
            if (protocol[i * GAS_INSLENGTH + GAS_OFFSET_INSTYPE] == GAS_INS_SUBFUNCTCALL) {
                // FNC found
                pStrategy->sfncSlots++;    
            }
            else {
                // break on first non-FNC instruction
            }
             
            i--;
        } 
        
        if (bGroupProtocol) {
            pStrategy->sfncSlots = fncSlot + 1;
        }
        else {
            pStrategy->maxInstruct = slot - pStrategy->sfncSlots;
            if (pStrategy->sfncSlots > 0) pStrategy->sfncSize = pStrategy->maxInstruct / pStrategy->sfncSlots;
        }
    } 
    
    
    return status;
}

int CSensorEnv::GAS_ParseRelativeNodeIdentif(CString value, GAS_ELEMENT_TYPE* pRelIdentif) {
    int status = STAT_OK;
    
    // EITHER IDENTIFICATION N1 or N2 IS GIVEN, OR RELATIVE DIST BY FLOAT VALUE
    if (value == "1_" || value == "2_") {
        *pRelIdentif = (value == "1_") ? NODE_DIST_GRANULARITY + 1 : NODE_DIST_GRANULARITY + 2; // ODD VALUE FOR NODE 1, EVEN VALUE FOR NODE 2    
    }
    else {
        // RELATIVE DIST BY FLOAT
        int pos5; 
        if ((pos5 = value.Find("_")) != -1) {
            CString dist1;CString dist2;
            dist1 = value.Left(pos5);
            dist2 = value.Right(value.GetLength() - pos5 - 1);
            float fCentralDist = atof(dist1); float fNeighDist = atof(dist2);
            *pRelIdentif = round(fNeighDist * NODE_DIST_GRANULARITY);
            *pRelIdentif  = *pRelIdentif << 8;
            *pRelIdentif += round(fCentralDist * NODE_DIST_GRANULARITY);
        }
        else status = STAT_DATA_CORRUPTED;
    }
    
    return status;
}
// SET UNUSED PARAMETERS TO ZERO
void CSensorEnv::GAS_CanonizeInstruction(GAS_ELEMENT_TYPE* ins, GA_STRATEGY* pStrategy){
	switch (ins[GAS_OFFSET_INSTYPE]) {
		case GAS_INS_NOP:
		case GAS_INS_SWITCH:
			ins[GAS_OFFSET_NODE] = 0;
			ins[GAS_OFFSET_PARAM1] = 0;			
		case GAS_INS_RNG:
		case GAS_INS_SUBFUNCTCALL:
			ins[GAS_OFFSET_PARAM2] = 0;
			ins[GAS_OFFSET_PARAM3] = 0;
			break;
	}
}

// SINGLE ARRAY WITH LENGTH GAS_INSLENGTH IS EXPECTED
int CSensorEnv::GAS_ValidateInstructionCorrectness(GAS_ELEMENT_TYPE* ins, GA_STRATEGY* pStrategy, DWORD flags) {
    int status = STAT_OK;

	// INSTRUCTION
	if (flags & GAS_FLAG_INSTYPE) {
		if (ins[GAS_OFFSET_INSTYPE] < GAS_INS_NOP || ins[GAS_OFFSET_INSTYPE] > GAS_INS_MAX) 
			status = STAT_USERDATA_BAD;
	}

	// ACTIVE
	if (flags & GAS_FLAG_INSTAT) {
		if (ins[GAS_OFFSET_INSTAT] < 0 || ins[GAS_OFFSET_INSTAT] > 1) 
			status = STAT_USERDATA_BAD;
	}

	// NODE
	if (flags & GAS_FLAG_NODE) {
		switch (ins[GAS_OFFSET_INSTYPE]) {
			case GAS_INS_NOP:
			case GAS_INS_SWITCH:
				break;
			case GAS_INS_RNG:
			case GAS_INS_COMBINE:
			case GAS_INS_REMOTECOMBINE:
			case GAS_INS_SEND:
			case GAS_INS_ENCRYPT:
			case GAS_INS_DECRYPT:
			
				if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_NODESORIENTED && (ins[GAS_OFFSET_NODE] < 1 || ins[GAS_OFFSET_NODE] > pStrategy->maxParties))
					status = STAT_USERDATA_BAD;

				if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED && (ins[GAS_OFFSET_NODE] < 0 || ins[GAS_OFFSET_NODE] > 0xFFFF))
					status = STAT_USERDATA_BAD;

				break;
			case GAS_INS_SUBFUNCTCALL:

				if (ins[GAS_OFFSET_NODE] < 0 || ins[GAS_OFFSET_NODE] > pStrategy->sfncCount-1) status = STAT_USERDATA_BAD;

				break;
			default:
				status = STAT_USERDATA_BAD;
		}
	}

	// PARAM1
	if (flags & GAS_FLAG_PARAM1) {
		switch (ins[GAS_OFFSET_INSTYPE]) {
			case GAS_INS_NOP:
			case GAS_INS_SWITCH:
				break;
			case GAS_INS_RNG: 
				if (ins[GAS_OFFSET_PARAM1] < 1 || ins[GAS_OFFSET_PARAM1] > pStrategy->partyMemorySize) 
						status = STAT_USERDATA_BAD;
				break;
			case GAS_INS_REMOTECOMBINE: 
			case GAS_INS_SEND: 
				if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_NODESORIENTED && (ins[GAS_OFFSET_PARAM1] < 1 || ins[GAS_OFFSET_PARAM1] > pStrategy->partyMemorySize))
					status = STAT_USERDATA_BAD;
				if (pStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED && (ins[GAS_OFFSET_PARAM1] < 0 || ins[GAS_OFFSET_PARAM1] > 0xFFFF))
					status = STAT_USERDATA_BAD;
				break;
			case GAS_INS_COMBINE: 
			case GAS_INS_ENCRYPT: 
			case GAS_INS_DECRYPT: 
				if (ins[GAS_OFFSET_PARAM1] < 1 || ins[GAS_OFFSET_PARAM1] > pStrategy->partyMemorySize) status = STAT_USERDATA_BAD;
				break;
			case GAS_INS_SUBFUNCTCALL:
				if (ins[GAS_OFFSET_PARAM1] < 0 || ins[GAS_OFFSET_PARAM1] > pStrategy->sfncSize-1) status = STAT_USERDATA_BAD;
				break;
			default:
				status = STAT_USERDATA_BAD;
		}
	}

	// PARAM2
	if (flags & GAS_FLAG_PARAM2) {
		switch (ins[GAS_OFFSET_INSTYPE]) {
			case GAS_INS_NOP:
			case GAS_INS_SWITCH:
			case GAS_INS_RNG: 
			case GAS_INS_SUBFUNCTCALL:
				break;
			case GAS_INS_REMOTECOMBINE: 
			case GAS_INS_SEND: 
			case GAS_INS_COMBINE: 
			case GAS_INS_ENCRYPT: 
			case GAS_INS_DECRYPT: 
				if (ins[GAS_OFFSET_PARAM2] < 1 || ins[GAS_OFFSET_PARAM2] > pStrategy->partyMemorySize) status = STAT_USERDATA_BAD;
				break;
			default:
				status = STAT_USERDATA_BAD;
		}
	}

	// PARAM3
	if (flags & GAS_FLAG_PARAM3) {
		switch (ins[GAS_OFFSET_INSTYPE]) {
			case GAS_INS_NOP:
			case GAS_INS_SWITCH:
			case GAS_INS_RNG: 
			case GAS_INS_SUBFUNCTCALL:
				break;
			case GAS_INS_REMOTECOMBINE: 
			case GAS_INS_SEND: 
			case GAS_INS_COMBINE: 
			case GAS_INS_ENCRYPT: 
			case GAS_INS_DECRYPT: 
				if (ins[GAS_OFFSET_PARAM3] < 1 || ins[GAS_OFFSET_PARAM3] > pStrategy->partyMemorySize) status = STAT_USERDATA_BAD;
				break;
			default:
				status = STAT_USERDATA_BAD;
		}
	}

#ifdef _DEBUG
	if (status!=STAT_OK){
		CString message;
		message.Format("Ins is not valid (%d %d %d %d %d %d, flags %d)", ins[GAS_OFFSET_INSTYPE], ins[GAS_OFFSET_INSTAT], ins[GAS_OFFSET_NODE], ins[GAS_OFFSET_PARAM1], ins[GAS_OFFSET_PARAM2], ins[GAS_OFFSET_PARAM3], flags);
		LOG_INSERTSTRING_NOTIME(message);
	}
#endif

    return status;
}

int CSensorEnv::GAS_UncompromisedKeyShared(GA_STRATEGY* pStrategy, SENSOR_NODE* pNode1, SENSOR_NODE* pNode2) {
    _PERFTIME_BEGIN_
    
    BOOL    bSecureLink = FALSE;
    // FIND ALL UNCOMPROMISED KEYS FROM A's MEMORY
    BOOL    bAtLeastOneKeyShared = FALSE;
    for (int k = 0; k < pNode1->valueMemSize; k++) {
        if (pNode1->getValueMem(k) != GAS_NO_VALUE) {
            DWORD valueID = pNode1->getValueMem(k);
            // IF SIMPLE KEY, THEN CHECK DIRECTLY IF BOTH KNOWS THE KEY
            if ((pStrategy->valuesInfo[valueID]).type == GAS_SIMPLEVALUE || (pStrategy->valuesInfo[valueID]).type == GAS_COMPOSITEVALUE) {
                // THIS KEY IS SECURE, CHECK IF ALSO NODE N KNOWS IT 
                for (int j = 0; j < pNode2->valueMemSize; j++) {
                    if (pNode2->getValueMem(j) != GAS_NO_VALUE && valueID == pNode2->getValueMem(j)) {
                        bAtLeastOneKeyShared = TRUE;
                        if ((pStrategy->valuesInfo[valueID]).flags & FLAG_GAS_COMPROMISED) {
                            // THIS KEY IS COMPROMISED
                        }
                        else {                    
                            // B ALSO KNOWS THE KEY => SECURE LINK CAN BE ESTABLISHED
                            bSecureLink = TRUE;
                        }
                    }
                }
            }
/*            
            if ((pStrategy->valuesInfo[valueID]).type == GAS_COMPOSITEVALUE) { 
                // CHECK IF SECOND NODE ALSO KNOW THE VALUE OR AT LEAST BOTH SUBCOMPONENTS
                for (int j = 0; j < pNode2->valueMemSize; j++) {
                    int actValueID = pNode2->getValueMem(j);
                    if (actValueID != GAS_NO_VALUE && valueID == actValueID) {
                        if ((pStrategy->valuesInfo[valueID]).flags & FLAG_GAS_COMPROMISED) {
                            // THIS KEY IS COMPROMISED
                        }
                        else {                    
                            // B ALSO KNOWS THE KEY => SECURE LINK CAN BE ESTABLISHED
                            bSecureLink = TRUE;
                        }
                    }
                    else {
                        // SIMPLE TEST, IF TWO COMPOSITE VALUES ARE NOT IN FACT FROM SAME TWO COMPONENTS
                        if (actValueID != GAS_NO_VALUE && (pStrategy->valuesInfo[actValueID]).type == GAS_COMPOSITEVALUE) {
                            if (((pStrategy->valuesInfo[valueID]).subValueID1 == (pStrategy->valuesInfo[actValueID]).subValueID1 && 
                                (pStrategy->valuesInfo[valueID]).subValueID2 == (pStrategy->valuesInfo[actValueID]).subValueID2) ||
                                ((pStrategy->valuesInfo[valueID]).subValueID2 == (pStrategy->valuesInfo[actValueID]).subValueID1 && 
                                (pStrategy->valuesInfo[valueID]).subValueID1 == (pStrategy->valuesInfo[actValueID]).subValueID2)) {
                                
                                if ((pStrategy->valuesInfo[valueID]).flags & FLAG_GAS_COMPROMISED) {
                                    // THIS KEY IS COMPROMISED
                                }
                                else {                    
                                    // B ALSO KNOWS THE KEY => SECURE LINK CAN BE ESTABLISHED
                                    bSecureLink = TRUE;
                                }
                            }
                        }
                    }
                }
            }
/**/
            
/*  JE POTREBA UDELAT PASOVANO KOMPOSITU V LIBOVOLNE UROVNI, TJ. POKUD ZNAM DVA PODKOMPOSITY, TAK MUZU VYTVORIT NAD KOMPOSIT            
 -- projit vsechny simple hodnoty vegenerovatelne z hodnot nesenych uzlem A i B (simple + composite). 
 Pokud existuje nekompromitovany prunik, tak ten lze pouzit ke konstrukci nekompromitovaneho klice
 pozor - rozlozeni na simple hodnoty neznamena, ze uzel je opravdu zna => lze je pouzit pro konstrukci? 
 
            // CHECK IF SECOND NODE DO NOT KNOW ALL COMPONENTS TO RECONSTRUCT COMPOSITE 
            if (bSecureLink == FALSE && (pStrategy->valuesInfo[valueID]).type == GAS_COMPOSITEVALUE) {
                lii simpleList;
                lii::iterator   iter;
                if (GAS_GetAllSimpleComponents(pStrategy, valueID, &simpleList) == STAT_OK) {
                    BOOL    allComponentsKnown = TRUE;       
                    for (iter = simpleList.begin(); iter != simpleList.end(); iter++) {
                        int componentID = *iter;
                        // CHECK IF ALSO SECOND NODE KNOWS GIVEN SIMPLE VALUE 
                        BOOL    node2Knows = FALSE;
                        for (int j = 0; j < pNode2->valueMemSize; j++) {
                            if (pNode2->getValueMem(j) != GAS_NO_VALUE && componentID == pNode2->getValueMem(j)) {
                                node2Knows = TRUE;  
                                break; 
                            }   
                        }
                        // SKIP SEARCHING REST VALUES, COMPONENT NOT SHARED
                        if (node2Knows) {
                            // OK, TRY NEXT COMPONENT 
                        }
                        else {
                            // COMPONENT NOT KNOWN TO NODE2, BREAK
                            allComponentsKnown = FALSE;
                            break;
                        }
                    }
                    
                    if (allComponentsKnown) {
                        // BOTH NODES KNOWS ALL COMPONENTS OF THE KEY => SECURE LINK CAN BE ESTABLISHED
                        bSecureLink = TRUE;
                    }
                }
            }
/**/                
            
        }
    }
    
    _PERFTIME_END_
    
    return (bSecureLink) ? STAT_OK : STAT_KEY_MISSING;
}  

int CSensorEnv::GAS_GetAllSimpleComponents(GA_STRATEGY* pStrategy, int compositeID, lii* pSimpleList) {
    int     status = STAT_OK;
    
    if ((pStrategy->valuesInfo[compositeID]).type == GAS_COMPOSITEVALUE) {
        // CALL RECURSIVELY FOR FIRST COMPONENT 
        GAS_GetAllSimpleComponents(pStrategy, (pStrategy->valuesInfo[compositeID]).subValueID1, pSimpleList);
        // CALL RECURSIVELY FOR SECOND COMPONENT 
        GAS_GetAllSimpleComponents(pStrategy, (pStrategy->valuesInfo[compositeID]).subValueID2, pSimpleList);
    }
    else {
        if ((pStrategy->valuesInfo[compositeID]).type == GAS_SIMPLEVALUE) {
            // SIMPLE VALUE, JUST PUT INTO LIST
            pSimpleList->push_back(compositeID);
        }
    }
            
    return status;
}

void CSensorEnv::GAS_initializer_random(GAGenome &g) {	
	GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;	
	int fncSlots = (pSensorSim->m_pGAStrategy->useSubFunct) ? pSensorSim->m_pGAStrategy->sfncSlots : 0;

    for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct + fncSlots; slot++)
		for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) 
			genome.gene(slot * GAS_INSLENGTH + insOffs, 0);
	
	GAS_mutator(g, 1);

    for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct; slot++) {
		genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GAS_INSACTIVE);
	}
}

void CSensorEnv::GAGS_initializer_random(GAGenome &g) {
	GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;	
	int fncSlots = (pSensorSim->m_pGAStrategy->useSubFunct) ? pSensorSim->m_pGAStrategy->sfncSlots : 0;

    for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct + fncSlots; slot++)
		for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) 
			genome.gene(slot * GAS_INSLENGTH + insOffs, 0);
	
    GAGS_mutator(g, 1);

	for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct; slot++) {
		genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GAS_INSACTIVE);
	}

    // INITILIALIZE SUBFUNCTIONS
    for (int slot = pSensorSim->m_pGAStrategy->maxInstruct; slot < (pSensorSim->m_pGAStrategy->maxInstruct + fncSlots); slot++) {
        genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GAS_INSACTIVE);   // subfunction call is active every time
        genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE, slot - pSensorSim->m_pGAStrategy->maxInstruct); // initialize by respective number of subfunction, e.g. 1st slot => 1
		//genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, (fncSlots == 1) ? pSensorSim->m_pGAStrategy->sfncSize : GARandomInt(0, pSensorSim->m_pGAStrategy->sfncSize - 1));
		genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, pSensorSim->m_pGAStrategy->sfncSize - 1);
        
        ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
        // LENGTH OF SUBFUNCTION STORED IN GAS_OFFSET_NODE ARGUMENT
        ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE) < pSensorSim->m_pGAStrategy->sfncCount);
    }
}

void CSensorEnv::GA_initializer_load(GAGenome &g, DWORD protocolTypeFlags) {
	// IF ENOUGH GENOMS AVAILABLE, FILL FROM LOADED 
    if (pSensorSim->m_sensorEnv.m_lastLoadedGenom > 0) {
        GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;
        
        // GET NEXT UNUSED GENOM FROM LOADED POPULATION
        GAS_Genome<GAS_ELEMENT_TYPE> loadGenome 
			= (GAS_Genome<GAS_ELEMENT_TYPE>&) 
				pSensorSim->m_sensorEnv.m_pLoadedPop->individual(
					//pSensorSim->m_pGAStrategy->popSize - pSensorSim->m_sensorEnv.m_lastLoadedGenom 
					pSensorSim->m_sensorEnv.m_lastLoadedGenom - 1
				); 
		            
        // LOAD FROM LOADED GENOME        
        for (int i = 0; i < genome.size(); i++) {
            genome.gene(i, loadGenome.gene(i));    
        }   
        // SHIFT TO NEXT GENOM
        pSensorSim->m_sensorEnv.m_lastLoadedGenom--;
    }
    else {
        // OTHERWISE FILL RANDOMLY
		if (protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED)
			GAGS_initializer_random(g);
		else
			GAS_initializer_random(g);
    }
}

void CSensorEnv::GAS_initializer_load(GAGenome &g) {
	GA_initializer_load(g, FLAG_PROTOCOL_NODESORIENTED);
}
void CSensorEnv::GAGS_initializer_load(GAGenome &g) {
	GA_initializer_load(g, FLAG_PROTOCOL_GROUPORIENTED);
}

float CSensorEnv::GAS_fitness(GAGenome &g) {
    _PERFTIME_BEGIN_

    int                     status = STAT_OK;
    GAS_Genome<GAS_ELEMENT_TYPE>   &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    GAS_ELEMENT_TYPE*       prunnedGenom = NULL;
    GAS_ELEMENT_TYPE*       origGenom = NULL;
    float                   fit = 0;

    DWORD strategyOffset = 0;
    // CLEAR STRATEGY STRING
    memset(pSensorSim->m_pGAStrategy->stratString, 0, pSensorSim->m_pGAStrategy->stratStringSize * sizeof(GAS_ELEMENT_TYPE));
    
    //
    //  OLD STYLE, NODES-ORIENTED PROTOCOL
    //
    if (pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_NODESORIENTED) {
        // COPY GENOM TO TEST ARRAY
        if (pSensorSim->m_pGAStrategy->useSubFunct) {
            // GENERATE PROTOCOL FROM SUBFUNCTIONS AS CONCATENATION
            for (int slot = pSensorSim->m_pGAStrategy->maxInstruct; slot < pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots; slot++) {
                ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
                if ((genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL) &&
                    (genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT) == GAS_INSACTIVE)) {
                    // COMPUTE OFFSET OF SUBFUNCTION IN EXISTING STRATEGY
                    int sfncOffset = genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE);
                    ASSERT(sfncOffset < pSensorSim->m_pGAStrategy->sfncCount);
                    sfncOffset *= (pSensorSim->m_pGAStrategy->sfncSize * GAS_INSLENGTH);
                    ASSERT(sfncOffset < pSensorSim->m_pGAStrategy->stratStringSize);
                    // OBTAIN NUMBER OF INSTRUCTION FOR THIS SUBFUNCTION
                    int insCount = genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1);
                    ASSERT(insCount < pSensorSim->m_pGAStrategy->sfncSize);
                    // COPY SUBFUNCTION CONTENT INTO MAIN STRATEGY STRING
                    for (int i = 0; i < insCount; i++) {
						DWORD insBase = i * GAS_INSLENGTH;
						for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) {
							pSensorSim->m_pGAStrategy->stratString[strategyOffset] = genome.gene(sfncOffset + insBase + insOffs);
							// CALCULATE OFFSET FOR THIS SUBFUNCTION IN NEW STRATEGY
							strategyOffset++;
						}
                    }
                }
            }
            
            // COMPUTE LENGTH OF RESULTING STRATEGY (IN INSTRUCTION)
            strategyOffset /= GAS_INSLENGTH;
        }
        else {
            // COPY DIRECT PROTOCOL (WITHOUT SUBFUNCTIONS)
            for (int i = 0; i < pSensorSim->m_pGAStrategy->maxInstruct; i++) {
				DWORD insBase = i * GAS_INSLENGTH;
				for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) pSensorSim->m_pGAStrategy->stratString[insBase + insOffs] = genome.gene(insBase + insOffs); 
                ASSERT(pSensorSim->m_pGAStrategy->stratString[i * GAS_INSLENGTH] != 0);
            }
            
            strategyOffset = pSensorSim->m_pGAStrategy->maxInstruct;
        }    
    }    

    //
    //  GROUP-ORIENTED PROTOCOL: protocol may contain different instructions (different protocol execution) for different neighbour based on its distance from the central node. Protocol is executed only once for group around given central node.   
    //
    if (pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUPORIENTED) {
        // COPY DIRECT PROTOCOL *WITH* SUBFUNCTIONS PART (SUBFUNCTION SERVES AS POINTER )
        for (int i = 0; i < genome.size(); i++) {
            pSensorSim->m_pGAStrategy->stratString[i] = genome.gene(i); 
            if (i % GAS_INSLENGTH == 0) {
                ASSERT(pSensorSim->m_pGAStrategy->stratString[i] != 0);
            }
        }
        
        strategyOffset = pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots;
    }
    
    //
    // EVALUATE FITNESS
    //
    pSensorSim->m_pGAStrategy->fitness = 0;
    float   pureFitness = 0;
	if ((status = pSensorSim->m_sensorEnv.GAS_ExecuteStrategy(pSensorSim->m_pGAStrategy, &genome.stats)) == STAT_OK) {
        // STORE ORIGINAL FITNESS
        pureFitness = pSensorSim->m_pGAStrategy->fitness;
        
        if (pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION) {
            if (pSensorSim->m_pGAStrategy->fitness < pSensorSim->m_pGAStrategy->minimizeMsgFit) {
                // PREVIOUS LIMIT NOT REACHED, SET FITNESS TO ZERO
                pSensorSim->m_pGAStrategy->fitness = 0;    
            }
            else {
                // SET FITNESS ACCORDING TO NUMBER OF USED MESSAGES 
                pSensorSim->m_pGAStrategy->fitness += pSensorSim->m_pGAStrategy->totalMaxMsg - genome.stats.getTotalMessages(); 
            }
        }
         
        fit = pSensorSim->m_pGAStrategy->fitness;
    }
    else {
        message.Format("GAS_ExecuteStrategy() fail with %d %s", status, ERROR_TO_STRING(status));
        LOG_INSERTSTRING(message);
    }
    
    if (status == STAT_OK) {
        // DISPLAY IMPROVEMENT IF ANY
        if (pSensorSim->m_pGAStrategy->maxFit < pSensorSim->m_pGAStrategy->fitness) {
            pSensorSim->m_pGAStrategy->maxFit = pSensorSim->m_pGAStrategy->fitness;
            pSensorSim->m_pGAStrategy->generationOfMaxFit = pSensorSim->m_pGAStrategy->actGeneration;

			for (int slot = 0; slot < strategyOffset; slot++) {
                pSensorSim->m_pGAStrategy->toString(slot, &message, pSensorSim->m_pGAStrategy->protocolTypeFlags);
                value.Format("%2d:", slot);
                value += message;
                LOG_INSERTSTRING_NOTIME(value); 
            }
			
            message.Format("New maximum found: %f (%0.4f%% secured, %0.2f messages, %d/%d SND ins)", 
				pSensorSim->m_pGAStrategy->fitness,
				genome.stats.getSecuredRatio()*100,
				genome.stats.getAverageMessages(),
				genome.stats.insCount[GAS_INS_SEND],
				genome.stats.insTotalCount
			);
			LOG_INSERTSTRING(message);
            
            if (pSensorSim->m_pGAStrategy->prunneResults) {
                if (prunnedGenom == NULL) prunnedGenom = new GAS_ELEMENT_TYPE[(pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots) * GAS_INSLENGTH];
                memset(prunnedGenom, 0, (pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots) * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
                if (origGenom == NULL) origGenom = new GAS_ELEMENT_TYPE[(pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots) * GAS_INSLENGTH];
                memset(origGenom, 0, (pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots) * GAS_INSLENGTH * sizeof(GAS_ELEMENT_TYPE));
			    int fncSlots = (pSensorSim->m_pGAStrategy->useSubFunct) ? pSensorSim->m_pGAStrategy->sfncSlots : 0;
                for (int i = 0; i < (pSensorSim->m_pGAStrategy->maxInstruct + fncSlots); i++) {
					DWORD insBase = i * GAS_INSLENGTH;
					for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) origGenom[insBase + insOffs] = pSensorSim->m_pGAStrategy->stratString[insBase + insOffs]; 
                }
                float*  insFitnessGain = NULL;
                insFitnessGain = new float[(pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots)]; 
                memset(insFitnessGain, 0, (pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots) * sizeof(float));
                
                status = pSensorSim->m_sensorEnv.GAS_PruneProtocol(origGenom, prunnedGenom, pSensorSim->m_pGAStrategy->maxInstruct, pureFitness, 3, insFitnessGain, pSensorSim->m_pGAStrategy->sfncSlots);
                
    /*          
                // COPY PRUNNED STRATEGY TO GENE
                for (int i = 0; i < pSensorSim->m_pGAStrategy->maxInstruct * GAS_INSLENGTH; i++) {
                    genome.gene(i, pSensorSim->m_pGAStrategy->stratString[i]); 
                }
    /**/            
    
                if (insFitnessGain != NULL) {
					delete[] insFitnessGain;
					insFitnessGain = NULL;
				}
            }
            
            // CHECK IF FITNESS THRESHOLD WAS REACHED AND EVENTUALLY DEPLOY NEW NETWORKS
            if (pSensorSim->m_pGAStrategy->bAddDeploymentsFitnessTreshold && pSensorSim->m_pGAStrategy->maxFit >= pSensorSim->m_pGAStrategy->moreDeploymentsTreshold) {
                // DEPLOY
                int remainingDeploy = pSensorSim->m_pGAStrategy->moreDeploymentsCounter;
                if (pSensorSim->m_sensorEnv.m_basicSettings.numDeployments + remainingDeploy <= MAX_DEPLOYMENTS) {
                    pSensorSim->m_sensorEnv.m_basicSettings.numDeployments += remainingDeploy;
            	    for (int dep = 0; dep < pSensorSim->m_sensorEnv.m_basicSettings.numDeployments; dep++) {
                        if (remainingDeploy > 0 && pSensorSim->m_sensorEnv.m_deploymentList[dep]->sensorsDeployed == FALSE) {
			                // set actual deployment
			                pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[dep];

			                // DEPLOY SENSORS
			                pSensorSim->m_sensorEnv.DeploySensors(&(pSensorSim->m_sensorEnv.m_basicSettings));
                            
                            remainingDeploy--;
                        }
                        if (remainingDeploy == 0) break;
                    }   
		            // set again first deployment struct
		            pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[0];
                    LOG_INSERTSTRING("MORE DEPLOYMENTS ADDED");
                    
                    // reset maximum
                    pSensorSim->m_pGAStrategy->maxFit = 0;
                    LOG_INSERTSTRING("MAXIMUM RESETED");

                    // FORCE TO REAVALUATE ALL GENOMS
                    pSensorSim->m_pGAStrategy->pGA->pop->evaluated = gaFalse;
		        }
		        else {
                    LOG_INSERTSTRING("ERROR: Unable to deploy more deployments (total count higher than maximum)");
		        }
            }
        } // end fit > maxFit

        // CHECK IF LIMIT OF GENERATIONS WITHOUT CHANGE HAS BEEN REACHED AND EVENTUALLY SWITCH TO 
        // MESSAGE MINIMIZING MODE
        if ((pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES) && 
            !(pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION)) {
            if ((pSensorSim->m_pGAStrategy->actGeneration - pSensorSim->m_pGAStrategy->generationOfMaxFit) > pSensorSim->m_pGAStrategy->fitnessStableGenersTreshold) {
                // WE WILL MINIMIZING MESSAGES NOW => PROTOCOL MUST ACHIVE AT LEAST ACTUAL FITNESS AND GET FITNESS BASED ON USED MESSAGES (LOWER THE BETTER) 
                pSensorSim->m_pGAStrategy->protocolTypeFlags |= FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION;
                pSensorSim->m_pGAStrategy->minimizeMsgFit = pSensorSim->m_pGAStrategy->maxFit;
                LOG_INSERTSTRING("###MESSAGE MINIMIZATION STARTED");
            }
        }
    }

	pSensorSim->m_pGAStrategy->totalFitEvals++;

    if (prunnedGenom == NULL)  delete[] prunnedGenom;
    if (origGenom == NULL)  delete[] origGenom;


    _PERFTIME_END_

    return fit;
}    

int CSensorEnv::GAS_mutator(GAGenome &g, float pmut) {
    _PERFTIME_BEGIN_
    
    GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;
    float   result = 0;
	DWORD* ins;
    
    /*
        // SWAP ACTUAL WITH OTHER RANDOMLY SELECTED
        GAS_ELEMENT_TYPE    help[GAS_INSLENGTH];
        int     ins1;
        int     i;    
        for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct; slot++) {
            if (GAFlipCoin(pmut)) {
                ins1 = GARandomInt(0, pSensorSim->m_pGAStrategy->maxInstruct - 1);  
                
                for (i = 0; i < GAS_INSLENGTH; i++) {
                    help[i] = genome.gene(i + slot * GAS_INSLENGTH);
                } 
                for (i = 0; i < GAS_INSLENGTH; i++) {
                    genome.gene(i + slot * GAS_INSLENGTH, genome.gene(i + ins1 * GAS_INSLENGTH));
                } 
                for (i = 0; i < GAS_INSLENGTH; i++) {
                    genome.gene(i + ins1 * GAS_INSLENGTH, help[i]);
                } 
            }
        }
    /**/    

    // 
    //  MUTATE PROTOCOL INSTRUCTIONS 
    //
        
    for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct; slot++) {			
		ins = &(*genome)+slot * GAS_INSLENGTH;
#ifdef _DEBUG
		_LOG_GA_SEED_
		CString str;
		str.Format("slot %3d, ins %2d %2d %2d %2d %2d %2d", slot, ins[0], ins[1], ins[2], ins[3], ins[4], ins[5]);
		LOG_INSERTSTRING_NOTIME(str);
#endif
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTYPE) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE, GARandomInt(GAS_INS_RNG, GAS_INS_DECRYPT));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_NODE) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE, GARandomInt(1, pSensorSim->m_pGAStrategy->maxParties));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM1) != STAT_OK || GAFlipCoin(pmut)) {
            if (genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SEND ||
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_REMOTECOMBINE) { 
                // PARAM1 == NODE IDENTIFICATION
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(1, pSensorSim->m_pGAStrategy->maxParties));
            }
            else {
                // NOT A NODE IDENTIFICATION
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
            }
        }
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM2) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM2, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM3) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM3, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTAT) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GARandomInt(0, 1));

        ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) <= GAS_INS_MAX);
    }
        
    // 
    //  MUTATATE SUBFUNCTIONS SLOTS, IF USED  
    //
    if (pSensorSim->m_pGAStrategy->useSubFunct) {
        for (int slot = pSensorSim->m_pGAStrategy->maxInstruct; slot < pSensorSim->m_pGAStrategy->maxInstruct + pSensorSim->m_pGAStrategy->sfncSlots; slot++) {
			ins = &(*genome)+slot * GAS_INSLENGTH;
            
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE, GAS_INS_SUBFUNCTCALL);
            if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTAT) != STAT_OK || GAFlipCoin(pmut)) 
				genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GARandomInt(0,1));
            if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_NODE) != STAT_OK || GAFlipCoin(pmut)) 
				genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE, GARandomInt(0, pSensorSim->m_pGAStrategy->sfncCount-1));
            if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM1) != STAT_OK || GAFlipCoin(pmut)) 
				genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(0, pSensorSim->m_pGAStrategy->sfncSize-1));

            ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) <= GAS_INS_MAX);
            ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE) < pSensorSim->m_pGAStrategy->sfncCount);
            ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1) < pSensorSim->m_pGAStrategy->sfncSize);
        }
    }

    _PERFTIME_END_
    
    return result;
}

int CSensorEnv::GAGS_mutator(GAGenome &g, float pmut) {
    _PERFTIME_BEGIN_
    
    GAS_Genome<GAS_ELEMENT_TYPE> &genome = (GAS_Genome<GAS_ELEMENT_TYPE>&) g;
    float   result = STAT_OK;
    
    // 
    //  MUTATE PROTOCOL INSTRUCTIONS 
    //
	DWORD* ins;
    for (int slot = 0; slot < pSensorSim->m_pGAStrategy->maxInstruct; slot++) {
		ins = &(*genome)+slot * GAS_INSLENGTH;
#ifdef _DEBUG
		_LOG_GA_SEED_
#endif    	
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTYPE) != STAT_OK || GAFlipCoin(pmut)) {
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE, GARandomInt(GAS_INS_RNG, GAS_INS_DECRYPT));			
		}
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_NODE) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE, GARandomInt(0, 0xFFFF));
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM1) != STAT_OK || GAFlipCoin(pmut)) {
            if (genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SEND ||
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_REMOTECOMBINE) { 
                // PARAM1 == NODE IDENTIFICATION
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(0, 0xFFFF));
            }
            else {
                // NOT A NODE IDENTIFICATION
                genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
            }
        }
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM2) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM2, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM3) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM3, GARandomInt(1, pSensorSim->m_pGAStrategy->partyMemorySize));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTAT) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GARandomInt(0, 1));

		ASSERT(genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) <= GAS_INS_MAX);
    }

    // 
    //  MUTATE SUBFUNCTIONS SLOTS   
    //
    int fncSlots = (pSensorSim->m_pGAStrategy->useSubFunct) ? pSensorSim->m_pGAStrategy->sfncSlots : 0;
    for (int slot = pSensorSim->m_pGAStrategy->maxInstruct; slot < pSensorSim->m_pGAStrategy->maxInstruct + fncSlots; slot++) {
		ins = &(*genome)+slot * GAS_INSLENGTH;
		genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE, GAS_INS_SUBFUNCTCALL);

//		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_INSTAT) != STAT_OK || GAFlipCoin(pmut)) 
//			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_INSTAT, GARandomInt(0, 1));
		if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_NODE) != STAT_OK /*|| GAFlipCoin(pmut)*/) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_NODE, GARandomInt(0, pSensorSim->m_pGAStrategy->sfncCount - 1));
        if (GAS_ValidateInstructionCorrectness(ins, pSensorSim->m_pGAStrategy, GAS_FLAG_PARAM1) != STAT_OK || GAFlipCoin(pmut)) 
			genome.gene(slot * GAS_INSLENGTH + GAS_OFFSET_PARAM1, GARandomInt(0, pSensorSim->m_pGAStrategy->sfncSize - 1));
    }
    
    _PERFTIME_END_
    
    return result;
}

int CSensorEnv::GAS_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    _PERFTIME_BEGIN_
    
    GAS_Genome<GAS_ELEMENT_TYPE> &parent1 = (GAS_Genome<GAS_ELEMENT_TYPE>&) p1;
    GAS_Genome<GAS_ELEMENT_TYPE> &parent2 = (GAS_Genome<GAS_ELEMENT_TYPE>&) p2;
    GAS_Genome<GAS_ELEMENT_TYPE> &offspring1 = (GAS_Genome<GAS_ELEMENT_TYPE>&) *o1;
    GAS_Genome<GAS_ELEMENT_TYPE> &offspring2 = (GAS_Genome<GAS_ELEMENT_TYPE>&) *o2;
    int cpoint = GARandomInt(1, pSensorSim->m_pGAStrategy->maxInstruct - 1) * GAS_INSLENGTH; // bod ken (v mezch od,do)
    for (int i = 0; i < pSensorSim->m_pGAStrategy->maxInstruct; i++) {
		DWORD insBase = i * GAS_INSLENGTH;
		for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) {
			if (insBase + insOffs < cpoint) {
				if (o1 != NULL) offspring1.gene(insBase + insOffs, parent1.gene(insBase + insOffs));
				if (o2 != NULL) offspring2.gene(insBase + insOffs, parent2.gene(insBase + insOffs));
			}
			else {
				if (o1 != NULL) offspring1.gene(insBase + insOffs, parent2.gene(insBase + insOffs));
				if (o2 != NULL) offspring2.gene(insBase + insOffs, parent1.gene(insBase + insOffs));
			}
		}
    }

    _PERFTIME_END_
    
    return 1; // opt pouze pro statistiku
}

int CSensorEnv::GAGS_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    _PERFTIME_BEGIN_
    
    GAS_Genome<GAS_ELEMENT_TYPE> &parent1 = (GAS_Genome<GAS_ELEMENT_TYPE>&) p1;
    GAS_Genome<GAS_ELEMENT_TYPE> &parent2 = (GAS_Genome<GAS_ELEMENT_TYPE>&) p2;
    GAS_Genome<GAS_ELEMENT_TYPE> &offspring1 = (GAS_Genome<GAS_ELEMENT_TYPE>&) *o1;
    GAS_Genome<GAS_ELEMENT_TYPE> &offspring2 = (GAS_Genome<GAS_ELEMENT_TYPE>&) *o2;

    // COPY FNC CALL (NO CHANGE)
    int fncSlots = (pSensorSim->m_pGAStrategy->useSubFunct) ? pSensorSim->m_pGAStrategy->sfncSlots : 0;
    for (int k = (pSensorSim->m_pGAStrategy->maxInstruct * GAS_INSLENGTH); k < ((pSensorSim->m_pGAStrategy->maxInstruct + fncSlots) * GAS_INSLENGTH); k++) {
        if (o1 != NULL) offspring1.gene(k, parent1.gene(k));
        if (o2 != NULL) offspring2.gene(k, parent2.gene(k));
    }

    // CROSSING ONLY WHOLE SUBFUNCTION PARTS
    int cpoint = GARandomInt(1, fncSlots - 1) * pSensorSim->m_pGAStrategy->sfncSize; // crossing point
    for (int i = 0; i < pSensorSim->m_pGAStrategy->maxInstruct; i++) {
		DWORD insBase = i * GAS_INSLENGTH;
		for (int insOffs = 0; insOffs < GAS_INSLENGTH; insOffs++) {
			if (insBase + insOffs < cpoint) {
				if (o1 != NULL) offspring1.gene(insBase + insOffs, parent1.gene(insBase + insOffs));
				if (o2 != NULL) offspring2.gene(insBase + insOffs, parent2.gene(insBase + insOffs));
			}
			else {
				if (o1 != NULL) offspring1.gene(insBase + insOffs, parent2.gene(insBase + insOffs));
				if (o2 != NULL) offspring2.gene(insBase + insOffs, parent1.gene(insBase + insOffs));
			}
		}
    }
    
/*
    int cpoint = GARandomInt(1, pSensorSim->m_pGAStrategy->sfncSlots - 1); // crossing point
    // CROSSING ONLY IN SUNCFUNCTION CALL PART OFFSETS
    for (int i = 0; i < pSensorSim->m_pGAStrategy->sfncSlots; i++) {
        if (i < cpoint) {
            // COPY SUBFUNCTION AREA 
            for (int k = i * pSensorSim->m_pGAStrategy->sfncSize; k < ((i + 1) * pSensorSim->m_pGAStrategy->sfncSize); k++) {
                if (offspring1) offspring1.gene(k, parent1.gene(k));
                if (offspring2) offspring2.gene(k, parent2.gene(k));
            }
            // COPY FNC CALL
            int sfncSlot = pSensorSim->m_pGAStrategy->maxInstruct + i;
            for (int k = sfncSlot * GAS_INSLENGTH; k < ((sfncSlot + 1) * GAS_INSLENGTH); k++) {
                if (offspring1) offspring1.gene(k, parent1.gene(k));
                if (offspring2) offspring2.gene(k, parent2.gene(k));
            }
            ASSERT(offspring1.gene(sfncSlot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
            ASSERT(offspring2.gene(sfncSlot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
        }
        else {
            // COPY SUBFUNCTION AREA 
            for (int k = i * pSensorSim->m_pGAStrategy->sfncSize; k < ((i + 1) * pSensorSim->m_pGAStrategy->sfncSize); k++) {
                if (offspring1) offspring1.gene(k, parent2.gene(k));
                if (offspring2) offspring2.gene(k, parent1.gene(k));
            }
            // COPY FNC CALL
            int sfncSlot = pSensorSim->m_pGAStrategy->maxInstruct + i;
            for (int k = sfncSlot * GAS_INSLENGTH; k < ((sfncSlot + 1) * GAS_INSLENGTH); k++) {
                if (offspring1) offspring1.gene(k, parent2.gene(k));
                if (offspring2) offspring2.gene(k, parent1.gene(k));
            }
            ASSERT(offspring1.gene(sfncSlot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
            ASSERT(offspring2.gene(sfncSlot * GAS_INSLENGTH + GAS_OFFSET_INSTYPE) == GAS_INS_SUBFUNCTCALL);
        }
    }
/**/    
    _PERFTIME_END_
    
    return 1;
}
int  CSensorEnv::GABP_Evolve(GA_STRATEGY* pStrategy, BOOL bBlackPattern) {
    int                 status = STAT_OK;
    CString             message;
    CString             value;
    CTimeSpan           ts(0, 3, 0, 0 ); 
    CTime               lastSave = CTime::GetCurrentTime();
    CTime               actualTime = CTime::GetCurrentTime();
    GA1DArrayGenome<int>* pGenomeTemp = NULL;
    
    pSensorSim->m_pGAStrategy->maxFit = 0;

    message.Format("GA# Num. parties: %d, MaxInstr: %d, Memory size: %d, Protocol repeat: %d", pSensorSim->m_pGAStrategy->maxParties, pSensorSim->m_pGAStrategy->maxInstruct, pSensorSim->m_pGAStrategy->partyMemorySize, pSensorSim->m_pGAStrategy->numRepeat);    
    LOG_INSERTSTRING(message); 
    message.Format("GA# PopSize: %d, NumGener: %d, MutProb: %f, CrossProb: %f", pSensorSim->m_pGAStrategy->popSize, pSensorSim->m_pGAStrategy->nGeners, pSensorSim->m_pGAStrategy->pMutt, pSensorSim->m_pGAStrategy->pCross);    
    LOG_INSERTSTRING(message); 
    
    if (bBlackPattern) {
        int blackPerCell = pSensorSim->m_sensorEnv.m_basicSettings.blackNum / pow((float) pStrategy->blackDeployPattern.numCellPerAxis, 2);
        message.Format("GABP# BlackPerCell: %d, CellsPerAxis: %d, TotalWhite: %d, TotalBlack: %d", blackPerCell, pStrategy->blackDeployPattern.numCellPerAxis, pSensorSim->m_sensorEnv.m_basicSettings.whiteNum, pSensorSim->m_sensorEnv.m_basicSettings.blackNum);    
        LOG_INSERTSTRING(message); 
        pGenomeTemp = new GA1DArrayGenome<int>(blackPerCell * 2, CSensorEnv::GABP_fitness);
    }
    else {
        int whitePerCell = pSensorSim->m_sensorEnv.m_basicSettings.whiteNum / pow((float) pStrategy->whiteDeployPattern.numCellPerAxis, 2);
        message.Format("GAWP# WhitePerCell: %d, CellsPerAxis: %d, TotalWhite: %d, TotalBlack: %d", whitePerCell, pStrategy->whiteDeployPattern.numCellPerAxis, pSensorSim->m_sensorEnv.m_basicSettings.whiteNum, pSensorSim->m_sensorEnv.m_basicSettings.blackNum);    
        LOG_INSERTSTRING(message); 
        pGenomeTemp = new GA1DArrayGenome<int>(whitePerCell * 2, CSensorEnv::GAWP_fitness);
    }    
    
    //
    //  EVOLVE STEP-BY-STEP
    //
    int actGener = 1;
    pStrategy->pGA->pop->evaluated = gaFalse;
    while (pSensorSim->m_pGAStrategy->pGA->statistics().maxEver() < 1 && actGener < pSensorSim->m_pGAStrategy->nGeners) {
        // CHECK FOR SOFT INTERUPT
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            CString value;
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);

            // SET ACTIVE THREAD PTR TO NULL
            pSensorSim->m_pActualThread = NULL;
            
            return 0;
        }        

		pSensorSim->m_pGAStrategy->pGA->step(); // krok GA
        
        *pGenomeTemp = (GA1DArrayGenome<int>&) pSensorSim->m_pGAStrategy->pGA->statistics().bestIndividual(); 
        float fit = (bBlackPattern) ? CSensorEnv::GABP_fitness(*pGenomeTemp) : CSensorEnv::GAWP_fitness(*pGenomeTemp);
        message.Format("(%d gen.): %f best", actGener, fit); 
        LOG_INSERTSTRING(message); 
        
		// SAVE ACTUAL STATUS INTO RUNNING INSTANCES STRING
        SaveInstanceLog(message);

        // SAVE POPULATION TO FILE EACH 3 HOURS OR 1000 GENERATION
        actualTime = CTime::GetCurrentTime();
        if ((actGener % 1000 == 1) || (actualTime - ts  > lastSave)) {
            lastSave = CTime::GetCurrentTime();

            CString fileName;
            fileName.Format("population_%d_%d.txt", pSensorSim->m_numRepetitionsElapsed, actGener);
            DeleteFile(fileName);
            // SAVE ACTUAL GENERATION HEADER
            pSensorSim->Log_InsertToFile(fileName, "########## ACTUAL POPULATION #############\n"); 
            message.Format("%s\n", GABP_HEADER_BEGIN);
            pSensorSim->Log_InsertToFile(fileName, message); 
            message = "";
            pStrategy->settingsToString(&message);
            pSensorSim->Log_InsertToFile(fileName, message); 
            pSensorSim->Log_InsertToFile(fileName, GABP_HEADER_END); 
            pSensorSim->Log_InsertToFile(fileName, GABP_SEPARATOR); 
            // SAVE GENOMS
            GADP_SavePopulation(fileName, pSensorSim->m_pGAStrategy->pGA, pGenomeTemp);
        }
        
        actGener++;
    }
    
    // 
    // SHOW BEST GENOM EVER
    //
    *pGenomeTemp = (GA1DArrayGenome<int>&) pSensorSim->m_pGAStrategy->pGA->statistics().bestIndividual(); 

    for (int i = 0; i < pGenomeTemp->size() / 2; i++) {
        value.Format("%3d: [%d, %d]", i, pGenomeTemp->gene(i * 2), pGenomeTemp->gene(i * 2 + 1));
        value += "\n";
        LOG_INSERTSTRING_NOTIME(value); 
    }
    for (int i = 0; i < pGenomeTemp->size() / 2; i++) {
        value.Format("%.3f %.3f\n", pGenomeTemp->gene(i * 2) / (float) INT_MAX, pGenomeTemp->gene(i * 2 + 1) / (float) INT_MAX);
        LOG_INSERTSTRING_NOTIME(value); 
    }
    GABP_FormatAsciiPattern(pGenomeTemp, &value, TRUE); 
    //LOG_INSERTSTRING(value, FALSE, FALSE); 
    LOG_INSERTSTRING_NOTIME(value); 
    GABP_FormatAsciiPattern(pGenomeTemp, &value, FALSE); 
    pSensorSim->Log_InsertToFile(pSensorSim->m_logFile, value); 
                    
    
    float fit = (bBlackPattern) ? CSensorEnv::GABP_fitness(*pGenomeTemp) : CSensorEnv::GAWP_fitness(*pGenomeTemp);
    message.Format("Best fitness: %f", fit); 
    LOG_INSERTSTRING(message); 
    
    if (pGenomeTemp) delete pGenomeTemp;
    return status;    
}

int CSensorEnv::GABWP_Evolve(GA_STRATEGY* pWhiteStrategy, GA_STRATEGY* pBlackStrategy) {
    int                 status = STAT_OK;
    CString             message;
    CString             value;
    CTimeSpan           ts(0, 3, 0, 0 ); 
    CTime               lastSave = CTime::GetCurrentTime();
    CTime               actualTime = CTime::GetCurrentTime();
    GA1DArrayGenome<int>* pBlackGenomeTemp = NULL;
    GA1DArrayGenome<int>* pWhiteGenomeTemp = NULL;
    GA_STRATEGY*        pOldStrategy;
    
    pWhiteStrategy->maxFit = 0;
    pBlackStrategy->maxFit = 0;

    message.Format("# Num. parties: %d, MaxInstr: %d, Memory size: %d, Protocol repeat: %d", pWhiteStrategy->maxParties, pWhiteStrategy->maxInstruct, pWhiteStrategy->partyMemorySize, pWhiteStrategy->numRepeat);    
    LOG_INSERTSTRING(message); 
    message.Format("GA# PopSize: %d, NumGener: %d, MutProb: %f, CrossProb: %f", pWhiteStrategy->popSize, pWhiteStrategy->nGeners, pWhiteStrategy->pMutt, pWhiteStrategy->pCross);    
    LOG_INSERTSTRING(message); 
    
    int blackPerCell = pSensorSim->m_sensorEnv.m_basicSettings.blackNum / pow((float) pBlackStrategy->blackDeployPattern.numCellPerAxis, 2);
    message.Format("GABP# BlackPerCell: %d, CellsPerAxis: %d, TotalBlack: %d", blackPerCell, pBlackStrategy->blackDeployPattern.numCellPerAxis, pSensorSim->m_sensorEnv.m_basicSettings.blackNum);    
    LOG_INSERTSTRING(message); 
    pBlackGenomeTemp = new GA1DArrayGenome<int>(blackPerCell * 2, CSensorEnv::GABP_fitness);
    pBlackStrategy->pBestGenom = new GA1DArrayGenome<int>(blackPerCell * 2, CSensorEnv::GABP_fitness);

    int whitePerCell = pSensorSim->m_sensorEnv.m_basicSettings.whiteNum / pow((float) pWhiteStrategy->whiteDeployPattern.numCellPerAxis, 2);
    message.Format("GAWP# WhitePerCell: %d, CellsPerAxis: %d, TotalWhite: %d", whitePerCell, pWhiteStrategy->whiteDeployPattern.numCellPerAxis, pSensorSim->m_sensorEnv.m_basicSettings.whiteNum);    
    LOG_INSERTSTRING(message); 
    pWhiteGenomeTemp = new GA1DArrayGenome<int>(whitePerCell * 2, CSensorEnv::GAWP_fitness);
    pWhiteStrategy->pBestGenom = new GA1DArrayGenome<int>(whitePerCell * 2, CSensorEnv::GAWP_fitness);
    
    //
    //  EVOLVE STEP-BY-STEP, ONE STEP FOR BLACK, ONE STEP FOR WHITE
    //
    int actGener = 1;
    // SAVE OROGINAL STRATEGY POINTER
    pOldStrategy = pSensorSim->m_pGAStrategy;
    while (pWhiteStrategy->pGA->statistics().maxEver() < 1 && actGener < pWhiteStrategy->nGeners) {
        // SWITCH GLOBAL STRUCT TO WHITE STRATEGY 
        pSensorSim->m_pGAStrategy = pWhiteStrategy;
        pWhiteStrategy->maxFitGener = -1;
        
        // STEP OF WHITE POPULATION
        pWhiteStrategy->pGA->step();  
        
        // OBTAIN BEST WHITE GENOM AND DEPLOY WHITE NODES ACCORDING IT (DONE INSIDE CSensorEnv::GAWP_fitness FUNCTION) 
        message.Format("Best white (%d gen.): %f", actGener, CSensorEnv::GAWP_fitness(*(pWhiteStrategy->pBestGenom))); 
        LOG_INSERTSTRING(message); 
        
        // SWITCH GLOBAL STRUCT TO BLACK STRATEGY 
        pSensorSim->m_pGAStrategy = pBlackStrategy;
        pBlackStrategy->maxFitGener = -1;
        
        // STEP OF BLACK POPULATION
        pBlackStrategy->pGA->step();  

        // OBTAIN BEST BLACK GENOM AND DEPLOY BLACK NODES ACCORDING IT (DONE INSIDE CSensorEnv::GABP_fitness FUNCTION) 
        message.Format("Best black (%d gen.): %f", actGener, CSensorEnv::GABP_fitness(*(pBlackStrategy->pBestGenom))); 
        LOG_INSERTSTRING(message); 

        // SHOW BEST WHITE AND BLACK EACH 50th GENERATION
        if (actGener % 10 == 1) {
            LOG_INSERTSTRING("Best white genom"); 
            GABP_FormatAsciiPattern(pWhiteStrategy->pBestGenom, &value, TRUE); 
            //LOG_INSERTSTRING_NOTIME(value, FALSE, FALSE); 
            LOG_INSERTSTRING_NOTIME(value); 
            LOG_INSERTSTRING("Best black genom"); 
            GABP_FormatAsciiPattern(pBlackStrategy->pBestGenom, &value, TRUE); 
//            LOG_INSERTSTRING_NOTIME(value, FALSE, FALSE); 
            LOG_INSERTSTRING_NOTIME(value); 
        }
        
        // SAVE POPULATION TO FILE EACH 3 HOURS OR 1000 GENERATION
        actualTime = CTime::GetCurrentTime();
        if ((actGener % 1000 == 1) || (actualTime - ts  > lastSave)) {
            lastSave = CTime::GetCurrentTime();

            CString fileName;
            //
            // SAVE ACTUAL GENERATION OF BLACK
            //
            fileName.Format("population_black_%d_%d.txt", pSensorSim->m_numRepetitionsElapsed, actGener);
            DeleteFile(fileName);
            // SAVE ACTUAL GENERATION HEADER
            pSensorSim->Log_InsertToFile(fileName, "########## ACTUAL POPULATION #############\n"); 
            message.Format("%s\n", GABP_HEADER_BEGIN);
            pSensorSim->Log_InsertToFile(fileName, message); 
            message = ""; pBlackStrategy->settingsToString(&message);
            pSensorSim->Log_InsertToFile(fileName, message); 
            pSensorSim->Log_InsertToFile(fileName, GABP_HEADER_END); 
            pSensorSim->Log_InsertToFile(fileName, GABP_SEPARATOR); 

            GADP_SavePopulation(fileName, pBlackStrategy->pGA, pBlackGenomeTemp);
            
            
            //
            // SAVE ACTUAL GENERATION OF WHITE
            //
            fileName.Format("population_white_%d_%d.txt", pSensorSim->m_numRepetitionsElapsed, actGener);
            DeleteFile(fileName);
            // SAVE ACTUAL GENERATION HEADER
            pSensorSim->Log_InsertToFile(fileName, "########## ACTUAL POPULATION #############\n"); 
            message.Format("%s\n", GABP_HEADER_BEGIN);
            pSensorSim->Log_InsertToFile(fileName, message); 
            message = ""; pWhiteStrategy->settingsToString(&message);
            pSensorSim->Log_InsertToFile(fileName, message); 
            pSensorSim->Log_InsertToFile(fileName, GABP_HEADER_END); 
            pSensorSim->Log_InsertToFile(fileName, GABP_SEPARATOR); 

            GADP_SavePopulation(fileName, pBlackStrategy->pGA, pWhiteGenomeTemp);
        }
        
        actGener++;
    }

    if (pBlackGenomeTemp) delete pBlackGenomeTemp;
    if (pWhiteGenomeTemp) delete pWhiteGenomeTemp;
    if (pWhiteStrategy->pBestGenom) {
        delete pWhiteStrategy->pBestGenom;
        pWhiteStrategy->pBestGenom = NULL;
    }
    if (pBlackStrategy->pBestGenom) {
        delete pBlackStrategy->pBestGenom;
        pBlackStrategy->pBestGenom = NULL;
    }
    
    // RESTORE OROGINAL STRATEGY POINTER
    pSensorSim->m_pGAStrategy = pOldStrategy;
    
    return status;    
}  

   
void CSensorEnv::GABP_initializer_random(GAGenome &g) {
    GA1DArrayGenome<int> &genome = (GA1DArrayGenome<int>&) g;
    
    for (int i = 0; i < genome.size() / 2; i++) {
        int rnd = 0;
        // INITILIZE X AXIS
        pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(INT_MAX, &rnd);
        genome.gene(i * 2, rnd); 
        // INITIALIZE Y AXIS
        pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(INT_MAX, &rnd);
        genome.gene(i * 2 + 1, rnd);
    }   
    
    GABP_sort(&genome);
}

float CSensorEnv::GABP_fitness(GAGenome &g) {
    return GADP_fitness(g, TRUE, FALSE);
}

float CSensorEnv::GAWP_fitness(GAGenome &g) {
    return GADP_fitness(g, FALSE, TRUE);
}

float CSensorEnv::GADP_fitness(GAGenome &g, BOOL bBlack, BOOL bWhite) {
    int                     status = STAT_OK;
    GA1DArrayGenome<int>   &genome = (GA1DArrayGenome<int>&) g;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;

    // #ALL DEPLOYMENTS
	for (int dep = 0; dep < pSensorSim->m_sensorEnv.m_basicSettings.numDeployments; dep++) {
		// set actual deployment
		pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[dep];
    
        // REMOVE PREVIOUS BLACK NODES FROM DEPLYED POPULATION, IF REQUESTED
        if (status == STAT_OK && bBlack) {
            status = RemoveDeployedNodes(FLAG_SENSOR_BLACK);
        }
        // REMOVE PREVIOUS WHITE NODES FROM DEPLOYED POPULATION, IF REQUESTED
        if (status == STAT_OK && bWhite) {
            status = RemoveDeployedNodes(FLAG_SENSOR_WHITE);
        }
            
        // DEPLOY NEW BLACK NODES ACCORDING TO GENOM, IF REQUESTED
        if (status == STAT_OK && bBlack) {
            for (int i = 0; i < genome.size(); i++) {
                pSensorSim->m_pGAStrategy->blackDeployPattern.pattern[i] = genome.gene(i);
            }
            pSensorSim->m_pGAStrategy->blackDeployPattern.numNodes = genome.size() / 2;

            status = DeployBlackNodesByPattern(&(pSensorSim->m_sensorEnv.m_basicSettings), &(pSensorSim->m_pGAStrategy->blackDeployPattern));
        }
        // DEPLOY NEW WHITE NODES ACCORDING TO GENOM, IF REQUESTED
        if (status == STAT_OK && bWhite) {
            for (int i = 0; i < genome.size(); i++) {
                pSensorSim->m_pGAStrategy->whiteDeployPattern.pattern[i] = genome.gene(i);
            }
            pSensorSim->m_pGAStrategy->whiteDeployPattern.numNodes = genome.size() / 2;

            status = DeployWhiteNodesByPattern(&(pSensorSim->m_sensorEnv.m_basicSettings), &(pSensorSim->m_pGAStrategy->whiteDeployPattern));
        }
    }
    
	// set again first deployment struct
	pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[0];
    
    // EVALUATE FITNESS
    pSensorSim->m_pGAStrategy->fitness = 0;
	GA_PROTOCOL_STATS stats;
    if ((status = pSensorSim->m_sensorEnv.GAS_ExecuteStrategy(pSensorSim->m_pGAStrategy, &stats)) == STAT_OK) {
        // LOWER pSensorSim->m_pGAStrategy->fitness MEANS BETTER FITNESS FOR BLACK PATTERN
        if (bBlack) {
            fit = 1 - pSensorSim->m_pGAStrategy->fitness;
            value.Format("Black genom fitness: %f", fit);
//            LOG_INSERTSTRING(value); 
        }
        if (bWhite) {
            fit = pSensorSim->m_pGAStrategy->fitness;
            value.Format("White genom fitness: %f", fit);
//            LOG_INSERTSTRING(value); 
        }

        // SAVE CONTENT OF BEST GENOM FOR THIS GENERATION, IF REQUIRED
        if (pSensorSim->m_pGAStrategy->pBestGenom != NULL && pSensorSim->m_pGAStrategy->maxFitGener < fit) {
            pSensorSim->m_pGAStrategy->maxFitGener = fit;   
            *(pSensorSim->m_pGAStrategy->pBestGenom) = (GAGenome&) g;   
        }


        // DISPLAY IMPROVEMENT IF ANY
        if (pSensorSim->m_pGAStrategy->maxFit < fit) {
            pSensorSim->m_pGAStrategy->maxFit = fit;

            if (bBlack) LOG_INSERTSTRING("Black deploy pattern"); 
            if (bWhite) LOG_INSERTSTRING("White deploy pattern"); 

            GA1DArrayGenome<int> &genomeTemp = (GA1DArrayGenome<int>&) g;
            for (int i = 0; i < genomeTemp.size() / 2; i++) {
                value.Format("%3d: [%d, %d]", i, genome.gene(i * 2), genome.gene(i * 2 + 1));
                value += "\n";
                LOG_INSERTSTRING_NOTIME(value); 
            }
            for (int i = 0; i < genomeTemp.size() / 2; i++) {
                value.Format("%.3f %.3f\n", genomeTemp.gene(i * 2) / (float) INT_MAX, genomeTemp.gene(i * 2 + 1) / (float) INT_MAX);
                LOG_INSERTSTRING_NOTIME(value); 
            }
            
            GABP_FormatAsciiPattern(&genomeTemp, &value, TRUE); 
            //LOG_INSERTSTRING_NOTIME(value, FALSE, FALSE); 
            LOG_INSERTSTRING_NOTIME(value); 
            GABP_FormatAsciiPattern(&genomeTemp, &value, FALSE); 
            pSensorSim->Log_InsertToFile(pSensorSim->m_logFile, value); 

            message.Format("New maximum found: %f", fit);
            LOG_INSERTSTRING(message);
            
            // CHECK IF FITNESS TRESHOLD REACHED AND EVENTUALLY DEPLOY NEW NETWORKS
            if (pSensorSim->m_pGAStrategy->bAddDeploymentsFitnessTreshold && fit >= pSensorSim->m_pGAStrategy->moreDeploymentsTreshold) {
                // DEPLOY
                int remainingDeploy = pSensorSim->m_pGAStrategy->moreDeploymentsCounter;
                if (pSensorSim->m_sensorEnv.m_basicSettings.numDeployments + remainingDeploy <= MAX_DEPLOYMENTS) {
                    pSensorSim->m_sensorEnv.m_basicSettings.numDeployments += remainingDeploy;
            	    for (int dep = 0; dep < pSensorSim->m_sensorEnv.m_basicSettings.numDeployments; dep++) {
                        if (remainingDeploy > 0 && pSensorSim->m_sensorEnv.m_deploymentList[dep]->sensorsDeployed == FALSE) {
			                // set actual deployment
			                pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[dep];

			                // DEPLOY SENSORS
			                pSensorSim->m_sensorEnv.DeploySensors(&(pSensorSim->m_sensorEnv.m_basicSettings));
                            
                            remainingDeploy--;
                        }
                        if (remainingDeploy == 0) break;
                    }   
		            // set again first deployment struct
		            pSensorSim->m_sensorEnv.m_pActualDeployment = pSensorSim->m_sensorEnv.m_deploymentList[0];
                    LOG_INSERTSTRING("MORE DEPLOYMENTS ADDED");
                    
                    // reset maximum
                    pSensorSim->m_pGAStrategy->maxFit = 0;
                    LOG_INSERTSTRING("MAXIMUM RESETED");
                    
                    // FORCE TO REAVALUATE ALL GENOMS
                    pSensorSim->m_pGAStrategy->pGA->pop->evaluated = gaFalse;
		        }
		        else {
                    LOG_INSERTSTRING("ERROR: Unable to deploy more deployments (total count higher than maximum)");
		        }
            }
        }
    }
    else {
        message.Format("GAS_ExecuteStrategy() fail with %d %s", status, ERROR_TO_STRING(status));
        LOG_INSERTSTRING(message);
    }

    return fit;
}

int CSensorEnv::GABP_mutator(GAGenome &g, float pmut) {
    GA1DArrayGenome<int> &genome = (GA1DArrayGenome<int>&) g;
    
    for (int i = 0; i < genome.size() / 2; i++) {
        // COMPETELY RANDOM MUTATION
        int rnd = 0;
        if (GAFlipCoin(pmut)) {
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(INT_MAX, &rnd);
            genome.gene(i * 2, rnd); 
        }
        if (GAFlipCoin(pmut)) {
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(INT_MAX, &rnd);
            genome.gene(i * 2 + 1, rnd);
        }
       

/* 
        
        //SLIDE MUTATION (EXISTING VALUES ARE ONLY CHANGED BY LIMITED STEP)
        // X AXIS
        if (GAFlipCoin(pmut)) {
            // OBTAIN ORIGINAL VALE
            int orig = genome.gene(i * 2);
            // GENERATE +- MUTATION SHIFT
            int mutStep;
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(GABP_MAXMUTSTEP, &mutStep);
            if (GARandomInt(0,1) == 0) mutStep = -mutStep;
            // DO NOT GO OUTSIDE RANGE, CYCLIC TRANSITION FROM ONE SIDE TO OTHER
            if (orig + mutStep > INT_MAX) orig = orig + mutStep - INT_MAX;
            if (orig + mutStep < 0) orig = INT_MAX + (orig + mutStep);
            // SAVE TO GENOM
            genome.gene(i * 2, orig);
        }
        // Y AXIS
        if (GAFlipCoin(pmut)) {
            // OBTAIN ORIGINAL VALE
            int orig = genome.gene(i * 2 + 1);
            // GENERATE +- MUTATION SHIFT
            int mutStep;
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(GABP_MAXMUTSTEP, &mutStep);
            if (GARandomInt(0, 1) == 0) mutStep = -mutStep;
            // DO NOT GO OUTSIDE RANGE, CYCLIC TRANSITION FROM ONE SIDE TO OTHER
            if (orig + mutStep > INT_MAX) orig = orig + mutStep - INT_MAX;
            if (orig + mutStep < 0) orig = INT_MAX + (orig + mutStep);
            // SAVE TO GENOM
            genome.gene(i * 2 + 1, orig);
        }
/**/        
    }   
    
    GABP_sort(&genome);
    
    return 0;
}

int CSensorEnv::GABP_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    GA1DArrayGenome<int> &parent1 = (GA1DArrayGenome<int>&) p1;
    GA1DArrayGenome<int> &parent2 = (GA1DArrayGenome<int>&) p2;
    GA1DArrayGenome<int> &offspring1 = (GA1DArrayGenome<int>&) *o1;
    GA1DArrayGenome<int> &offspring2 = (GA1DArrayGenome<int>&) *o2;
    int cpoint = GARandomInt(1, parent1.size() / 2) * 2; // CROSSING POINT, WILL NOT SPLIT X AND Y AXIS
    for (int i = 0; i < parent1.size(); i++) {
        if (i < cpoint) {
            if (o1 != NULL) offspring1.gene(i, parent1.gene(i));
            if (o2 != NULL) offspring2.gene(i, parent2.gene(i));
        }
        else {
            if (o1 != NULL) offspring1.gene(i, parent2.gene(i));
            if (o2 != NULL) offspring2.gene(i, parent1.gene(i));
        }
    }
    
    // SORT POINTS CODED IN GENOM BY AXIS VALUES
    if (o1 != NULL) GABP_sort(&offspring1);
    if (o2 != NULL) GABP_sort(&offspring2);
    
    return 1; 
}

int CSensorEnv::GABP_sort(GA1DArrayGenome<int>* pGenom) {
    int     status = STAT_OK;
    int*    tempArray = NULL;
    
    tempArray = new int[pGenom->size()];
    memset(tempArray, 0, pGenom->size() * sizeof(int));
    
    // SORT POINT IN GENOM FIRST BASED ON X-AXIs, THEN Y-AXIS
    int minX = INT_MAX;
    int minY = INT_MAX;
    for (int i = 0; i < pGenom->size() / 2; i++) {
        minX = INT_MAX;
        minY = INT_MAX;
        for (int j = 0; j < pGenom->size() / 2; j++) {
            // FIND LOWEST X
            if (pGenom->gene(j * 2) < minX) {
                minX = pGenom->gene(j * 2);        
                minY = pGenom->gene(j * 2 + 1); 
            }
            else {
                // SAME X, SORT BY Y 
                if ((pGenom->gene(j * 2) == minX) && (pGenom->gene(j * 2 + 1) < minY)) {
                    minY = pGenom->gene(j * 2 + 1);     
                }        
            }
        }
        
        // CLEAR ACTUAL MINIMUM FOR THIS ROUND TO PREVENT TO BE CHOOSED AGAIN
        for (int j = 0; j < pGenom->size() / 2; j++) {
            if (pGenom->gene(j * 2) == minX && pGenom->gene(j * 2 + 1) == minY) {
                pGenom->gene(j * 2, INT_MAX);        
                pGenom->gene(j * 2 + 1, INT_MAX); 
                break;
            }
            
        }        
        
        // COPY MINIMUm TO TEMP ARRAY
        tempArray[i * 2] = minX;
        tempArray[i * 2 + 1] = minY;
    }
    
    // COPY BACK TO ORIGINAL GENOM
    for (int j = 0; j < pGenom->size() / 2; j++) {
        pGenom->gene(j * 2, tempArray[j * 2]);        
        pGenom->gene(j * 2 + 1, tempArray[j * 2 + 1]);        
    }        
    
    if (tempArray) delete[] tempArray;
    
    return status;       
}

int CSensorEnv::GABP_FormatAsciiPattern(GAGenome* pG, CString* pValue, BOOL bToScreen) {
    GA1DArrayGenome<int>* pGenome = (GA1DArrayGenome<int>*) pG;
#define     RESOLUTION  10
    CString lineChar;
    CString nodeChar;
    CString emptyChar;
    
    if (bToScreen) {
        // PROPORTIONAL FONT
        lineChar = "----"; 
        nodeChar = " + ";   
        emptyChar = "    ";
    } 
    else {
        // NONPROPRTIONAL FONT
        lineChar = "--"; 
        nodeChar = " +";   
        emptyChar = "  ";
    }

    *pValue = "";
    for (int i = 0; i < 2 * RESOLUTION; i++) *pValue += lineChar;
    *pValue += "---\r\n";

    float   resStep = 1 / (float) RESOLUTION;
    for (int j = 0; j < RESOLUTION; j++) {
        CString line;
        line += "|";
    
        for (int k = 0; k < RESOLUTION; k++) {
            CString character = emptyChar;
            for (int i = 0; i < pGenome->size() / 2; i++) {
                float x = pGenome->gene(i * 2) / (float) INT_MAX;
                float y = pGenome->gene(i * 2 + 1) / (float) INT_MAX;
                if (x >= (k * resStep) && x < ((k + 1) * resStep) && y >= (j * resStep)  && y < ((j + 1) * resStep)) { 
                    character = nodeChar;
                }
            }
            line += character;
        }  
        *pValue += line + line + "|\r\n";
    }
    
    CString temp = *pValue;
    *pValue += temp;

    for (int i = 0; i < 2 * RESOLUTION; i++) *pValue += lineChar;
    *pValue += "---\r\n";
        
    pValue->Insert(0, "\r\n");        
    return STAT_OK;    
} 

int CSensorEnv::GABP_FormatAsciiPattern(NODES_DEPLOY_PATTERN* pDeployPattern, CString* pValue, BOOL bToScreen) {
    int         status = STAT_OK;
    GA1DArrayGenome<int> genomeTemp(pDeployPattern->numNodes * 2, CSensorEnv::GAS_fitness);

    // COPY TO GENOME STRUCTURE
    for (int i = 0; i < pDeployPattern->numNodes * 2; i++) {
        genomeTemp.gene(i, pDeployPattern->pattern[i]);
    }
    // FORMAT STRING
    status = GABP_FormatAsciiPattern(&genomeTemp, pValue, bToScreen);
    
    return status;
} 

int CSensorEnv::GADP_SavePopulation(CString fileName, GASteadyStateGA* pGA, GA1DArrayGenome<int>* pGenomeTemp) {
    int     status = STAT_OK;
    CString message;
    
    // SAVE ACTUAL SETTINGS
    GAPopulation pop = pGA->population();
    for (int nGen = 0; nGen < pop.size(); nGen++) {
        message.Format("%s\n", GABP_GENOM_BEGIN);
        pSensorSim->Log_InsertToFile(fileName, message); 
        *pGenomeTemp = (GA1DArrayGenome<int>&) pop.individual(nGen); 
        
        CString value;
        for (int i = 0; i < pGenomeTemp->size() / 2; i++) {
            value.Format("%3d: [%d, %d]", i, pGenomeTemp->gene(i * 2), pGenomeTemp->gene(i * 2 + 1));
            value += "\n";
            pSensorSim->Log_InsertToFile(fileName, value); 
        }
        for (int i = 0; i < pGenomeTemp->size() / 2; i++) {
            value.Format("%.3f %.3f\n", pGenomeTemp->gene(i * 2) / (float) INT_MAX, pGenomeTemp->gene(i * 2 + 1) / (float) INT_MAX);
            pSensorSim->Log_InsertToFile(fileName, value); 
        }
        GABP_FormatAsciiPattern(pGenomeTemp, &value, FALSE); 
        pSensorSim->Log_InsertToFile(fileName, value); 

        pSensorSim->Log_InsertToFile(fileName, GABP_GENOM_END); 
        pSensorSim->Log_InsertToFile(fileName, GABP_SEPARATOR); 
    }
    
    pSensorSim->Log_InsertToFile(fileName, "########## END ACTUAL POPULATION #############\n"); 

    return status;
}

int CSensorEnv::GADP_ParsePattern(CString pattern, NODES_DEPLOY_PATTERN* pPattern) {
    int     status = STAT_OK;
    int     pos = 0;
    int     pos2 = 0;
    int     pos3 = 0;
    int     pos4 = 0;
    DWORD   slot = 0;
    CString line;
    CString value;
    
    pPattern->numNodes = 0;
    
    pattern += "\n";
    while (pos < pattern.GetLength()) {
        if ((pos2 = pattern.Find('\n', pos)) != -1) {
            line = pattern.Mid(pos, pos2 - pos);        
            line.TrimLeft();
            line.TrimRight();
            
            // NOTE: line IN FORMAT 'num:[x_axis, y_axis]'\n EXPECTED 
            
            if (line != "") {
                // IGNORE EVERYTHING UNTIL LAST ':' FOUND
                if ((pos3 = line.ReverseFind(':')) != -1) {
                    line = line.Mid(pos3 + 1);
                }
                line.TrimLeft();
                line.TrimRight();
                line += " ";
                
                // SEARCH FOR ',' - X AXIS
                if ((pos3 = line.Find(',')) != -1) {
                    value = line.Mid(1, pos3-1);
                    pPattern->pattern[pPattern->numNodes * 2] = atoi(value);

                    // SEARCH FOR ']' - Y AXIS
                    if ((pos4 = line.Find(']', pos3)) != -1) {
                        value = line.Mid(pos3 + 1, pos4 - pos3 - 1);
                        value.TrimLeft();
                        value.TrimRight();
                        pPattern->pattern[pPattern->numNodes * 2 + 1] = atoi(value);
        
                        // INCREASE NUMBER OF FOUND POSITIONS
                        pPattern->numNodes++;                
                    }
                }        
            }
        }
        else {
            break;
        }    
        
        pos = pos2 + 1;
    }

    return status;
}

/*
int CSensorEnv::GA_GetBestGenom(GA_STRATEGY* pStrategy, GA1DArrayGenome<int>* pBestGenom, GA1DArrayGenome<int>* pGenomeTemp) {
    int     status = STAT_OK;
    float   maxFit = 0;
    // SAVE ACTUAL SETTINGS
    GAPopulation pop = pStrategy->pGA->population();
    for (int nGen = 0; nGen < pop.size(); nGen++) {
        *pGenomeTemp = (GA1DArrayGenome<int>&) pop.individual(nGen); 
        if (pGenomeTemp->score() > maxFit) {
            // BETTER FOUND
            *pBestGenom = (GA1DArrayGenome<int>&) pop.individual(nGen); 
            maxFit = pGenomeTemp->score();
        } 
    }
    return status;
}
/**/
int CSensorEnv::GetRandomNodes(SENSOR_NODE** ppNodesList, DWORD numNodes) {
    int     status = STAT_OK;
    int     j;
    DWORD   random;
    BYTE*   selectedNodesMask;    
    
    // CHECK FOR TOO MANY NODES TO SELECTED
    if (numNodes > m_pActualDeployment->whiteSensorList.numSensors) status = STAT_DATA_INCORRECT_LENGTH;

    if (status == STAT_OK) {
        // MASK INDICATING THAT GIVEN NODE WAS NOT SELECTED BEFORE
        selectedNodesMask = new BYTE[m_pActualDeployment->whiteSensorList.numSensors];
        memset(selectedNodesMask, 0, m_pActualDeployment->whiteSensorList.numSensors);

        j = 0;
        while (j < numNodes) {
            m_rndGen.GetRandomFromInterval(m_pActualDeployment->whiteSensorList.numSensors, &random);        
            if (selectedNodesMask[random] == 0) {
                if (m_pActualDeployment->whiteSensorList.sensors[random]->flags & FLAG_SENSOR_BASE_STATION) {
                    // BS CANNOT BE SELECTED
                }
                else {
                    // NODE WAS NOT SELECTED BEFORE, SELECT NOW 
                    ppNodesList[j] = m_pActualDeployment->whiteSensorList.sensors[random]; 
                    // SET MASK FOR USED NODE
                    selectedNodesMask[random] = 1;

                    j++;
                }
            }
        } 

        delete[] selectedNodesMask;
    }
    
    return status;
}

int CSensorEnv::GetNodesByID(SENSOR_NODE** ppNodesList, DWORD numNodes, lii* pIDList) {
    int     status = STAT_OK;
    int     j = 0;
    lii::iterator   iter;

    for (iter = pIDList->begin(); iter != pIDList->end(); iter++) {
        if (m_pActualDeployment->whiteSensorList.sensors[*iter]->flags & FLAG_SENSOR_BASE_STATION) {
            // BS CANNOT BE SELECTED
        }
        else {
            // NODE WAS NOT SELECTED BEFORE, SELECT NOW 
            ppNodesList[j] = m_pActualDeployment->whiteSensorList.sensors[*iter]; 

            j++;
        }
    }
    
    return status;
}

int CSensorEnv::EG_SharedKeysProb(DWORD poolSize, DWORD ringSize, DWORD numSharedKeys, double* pProb) {
    int     status = STAT_OK;
    double  partA = 1;
    double  partB = 1;
    double  partC = 1;
    double  tmp = 1;
    int     j;
    int     k;
    DWORD   numNodes;
    DWORD   actNumNodes;
    int     minRange;
    int     maxRange;   

    // HIGHLY OPTIMISED VERSION, WORKS ONLY FOR TWO NODES
    numNodes = 2;
    actNumNodes = 2;

    // part A
    partA = 1;
    for (j = (poolSize - numSharedKeys + 1); j <= poolSize; j++) {
        partA *= j;
    }

    partA = partA / Fact(numSharedKeys);

    // part B1 evaluated together with part C2 
    DWORD minRangeB1 = poolSize - numSharedKeys - numNodes * (ringSize - numSharedKeys) + 1;
    DWORD maxRangeB1 = poolSize - ringSize;
    DWORD minRangeC2 = poolSize - ringSize + 1;
    DWORD maxRangeC2 = poolSize;
    DWORD shiftC2B = (poolSize - ringSize + 1) - (poolSize - numSharedKeys - numNodes * (ringSize - numSharedKeys) + 1);

    tmp = 1;
    for (j = (poolSize - numSharedKeys - numNodes * (ringSize - numSharedKeys) + 1); j <= poolSize; j++) {
        // multiply by B1 factor if in B1 range
        if (j >= minRangeB1 && j <= maxRangeB1) tmp = tmp * j; 
        // divide by C2 factor if in C2 range
        if ((j + shiftC2B) >= minRangeC2 && (j + shiftC2B) <= maxRangeC2) tmp = tmp / (j + shiftC2B); 
    }

    partB = tmp;

    // part B2 evaluated together with part C1 
    DWORD minRangeB2 = ringSize + 1;  
    DWORD maxRangeB2 = numNodes * (ringSize - numSharedKeys);
    DWORD minRangeC1 = actNumNodes * (ringSize - numSharedKeys) - (ringSize - numSharedKeys) + 1;
    DWORD maxRangeC1 = actNumNodes * (ringSize - numSharedKeys);

    if (minRangeB2 < minRangeC1) minRange = minRangeB2; else minRange = minRangeC1; 
    if (maxRangeC1 > maxRangeB2) maxRange = maxRangeC1; else maxRange = maxRangeB2; 

    tmp = 1;
    for (j = minRange; j<= maxRange; j++) {
        if (j >= minRangeB2 && j <= maxRangeB2 && j >= minRangeC1 && j <= maxRangeC1) {
            // factors from B2 and C1 eats itselfs
        }
        else {
            // divide by B2 factor if in B2 range
            if (j >= minRangeB2 && j <= maxRangeB2) tmp = tmp / j; 
            // multiply by C1 factor if in C1 range
            if (j >= minRangeC1 && j <= maxRangeC1) tmp = tmp * j; 
        }
    
    }
    partB = partB * tmp; 

    tmp = 1;
    for (j = (poolSize - numSharedKeys + 1); j <= poolSize; j++) {
        tmp = tmp * j;
    }
    partB = partB / tmp; 

    partC = 1;
    tmp = 1;
    for (k = (ringSize - numSharedKeys + 1); k <= ringSize; k++) {
        tmp = tmp * k;
    }
    partC = partC * tmp; 

    *pProb = partA * partB * partC;

    return status;
}

int CSensorEnv::EG_NodeCaptureResilience(DWORD poolSize, DWORD ringSize, DWORD numComprNodes, double* pFraction) {
    int     status = STAT_OK;
    return status;
}


int CSensorEnv::NCR_Evolve(GA_STRATEGY* pStrategy) {
    int                 status = STAT_OK;
    CString             message;
    CString             value;
    CTimeSpan           ts(0, 3, 0, 0 ); 
    CTime               lastSave = CTime::GetCurrentTime();
    CTime               actualTime = CTime::GetCurrentTime();
    
    GA1DArrayGenome<NCR_ELEMENT_TYPE> genomeTemp(pStrategy->gLength, CSensorEnv::NCR_fitness);

    pStrategy->maxFit = 0;

    DeleteFile("NCR_Evolve_fitnessProgress.txt");

    //
    //  EVOLVE STEP-BY-STEP
    //
    int actGener = 1;
    pStrategy->pGA->flushScores();
    pStrategy->pGA->pop->evaluated = gaFalse;
    //while (actGener < pStrategy->nGeners) {
    while (TRUE) {
        pStrategy->actGeneration = actGener;
        
        // CHECK FOR SOFT INTERUPT
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            CString value;
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);

            // SET ACTIVE THREAD PTR TO NULL
            pSensorSim->m_pActualThread = NULL;
            
            return 0;
        }        

        pStrategy->pGA->pop->flushEvalution();
        pStrategy->pGA->step(); // krok GA        
/*        
        genomeTemp = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) pStrategy->pGA->statistics().bestIndividual(); 
        float fit = CSensorEnv::NCR_fitness(genomeTemp);
/**/        

        message.Format("(%d gen.): %f best", actGener, pStrategy->maxFit); 
        LOG_INSERTSTRING(message); 
        
		// SAVE ACTUAL STATUS INTO RUNNING INSTANCES STRING
		SaveInstanceLog(message);

        // SAVE FITNESS PROGRESS
        message.Format("%d %f\r\n", actGener, pStrategy->maxFit); 
        CCommonFnc::File_AppendString("NCR_Evolve_fitnessProgress.txt", message);


/**/
/*
		// SAVE ACTUAL STATUS INTO RUNNING INSTANCES STRING
        SaveInstanceLog(message);
/*        
        // SAVE POPULATION TO FILE EACH 3 HOURS OR 1000 GENERATION
        actualTime = CTime::GetCurrentTime();
        if ((actGener % 1000 == 1) || (actualTime - ts  > lastSave)) {
            lastSave = CTime::GetCurrentTime();

            CString fileName;
            fileName.Format("population_%d_%d.txt", pSensorSim->m_numRepetitionsElapsed, actGener);
            DeleteFile(fileName);
            
            // SAVE ACTUAL SETTINGS
            pSensorSim->Log_InsertToFile(fileName, "########## ACTUAL POPULATION #############\n"); 
            message.Format("%s\n", GAS_HEADER_BEGIN);
            pSensorSim->Log_InsertToFile(fileName, message); 
            message = "";
            pStrategy->settingsToString(&message);
            pSensorSim->Log_InsertToFile(fileName, message); 
            pSensorSim->Log_InsertToFile(fileName, GAS_HEADER_END); 
            pSensorSim->Log_InsertToFile(fileName, GAS_SEPARATOR); 

            GAPopulation pop = pStrategy->pGA->population();
            for (int nGen = 0; nGen < pStrategy->popSize; nGen++) {
                message.Format("%s\n", GAS_GENOM_BEGIN);
                pSensorSim->Log_InsertToFile(fileName, message); 
                genomeTemp = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) pop.individual(nGen); 

                for (int i = 0; i < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; i++) {
                    pStrategy->stratString[i] = genomeTemp.gene(i); 
                }
                for (int slot = 0; slot < m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; slot++) {
                    pStrategy->toString(slot, &message);
                    value.Format("%d:", slot);
                    value += message;
                    value += "\n";
                    pSensorSim->Log_InsertToFile(fileName, value); 
                }
                
                pSensorSim->Log_InsertToFile(fileName, GAS_GENOM_END); 
                pSensorSim->Log_InsertToFile(fileName, GAS_SEPARATOR); 
            }
            
            pSensorSim->Log_InsertToFile(fileName, "########## END ACTUAL POPULATION #############\n"); 
        }
/**/        
        actGener++;
    }
/*    
    genomeTemp = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) pStrategy->pGA->statistics().bestIndividual(); 
/*
    for (int i = 0; i < pStrategy->maxInstruct * GAS_INSLENGTH; i++) {
        pStrategy->stratString[i] = genomeTemp.gene(i); 
    }
    for (int slot = 0; slot < pStrategy->maxInstruct; slot++) {
        pStrategy->toString(slot, &message);
        value.Format("%2d:", slot);
        value += message;
        LOG_INSERTSTRING_NOTIME(value, FALSE); 
    }
    float fit = CSensorEnv::NCR_fitness(genomeTemp);
    message.Format("Best fitness: %f", fit); 
    LOG_INSERTSTRING(message); 
/**/    
    return status;    
}

float CSensorEnv::NCR_fitness(GAGenome &g) {
    int                     status = STAT_OK;
    GA1DArrayGenome<NCR_ELEMENT_TYPE>   &genome = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) g;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;

    SENSOR_NODE**       pSensorsList = NULL;
    KEY_ID*             keysList = NULL;
    DWORD               numKeys = 0;       
    DWORD               secure = 0;
    DWORD               compr = 0;
    DWORD               missing = 0;
    // CREATE KEYS COMPROMISE MASK
    BYTE*               keyComprStatus = new BYTE[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    memset(keyComprStatus, 0, pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1);
    
    // RANDOM PICK OR PICK BY LIST
    if (status == STAT_OK) {
        pSensorsList = new SENSOR_NODE*[genome.size()];

        // SELECT NODES GIVEN BY THE GENOM AS CAPTURED
        for (int i = 0; i < genome.size(); i++) {
            // IF ZERO, THAN NO NODE IS CAPTURED FOR GIVEN GENE
            if (genome.gene(i) < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors) {
                if (genome.gene(i) != 0) pSensorsList[i] = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[genome.gene(i)];
                else pSensorsList[i] = NULL;
            }
            else {
                // INCORRECT VALUE
                ASSERT(FALSE);
            }
        }

        status = pSensorSim->m_sensorEnv.GetKeysFromNodes(pSensorsList, pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes, keyComprStatus);
    }   

    if (status == STAT_OK) {
        pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
        // MARK LINK STATUS ACCORDING TO COMPROMISED KEYS   
        //status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList(&pSensorSim->m_sensorEnv.m_basicSettings, &secure, &compr, keysList, numKeys);  

        // COMPUTE STATUS OF LINK COMPROMISE
        if (pSensorSim->m_sensorEnv.m_basicSettings.danAmplifWhisper) {
            // IF REQUIRED, PERFORM SECRECY AMPLIFICATION
            // NOTE: LINKS WITH MISSING KEYS *ARE* COMPUTED AS COMPROMISED
            if ((status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList(&pSensorSim->m_sensorEnv.m_basicSettings, &secure, &compr, keyComprStatus, FALSE)) == STAT_OK) {
                BYTE repeat = pSensorSim->m_sensorEnv.m_basicSettings.amplifRepeat;
                if (repeat == 0) repeat = 1;
                for (BYTE i = 0; i < repeat; i++) {
                    secure = 0;
                    compr = 0;
                    status = DanAmplif(&pSensorSim->m_sensorEnv.m_basicSettings, &secure, &compr);
                }
            } 
        }
        else {
            // WITHOUT SEC. AMPLIF
            // NOTE: LINKS WITH MISSING KEYS ARE *NOT* COMPUTED AS COMPROMISED
            status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList(&pSensorSim->m_sensorEnv.m_basicSettings, &secure, &compr, keyComprStatus, FALSE);  
        }



        // FITNESS TO MAXIMIZE COMPROMISED LINKS        
        if (secure + compr == 0) fit = 0;
        else fit = 1 - (secure / (float) (secure + compr));

        
        if ((pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES) && 
            !(pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION)) {
            if ((pSensorSim->m_pGAStrategy->actGeneration - pSensorSim->m_pGAStrategy->generationOfMaxFit) > pSensorSim->m_pGAStrategy->fitnessStableGenersTreshold) {
                // WE WILL MINIMIZING MESSAGES NOW => PROTOCOL MUST ACHIVE AT LEAST ACTUAL FITNESS AND GET FITNESS BASED ON USED MESSAGES (LOWER THE BETTER) 
                pSensorSim->m_pGAStrategy->protocolTypeFlags |= FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION;

                pSensorSim->m_pGAStrategy->minimizeMsgFit = pSensorSim->m_pGAStrategy->maxFit - pSensorSim->m_pGAStrategy->maxFit / (float) 100;
                pSensorSim->m_pGAStrategy->maxFit = 0;
                LOG_INSERTSTRING("###MESSAGE MINIMIZATION STARTED");
            }
        }

        // FITNESS TO MINIMIZE CAPTURED NODES
        if (pSensorSim->m_pGAStrategy->protocolTypeFlags & FLAG_PROTOCOL_GROUP_MINIMIZEMESSAGES_INACTION) {
            if (secure + compr == 0) fit = 0;
            else {
                if (1 - (secure / (float) (secure + compr)) < pSensorSim->m_pGAStrategy->minimizeMsgFit) fit = 0; // 30% compromise required
                else {
                    // REMOVE DUPLICITIES IF ANY
                    for (int i = 0; i < genome.size(); i++) {
                        for (int j = i + 1; j < genome.size(); j++) {
                            // IF SAME NODE ALREDY ASSIGNED, SET AS NULL
                            if (pSensorsList[i] == pSensorsList[j]) pSensorsList[j] = NULL;
                        }    
                    }   
                    DWORD numNodes = 0;
                    for (int i = 0; i < genome.size(); i++) {
                        if (pSensorsList[i] != NULL) numNodes++;
                    }
                    fit = 1 - (numNodes / (float) genome.size()); 
                }
            }
        }

        pSensorSim->m_pGAStrategy->fitness = fit;
    }
    
    if (keyComprStatus) delete[] keyComprStatus;
    if (pSensorsList) delete[] pSensorsList;
    if (keysList) delete[] keysList;

    if (status == STAT_OK) {
        // DISPLAY IMPROVEMENT IF ANY
        if (pSensorSim->m_pGAStrategy->maxFit < pSensorSim->m_pGAStrategy->fitness) {
            pSensorSim->m_pGAStrategy->maxFit = pSensorSim->m_pGAStrategy->fitness;
            pSensorSim->m_pGAStrategy->generationOfMaxFit = pSensorSim->m_pGAStrategy->actGeneration;

            message.Format("New maximum found: %f", pSensorSim->m_pGAStrategy->fitness);
            LOG_INSERTSTRING(message);

            message = "";
            CString value;
            for (int i = 0; i < genome.size(); i++) {
                value.Format("%d ", genome.gene(i));
                message += value;
            }
            LOG_INSERTSTRING(message);
        }
    }

    return fit;
}    

void CSensorEnv::NCR_initializer_random(GAGenome &g) {

    GA1DArrayGenome<NCR_ELEMENT_TYPE> &genome = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) g;
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; i++) {
        DWORD   id = 0;
        pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(pSensorSim->m_sensorEnv.m_basicSettings.whiteNum, &id);        
        genome.gene(i, id);
    }   
/**/    
//    NCR_initializer_best(g);
}

void CSensorEnv::NCR_initializer_best(GAGenome &g) {
    // COMPUTE BEST PICK IF NOT BEFORE
    if (pSensorSim->m_sensorEnv.bestNC[0] == 0) {
        NCR_PickAlg3(1);
    }
    
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &genome = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) g;
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; i++) {
        genome.gene(i, pSensorSim->m_sensorEnv.bestNC[i]);
    }   
}

int CSensorEnv::NCR_mutator(GAGenome &g, float pmut) {
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &genome = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) g;
    float   result = 0;
    float   mut = pmut;
    
    // 
    //  MUTATE PROTOCOL INSTRUCTIONS 
    //
        
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; i++) {
        mut = pmut;
        if (GAFlipCoin(mut)) {
            DWORD   id = 0;
            pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(pSensorSim->m_sensorEnv.m_basicSettings.whiteNum, &id);        
            genome.gene(i, id);
        }
    }

    return result;
}

int CSensorEnv::NCR_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &parent1 = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) p1;
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &parent2 = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) p2;
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &offspring1 = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) *o1;
    GA1DArrayGenome<NCR_ELEMENT_TYPE> &offspring2 = (GA1DArrayGenome<NCR_ELEMENT_TYPE>&) *o2;

    // 2 CROSSING POINTS EVERYWHERE
    int cpoint1 = GARandomInt(1, pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes - 1); // crossing point
    int cpoint2 = GARandomInt(1, pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes - 1); // crossing point

    // IF NECESSARY, SWAP c1 AND c2 => c1 < c2
    if (cpoint1 > cpoint2) {
        cpoint1 += cpoint2;
        cpoint2 = cpoint1 - cpoint2;
        cpoint1 = cpoint1 - cpoint2;
    }
    
    // | xxxx c1 xxxx c2 xxxx |
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; i++) {
        if (i < cpoint1) {
            if (o1 != NULL) offspring1.gene(i, parent1.gene(i));
            if (o2 != NULL) offspring2.gene(i, parent2.gene(i));
        }
        else {
            if (i < cpoint2) {
                if (o1 != NULL) offspring1.gene(i, parent2.gene(i));
                if (o2 != NULL) offspring2.gene(i, parent1.gene(i));
            }
            else {
                if (o1 != NULL) offspring1.gene(i, parent1.gene(i));
                if (o2 != NULL) offspring2.gene(i, parent2.gene(i));
            }
        }
    }

    return 1;
}

float CSensorEnv::NCR_PickAlg(int type) {
    int                     status = STAT_OK;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;
    DWORD                   numKeys = 0;       
    DWORD                   secure = 0;
    DWORD                   compr = 0;
    DWORD                   missing = 0;
    GA1DArrayGenome<NCR_ELEMENT_TYPE>* genome = new GA1DArrayGenome<NCR_ELEMENT_TYPE>(pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes, CSensorEnv::NCR_fitness);

    // CREATE KEYS FREQUENCY MASK AND PRICE MASK
    DWORD*               keyFreqStatus = new DWORD[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    memset(keyFreqStatus, 0, pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1);
    DWORD*               nodePrice = new DWORD[pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1];
    memset(keyFreqStatus, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
    memset(nodePrice, 0, (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1) * sizeof(DWORD));
    
    // PROCESS ALL NODES AND COMPUTE ASSIGNED KEYS FREQUENCY
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

        for (int j = 0; j < pSensorA->keyRingSize; j++) {
            keyFreqStatus[pSensorA->keyRing[j]] = keyFreqStatus[pSensorA->keyRing[j]] + 1; 
        }
    }
    
    // COMPUTE PRICE OF EACH NODE BASED ON THE CARRIED KEYS
    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
        SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
        DWORD   nodeValue = 0;
        for (int j = 0; j < pSensorA->keyRingSize; j++) {
            nodeValue += keyFreqStatus[pSensorA->keyRing[j]];
        }
        nodePrice[i] = nodeValue;
    }
    
    // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
    if (status == STAT_OK) {
        DWORD   bestID = 0;
        DWORD   bestValue = INT_MAX;
        for (int offset = 0; offset < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; offset++) {
            switch (type) {
                // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                case 0: {
                    bestValue = INT_MAX;                
                    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                        if (nodePrice[i] < bestValue) {
                            bestValue = nodePrice[i];
                            bestID = i;
                        }    
                    }    
                    genome->gene(offset, bestID);
                    // SET TO MAXIMUM TO PREVENT PICK AGAIN
                    nodePrice[bestID] = INT_MAX;
                    bestValue = INT_MAX;
                    break;
                }
                // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                case 1: {
                    bestValue = 0;                
                    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                        if (nodePrice[i] > bestValue) {
                            bestValue = nodePrice[i];
                            bestID = i;
                        }    
                    }    
                    genome->gene(offset, bestID);
                    // SET TO MINIMUM TO PREVENT PICK AGAIN
                    nodePrice[bestID] = 0;
                    bestValue = 0;
                    break;
                }
            }
        }
    }
    
    fit = CSensorEnv::NCR_fitness(*genome);
    
    return fit;
}

float CSensorEnv::NCR_PickAlg2(int type) {
    int                     status = STAT_OK;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;
    DWORD                   numKeys = 0;       
    DWORD                   secure = 0;
    DWORD                   compr = 0;
    DWORD                   missing = 0;
    GA1DArrayGenome<NCR_ELEMENT_TYPE>* genome = new GA1DArrayGenome<NCR_ELEMENT_TYPE>(pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes, CSensorEnv::NCR_fitness);

    // CREATE KEYS FREQUENCY MASK AND PRICE MASK
    DWORD*               keyFreqStatus = new DWORD[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    DWORD*               keyAlreadyCaptured = new DWORD[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    DWORD*               nodePrice = new DWORD[pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1];
    memset(keyFreqStatus, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
    memset(keyAlreadyCaptured, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
    memset(nodePrice, 0, (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1) * sizeof(DWORD));


    for (int offset = 0; offset < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; offset++) {
        memset(keyFreqStatus, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
        memset(nodePrice, 0, (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1) * sizeof(DWORD));
    
        // PROCESS ALL NODES AND COMPUTE ASSIGNED KEYS FREQUENCY
        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

            for (int j = 0; j < pSensorA->keyRingSize; j++) {
                if (keyAlreadyCaptured[pSensorA->keyRing[j]] == 0) {
                    // COUNT ONLY PRICE OF KEYS THAT WERE NOT ALREADY CAPTURED 
                    keyFreqStatus[pSensorA->keyRing[j]] = keyFreqStatus[pSensorA->keyRing[j]] + 1; 
                }
                else {
                    // KEY ALREADY CAPTURED, NO BONUS - ASSIGN DEPENDS ON CAPTURE TYPE
                    switch (type) {
                        // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                        case 0: {
                            keyFreqStatus[pSensorA->keyRing[j]] = INT_MAX;
                            break;
                        }
                        // PICK NODES WITH HIGHEST VALUE (= LOWEST UNIQUENESS)
                        case 1: {
                            keyFreqStatus[pSensorA->keyRing[j]] = 0;
                            break;
                        }
                    }
                }
            }
        }
        
        // COMPUTE PRICE OF EACH NODE BASED ON THE CARRIED KEYS
        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
            DWORD   nodeValue = 0;
            for (int j = 0; j < pSensorA->keyRingSize; j++) {
                nodeValue += keyFreqStatus[pSensorA->keyRing[j]];
            }
            nodePrice[i] = nodeValue;
        }
    
        // PICK NODE WITH LOWEST/HIGHEST VALUE 
        if (status == STAT_OK) {
            DWORD   bestID = 0;
            DWORD   bestValue = INT_MAX;

            switch (type) {
                // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                case 0: {
                    bestValue = INT_MAX;                
                    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                        if (nodePrice[i] < bestValue) {
                            bestValue = nodePrice[i];
                            bestID = i;
                        }    
                    }    
                    genome->gene(offset, bestID);
                    break;
                }
                // PICK NODES WITH HIGHEST VALUE (= LOWEST UNIQUENESS)
                case 1: {
                    bestValue = 0;                
                    for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                        if (nodePrice[i] > bestValue) {
                            bestValue = nodePrice[i];
                            bestID = i;
                        }    
                    }    
                    genome->gene(offset, bestID);
                    break;
                }
            }
            
            if (status == STAT_OK) {
                // SET CAPTURED KEYS SO NO BONUS FOR CAPTURING THEM AGAIN OCCURE
                for (int j = 0; j < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[bestID]->keyRingSize; j++) {
                    int keyID = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[bestID]->keyRing[j];
                    keyAlreadyCaptured[keyID] = 1; 
                }
                
            }
        }
    }
    
    fit = CSensorEnv::NCR_fitness(*genome);


    if (keyFreqStatus) delete[] keyFreqStatus;
    if (keyAlreadyCaptured) delete[] keyAlreadyCaptured;
    if (nodePrice) delete[] nodePrice;

    return fit;
}

float CSensorEnv::NCR_PickAlg3(int type, int numRandomlyCaptured) {
    int                     status = STAT_OK;
    CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;
    DWORD                   numKeys = 0;       
    DWORD                   secure = 0;
    DWORD                   compr = 0;
    DWORD                   missing = 0;
    GA1DArrayGenome<NCR_ELEMENT_TYPE>* genome = new GA1DArrayGenome<NCR_ELEMENT_TYPE>(pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes, CSensorEnv::NCR_fitness);

    // CREATE KEYS FREQUENCY MASK AND PRICE MASK
    DWORD*               keyFreqStatus = new DWORD[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    DWORD*               keyAlreadyCaptured = new DWORD[pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1];
    DWORD*               nodePrice = new DWORD[pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1];
    memset(keyFreqStatus, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
    memset(keyAlreadyCaptured, 0, (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.keyPoolSize + 1) * sizeof(DWORD));
    memset(nodePrice, 0, (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1) * sizeof(DWORD));

        // PROCESS ALL NEIGHBOURS AND COMPUTE ASSIGNED KEYS FREQUENCY
    if (status == STAT_OK) {
        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];

            // PROCESS NEIGHBOURS 
            NEIGH_LIST_NODE* pListNode = pSensorA->whiteNeigh;
            while ((pListNode != NULL)) {
                SENSOR_NODE* pSensorB = (SENSOR_NODE*) pListNode->pItem;

                if (pSensorB->flags & FLAG_SENSOR_WHITE) {
                    if (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.ringRepresentMethod == FLAG_KEYRING_BITMASK) {
                        ASSERT(FALSE);
                        // TODO: add support for bit mask style of key distribution
                    }
                    else if (pSensorSim->m_sensorEnv.m_basicSettings.egScheme.ringRepresentMethod == FLAG_KEYRING_IDSET) {
                        // COMPUTE FREQUENCY BASED ON KEYS USED TO SECURE THE LINK 
                        for (int j = 0; j < MAX_SHARED_KEYS; j++) {
                            if (pListNode->EG_sharedKeys[j].id != 0) {
                                // INCREASE SHARED KEYS COUNTER
                                if (keyAlreadyCaptured[pListNode->EG_sharedKeys[j].id] == 0) {
                                    // COUNT ONLY PRICE OF KEYS THAT WERE NOT ALREADY CAPTURED 
                                    keyFreqStatus[pListNode->EG_sharedKeys[j].id] = keyFreqStatus[pListNode->EG_sharedKeys[j].id] + 1; 
                                }
                                else {
                                    // KEY ALREADY CAPTURED, NO BONUS - ASSIGN DEPENDS ON CAPTURE TYPE
                                    switch (type) {
                                        // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                                        case 0: {
                                            keyFreqStatus[pListNode->EG_sharedKeys[j].id] = INT_MAX;
                                            break;
                                        }
                                        // PICK NODES WITH HIGHEST VALUE (= LOWEST UNIQUENESS)
                                        case 1: {
                                            keyFreqStatus[pListNode->EG_sharedKeys[j].id] = 0;
                                            break;
                                        }
                                    }
                                }
                            }
                            else {
                                // BREAK ON FIRST EMPTY SLOT
                                break;
                            }
                        }
                    }
                }

                pListNode = pListNode->next;
           }
        }
    }
    
    for (int offset = 0; offset < pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes; offset++) {
        memset(nodePrice, 0, (pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors + 1) * sizeof(DWORD));
        
        // COMPUTE PRICE OF EACH NODE BASED ON THE CARRIED KEYS
        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
            SENSOR_NODE* pSensorA = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[i];
            DWORD   nodeValue = 0;
            for (int j = 0; j < pSensorA->keyRingSize; j++) {
                nodeValue += keyFreqStatus[pSensorA->keyRing[j]];
            }
            nodePrice[i] = nodeValue;
        }
    
        // PICK NODE WITH LOWEST/HIGHEST VALUE 
        if (status == STAT_OK) {
            DWORD   bestID = 0;
            DWORD   bestValue = INT_MAX;

            if (offset < numRandomlyCaptured) {
                // CAPTURE NODE RANDOMLY
                pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(pSensorSim->m_sensorEnv.m_basicSettings.whiteNum, &bestID);        
                pSensorSim->m_sensorEnv.bestNC[offset] = bestID;
                bestValue = nodePrice[bestID];
            }
            else {
                switch (type) {
                    // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                    case 0: {
                        bestValue = INT_MAX;                
                        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                            if (nodePrice[i] < bestValue) {
                                bestValue = nodePrice[i];
                                bestID = i;
                            }    
                        }    
                        genome->gene(offset, bestID);
                        pSensorSim->m_sensorEnv.bestNC[offset] = bestID;
                        break;
                    }
                    // PICK NODES WITH HIGHEST VALUE (= LOWEST UNIQUENESS)
                    case 1: {
                        bestValue = 0;                
                        for (int i = 0; i < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.numSensors; i++) {
                            if (nodePrice[i] > bestValue) {
                                bestValue = nodePrice[i];
                                bestID = i;
                            }    
                        }    
                        genome->gene(offset, bestID);
                        
                        pSensorSim->m_sensorEnv.bestNC[offset] = bestID;
                        
                        break;
                    }
                }
            }
            if (status == STAT_OK) {
                // SET CAPTURED KEYS SO NO BONUS FOR CAPTURING THEM AGAIN OCCURE
                for (int j = 0; j < pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[bestID]->keyRingSize; j++) {
                    int keyID = pSensorSim->m_sensorEnv.m_pActualDeployment->whiteSensorList.sensors[bestID]->keyRing[j];
                    keyAlreadyCaptured[keyID] = 1; 
                    
                    // KEY CAPTURED, NO BONUS FOR NODE - ASSIGN DEPENDS ON CAPTURE TYPE
                    switch (type) {
                        // PICK NODES WITH LOWEST VALUE (= HIGHEST UNIQUENESS)
                        case 0: {
                            keyFreqStatus[keyID] = INT_MAX;
                            break;
                        }
                        // PICK NODES WITH HIGHEST VALUE (= LOWEST UNIQUENESS)
                        case 1: {
                            keyFreqStatus[keyID] = 0;
                            break;
                        }
                    }
                }
            }
        }
    }
    
    fit = CSensorEnv::NCR_fitness(*genome);


    if (keyFreqStatus) delete[] keyFreqStatus;
    if (keyAlreadyCaptured) delete[] keyAlreadyCaptured;
    if (nodePrice) delete[] nodePrice;

    return fit;
}


//CALCULATE NODE CAPTURE RESILIENCY 
int CSensorEnv::NCR_compute(BASIC_INIT_DATA* pBasicSettings, int numCaptured, DWORD *pSecure, DWORD *pCompr)
{
	int status = STAT_OK;
	DWORD* comprNodes = new DWORD[numCaptured];
	memset(comprNodes,0,sizeof(DWORD));

	CString                 message;
    CString                 value;
    float                   oldFitness = 0;
    float                   fit = 0;

    SENSOR_NODE**       pSensorsList = NULL;
    KEY_ID*             keysList = NULL;
    DWORD               numKeys = 0;       
    DWORD               secure = 0;
    DWORD               compr = 0;
    DWORD               missing = 0;

    // CREATE KEYS COMPROMISE MASK
    BYTE*               keyComprStatus = new BYTE[pBasicSettings->egScheme.keyPoolSize + 1];
    memset(keyComprStatus, 0, pBasicSettings->egScheme.keyPoolSize + 1);
	// CREATE KEYS COMPROMISE MASK SECOND DIMENSION
	KEY_2DID*           keyComprStatus2D = new KEY_2DID[pBasicSettings->egScheme.keyPoolSize + 1];
    memset(keyComprStatus2D, 0xff, sizeof(KEY_2DID)*pBasicSettings->egScheme.keyPoolSize + 1);
	    

    // PICK RANDOM NODES  
    if (status == STAT_OK) {
        pSensorsList = new SENSOR_NODE*[numCaptured];
       status = pSensorSim->m_sensorEnv.GetRandomNodes(pSensorsList, numCaptured);
	}

	// GET KEYS FROM THE NODE
	if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.GetKeysFromNodes2D(pSensorsList, numCaptured, keyComprStatus, keyComprStatus2D);
    }   

    if (status == STAT_OK) {
        pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
        // MARK LINK STATUS ACCORDING TO COMPROMISED KEYS   
        //status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList(&pSensorSim->m_sensorEnv.m_basicSettings, &secure, &compr, keysList, numKeys);  

        // COMPUTE STATUS OF LINK COMPROMISE
        
        // WITHOUT SEC. AMPLIF
        // NOTE: LINKS WITH MISSING KEYS ARE *NOT* COMPUTED AS COMPROMISED
		if (pBasicSettings->egScheme.useHashChain)
			status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList2D(pBasicSettings, pSecure, pCompr, keyComprStatus, keyComprStatus2D, pSensorsList, numCaptured, FALSE);  
		else
			status = pSensorSim->m_sensorEnv.DistributeKeyStatusByComprList2D(pBasicSettings, pSecure, pCompr, keyComprStatus, NULL, pSensorsList, numCaptured, FALSE);  
		
	}
    
    if (keyComprStatus) delete[] keyComprStatus;
	if (keyComprStatus2D) delete[] keyComprStatus2D;
    if (pSensorsList) delete[] pSensorsList;
    if (keysList) delete[] keysList;
	if (comprNodes) delete[] comprNodes;

    return status;
}


int CSensorEnv::SaveConfigScript(CString filePath, BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    CString value;
    
    //
    //  PROGRAM VERSION AND DATE
    //
    if (!WritePrivateProfileString("HEADER", "SWVERSION", (LPCTSTR) pBasicSettings->simulSWVersion, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    if (!WritePrivateProfileString("HEADER", "SIMULDATE", (LPCTSTR) pBasicSettings->simulDate, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("HEADER", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;


    //
    // TARGET PLANE PARAMS
    //
    value.Format("%d", pBasicSettings->planeType);
    if (!WritePrivateProfileString("TARGET_PLANE", "PLANE_TYPE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->planeX);
    if (!WritePrivateProfileString("TARGET_PLANE", "PLANE_X", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->planeY);
    if (!WritePrivateProfileString("TARGET_PLANE", "PLANE_Y", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->planeRadius);
    if (!WritePrivateProfileString("TARGET_PLANE", "PLANE_RADIUS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("TARGET_PLANE", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // SENSORS SETTINGS
    // 
    // WHITE SENSORS
    value.Format("%d", pBasicSettings->whiteNum);
    if (!WritePrivateProfileString("WHITE_SENSORS", "WHITE_NUM", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->whiteTransmRange);
    if (!WritePrivateProfileString("WHITE_SENSORS", "WHITE_TRANSM_RANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->whiteRatioReceiveRange);
    if (!WritePrivateProfileString("WHITE_SENSORS", "WHITE_RATION_RECEIVE_RANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->whiteFailureRate);
    if (!WritePrivateProfileString("WHITE_SENSORS", "WHITE_FAILURE_RATE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("WHITE_SENSORS", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    
    // BLACK SENSORS
    value.Format("%d", pBasicSettings->blackNum);
    if (!WritePrivateProfileString("BLACK_SENSORS", "BLACK_NUM", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->blackTransmRange);
    if (!WritePrivateProfileString("BLACK_SENSORS", "BLACK_TRANSM_RANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->blackRatioReceiveRange);
    if (!WritePrivateProfileString("BLACK_SENSORS", "BLACK_RATION_RECEIVE_RANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->blackCountType);
    if (!WritePrivateProfileString("BLACK_SENSORS", "BLACK_COUNT_TYPE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->blackRatioWhite);
    if (!WritePrivateProfileString("BLACK_SENSORS", "BLACK_RATIO_TO_WHITE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("BLACK_SENSORS", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // BASE STATIONS SETTINGS
    //
    value.Format("%d", pBasicSettings->bsConfig.bsCount);
    if (!WritePrivateProfileString("BASE_STATION", "BS_COUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->bsConfig.bsActive);
    if (!WritePrivateProfileString("BASE_STATION", "BS_ACTIVE_COUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->bsConfig.activeBSSelect);
    if (!WritePrivateProfileString("BASE_STATION", "ACTIVE_BS_SELECT_RULE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->bsConfig.bsPlacement);
    if (!WritePrivateProfileString("BASE_STATION", "ACTIVE_BS_PLACEMENT_RULE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("BASE_STATION", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    //  DISTRIBUTION RELATED SETTINGS
    //
    value.Format("%d", pBasicSettings->useOnlyXNeigh);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "USE_ONLY_X_NEIGH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->useNeighFlags);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "USE_NEIGH_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->distribFlags);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "DISTRIB_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->distribKeysCompr);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "DISTRIB_KEYS_COMPR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->removeMostComprBlack);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "REMOVE_MOST_COMPR_BLACK", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->numDeployments);
    if (!WritePrivateProfileString("DISTRIB_SETTINGS", "NUM_DEPLOYMENTS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("DISTRIB_SETTINGS", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;


    //
    // TYPE OF TEST SETTINGS
    //
    value.Format("%d", pBasicSettings->plainMaxScream);
    if (!WritePrivateProfileString("TEST_SETTINGS", "PLAINTEXT_MAX_SCREAM", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->basicWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "BASIC_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->mutualWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "MUTUAL_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->rossAmplifWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "PUSH_AMPLIF_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->danAmplifWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "PULL_AMPLIF_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->rossAmplifMutualWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "PUSH_AMPLIF_MUTUAL_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->danAmplifMutualWhisper);
    if (!WritePrivateProfileString("TEST_SETTINGS", "PULL_AMPLIF_MUTUAL_WHISPER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->maintainCompromissedBy);
    if (!WritePrivateProfileString("TEST_SETTINGS", "MAINTAIN_COMPR_BY_LIST", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->amplifRepeat);
    if (!WritePrivateProfileString("TEST_SETTINGS", "AMPLIF_REPEAT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    
    value = (pBasicSettings->testType & FLAG_TESTTYPE_FIXEDTRANSMRANGE) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "TESTTYPE_FIXEDTRANSMRANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLETRANSMRANGE) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLETRANSMRANGE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLEBWCOUNTRATIO", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEDROPCHECKPROB) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEDROPCHECKPROB", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLECOMPRNODES) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLECOMPRNODES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EAPROTOCOLSEARCH) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EAPROTOCOLSEARCH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EABLACKPATTERNSEARCH) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EABLACKPATTERNSEARCH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EAWHITEPATTERNSEARCH) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EAWHITEPATTERNSEARCH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EABLACKWHITEEVOLUTIONSEARCH) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EABLACKWHITEEVOLUTIONSEARCH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EVOLVECIRCUIT) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EVOLVECIRCUIT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->testType & FLAG_TESTTYPE_EVOLVENODEPICK) ? "1" : "0";
    if (!WritePrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EVOLVENODEPICK", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	if (!WritePrivateProfileString("TEST_SETTINGS", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    // 
    //  BATCH TEST SETTINGS
    //
    value.Format("%d", pBasicSettings->batchTest.whatSaveFlags);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "WHAT_SAVE_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->batchTest.methodsFlags);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "METHODS_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->batchTest.batchRepeat);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "BATCH_REPEAT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "BASE_OUTPUT_PATH", (LPCTSTR) pBasicSettings->batchTest.outputPath, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "NOTE", (LPCTSTR) pBasicSettings->batchTest.note, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->batchTest.rangeFrom);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_FROM", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->batchTest.rangeTo);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_TO", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->batchTest.rangeStep);
    if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_STEP", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("BATCH_TEST_SETTINGS", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;



    //
    // EG PREDISTRIBUTION SCHEME SETTINGS
    // 
	value.Format("%d", pBasicSettings->egScheme.useHashChain);
    if (!WritePrivateProfileString("EG_SCHEME", "USE_HASH_CHAIN", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.hashChainLen);
    if (!WritePrivateProfileString("EG_SCHEME", "HASH_CHAIN_LENGTH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.countComprLinks);
    if (!WritePrivateProfileString("EG_SCHEME", "COUNT_COMPR_LINKS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.captureTestRepeat);
    if (!WritePrivateProfileString("EG_SCHEME", "CAPTURE_TEST_REPEAT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->egScheme.useIt);
    if (!WritePrivateProfileString("EG_SCHEME", "USE_EG_SCHEME", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.useIntermKeys);
    if (!WritePrivateProfileString("EG_SCHEME", "USE_INTERMEDIATE_KEYS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->egScheme.keyPoolSize);
    if (!WritePrivateProfileString("EG_SCHEME", "KEY_POOL_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->egScheme.keyRingSize);
    if (!WritePrivateProfileString("EG_SCHEME", "KEY_RING_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->egScheme.qIntersect);
    if (!WritePrivateProfileString("EG_SCHEME", "Q_INTERSECT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.normalKeys);
    if (!WritePrivateProfileString("EG_SCHEME", "NORMAL_KEYS_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->egScheme.intermKeys);
    if (!WritePrivateProfileString("EG_SCHEME", "INTERMEDIATE_KEYS_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->egScheme.maxComprNodes);
    if (!WritePrivateProfileString("EG_SCHEME", "MAX_COMPR_NODES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("EG_SCHEME", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;


	//
    // MULTI-PATH MESSAGE DELIVERY CONFIG
    //
    value.Format("%d", pBasicSettings->mmdConfig.routingStrategy);
    if (!WritePrivateProfileString("MULTIPATH_MESSAGE_DELIVERY", "ROUTING_STRATEGY", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->mmdConfig.nextNeighDistance);
    if (!WritePrivateProfileString("MULTIPATH_MESSAGE_DELIVERY", "NEXT_NEIGH_DISTANCE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("MULTIPATH_MESSAGE_DELIVERY", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // RANDOM GENERATOR SETTINGS
    //
    value.Format("%d", pBasicSettings->rndGen.randomSeed);
    if (!WritePrivateProfileString("RANDOM_GENERATOR", "RANDOM_SEED", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->rndGen.useFixedSeed);
    if (!WritePrivateProfileString("RANDOM_GENERATOR", "USE_FIXED_SEED", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("RANDOM_GENERATOR", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	
    //
    // GA RANDOM GENERATOR SETTINGS
    //
    value.Format("%d", pBasicSettings->rndGenGA.randomSeed);
    if (!WritePrivateProfileString("GA_RANDOM_GENERATOR", "RANDOM_SEED", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->rndGenGA.useFixedSeed);
    if (!WritePrivateProfileString("GA_RANDOM_GENERATOR", "USE_FIXED_SEED", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("GA_RANDOM_GENERATOR", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

	// FITNESS COMPUTATION SETTINGS
	value.Format("%.2f", pBasicSettings->fitnessWeightMessages);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_WEIGHT_MESSAGES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%.2f", pBasicSettings->fitnessWeightLinks);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_WEIGHT_LINKS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%df", pBasicSettings->fitnessMaxSndCount);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_MAX_SND_COUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%.2f", pBasicSettings->fitnessLinksLowerBound);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_LINKS_LOWER_BOUND", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%.2f", pBasicSettings->fitnessMessagesUpperBound);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_MESSAGES_UPPER_BOUND", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    
    //
    // FORWARD ONION ROUTING SETTINGS
    //
    value.Format("%d", pBasicSettings->foeConfig.useLocalBoost);
    if (!WritePrivateProfileString("FOE_CONFIG", "FOE_USE_LOCAL_BOOST", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->foeConfig.useBoostAllDirectNeigh);
    if (!WritePrivateProfileString("FOE_CONFIG", "FOE_USE_BOOST_ALL_DIRECT_NEIGH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->foeConfig.boostedNodesCount);
    if (!WritePrivateProfileString("FOE_CONFIG", "FOE_BOOSTED_NODES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->foeConfig.onlySenderTarget);
    if (!WritePrivateProfileString("FOE_CONFIG", "FOE_ONLY_SENDER_TARGET", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("FOE_CONFIG", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // NEIGHBOURS LEASHES SETTINGS
    //
    value.Format("%.2f", pBasicSettings->neighLeash.dropMessageProb);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "DROP_MESSAGE_PROB", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->neighLeash.injectMessageProb);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "INJECT_MESSAGE_PROB", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->neighLeash.changeMessageDestProb);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "CHANGEDEST_MESSAGE_PROB", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->neighLeash.neighTransmitCheckProb);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_TRANSMITCHECK_PROB", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%.2f", pBasicSettings->neighLeash.fractComprNodes);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_FRACT_COMPR_NODES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->neighLeash.transmitCheckFlags);
    if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_TRANSMITCHECK_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("NEIGHLEASH_CONFIG", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

	//
	//	REMOTE CONTROL SETTINGS
	//
	if (!WritePrivateProfileString("REMOTECONTROL_CONFIG", "REMOTE_HOSTS", (LPCTSTR) pBasicSettings->remoteConfig.remoteHosts, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("REMOTECONTROL_CONFIG", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // GA SETTINGS
    //
	value = (pBasicSettings->gaConfig.prunneResults) ? "1" : "0";
    if (!WritePrivateProfileString("GA_CONFIG", "PRUNNE_RESULTS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->gaConfig.autoKeyDiscovery) ? "1" : "0";
    if (!WritePrivateProfileString("GA_CONFIG", "AUTO_KEY_DISCOVERY", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.numRepeat);
    if (!WritePrivateProfileString("GA_CONFIG", "NUM_REPEAT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.maxParties);
    if (!WritePrivateProfileString("GA_CONFIG", "MAX_PARTIES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.partyMemorySize);
    if (!WritePrivateProfileString("GA_CONFIG", "PARTY_MEMORY_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.maxInstruct);
    if (!WritePrivateProfileString("GA_CONFIG", "MAX_INSTRUCTS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.sfncSlots);
    if (!WritePrivateProfileString("GA_CONFIG", "SFNC_SLOTS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.sfncSize);
    if (!WritePrivateProfileString("GA_CONFIG", "SFNC_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.sfncCount);
    if (!WritePrivateProfileString("GA_CONFIG", "SFNC_COUNT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.gLength);
    if (!WritePrivateProfileString("GA_CONFIG", "GENOM_LENGTH", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->gaConfig.useSubFunct) ? "1" : "0";
    if (!WritePrivateProfileString("GA_CONFIG", "USE_SUBFUNCTIONS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%f", pBasicSettings->gaConfig.pMutt);
    if (!WritePrivateProfileString("GA_CONFIG", "PROBABILITY_MUTATION", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%f", pBasicSettings->gaConfig.pCross);
    if (!WritePrivateProfileString("GA_CONFIG", "PROBABILITY_CROSSING", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.popSize);
    if (!WritePrivateProfileString("GA_CONFIG", "POPULATION_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.nGeners);
    if (!WritePrivateProfileString("GA_CONFIG", "NUM_GENERATIONS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.nGeners);
    if (!WritePrivateProfileString("GA_CONFIG", "NUM_GENERATIONS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->gaConfig.bAddDeploymentsFitnessTreshold) ? "1" : "0";
    if (!WritePrivateProfileString("GA_CONFIG", "ADD_DEPLOYMENTS_ON_FITNESS_TRESHOLD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.moreDeploymentsCounter);
    if (!WritePrivateProfileString("GA_CONFIG", "MORE_DEPLOYMENTS_COUNTER", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%f", pBasicSettings->gaConfig.moreDeploymentsTreshold);
    if (!WritePrivateProfileString("GA_CONFIG", "MORE_DEPLOYMENTS_THRESHOLD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.fitnessStableGenersTreshold);
    if (!WritePrivateProfileString("GA_CONFIG", "FITNESS_STABLE_GENERS_THRESHOLD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.protocolTypeFlags);
    if (!WritePrivateProfileString("GA_CONFIG", "PROTOCOL_TYPE_FLAGS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.blackDeployPattern.numCellPerAxis);
    if (!WritePrivateProfileString("GA_CONFIG", "BLACK_NUM_CELLS_PER_AXIS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaConfig.whiteDeployPattern.numCellPerAxis);
    if (!WritePrivateProfileString("GA_CONFIG", "WHITE_NUM_CELLS_PER_AXIS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("GA_CONFIG", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    //
    // GA CIRCUIT SETTINGS
    //
    value.Format("%d", pBasicSettings->gaCircuitConfig.numLayers);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_LAYERS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.numSelectorLayers);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_SELECTOR_LAYERS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.numInputs);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "INPUT_LAYER_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->gaCircuitConfig.internalLayerSize);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "INTERNAL_LAYER_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.outputLayerSize);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "OUTPUT_LAYER_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.numLayerConnectors);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_LAYER_CONNECTORS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.predictMethod);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "PREDICTION_METHOD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.genomeSize);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "GENOME_SIZE", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.bSectorInputData);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "SECTOR_INPUT_DATA", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;


    value.Format("%d", pBasicSettings->gaCircuitConfig.numTestVectors);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_TEST_VECTORS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value = (pBasicSettings->gaCircuitConfig.limitAlgRounds) ? "1" : "0";
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "LIMIT_ALG_ROUNDS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.limitAlgRoundsCount);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_ALG_ROUNDS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.representBitAsBytes);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "REPRESENT_BIT_AS_BYTES", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
	value.Format("%d", pBasicSettings->gaCircuitConfig.testVectorGenerMethod);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "TEST_VECTOR_GENERATION_METHOD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.testVectorChangeGener);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "TEST_VECTOR_CHANGE_GENERATION", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.numBestPredictors);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_BEST_PREDICTORS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.bSplitTestVectors);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "SPLIT_TEST_VECTOR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NOP]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NOP", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_OR]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_OR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_AND]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_AND", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_CONST]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_CONST", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_XOR]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_XOR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NOR]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NOR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NAND]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NAND", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ROTL]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ROTL", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ROTR]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ROTR", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_SUM]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_SUM", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_SUBS]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_SUBS", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ADD]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ADD", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_MULT]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_MULT", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
    value.Format("%d", pBasicSettings->gaCircuitConfig.allowedFNC[FNC_DIV]);
    if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_DIV", (LPCTSTR) value, filePath)) status = STAT_INI_DATA_WRITE_FAIL;
  	if (!WritePrivateProfileString("GA_CIRCUIT_CONFIG", "#", "##################################################################", filePath)) status = STAT_INI_DATA_WRITE_FAIL;

    return status;
}

int CSensorEnv::LoadConfigScript(CString filePath, BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    CString value;
    char    iniValue[MAX_INI_VALUE_CHAR];

	// MAKE PATH RELATIVE TO CURRENT WORKING DIRECTORY
	BOOL isRelative = PathIsRelative(filePath);
	if (isRelative) {
		TCHAR path[MAX_PATH+1];
		memset(&path, 0, sizeof(path));
		GetCurrentDirectory(MAX_PATH, path);
		if (PathAppend(path,filePath))
			filePath = path;
	}

    //
    //  PROGRAM VERSION AND DATE
    //
	if (GetPrivateProfileString("HEADER", "SWVERSION", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) {
		status = STAT_INI_DATA_READ_FAIL;
	}
	else {
		pBasicSettings->simulSWVersion = iniValue;
		GetPrivateProfileString("HEADER", "SIMULDATE", "", iniValue, MAX_INI_VALUE_CHAR, filePath);
		pBasicSettings->simulDate = iniValue;

		//
		// TARGET PLANE PARAMS
		//
		if (GetPrivateProfileString("TARGET_PLANE", "PLANE_TYPE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->planeType = atoi(iniValue);
		if (GetPrivateProfileString("TARGET_PLANE", "PLANE_X", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->planeX = (float) atof(iniValue);
		if (GetPrivateProfileString("TARGET_PLANE", "PLANE_Y", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->planeY = (float) atof(iniValue);
		if (GetPrivateProfileString("TARGET_PLANE", "PLANE_RADIUS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->planeRadius = (float) atof(iniValue);


		//
		// SENSORS SETTINGS
		// 
		// WHITE SENSORS
		if (GetPrivateProfileString("WHITE_SENSORS", "WHITE_NUM", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->whiteNum = atoi(iniValue);
		if (GetPrivateProfileString("WHITE_SENSORS", "WHITE_TRANSM_RANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->whiteTransmRange = (float) atof(iniValue);
		if (GetPrivateProfileString("WHITE_SENSORS", "WHITE_RATION_RECEIVE_RANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->whiteRatioReceiveRange = (float) atof(iniValue);
		if (GetPrivateProfileString("WHITE_SENSORS", "WHITE_FAILURE_RATE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->whiteFailureRate = (float) atof(iniValue);

	    
		// BLACK SENSORS
		if (GetPrivateProfileString("BLACK_SENSORS", "BLACK_NUM", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->blackNum = atoi(iniValue);
		if (GetPrivateProfileString("BLACK_SENSORS", "BLACK_TRANSM_RANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->blackTransmRange = (float) atof(iniValue);
		if (GetPrivateProfileString("BLACK_SENSORS", "BLACK_RATION_RECEIVE_RANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->blackRatioReceiveRange = (float) atof(iniValue);
		if (GetPrivateProfileString("BLACK_SENSORS", "BLACK_COUNT_TYPE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->blackCountType = atoi(iniValue);
		if (GetPrivateProfileString("BLACK_SENSORS", "BLACK_RATIO_TO_WHITE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->blackRatioWhite = (float) atof(iniValue);

		//
		// BASE STATIONS SETTINGS
		//
		if (GetPrivateProfileString("BASE_STATION", "BS_COUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->bsConfig.bsCount = atoi(iniValue);
		if (GetPrivateProfileString("BASE_STATION", "BS_ACTIVE_COUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->bsConfig.bsActive = atoi(iniValue);
		if (GetPrivateProfileString("BASE_STATION", "ACTIVE_BS_SELECT_RULE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->bsConfig.activeBSSelect = atoi(iniValue);
		if (GetPrivateProfileString("BASE_STATION", "ACTIVE_BS_PLACEMENT_RULE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->bsConfig.bsPlacement = atoi(iniValue);


		//
		//  DISTRIBUTION RELATED SETTINGS
		//
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "USE_ONLY_X_NEIGH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->useOnlyXNeigh = atoi(iniValue);
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "USE_NEIGH_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->useNeighFlags = atoi(iniValue);
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "DISTRIB_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->distribFlags = atoi(iniValue);
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "DISTRIB_KEYS_COMPR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->distribKeysCompr = (float) atof(iniValue);
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "REMOVE_MOST_COMPR_BLACK", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->removeMostComprBlack = (float) atof(iniValue);
		if (GetPrivateProfileString("DISTRIB_SETTINGS", "NUM_DEPLOYMENTS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->numDeployments = (BYTE) atoi(iniValue);

		//
		// TYPE OF TEST SETTINGS
		//
		if (GetPrivateProfileString("TEST_SETTINGS", "PLAINTEXT_MAX_SCREAM", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->plainMaxScream = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "BASIC_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->basicWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "MUTUAL_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->mutualWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "PUSH_AMPLIF_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rossAmplifWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "PULL_AMPLIF_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->danAmplifWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "PUSH_AMPLIF_MUTUAL_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rossAmplifMutualWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "PULL_AMPLIF_MUTUAL_WHISPER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->danAmplifMutualWhisper = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "MAINTAIN_COMPR_BY_LIST", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->maintainCompromissedBy = atoi(iniValue);
		if (GetPrivateProfileString("TEST_SETTINGS", "AMPLIF_REPEAT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->amplifRepeat = atoi(iniValue);

		pBasicSettings->testType = 0;
		if (GetPrivateProfileString("TEST_SETTINGS", "TESTTYPE_FIXEDTRANSMRANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_FIXEDTRANSMRANGE;
		if (GetPrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLETRANSMRANGE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLETRANSMRANGE;
		if (GetPrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLEBWCOUNTRATIO", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO;
		if (GetPrivateProfileString("TEST_SETTINGS", "TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY;

		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLEDROPCHECKPROB", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLEDROPCHECKPROB;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_VARIABLECOMPRNODES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_VARIABLECOMPRNODES;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EAPROTOCOLSEARCH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EAPROTOCOLSEARCH;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EABLACKPATTERNSEARCH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EABLACKPATTERNSEARCH;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EAWHITEPATTERNSEARCH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EAWHITEPATTERNSEARCH;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EABLACKWHITEEVOLUTIONSEARCH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EABLACKWHITEEVOLUTIONSEARCH;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EVOLVECIRCUIT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EVOLVECIRCUIT;
		if (GetPrivateProfileString("TEST_SETTINGS", "FLAG_TESTTYPE_EVOLVENODEPICK", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (atoi(iniValue) == 1) pBasicSettings->testType |= FLAG_TESTTYPE_EVOLVENODEPICK;

		// 
		//  BATCH TEST SETTINGS
		//
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "WHAT_SAVE_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.whatSaveFlags = atoi(iniValue);
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "METHODS_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.methodsFlags = atoi(iniValue);
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "BATCH_REPEAT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.batchRepeat = atoi(iniValue);
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "BASE_OUTPUT_PATH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.outputPath = iniValue;
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "NOTE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.note = iniValue;
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_FROM", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.rangeFrom = (float) atof(iniValue);
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_TO", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.rangeTo = (float) atof(iniValue);
		if (GetPrivateProfileString("BATCH_TEST_SETTINGS", "RANGE_STEP", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->batchTest.rangeStep = (float) atof(iniValue);

		//
		// EG PREDISTRIBUTION SCHEME SETTINGS
		// 
		if (GetPrivateProfileString("EG_SCHEME", "COUNT_COMPR_LINKS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.countComprLinks = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "CAPTURE_TEST_REPEAT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.captureTestRepeat = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "USE_HASH_CHAIN", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.useHashChain = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "HASH_CHAIN_LENGTH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.hashChainLen = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "USE_EG_SCHEME", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.useIt = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "KEY_POOL_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.keyPoolSize = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "KEY_RING_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.keyRingSize = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "Q_INTERSECT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.qIntersect = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "NORMAL_KEYS_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.normalKeys = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "INTERMEDIATE_KEYS_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.intermKeys = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "MAX_COMPR_NODES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.maxComprNodes = atoi(iniValue);
		if (GetPrivateProfileString("EG_SCHEME", "USE_INTERMEDIATE_KEYS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->egScheme.useIntermKeys = atoi(iniValue);

		//
		// MULTI-PATH MESSAGE DELIVERY CONFIG
		//
		if (GetPrivateProfileString("MULTIPATH_MESSAGE_DELIVERY", "ROUTING_STRATEGY", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->mmdConfig.routingStrategy = atoi(iniValue);
		if (GetPrivateProfileString("MULTIPATH_MESSAGE_DELIVERY", "NEXT_NEIGH_DISTANCE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->mmdConfig.nextNeighDistance = (float) atof(iniValue);

		//
		// RANDOM GENERATOR SETTINGS
		//
		if (GetPrivateProfileString("RANDOM_GENERATOR", "RANDOM_SEED", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rndGen.randomSeed = atoi(iniValue);
		if (GetPrivateProfileString("RANDOM_GENERATOR", "USE_FIXED_SEED", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rndGen.useFixedSeed = atoi(iniValue);
	    
		//
		// GA RANDOM GENERATOR SETTINGS
		//
		if (GetPrivateProfileString("GA_RANDOM_GENERATOR", "RANDOM_SEED", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rndGenGA.randomSeed = atoi(iniValue);
		if (GetPrivateProfileString("GA_RANDOM_GENERATOR", "USE_FIXED_SEED", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->rndGenGA.useFixedSeed = atoi(iniValue);

		// FITNESS COMPUTATION SETTINGS
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_WEIGHT_MESSAGES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (strlen(iniValue)>0) pBasicSettings->fitnessWeightMessages = (float) atof(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_WEIGHT_LINKS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (strlen(iniValue)>0) pBasicSettings->fitnessWeightLinks = (float) atof(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_MAX_SND_COUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (strlen(iniValue)>0) pBasicSettings->fitnessMaxSndCount = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_LINKS_LOWER_BOUND", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (strlen(iniValue)>0) pBasicSettings->fitnessLinksLowerBound = (float) atof(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_MESSAGES_UPPER_BOUND", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		if (strlen(iniValue)>0) pBasicSettings->fitnessMessagesUpperBound = (float) atof(iniValue);		

		//
		// FORWARD ONION ROUTING SETTINGS
		//
		if (GetPrivateProfileString("FOE_CONFIG", "FOE_USE_LOCAL_BOOST", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->foeConfig.useLocalBoost = atoi(iniValue);
		if (GetPrivateProfileString("FOE_CONFIG", "FOE_USE_BOOST_ALL_DIRECT_NEIGH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->foeConfig.useBoostAllDirectNeigh = atoi(iniValue);
		if (GetPrivateProfileString("FOE_CONFIG", "FOE_BOOSTED_NODES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->foeConfig.boostedNodesCount = atoi(iniValue);
		if (GetPrivateProfileString("FOE_CONFIG", "FOE_ONLY_SENDER_TARGET", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->foeConfig.onlySenderTarget = atoi(iniValue);
	    
		//
		// NEIGHBOURS LEASHES SETTINGS
		//
	    
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "DROP_MESSAGE_PROB", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.dropMessageProb = atof(iniValue);
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "INJECT_MESSAGE_PROB", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.injectMessageProb = atof(iniValue);
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "CHANGEDEST_MESSAGE_PROB", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.changeMessageDestProb = atof(iniValue);
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_TRANSMITCHECK_PROB", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.neighTransmitCheckProb = atof(iniValue);
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_FRACT_COMPR_NODES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.fractComprNodes = atof(iniValue);
		if (GetPrivateProfileString("NEIGHLEASH_CONFIG", "NEIGH_TRANSMITCHECK_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->neighLeash.transmitCheckFlags = atol(iniValue);

		//
		// GA SETTINGS
		//
		if (GetPrivateProfileString("GA_CONFIG", "PRUNNE_RESULTS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.prunneResults = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "AUTO_KEY_DISCOVERY", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.autoKeyDiscovery = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "NUM_REPEAT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.numRepeat = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "MAX_PARTIES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.maxParties = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "PARTY_MEMORY_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.partyMemorySize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "MAX_INSTRUCTS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.maxInstruct = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "SFNC_SLOTS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.sfncSlots = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "SFNC_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.sfncSize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "SFNC_COUNT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.sfncCount = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "GENOM_LENGTH", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.gLength = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "USE_SUBFUNCTIONS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.useSubFunct = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "PROBABILITY_MUTATION", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.pMutt = atof(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "PROBABILITY_CROSSING", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.pCross = atof(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "POPULATION_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.popSize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "NUM_GENERATIONS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.nGeners = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "ADD_DEPLOYMENTS_ON_FITNESS_TRESHOLD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.bAddDeploymentsFitnessTreshold = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "MORE_DEPLOYMENTS_COUNTER", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.moreDeploymentsCounter = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "MORE_DEPLOYMENTS_THRESHOLD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.moreDeploymentsTreshold = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "FITNESS_STABLE_GENERS_THRESHOLD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.fitnessStableGenersTreshold = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "PROTOCOL_TYPE_FLAGS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.protocolTypeFlags = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "BLACK_NUM_CELLS_PER_AXIS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.blackDeployPattern.numCellPerAxis = atoi(iniValue);
		if (GetPrivateProfileString("GA_CONFIG", "WHITE_NUM_CELLS_PER_AXIS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaConfig.whiteDeployPattern.numCellPerAxis = atoi(iniValue);


		//
		//	REMOTE CONTROL SETTINGS
		//
		if (GetPrivateProfileString("REMOTECONTROL_CONFIG", "REMOTE_HOSTS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->remoteConfig.remoteHosts = iniValue;

		//
		// GA CIRCUIT SETTINGS
		//

		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_LAYERS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numLayers = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_SELECTOR_LAYERS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numSelectorLayers = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "INPUT_LAYER_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numInputs = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "INTERNAL_LAYER_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.internalLayerSize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "OUTPUT_LAYER_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.outputLayerSize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_LAYER_CONNECTORS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numLayerConnectors = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "PREDICTION_METHOD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.predictMethod = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "GENOME_SIZE", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.genomeSize = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "SECTOR_INPUT_DATA", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.bSectorInputData = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_TEST_VECTORS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numTestVectors = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "LIMIT_ALG_ROUNDS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.limitAlgRounds = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_ALG_ROUNDS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.limitAlgRoundsCount = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "REPRESENT_BIT_AS_BYTES", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.representBitAsBytes = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "TEST_VECTOR_GENERATION_METHOD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.testVectorGenerMethod = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "TEST_VECTOR_CHANGE_GENERATION", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.testVectorChangeGener = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "NUM_BEST_PREDICTORS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.numBestPredictors = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "SPLIT_TEST_VECTOR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.bSplitTestVectors = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NOP", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NOP] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_OR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_OR] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_AND", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_AND] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_CONST", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_CONST] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_XOR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_XOR] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NOR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NOR] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_NAND", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_NAND] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ROTL", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ROTL] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ROTR", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ROTR] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_SUM", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_SUM] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_SUBS", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_SUBS] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_ADD", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_ADD] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_MULT", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_MULT] = atoi(iniValue);
		if (GetPrivateProfileString("GA_CIRCUIT_CONFIG", "FNC_DIV", "", iniValue, MAX_INI_VALUE_CHAR, filePath) == 0) status = STAT_INI_DATA_READ_FAIL;
		pBasicSettings->gaCircuitConfig.allowedFNC[FNC_DIV] = atoi(iniValue);

		// DO NOT CHECK FOR STATUS - SOME VALUES MAY BE MISSING
		if (status != STAT_OK) { 
			//ASSERT(FALSE);
			status = STAT_OK;
		}
	}
    return status;
}


int CSensorEnv::ExecuteTests(BASIC_INIT_DATA* pBasicSettings, DWORD flagsWhatSave, CString basicPath, DWORD numSuffix) {
    //
    //  BASIC TEST TEMPLATE
    //
    int     status = STAT_OK;
    CFile   file;
    DWORD   secure = 0;
    DWORD   compr = 0;
    CString basicCompRatio;
    CString amplifImpact;
    CString value;
    CString fileName;
    CString cfgScriptPath;
    CString header;
    CString mmd = "";
    CString nLeash = "";
    int     i;
    MMD_TEST_RESULTS        mmdResults;
    float   avgHops = 0;    

    //
    // DEPLOY SENSORS
    //
    DeploySensors(pBasicSettings);
    
    
    // CREATE ROUTING PATHS
    if (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEMULTIPATHMESSAGEDELIVERY) {
        status = this->MMD_CreateRoutingPaths(pBasicSettings);
    }

    // CREATE TARGET DIRECTORY IF NOT YET
    this->Dir_Create(basicPath);
    // CREATE TARGET DIRECTORY IF NOT YET
    fileName.Format("%s\\range%0.2f", basicPath, pBasicSettings->whiteTransmRange);
    this->Dir_Create(fileName);

    // SAVE CONFIGURATION SCRIPT
    cfgScriptPath.Format("%s\\!cfgScript_%d.ini", fileName, numSuffix);
    status = this->SaveConfigScript(cfgScriptPath, pBasicSettings);

    //
    // CREATE HEADER FROM ALL RELEVANT INFO
    //
    CalculateBasicDeployInfoString(pBasicSettings, &header);
    


    if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
        basicCompRatio = header;
    }
    if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
        mmd = header;
    }
    if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
        amplifImpact = header;
    }
    if (flagsWhatSave & FLAG_WHATSAVE_NEIGHBLEASH) {
        nLeash = header;
    }

    if (flagsWhatSave & FLAG_WHATSAVE_SENSORPOSITION) {
        // SAVE WHITE POSITIONS
        fileName.Format("%s\\range%0.2f\\posWhite_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
        this->SaveWhitePositions(fileName);            
        value.Format("  White sensors positions -> %s", fileName); 
        LOG_INSERTSTRING(value);

        // SAVE BLACK POSITIONS
        fileName.Format("%s\\range%0.2f\\posBlack_%d.dat", basicPath, pBasicSettings->whiteTransmRange,numSuffix);
        this->SaveBlackPositions(fileName);            
        value.Format("  Black sensors positions -> %s", fileName); 
        LOG_INSERTSTRING(value);
    }


    // ------------------------------------

	//
    //  MAX SCREAMING
    //
    if (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEKEYINFECT) {
        if (pBasicSettings->plainMaxScream) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->PlaintextMaxScream(pBasicSettings, &secure, &compr);
        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }

            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_ms_%d.dat", basicPath, pBasicSettings->whiteTransmRange,numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  PlaintextMaxScream -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_ms_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  PlaintextMaxScream -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_ms_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  PlaintextMaxScream -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_ms_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  PlaintextMaxScream -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }

	    //
        //  BASIC WHISPERING
        //
        if (pBasicSettings->basicWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->BasicWhisper(pBasicSettings, &secure, &compr);

            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }

            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_bw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  BasicWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_bw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  BasicWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_bw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  BasicWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_bw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  BasicWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_bw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }

        //
        //  MUTUAL WHISPERING
        //
        if (pBasicSettings->mutualWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->MutualWhisper(pBasicSettings, &secure, &compr);

            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_mw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_mw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_mw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_mw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_mw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        

            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }

	    // 
        //  ROSS AMPLIFICATION OF BASIC WHISPER
        //
        if (pBasicSettings->rossAmplifWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->RossAmplifWhisper(pBasicSettings, &secure, &compr, pBasicSettings->amplifRepeat);

            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_rabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  RossAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_rabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  RossAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_rabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  PlaintextMaxScream -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_rabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  RossAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_rabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            // NOTE: THIS SAVE OPTION MUST BE AFTER ALL PREVIOUS OPTIONS (ADDITIONALLY AFFECT RESULTS)
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 

                // ADDITIONAL AMPLIFICATIONS
                for (i = 2; i <= 5; i++) {
                    secure = compr = 0;
                    status = this->RossAmplif(pBasicSettings, &secure, &compr);
                    value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                    amplifImpact += value; 
                }
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                amplifImpact += "0 0 0 0 0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }
	    
	    // 
        //  ROSS AMPLIFICATION OF MUTUAL WHISPER
        //
        if (pBasicSettings->rossAmplifMutualWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->RossAmplifMutualWhisper(pBasicSettings, &secure, &compr, pBasicSettings->amplifRepeat);
        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_ramw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  RossAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_ramw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  RossAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_ramw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  RossAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_ramw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  RossAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_ramw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            // NOTE: THIS SAVE OPTION MUST BE AFTER ALL PREVIOUS OPTIONS (ADDITIONALLY AFFECT RESULTS)
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 

                // ADDITIONAL AMPLIFICATIONS
                for (i = 2; i <= 5; i++) {
                    secure = compr = 0;
                    status = this->RossAmplif(pBasicSettings, &secure, &compr);
                    value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                    amplifImpact += value; 
                }
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                amplifImpact += "0 0 0 0 0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }

	    // 
        //  DAN AMPLIFICATION OF BASIC WHISPER
        //
	    if (pBasicSettings->danAmplifWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->DanAmplifWhisper(pBasicSettings, &secure, &compr, pBasicSettings->amplifRepeat);
        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f ", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_dabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  DanAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_dabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  DanAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_dabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  DanAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_dabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  DanAmplifWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_dabw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            // NOTE: THIS SAVE OPTION MUST BE AFTER ALL PREVIOUS OPTIONS (ADDITIONALLY AFFECT RESULTS)
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 

                // ADDITIONAL AMPLIFICATIONS
                for (i = 2; i <= 5; i++) {
                    secure = compr = 0;
                    status = this->DanAmplif(pBasicSettings, &secure, &compr);
                    value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                    amplifImpact += value; 
                }
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                amplifImpact += "0 0 0 0 0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }

	    // 
        //  DAN AMPLIFICATION OF MUTUAL WHISPER
        //
	    if (pBasicSettings->danAmplifMutualWhisper) {
            secure = compr = 0;
            ClearLinkKeysStatus();
            this->DanAmplifMutualWhisper(pBasicSettings, &secure, &compr, pBasicSettings->amplifRepeat);
        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                basicCompRatio += value; 
            }

            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                if (MMD_EstablishBSPaths(pBasicSettings) == STAT_OK) {
                    if (MMD_Test(pBasicSettings, &mmdResults) == STAT_OK) {
                        value.Format("%d %d %d %.2f %.2f %.2f %.2f", mmdResults.foundPaths, mmdResults.notFoundPaths, mmdResults.totalMessages, mmdResults.secureMessages, mmdResults.comprMessages, mmdResults.avgHops, mmdResults.lostMessages);
                        mmd += value; 
                        avgHops = mmdResults.avgHops;
                    }
                }
            }
            
            if (flagsWhatSave & FLAG_WHATSAVE_KEYSDISTRIBUTION) {
                fileName.Format("%s\\range%0.2f\\keysDistrib_damw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveSecureKeyDistrib(fileName);
                value.Format("  DanAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
                fileName.Format("%s\\range%0.2f\\comprByBlack_damw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByBlack(fileName);
                value.Format("  DanAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }

            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE) {
                fileName.Format("%s\\range%0.2f\\comprByDist_damw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByDistance(pBasicSettings, fileName);
                value.Format("  DanAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }
            if (flagsWhatSave & FLAG_WHATSAVE_TRIANGLENEIGHPROPERTIES) {
                fileName.Format("%s\\range%0.2f\\triangleWhite_damw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveWhiteTriangleProperties(fileName);            
                value.Format("  DanAmplifMutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        
            if (flagsWhatSave & FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY) {
                fileName.Format("%s\\range%0.2f\\comprByDens_damw_%d.dat", basicPath, pBasicSettings->whiteTransmRange, numSuffix);
                this->SaveCompromissedByNodeDensity(pBasicSettings, fileName);            
                value.Format("  MutualWhisper -> %s", fileName); 
                LOG_INSERTSTRING(value);
            }        

            // NOTE: THIS SAVE OPTION MUST BE AFTER ALL PREVIOUS OPTIONS (ADDITIONALLY AFFECT RESULTS)
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                amplifImpact += value; 

                // ADDITIONAL AMPLIFICATIONS
                for (i = 2; i <= 5; i++) {
                    secure = compr = 0;
                    status = this->DanAmplif(pBasicSettings, &secure, &compr);
                    value.Format("%.2f ", (compr / (float) (secure + compr)) * 100);
                    amplifImpact += value; 
                }
            }
        }
        else {
            if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
                basicCompRatio += "0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
                amplifImpact += "0 0 0 0 0 "; 
            }
            if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
                mmd += "0 0 0 0 0 0 "; 
            }
        }
    } 

    
    if (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USENEIGHBLEASH) {
        status = pSensorSim->m_sensorEnv.NLeash_Test(pBasicSettings, FALSE, &nLeash);    
    }

    // ------------------------------------

    
    if (flagsWhatSave & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
        // SAVE COLLECTED BASIC RATIO OF COMPROMISSED KEYS 
        fileName.Format("%s\\comprRatioNeigh_%d.dat", basicPath, numSuffix);

        basicCompRatio += "\r\n";
        if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
            file.SeekToEnd();
            file.Write((LPCTSTR) basicCompRatio, basicCompRatio.GetLength());
            file.Close();
        } 

        value.Format("  range=%.2f -> %s", pBasicSettings->whiteTransmRange, fileName); 
        LOG_INSERTSTRING(value);
    }

    if (flagsWhatSave & FLAG_WHATSAVE_MMD) {
        value.Format("%.2f", avgHops);
        mmd += value;
        
        // SAVE COLLECTED MESSAGE MULTIPATH METHOD  
        fileName.Format("%s\\mmd_%d.dat", basicPath, numSuffix);

        mmd += "\r\n";
        if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
            file.SeekToEnd();
            file.Write((LPCTSTR) mmd, mmd.GetLength());
            file.Close();
        } 

        value.Format("  range=%.2f -> %s", pBasicSettings->whiteTransmRange, fileName); 
        LOG_INSERTSTRING(value);
    }

    if (flagsWhatSave & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
        // SAVE COLLECTED AMPLIFICATION IMPACT
        fileName.Format("%s\\amplifImp_%d.dat", basicPath, numSuffix);

        amplifImpact += "\r\n";
        if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
            file.SeekToEnd();
            file.Write((LPCTSTR) amplifImpact, amplifImpact.GetLength());
            file.Close();
        } 

        value.Format("  range=%.2f -> %s", pBasicSettings->whiteTransmRange, fileName); 
        LOG_INSERTSTRING(value);
    }
    if (flagsWhatSave & FLAG_WHATSAVE_NEIGHBLEASH) {
        // SAVE COLLECTED RESULTS FOR NEIGHBOUR LEASHES
        fileName.Format("%s\\nLeash_%d.dat", basicPath, numSuffix);

        nLeash += "\r\n";
        if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
            file.SeekToEnd();
            file.Write((LPCTSTR) nLeash, nLeash.GetLength());
            file.Close();
        } 

        value.Format(" ---> %s", fileName); 
        LOG_INSERTSTRING(value);
    }

    return status;
}

int CSensorEnv::CalculateBasicDeployInfoString(BASIC_INIT_DATA* pBasicSettings, CString* pInfo) {
    int     status = STAT_OK;
    DWORD   min;
    DWORD   max;
    float   avg;
    CString value;
    
    //
    // CREATE HEADER FROM ALL RELEVANT INFO
    //
    *pInfo = "";
    value.Format("%d ", RESULTSOFFSET_COMPRATIOALLMETHODS);
    *pInfo += value; 
   
    // COLLECT AND SAVE BASIC RATIO OF COMPROMISSED KEYS 
    value.Format("%d %d ", pBasicSettings->whiteNum, pBasicSettings->blackNum);
    *pInfo += value; 

    // TRANSM RANGE AND RECEIVE SENSITIVITY 
    value.Format("%.2f ", pBasicSettings->whiteTransmRange);
    *pInfo += value; 

    this->CalcWhiteNeighStats(&max, &min, &avg);
    value.Format("%d %d %.2f ", max, min, avg);
    *pInfo += value; 

    this->CalcBlackNeighStats(&max, &min, &avg);
    value.Format("%d %d %.2f ", max, min, avg);
    *pInfo += value; 

    value.Format("%.2f ", pBasicSettings->blackRatioReceiveRange);
    *pInfo += value; 

    // NUMBER OF ACTIVE/TOTAL BASE STATIONS 
    value.Format("%d %d ", pBasicSettings->bsConfig.bsActive, pBasicSettings->bsConfig.bsCount);
    *pInfo += value; 

    return status;
}

int CSensorEnv::PrintBasicDeployInfo() {
    int     status = STAT_OK;
    DWORD   min;
    DWORD   max;
    float   avg;
    float   avgWhiteNeighBeforeBlack = 0;
    #define MAX_NUM_NEIGH   1000
    float distWhite[MAX_NUM_NEIGH];
    memset(distWhite, 0, MAX_NUM_NEIGH * sizeof(float));
    CString message;
    CString message2;

    // COMPUTE STATISTICS

    pSensorSim->m_sensorEnv.CalcWhiteNeighStats(&max, &min, &avg, &avgWhiteNeighBeforeBlack, distWhite);
    message.Format("White neighbours -- max: %d | min: %d | avg: %0.1f | avgWhiteBeforeBlack: %0.3f", max, min, avg, avgWhiteNeighBeforeBlack);
    LOG_INSERTSTRING(message);

    message = "";
    for (int i = 1; i < (int) avg; i++) {
        message2.Format("[%d:%0.2f] ", i, distWhite[i]);            
        message += message2;
    }
    LOG_INSERTSTRING(message);

    pSensorSim->m_sensorEnv.CalcBlackNeighStats(&max, &min, &avg);
    message.Format("Black neighbours -- max: %d | min: %d | avg: %0.1f", max, min, avg);
    LOG_INSERTSTRING(message);
    
    return status;
}

int CSensorEnv::ExecuteTestsByScript(CString filePath) {
    int                 status = STAT_OK;
    BASIC_INIT_DATA     basicSettings;
    TEST_THREAD_PARAM*  pThreadParam = NULL;
         
    // LOAD FROM SCRIPT INTO BASIC_INIT_DATA STRUCTURE 
    basicSettings.clear();
    if (status == STAT_OK) {
        status = LoadConfigScript(filePath, &basicSettings);
    }
        
    // EXECUTE BY LOADED SETTINGS
    if (status == STAT_OK) {
        status = ExecuteTestsBySettings(&basicSettings);
    }

    return status;
}

int CSensorEnv::ExecuteTestsBySettings(BASIC_INIT_DATA* pBasicSettings) {
    int                 status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = NULL;
         
    // INIT RANDOM GENERATOR
    if (status == STAT_OK) {
        status = InitRandomGenerator(pBasicSettings);
    }

    // DEPLOY SENSORS
    if (status == STAT_OK) {
        status = DeploySensors(pBasicSettings);
    }

    // RUN REQUIRED TESTS
    if (status == STAT_OK) {
        LOG_INSERTSTRING("Starting required tests");
        if (pBasicSettings->testType & FLAG_TESTTYPE_FIXEDTRANSMRANGE) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteFixedTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLETRANSMRANGE) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteVariableRangeTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteVariableBWRatioTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteVariableBlackReceiveRatioTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteVariableComprNodesTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EAPROTOCOLSEARCH) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEAProtocolSearchTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EABLACKPATTERNSEARCH) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEABlackPatternSearchTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EAWHITEPATTERNSEARCH) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEAWhitePatternSearchTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EABLACKWHITEEVOLUTIONSEARCH) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEABlackWhitePatternSearchTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EVOLVECIRCUIT) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEvolveCircuitTest(pThreadParam);
        }
        if (pBasicSettings->testType & FLAG_TESTTYPE_EVOLVENODEPICK) {
            pThreadParam = new TEST_THREAD_PARAM;
            pThreadParam->basicSettings = *pBasicSettings;
            ExecuteEvolveNodePickTest(pThreadParam);
        }
    }
    
    return status;
}

int CSensorEnv::DistributeEqualKeyStatusRandomly(BASIC_INIT_DATA* pBasicSettings, DWORD* pSecured, DWORD* pCompr, DWORD distribFlags) {
    //
    //  RESULT OF ORIGINAL KEY DISTRIBUTION IS EXPECTED IN pSecured AND pCompr
    //
    int                 status = STAT_OK;
    BASIC_INIT_DATA     tempBasicSettings = *pBasicSettings;

    // CALCULATE ORIGINAL PART OF COMPROMISSED KEYS
    tempBasicSettings.distribKeysCompr = (*pCompr / (float) ((*pCompr + *pSecured))) * 100;
    // PERFORM RANDOM DISTRIBUTION
    *pSecured = *pCompr = 0;
    ClearLinkKeysStatus();
    status = DistributeKeyStatusRandomly(&tempBasicSettings, pSecured, pCompr, distribFlags, TRUE);

    return status;
}


UINT CSensorEnv::ExecuteFixedTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    DWORD       exp;
    CString     value;
    CString     cfgScriptPath;

    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        

        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        // SAVE FLAG_WHATSAVE_COMPROMISSEDBYSTATS
        pSensorSim->m_sensorEnv.ExecuteTests(&(pThreadParam->basicSettings), pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}


UINT CSensorEnv::ExecuteVariableRangeTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, &(pThreadParam->basicSettings.whiteTransmRange), NULL);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;

/*
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    double  range = 0;
    CFile   file;
    DWORD   exp;
    CString value;
    CString fileName;
    BASIC_INIT_DATA basicSettings; 
        
    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        // INCREASE FLUENTLY TRANSIMITION RANGE
        for (range = pThreadParam->basicSettings.batchTest.rangeFrom; range <= pThreadParam->basicSettings.batchTest.rangeTo; range += pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }        

            // SET ACTUAL RANGE
            pThreadParam->basicSettings.whiteTransmRange = (float) range;

            value.Format(" range %.2f", range); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&(pThreadParam->basicSettings), pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
/**/
}


UINT CSensorEnv::ExecuteVariableBWRatioTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    
    // NUMBER OF WHITE NODES IS FIXED, SLIDE OVER BLACK NUM
    pThreadParam->basicSettings.batchTest.rangeFrom = (DWORD) (pThreadParam->basicSettings.batchTest.rangeFrom * (float) pThreadParam->basicSettings.whiteNum);           
    pThreadParam->basicSettings.batchTest.rangeTo = (DWORD) (pThreadParam->basicSettings.batchTest.rangeTo * (float) pThreadParam->basicSettings.whiteNum);           
    pThreadParam->basicSettings.batchTest.rangeStep = (DWORD) (pThreadParam->basicSettings.batchTest.rangeStep * (float) pThreadParam->basicSettings.whiteNum);           

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, NULL, &(pThreadParam->basicSettings.blackNum));

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    return status;


/*
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    double  ratio = 0;
    CFile   file;
    DWORD   exp;
    CString value;
    CString fileName;
    BASIC_INIT_DATA basicSettings; 
        
    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        // INCREASE FLUENTLY TRANSIMITION RANGE
        for (ratio = pThreadParam->basicSettings.batchTest.rangeFrom; ratio <= pThreadParam->basicSettings.batchTest.rangeTo; ratio += pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }        

            // SET ACTUAL NUMBER OF BLACK NODES 
            pThreadParam->basicSettings.blackNum = (DWORD) (pThreadParam->basicSettings.whiteNum * ratio);

            value.Format(" ratio %.2f", ratio); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&(pThreadParam->basicSettings), pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
/**/
}

UINT CSensorEnv::ExecuteVariableBlackReceiveRatioTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // !!!! BUGBUG: FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO FLAG IGNORED 

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, &(pThreadParam->basicSettings.blackRatioReceiveRange), NULL);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;

/*
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    double  ratio = 0;
    CFile   file;
    DWORD   exp;
    CString value;
    CString fileName;
    BASIC_INIT_DATA basicSettings; 
        
    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        // INCREASE FLUENTLY TRANSIMITION RANGE
        for (ratio = pThreadParam->basicSettings.batchTest.rangeFrom; ratio <= pThreadParam->basicSettings.batchTest.rangeTo; ratio += pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }    
            
            basicSettings = pThreadParam->basicSettings;    

            // SET ACTUAL BLACK NODE RECEVIVE SENSITIVITY RATIO
            basicSettings.blackRatioReceiveRange = (float) ratio;
            // IF W/B AND RECEIVE SENSITIVITY RATIO IS BINDED, THAN ADAPT NUMBER OF BLACK NODES
            if (pThreadParam->basicSettings.distribFlags & FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO) basicSettings.blackNum = (DWORD) (pThreadParam->basicSettings.blackNum / ratio);

            value.Format(" black receive ratio %.2f", ratio); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&basicSettings, pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
/**/
}


UINT CSensorEnv::ExecuteVariableTotalBSNumberTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    pThreadParam->basicSettings.bsConfig.bsActive = 1;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, NULL, &(pThreadParam->basicSettings.bsConfig.bsCount));

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;

/*
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    int     totalBS = 0;
    CFile   file;
    DWORD   exp;
    CString value;
    CString fileName;
    BASIC_INIT_DATA basicSettings; 

    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        //
        //  VARIABLE NUMBER OF TOTAL BS WITH 1 ACTIVE
        //

        // INCREASE NUMBER OF BS
        for (totalBS = (int) pThreadParam->basicSettings.batchTest.rangeFrom; totalBS <= (int) pThreadParam->basicSettings.batchTest.rangeTo; totalBS += (int) pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }    
            
            basicSettings = pThreadParam->basicSettings;    

            // SET NUMBER OF TOTAL BS
            basicSettings.bsConfig.bsCount = totalBS;
            // SET ACTIVE 
            basicSettings.bsConfig.bsActive = 1;

            value.Format(" Base station (active/total) %d/%d", basicSettings.bsConfig.bsActive, basicSettings.bsConfig.bsCount); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&basicSettings, pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    return status;
/**/
}
   

UINT CSensorEnv::ExecuteVariableActiveBSNumberTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, NULL, &(pThreadParam->basicSettings.bsConfig.bsActive));

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;

/*
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    DWORD       activeBS = 0;
    CFile       file;
    DWORD       exp;
    CString     value;
    CString     fileName;
    BASIC_INIT_DATA basicSettings; 


    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        //
        //  VARIABLE NUMBER OF TOTAL BS WITH 1 ACTIVE
        //

        // INCREASE NUMBER OF ACTIVE BS
        for (activeBS = (DWORD) pThreadParam->basicSettings.batchTest.rangeFrom; activeBS <= (DWORD) pThreadParam->basicSettings.batchTest.rangeTo; activeBS += (DWORD) pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }    
            
            basicSettings = pThreadParam->basicSettings;    

            // SET NUMBER OF TOTAL BS EQUAL ACTIVE IF ACTIVE IS BIGGER
            if (basicSettings.bsConfig.bsCount < activeBS) basicSettings.bsConfig.bsCount = activeBS;
            // SET ACTIVE 
            basicSettings.bsConfig.bsActive = activeBS;

            value.Format(" Base station (active/total) %d/%d", basicSettings.bsConfig.bsActive, basicSettings.bsConfig.bsCount); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&basicSettings, pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    return status;
/**/
}

UINT CSensorEnv::ExecuteVariableComprNodesTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
    DWORD       comprNodes = 0;
    CFile       file;
    DWORD       exp;
    CString     value;
    CString     fileName;
    BASIC_INIT_DATA basicSettings; 
    FOE_TEST_RESULTS    foeResults;
    FOE_MESSAGE_LIST_NODE*  pMessageProfile;
    FOE_MESSAGE_LIST_NODE*  pMessageProfile2;
    CString             message;
    DWORD               secured = 0;
    DWORD               compr = 0;
    DWORD               pathsFound = 0;
    DWORD               pathsNotFound = 0;
    CString             foe;
    CString             header;
    CString             hopsFileName;
    int                 i;
    INT_DATA_BLOB       countHopsToBS;
    INT_DATA_BLOB       countHopsToCapture;
    INT_DATA_BLOB       optPoolSizes;
    DWORD               optPoolSizesIndex;

    // ALLOCATE AND CLEAR FIELDS FOR HOPS DISTRIBUTION STATISTICS
    countHopsToBS.dwMaxLen = countHopsToBS.dwActLen = 10000;
    countHopsToCapture.dwMaxLen = countHopsToCapture.dwActLen = 10000;
    optPoolSizes.dwMaxLen = optPoolSizes.dwActLen = 10000;
    countHopsToBS.pData = new int[countHopsToBS.dwMaxLen];
    countHopsToCapture.pData = new int[countHopsToCapture.dwMaxLen];
    optPoolSizes.pData = new int [optPoolSizes.dwMaxLen];
    memset(countHopsToBS.pData, 0, countHopsToBS.dwMaxLen * sizeof(DWORD));
    memset(countHopsToCapture.pData, 0, countHopsToCapture.dwMaxLen * sizeof(DWORD));
    memset(optPoolSizes.pData, 0, optPoolSizes.dwMaxLen * sizeof(DWORD));

    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);


        //
        //  DELETE FILES USED FOR RESULTS
        //
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_FOE) {
            // CREATE DIRECTORY
            fileName.Format("%s\\range%0.2f\\", pThreadParam->basicSettings.batchTest.outputPath, pThreadParam->basicSettings.whiteTransmRange);
            Dir_Create(fileName);

            // DELETE TARGET FILE 
            fileName.Format("%s\\range%0.2f\\foe_NCR_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, pThreadParam->basicSettings.whiteTransmRange, exp);
            DeleteFile(fileName);

        }

        //
        // LOAD OPTIMAL POOL SIZES IF REQUIRED
        //
        if (pThreadParam->basicSettings.batchTest.methodsFlags & FLAG_METHOD_USEOPTIMALPOOLSIZE) {
            // TRY TO SEARCH AND LOAD DATA FROM FILE WITH NAME 'optimalPoolSizes.dat'
            status = Array_LoadFromFile("optimalPoolSizes.dat", &optPoolSizes);
        }

        basicSettings = pThreadParam->basicSettings;    

        //
        //  DEPLOY SENSORS 
        //

        // DEPLOY SENSORS, BASE STATIONS AND ESTABLISH LINK KEYS
        status = pSensorSim->m_sensorEnv.ExecuteTests(&basicSettings, pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);

        // RESULTS OFFSET
        header = "";
        value.Format("%d ", RESULTSOFFSET_FOE);
        header += value; 

        if (status == STAT_OK) {
            // CLEAR FIELDS FOR HOPS DISTRIBUTION STATISTICS
            memset(countHopsToBS.pData, 0, countHopsToBS.dwMaxLen * sizeof(DWORD));
            memset(countHopsToCapture.pData, 0, countHopsToCapture.dwMaxLen * sizeof(DWORD));

            // USE ONLY ONE BS PER NODE
            basicSettings.bsConfig.bsActive = 1;

            // PREPARE NODES ON PATH FOR EACH POSSIBLE MESSAGE FROM EACH NODE TO BS
            if (status == STAT_OK) {
                pMessageProfile = new FOE_MESSAGE_LIST_NODE;
                status = pSensorSim->m_sensorEnv.FOE_PrepareMessagesPath(&basicSettings, pMessageProfile, &pathsFound, &pathsNotFound, &countHopsToBS, &countHopsToCapture); 
            }

            foeResults.hopsToCaptureCounts.dwMaxLen = 10000; // no more than 10000 hops assumed
            foeResults.hopsToCaptureCounts.pData = new int[foeResults.hopsToCaptureCounts.dwMaxLen];
            memset(foeResults.hopsToCaptureCounts.pData, 0, sizeof(int) * foeResults.hopsToCaptureCounts.dwMaxLen);
            foeResults.hopsToBSCounts.dwMaxLen = 10000; // no more than 10000 hops assumed
            foeResults.hopsToBSCounts.pData = new int[foeResults.hopsToBSCounts.dwMaxLen];
            memset(foeResults.hopsToBSCounts.pData, 0, sizeof(int) * foeResults.hopsToBSCounts.dwMaxLen);
            foeResults.numComprOnHop.dwMaxLen = 10000; // no more than 10000 hops assumed
            foeResults.numComprOnHop.pData = new int[foeResults.numComprOnHop.dwMaxLen];
            memset(foeResults.numComprOnHop.pData, 0, sizeof(int) * foeResults.numComprOnHop.dwMaxLen);
            foeResults.numAppliedKeysCounts.dwMaxLen = basicSettings.egScheme.keyRingSize + 1; // no more than keyRingSize keys can be applied
            foeResults.numAppliedKeysCounts.pData = new int[foeResults.numAppliedKeysCounts.dwMaxLen];
            memset(foeResults.numAppliedKeysCounts.pData, 0, sizeof(int) * foeResults.numAppliedKeysCounts.dwMaxLen);


            // SAVE HOPS DISTRIBUTION STATISTICS
            hopsFileName.Format("%s\\range%0.2f\\foe_hopsStats_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, pThreadParam->basicSettings.whiteTransmRange, exp);
            if (file.Open(hopsFileName, CFile::modeReadWrite | CFile::modeCreate)) {
                DWORD max = (countHopsToBS.dwMaxLen > countHopsToCapture.dwMaxLen) ? countHopsToBS.dwMaxLen : countHopsToCapture.dwMaxLen; 
                DWORD   max1 = 0;
                DWORD   max2 = 0;
                // FIND MAXIMUM NON-ZERO COUNTER
                for (i = 0; i < max; i++) {
                    if (i < countHopsToBS.dwMaxLen) if (countHopsToBS.pData[i] > 0) max1 = i;      
                    if (i < countHopsToCapture.dwMaxLen) if (countHopsToCapture.pData[i] > 0) max2 = i;      
                }

                
                max = (max1 > max2) ? max1 : max2; 
                for (i = 0; i < max; i++) {
                    foe.Format("%d ", i);
                    if (i < countHopsToBS.dwMaxLen) value.Format("%d ", countHopsToBS.pData[i]);
                    else value.Format("0 ");
                    foe += value;
                    if (i < countHopsToCapture.dwMaxLen) value.Format("%d ", countHopsToCapture.pData[i]);
                    else value.Format("0 ");
                    foe += value;
                    foe += "\r\n";

                    file.Write((LPCTSTR) foe, foe.GetLength());
                }

                message.Format("FOE hops statistics -> '%s'", hopsFileName);
                LOG_INSERTSTRING(message);

                file.Close();
            }

            // SAVE RESULTS FROM FOE_PrepareMessagesPath
            //message.Format("FOE -- secure links: %d | compr. links: %d | foundPaths: %d | notFoundPaths: %d", secured, compr, pathsFound, pathsNotFound);
            //LOG_INSERTSTRING(message);
            value.Format("%d %d %d %d ", secured, compr, pathsFound, pathsNotFound);
            header += value;

            // CLEAR OLD PREDISTRIB. KEYS
            // PERFORM RANDOM-PREDISTRIB BASED ON SETTINGS
            if (status == STAT_OK) {
                status = pSensorSim->m_sensorEnv.KeyPredistrib_ClearKeys(&basicSettings);
                status = pSensorSim->m_sensorEnv.KeyPredistrib_EGScheme(&basicSettings);
            }

            // COMPUTE APPLIED KEYS OVER EACH MESSAGE
            if (status == STAT_OK) {
                status = pSensorSim->m_sensorEnv.FOE_MessageComputeAppliedKeys(&basicSettings, pMessageProfile);
            }

            //
            // PERFORM TESTS WITH VARIABLE NUMBER OF KEYS ETC.
            //
            optPoolSizesIndex = 0;
            // INCREASE NUMBER OF COMPROMISSED NODES
            for (comprNodes = (DWORD) pThreadParam->basicSettings.batchTest.rangeFrom; comprNodes <= (DWORD) pThreadParam->basicSettings.batchTest.rangeTo; comprNodes += (DWORD) pThreadParam->basicSettings.batchTest.rangeStep) { 
                // CHECK SOFT THREAD INTERRUPTION
                if (pSensorSim->m_stopThreadSoft) {
                    LOG_INSERTCLEARLINE
                    value.Format("!!! Soft thread interruption detected. Stopping now."); 
                    LOG_INSERTSTRING(value);
                    return 0;
                }    

                // PERFORM KEY PREDISTRIBUTION - ONLY WHEN CHANGING POOL SIZE
                if (pThreadParam->basicSettings.batchTest.methodsFlags & FLAG_METHOD_USEOPTIMALPOOLSIZE) {
                    // SET NEW OTIMAL POOL SIZE
                    basicSettings.egScheme.keyPoolSize = optPoolSizes.pData[optPoolSizesIndex];
                    optPoolSizesIndex++;

                    // CLEAR OLD PREDISTRIB. KEYS
                    // PERFORM RANDOM-PREDISTRIB BASED ON SETTINGS
                    if (status == STAT_OK) {
                        status = pSensorSim->m_sensorEnv.KeyPredistrib_ClearKeys(&basicSettings);
                        status = pSensorSim->m_sensorEnv.KeyPredistrib_EGScheme(&basicSettings);
                    }

                    // COMPUTE APPLIED KEYS OVER EACH MESSAGE
                    if (status == STAT_OK) {
                        status = pSensorSim->m_sensorEnv.FOE_MessageComputeAppliedKeys(&basicSettings, pMessageProfile);
                    }
                }

                foe = header;

                if ((status = pSensorSim->m_sensorEnv.FOE_Test(&basicSettings, pMessageProfile, comprNodes, &foeResults)) == STAT_OK) {
                    //message.Format("FOE -- hopsToBS: %0.1f | hopsToCapture: %0.1f | poolSize: %d | ringSize: %d", foeResults.avgHopsToBS, foeResults.avgHopsToCapture, basicSettings.egScheme.keyPoolSize, basicSettings.egScheme.keyRingSize);
                    //LOG_INSERTSTRING(message);
                    value.Format("%0.1f %0.1f %d %d ", foeResults.avgHopsToBS, foeResults.avgHopsToCapture, basicSettings.egScheme.keyPoolSize, basicSettings.egScheme.keyRingSize);
                    foe += value;

                    //message.Format("FOE -- comprRatio: %0.1f | sec.msg %d | compr.msg %d | compr. nodes: %d | compr. keys: %d", foeResults.comprMsg / (float) (foeResults.secureMsg + foeResults.comprMsg), foeResults.secureMsg, foeResults.comprMsg, foeResults.comprNodes, foeResults.comprKeys);
                    //LOG_INSERTSTRING(message);
                    value.Format("%d %d %0.3f %d %d %d %d ", foeResults.notCapturedMsg, foeResults.capturedMsg, foeResults.comprMsg / (float) (foeResults.secureMsg + foeResults.comprMsg), foeResults.secureMsg, foeResults.comprMsg, foeResults.comprNodes, foeResults.comprKeys);
                    foe += value;
                }

                // SAVE FOE RESULTS
                foe += "\r\n";
                if (file.Open(fileName, CFile::modeReadWrite | CFile::modeCreate | CFile::modeNoTruncate)) {
                    file.SeekToEnd();
                    file.Write((LPCTSTR) foe, foe.GetLength());
                    file.Close();
                } 

            }

            message.Format("FOE compr. nodes -> '%s'", fileName);
            LOG_INSERTSTRING(message);

            // CLEAR PROFILES OF MESSAGES
            while (pMessageProfile != NULL) {
                pMessageProfile2 = pMessageProfile->next;
                pMessageProfile->release();
                delete pMessageProfile;
                
                pMessageProfile = pMessageProfile2; 
            }
        }
    }   

    LOG_INSERTSTRING("finished.");

    delete[] countHopsToBS.pData;
    delete[] countHopsToCapture.pData;
    delete[] foeResults.hopsToCaptureCounts.pData;
    delete[] foeResults.hopsToBSCounts.pData;
    delete[] foeResults.numComprOnHop.pData;
    delete[] foeResults.numAppliedKeysCounts.pData;

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;
	pThreadParam = NULL;

    return status;
}

UINT CSensorEnv::ExecuteVariableWhiteNodesTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, NULL, &(pThreadParam->basicSettings.whiteNum), FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}

UINT CSensorEnv::ExecuteVariableDropCheckProbTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, &(pThreadParam->basicSettings.neighLeash.neighTransmitCheckProb), NULL, FLAG_TESTTYPE_VARIABLEDROPCHECKPROB);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}

UINT CSensorEnv::ExecuteVariableComprFractTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = ExecuteVariableTest(pThreadParam, &(pThreadParam->basicSettings.neighLeash.fractComprNodes), NULL, FLAG_TESTTYPE_VARIABLECOMPRNODES);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}

UINT CSensorEnv::ExecuteGABPEvolve(LPVOID pParam) {
    UINT        status = STAT_OK;
    GABP_THREAD_PARAM* pThreadParam = (GABP_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
    status = pSensorSim->m_sensorEnv.GABP_Evolve(pThreadParam->pGAStrategy, pThreadParam->bBlackPattern);


    //  RELEASE STRUCTS
    if (pThreadParam->pGAStrategy->pGenom) {
        delete pThreadParam->pGAStrategy->pGenom;
        pThreadParam->pGAStrategy->pGenom = NULL;
    }       
    if (pThreadParam->pGAStrategy->pGA) {
        delete pThreadParam->pGAStrategy->pGA;
        pThreadParam->pGAStrategy->pGA = NULL;
    }

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}


UINT CSensorEnv::ExecuteEAProtocolSearchTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;
	BASIC_INIT_DATA *pBasicSettings = &pThreadParam->basicSettings;
	CString message;
	
	// DEPLOY
	CKeyInfectTab::DeploySensors(pBasicSettings);

	// LOAD POPULATION
	if (!pSensorSim->m_defaultPopulationFile.IsEmpty()){
		if ((status = pSensorSim->m_sensorEnv.GAS_LoadFromFile(pSensorSim->m_defaultPopulationFile, pSensorSim->m_pGAStrategy)) == STAT_OK) {
            message.Format("Default population loaded from '%s'", pSensorSim->m_defaultPopulationFile);
			LOG_INSERTSTRING(message);
 		}
		else {
			message.Format("Default population could not be loaded from '%s' (error %d(%s))", pSensorSim->m_defaultPopulationFile, status, ERROR_TO_STRING(status));
			LOG_INSERTSTRING(message); 
		}
	}

    // RUN
	status = CKeyInfectTab::RunGAS(NULL);

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

	return status;
}

UINT CSensorEnv::ExecuteEABlackPatternSearchTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
	status = pSensorSim->m_sensorEnv.GABP_Evolve(&(pThreadParam->basicSettings.gaConfig), TRUE);

    //  RELEASE STRUCTS
    if (pThreadParam->basicSettings.gaConfig.pGenom) {
        delete pThreadParam->basicSettings.gaConfig.pGenom;
        pThreadParam->basicSettings.gaConfig.pGenom = NULL;
    }       
    if (pThreadParam->basicSettings.gaConfig.pGA) {
        delete pThreadParam->basicSettings.gaConfig.pGA;
        pThreadParam->basicSettings.gaConfig.pGA = NULL;
    }

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

	return status;
}

UINT CSensorEnv::ExecuteEAWhitePatternSearchTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // EXECUTE VARIABLE TEST
	status = pSensorSim->m_sensorEnv.GABP_Evolve(&(pThreadParam->basicSettings.gaConfig), FALSE);

    //  RELEASE STRUCTS
    if (pThreadParam->basicSettings.gaConfig.pGenom) {
        delete pThreadParam->basicSettings.gaConfig.pGenom;
        pThreadParam->basicSettings.gaConfig.pGenom = NULL;
    }       
    if (pThreadParam->basicSettings.gaConfig.pGA) {
        delete pThreadParam->basicSettings.gaConfig.pGA;
        pThreadParam->basicSettings.gaConfig.pGA = NULL;
    }

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

	return status;
}

UINT CSensorEnv::ExecuteEABlackWhitePatternSearchTest(LPVOID pParam) {
    UINT        status = STAT_OK;
    TEST_THREAD_PARAM* pThreadParam = (TEST_THREAD_PARAM*) pParam;

    // TODO: NOT IMPLEMENTED YET
    ASSERT(FALSE);

    // ...

    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

	return status;
}

UINT CSensorEnv::ExecuteEvolveCircuitTest(LPVOID pParam) {
    UINT                status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = (TEST_THREAD_PARAM*) pParam;
    

    CEACircuit::RunGACircuit(pThreadParam);

    
	return status;
}
UINT CSensorEnv::ExecuteEvolveNodePickTest(LPVOID pParam) {
    UINT        status = STAT_OK;
	return status;
}

UINT CSensorEnv::ExecuteVariableTest(TEST_THREAD_PARAM* pThreadParam, float* pFloatVariable, DWORD* pDWORDVariable, DWORD flags) {
    int     status = STAT_OK;
    double  range = 0;
    CFile   file;
    DWORD   exp;
    CString value;
    CString fileName;
    BASIC_INIT_DATA basicSettings; 
        
    // RUN m_batchRepeat-TIMES
    for (exp = 1; exp <= pThreadParam->basicSettings.batchTest.batchRepeat; exp++) {
        // CHECK SOFT THREAD INTERRUPTION
        if (pSensorSim->m_stopThreadSoft) {
            LOG_INSERTCLEARLINE
            value.Format("!!! Soft thread interruption detected. Stopping now."); 
            LOG_INSERTSTRING(value);
            return 0;
        }        
        
        LOG_INSERTSEPARATOR;
        value.Format("Batch run %d", exp); 
        LOG_INSERTSTRING(value);

        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPRATIOALLMETHODS) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\comprRatioNeigh_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_MMD) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\mmd_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_AMPLIFICATIONIMPACT) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\amplifImpact_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }
        if (pThreadParam->basicSettings.batchTest.whatSaveFlags & FLAG_WHATSAVE_NEIGHBLEASH) {
            // DELETE TARGET FILE 
            fileName.Format("%s\\nLeash_%d.dat", pThreadParam->basicSettings.batchTest.outputPath, exp);
            DeleteFile(fileName);
        }

        // ITERATE OVER GIVEN RANGE
        for (range = pThreadParam->basicSettings.batchTest.rangeFrom; range <= pThreadParam->basicSettings.batchTest.rangeTo; range += pThreadParam->basicSettings.batchTest.rangeStep) { 
            // CHECK SOFT THREAD INTERRUPTION
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                return 0;
            }        

            // SET ACTUAL RANGE BASED ON TYPE
            if (pFloatVariable) *pFloatVariable = (float) range;
            if (pDWORDVariable) *pDWORDVariable = (DWORD) range;

            if (flags & FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT) {
                // RECOMPUTE NUMBER OF BLACK NODES 
                if (pThreadParam->basicSettings.blackCountType == 1) {
                    pThreadParam->basicSettings.blackNum = (DWORD) (pThreadParam->basicSettings.whiteNum * pThreadParam->basicSettings.blackRatioWhite);
                }
            }
             

            value.Format(" variable value: %.2f ---> ", range); 
            LOG_INSERTSTRING(value);

            // EXECUTE TESTS
            pSensorSim->m_sensorEnv.ExecuteTests(&(pThreadParam->basicSettings), pThreadParam->basicSettings.batchTest.whatSaveFlags, pThreadParam->basicSettings.batchTest.outputPath, exp);
        }
    }

    LOG_INSERTSTRING("finished.");

    return status;
}



int CSensorEnv::Array_Add(INT_DATA_BLOB* pArray, INT_DATA_BLOB* pPlusArray) {
    return Array_Add(pArray, pPlusArray, 0, pPlusArray->dwActLen - 1);
}

int CSensorEnv::Array_Add(INT_DATA_BLOB* pArray, INT_DATA_BLOB* pPlusArray, int plusStartOffset, int plusEndOffset) {
    int     status = STAT_OK;
    int     i;
    int     limit; 

    // ADD UP TO LOWER LENGTH OF ARRAYS
    limit = (pPlusArray->dwActLen < pArray->dwActLen) ? pPlusArray->dwActLen : pArray->dwActLen;
    if (limit > (plusEndOffset - plusStartOffset - 1)) limit = plusEndOffset - plusStartOffset - 1;
    // ADD pPlusArray TO pArray
    for (i = 0; i < limit; i++) {
        pArray->pData[i] += pPlusArray->pData[i + plusStartOffset];
    }

    return status;
}

int CSensorEnv::Array_LoadFromFile(CString filePath, INT_DATA_BLOB* pBlob) {
    int     status = STAT_OK;
    int     radek=1;
    int     availableValues = 0;
    DWORD   readed = 0;
	FILE    *file; 
  	char    line[1024];
    
    if ((file = fopen((LPCTSTR) filePath,"rt")) != NULL) { 
	    readed = 0;
        while(fgets(line,sizeof(line),file))  {
		    // LOAD VALUE ONLY IF BEFORE END OF SUPPLIED ARRAY
            if (readed < pBlob->dwMaxLen) {
                pBlob->pData[readed] = atoi(line);
                readed++;
            }
            availableValues++;
	    }
	    fclose(file);
    }
    else status = STAT_FILE_OPEN_FAIL;

    if (status == STAT_OK) {
        // SET ERROR STATUS IF ARRAY TOO SHORT 
        if (availableValues > pBlob->dwMaxLen) status = STAT_DATA_TOO_LONG;
        // SET NUMBER OF READED VALUES 
        if (availableValues > readed) pBlob->dwActLen = availableValues;
        else pBlob->dwActLen = readed;
    }

    return status;
}

DWORD CSensorEnv::Fact(DWORD value) {
	int temp;

	if (value <= 1) return 1;

	temp = value * Fact(value - 1);
	return temp;
}
//ROUTING 
//JK
int CSensorEnv::Routing_Test(BASIC_INIT_DATA* pBasicSettings)
{    
	int status=STAT_OK;
    DWORD time = 100000; //INIT TIME COUNTER 
	DWORD numSensors = m_pActualDeployment->whiteSensorList.numSensors;
	DWORD numBS = m_pActualDeployment->BSList.itemCount;

	int data = 0;

	//ROUTE STRAT
	for(int i=0;i<numSensors;i++) {	
		CmcfRouteStrat* rs = new CmcfRouteStrat(m_pActualDeployment->whiteSensorList.sensors[i],m_pActualDeployment->whiteSensorList.sensors[i]->whiteNeigh,m_pActualDeployment->whiteSensorList.sensors[i]->blackNeigh);
		m_pActualDeployment->whiteSensorList.sensors[i]->setRouteStrat(rs);
	}


    //START ROUTING
	while (time>0) {
		time--;          //DECREASE TIME COUNTER
        
		//GENERATE MESSAGES
		int tmp = data;
		status = GenerateMessages(pBasicSettings,&tmp);
		if (tmp==-1){   //data were used, update data
			data++;
		}
		
		//PROCESS ALL NODES
		for(int i=0;i<numSensors;i++) {
			m_pActualDeployment->whiteSensorList.sensors[i]->pRouteStrat->timeProcess();	
		}

		for(int i=0;i<numSensors;i++) {
			m_pActualDeployment->whiteSensorList.sensors[i]->pRouteStrat->incomingToSend();	
		}


	}
    //END ROUTING
	return status;
}
//JK
int CSensorEnv::GenerateMessages(BASIC_INIT_DATA* pBasicSettings, int* data) {

#define CHANCE 100   //what is the chance, that the message will be generated

	int   rand;
	DWORD rand2;
	
	m_rndGen.GetRandomFromInterval(CHANCE,&rand2);
	if (rand2 == 0) {
		m_rndGen.GetRandomFromInterval(pBasicSettings->whiteNum,&rand);
		m_pActualDeployment->whiteSensorList.sensors[rand]->pRouteStrat->generateEvent(*data);
		(*data) = -1;
	}
	

	return STAT_OK;
}

//JK
void garoute_initializer(GAGenome &g) {
    GA1DArrayGenome<DWORD> &genome = (GA1DArrayGenome<DWORD>&) g;
    

	//RANDOM INIT
	for (int slot=0;slot<gaRoute.genomeSize;slot++) {
		DWORD value;
		pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(gaRoute.numRules, &value);
		genome.gene(slot,value);
	}
}
//JK

int garoute_mutator(GAGenome &g, float pmut) {
	float result = 0;

    return result;
}
//JK
int garoute_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2) {
    GA1DArrayGenome<DWORD> &parent1 = (GA1DArrayGenome<DWORD>&) p1;
    GA1DArrayGenome<DWORD> &parent2 = (GA1DArrayGenome<DWORD>&) p2;
    GA1DArrayGenome<DWORD> &offspring1 = (GA1DArrayGenome<DWORD>&) *o1;
    GA1DArrayGenome<DWORD> &offspring2 = (GA1DArrayGenome<DWORD>&) *o2;
    

	DWORD value;
	pSensorSim->m_sensorEnv.m_rndGen.GetRandomFromInterval(gaRoute.genomeSize, &value);
    int cpoint = (int) value;
	for (int i=0;i<gaRoute.genomeSize;i++) {
        if (i <= cpoint) {
                if (o1 != NULL) offspring1.gene(i, parent1.gene(i));
                if (o2 != NULL) offspring2.gene(i, parent2.gene(i));
        }
		else {   
                if (o1 != NULL) offspring1.gene(i, parent2.gene(i));
                if (o2 != NULL) offspring2.gene(i, parent1.gene(i));
        }
    }
    return 1;
}
//JK
float garoute_fitness(GAGenome &g) {
    int                     status = STAT_OK;
	CString                 message;
    float                   fit = 0;
	/*
    GA1DArrayGenome<DWORD>   &genome = (GA1DArrayGenome<DWORD>&) g;
    BYTE                    inputs[MAX_INTERNAL_LAYER_SIZE];    
    BYTE                    outputs[MAX_INTERNAL_LAYER_SIZE];  
    BYTE                    correctOutputs[MAX_OUTPUTS];  
    int                     match = 0;
    int                     numPredictions = 0; 
    
    
    for (int testSet = gaCirc.testVectStart; testSet < (gaCirc.testVectStart + gaCirc.numTestVectors); testSet++) { 
        // LOAD TEST SET INTO inputs
        memcpy(inputs, gaCirc.testVectors[testSet], gaCirc.numInputs);
        // LOAD TEST SET INTO outputs
        memcpy(correctOutputs, gaCirc.testVectors[testSet] + MAX_INPUTS, MAX_OUTPUTS);
        
        // EXECUTE CIRCUIT
        status = CMeshTab::ExecuteCircuit(&gaCirc, &genome, inputs, outputs);
        
        
        // EVALUATE SUCCESS OF CIRCUIT FOR THIST TEST VECTOR
        CMeshTab::EvaluateCircuitSuccess(outputs, correctOutputs, gaCirc.outputLayerSize, gaCirc.predictMethod, &match, &numPredictions);
    }

    // EVALUATE SUCCESS OF CIRCUIT OVER WHOLE TEST SET
    fit = (numPredictions > 0) ? (match / (float) numPredictions) : 0;
    
    // include into average fitness of whole generation
    gaCirc.avgGenerFit += fit;
    gaCirc.numAvgGenerFit++;
    gaCirc.avgPredictions += numPredictions;

    if (fit > gaCirc.bestGenerFit) gaCirc.bestGenerFit = fit;


    if (gaCirc.maxFit < fit) {
        gaCirc.maxFit = fit;
        
        message.Format("NEW MAXIMUM FOUND: %f", fit); 
        LOG_INSERTSTRING(message); 
        
        // DISPLAY CURRENTLY BEST
        CMeshTab::PrintCircuit(genome);
    }
    */
    return fit;
}
//JK
int CSensorEnv::RunGARouteEvol(BASIC_INIT_DATA* pBasicSettings) 
{
	int status = STAT_OK;

    GAROUTE_THREAD_PARAM* pThreadParam = new GAROUTE_THREAD_PARAM;
	
	pThreadParam->basicSettings = *pBasicSettings;
	//get GA strategy
	GA_STRATEGY strategy;
    // @TODO pSensorSim->m_tabKeyInfect.GetGASettings(&strategy);
    pThreadParam->gaStrategy = strategy;
	//get GA route data
	// @TODO pSensorSim->m_tabNeighLeash.GetGARouteData(&gaRoute);
	//set values just for debug purpose
	gaRoute.genomeSize=50;	


	pThreadParam->pGARoute = &gaRoute;


	// INTERRUPT PREVIOUS THREAD
    if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

    // RESET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = FALSE;

    // RUN WORKER THREAD FOR EXECUTION
    pSensorSim->m_pActualThread = AfxBeginThread(CSensorEnv::RunGARoute, pThreadParam, THREAD_PRIORITY_LOWEST); 
	
	return status;
}


//JK evolution
UINT CSensorEnv::RunGARoute(LPVOID pParam) {
    UINT                status = STAT_OK;
    GAROUTE_THREAD_PARAM*  pThreadParam = (GAROUTE_THREAD_PARAM*) pParam;
    GA_ROUTE*			pGARoute =  (GA_ROUTE*) pThreadParam->pGARoute;  
	CString				message;


	UINT genomeSize = 100; //tostruct
    
    if (status == STAT_OK) {
        //srand((unsigned int) time(NULL));
        
        // SAVE USED VALUES
       // CString                   message;  
        //message.Format("\n\n###NumLayers: %d, InLayerSize: %d, OutLayerSize: %d, NumInputs: %d, NumOutputs: %d, NumTestSets: %d, TestSetSize: %d\n\n",pGACircuit->numLayers, pGACircuit->internalLayerSize, pGACircuit->outputLayerSize, pGACircuit->numInputs, pGACircuit->numOutputs, NUM_TEST_SETS, pGACircuit->numTestVectors); 
        //LOG_INSERTSTRING(message); 

        
        // 
        // CREATE TEST DATA
        //
       // CMeshTab::GenerateTestData(pGACircuit->testVectorGenerMethod, pGACircuit->numTestVectors, pThreadParam->limitTestVectorAlgRounds);
        
        
            
        //
        //  CREATE GA STRUCTS
        //
        GA1DArrayGenome<DWORD>    genom(genomeSize, garoute_fitness);
        GA1DArrayGenome<DWORD>    genomeTemp(genomeSize, garoute_fitness);

        genom.initializer(garoute_initializer); 
        genom.mutator(garoute_mutator); 
        genom.crossover(garoute_crossover); 

        GASteadyStateGA ga(genom);
        ga.nReplacement(2 * pThreadParam->gaStrategy.popSize / 3);  
        ga.populationSize(pThreadParam->gaStrategy.popSize); 
        ga.nGenerations(pThreadParam->gaStrategy.nGeners);
        ga.pCrossover(pThreadParam->gaStrategy.pCross);
        ga.pMutation(pThreadParam->gaStrategy.pMutt); 

        ga.initialize();


        int     actGener = 1;
        
        ///pGACircuit->clearFitnessStats();
        
        //while (actGener < pThreadParam->gaStrategy.nGeners) {
        while (TRUE) {
            // CHECk FOR SOFT INTERUPT
            if (pSensorSim->m_stopThreadSoft) {
                LOG_INSERTCLEARLINE
                CString value;
                value.Format("!!! Soft thread interruption detected. Stopping now."); 
                LOG_INSERTSTRING(value);
                // DELETE THREAD PARAM STRUCT
                delete pThreadParam;
                // SET ACTIVE THREAD PTR TO NULL
                pSensorSim->m_pActualThread = NULL;
                
                return 0;
            }        
        
            // CHOOSE RANDOM TEST SUBSET ON WHICH INDIVIDIALS WILL BE TESTED
    
            
            ///pGACircuit->testVectStart = 0;        
        
			/*
            if ((pGACircuit->testVectorChangeGener == 0) || (actGener % pGACircuit->testVectorChangeGener == 0)) {
                GenerateTestData(pGACircuit->testVectorGenerMethod, pGACircuit->numTestVectors, pThreadParam->limitTestVectorAlgRounds);
            }/**/
            
            // RESET EVALUTION FOR ALL GENOMS
            ga.pop->flushEvalution();
            ga.step(); // GA evolution step

			//PRINT PRELIMINARY RESULTS
            if (actGener % 100 == 0) {
/*                
                genomeTemp = (GA1DArrayGenome<DWORD>&) ga.statistics().bestIndividual(); 
                PrintCircuit(genomeTemp);
/**/            
				/*
                message.Format("(%d gen.): best %f, avg: %f, avgPredict: %d, totalBest: %f", actGener, pGACircuit->bestGenerFit, pGACircuit->avgGenerFit / pGACircuit->numAvgGenerFit, pGACircuit->avgPredictions / pGACircuit->numAvgGenerFit, pGACircuit->maxFit); 
                LOG_INSERTSTRING(message); 

                pGACircuit->bestGenerFit = 0;
                pGACircuit->avgGenerFit = 0;
                pGACircuit->numAvgGenerFit = 0;
                pGACircuit->avgPredictions = 0;
				*/
            }
            
            actGener++;
        }    
        
        // GENERATE FRESH NEW SET
        //GenerateTestData(pGACircuit->testVectorGenerMethod, pGACircuit->numTestVectors, pThreadParam->limitTestVectorAlgRounds);
        
		//FINAL RESULT
        genomeTemp = (GA1DArrayGenome<DWORD>&) ga.statistics().bestIndividual(); 
        float fit = garoute_fitness(genomeTemp);
        message.Format("Total best: (%d gen.): %f", actGener, fit); 
        LOG_INSERTSTRING(message); 
    }    
    
    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;

    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;

    return status;
}

/**
 * If the population is already sorted, make sure it won't be sorted again 
 * (and individuals with equal fitness won't be swapped)
 */
void CSensorEnv::GAS_presort(GAPopulation* pop){	

#ifdef _DEBUG
	std::string dump;
	pop->Serialize(&dump);
	CString fileName;
	fileName.Format("GalibStateDump_GAS_presort_before.ini");
	DeleteFile(fileName);
	pSensorSim->Log_InsertToFile(fileName, dump.c_str());
#endif
	
	// reverse order (it should be from low to high)
	for (int i=0; i<pop->size(); i++){	
		if (i >= pop->size()-i-1) break;

		GAGenome &g1 = pop->individual(i);
		GAGenome &g2 = pop->individual(pop->size()-i-1);
		pop->replace(&g1, pop->size()-i-1);
		pop->replace(&g2, i);
	}
	// check whether the population is already sorted
	BOOL isSorted = TRUE;
	float lastScore = -1;
	for (int i=0; i<pop->size(); i++){
		if (lastScore>0 && lastScore > pop->individual(i).score()){
			isSorted = FALSE;
			break;
		}
		lastScore = pop->individual(i).score();
	}
	if (isSorted) {
		pop->setSorted(gaTrue);
	}
#ifdef _DEBUG
	dump = "";
	pop->Serialize(&dump);
	fileName.Format("GalibStateDump_GAS_presort_after.ini");
	DeleteFile(fileName);
	pSensorSim->Log_InsertToFile(fileName, dump.c_str());
#endif
}