#ifndef GAS_Genome_H
#define GAS_Genome_H

#include "StdAfx.h"
#include "SSconstants.h"
#include "ga/GA1DArrayGenome.h"

typedef struct _GA_PROTOCOL_STATS {
    int totalMessages;
	float totalSecuredRatio;
	unsigned int totalSecuredKeys;
	unsigned int totalCompromisedKeys;
	unsigned int numDeployments;
	float totalWhiteNeigh;
	unsigned int insCount[GAS_INS_MAX+1];
	unsigned int insTotalCount;
	float totalOrigSecuredRatio;

    _GA_PROTOCOL_STATS(void) {
        clear();
    }

    void clear() {
        totalMessages = 0;
		totalSecuredRatio = totalOrigSecuredRatio = 0.0;
		totalSecuredKeys = totalCompromisedKeys = 0;
		numDeployments = 0;
		totalWhiteNeigh = 0;
		insTotalCount = 0;
		memset(insCount, 0, sizeof(insCount));
    }

	int getTotalMessages(){
		return totalMessages;
	}
	unsigned int getNumDeployments(){
		return numDeployments;
	}
	unsigned int getTotalKeys(){
		return getTotalSecuredKeys()+getTotalCompromisedKeys();
	}
	unsigned int getTotalSecuredKeys(){
		return totalSecuredKeys;
	}
	unsigned int getTotalCompromisedKeys(){
		return totalCompromisedKeys;
	}
	float getTotalSecuredRatio(){
		return totalSecuredRatio;
	}
	float getAverageViaDeployment(float count){
		return getNumDeployments()>0?(count / (float) (getNumDeployments())):0.0;
	}
	float getAverageMessages(){
		return getAverageViaDeployment(getTotalMessages());
	}
	float getAverageSecuredKeys(){
		return getAverageViaDeployment(getTotalSecuredKeys());
	}
	float getAverageCompromisedKeys(){
		return getAverageViaDeployment(getTotalCompromisedKeys());
	}
	float getCompromisedRatio(){
		return 1 - getSecuredRatio();
	}
	float getSecuredRatio(){
		return getAverageViaDeployment(getTotalSecuredRatio());
	}
	float getOrigSecuredRatio(){
		return getAverageViaDeployment(totalOrigSecuredRatio);
	}
	float getAverageWhiteNeigh(){
		return getAverageViaDeployment(totalWhiteNeigh);
	}	

} GA_PROTOCOL_STATS;

template <class T>
class GAS_Genome : public GA1DArrayGenome<T>{
public:
	GADeclareIdentity();	

	GAS_Genome(unsigned int length,
			GAGenome::Evaluator f=(GAGenome::Evaluator)0,
			void * u=(void *)0);
	GAS_Genome(const GAS_Genome<T>&);
	GAS_Genome<T>& operator=(const GAGenome& arr) {copy(arr); return *this;}
	GAS_Genome<T>& operator=(const T array []) {GA1DArrayGenome<T>::operator=(array); return *this;}
	virtual ~GAS_Genome();
	virtual GAGenome *clone(GAGenome::CloneMethod flag=GAGenome::CONTENTS) const;
	virtual void copy(const GAGenome &);

	GA_PROTOCOL_STATS stats;	
};

#ifdef GALIB_USE_BORLAND_INST
#include "GAS_Genome.cpp"
#endif

#endif